import Oid.*;
import java.util.*;

/** 
 * An instance of <code>Algorithm</code> describes a class that
 * performs a <code>Task</code>.  It stores the algorithm's Class (and
 * can generate instances), its menu description, name, and an
 * "applies-to" class.
 * 
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 2 Oct 2001
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class Algorithm
{
  // private data members
  private Class     mClass;
  private String    mName;
  private Task      mTask;
  private Class     mAppliesTo;
  private String    mDescription;
  private boolean   mIsValid;

  /** 
   * constructs an instance of Algorithm with the indicated class name,
   * Task, applies-to class name and description
   *
   * @param className name of class that implements algorithm
   * @param algorithmTask reference to <code>Task</code> algorithm performs
   * @param appliesToClassName name of type required for algorithm
   * @param description description
   */
  public Algorithm(String className, 
		   Task algorithmTask, 
		   String appliesToClassName, 
		   String description)
  throws ClassNotFoundException
  {
    mClass = Class.forName(className);
    mAppliesTo = Class.forName(appliesToClassName);
    mTask = algorithmTask;
    mName = new String(className);
    mDescription = new String(description);
    validateAlgorithm();
  }

  /**
   * constructs an instance of Algorithm with the indicated class, Task,
   * applies-to class and description
   * 
   * @param algorithmClass class that implements algorithm
   * @param algorithmTask reference to <code>Task</code> algorithm performs
   * @param appliesTo type required for algorithm
   * @param description description of algorithm
   */
  public Algorithm(Class algorithmClass, Task algorithmTask, Class appliesTo, 
	      String description)
  {
    mClass = algorithmClass;
    mAppliesTo = appliesTo;
    mTask = algorithmTask;
    mName = algorithmClass.getName();
    mDescription = new String(description);
    validateAlgorithm();
  }

  /** 
   * returns the name of the class for this algorithm 
   *
   * @return name of class
   */
  public String getClassName()
  {
    return mName;
  }

  /** 
   * returns the Class for this Algorithm instance 
   *
   * @return algorithm's <code>Class</code>
   */
  public Class getAlgorithmClass()
  {
    return mClass;
  }

  /** 
   * returns the Task for this Algorithm instance 
   *
   * @return algorithm's <code>Task</code>
   */
  public Task getTask()
  {
    return mTask;
  }

  /** 
   * indicates whether the Class set for this instance is valid for the 
   * Task set for this instance
   *
   * @return <code>true</code> if and only if algorithm is valid for task
   */
  public boolean isValid()
  {
    return mIsValid;
  }

  /** 
   * returns the description for this algorithm instance 
   * 
   * @return algorithm description
   */
  public String getDescription()
  {
    return mDescription;
  }

  /** 
   * returns the name of the type this Algorithm applies to 
   *
   * @return name of type algorithm applies to
   */
  public String getAppliesToName()
  {
    return mAppliesTo.getName();
  }

  /** 
   * returns <code>Class</code> reference for type this Algorithm applies to 
   *
   * @return reference to <code>Class</code>
   */
  public Class getAppliesToClass()
  {
    return mAppliesTo;
  }

  /** 
   * indicates whether this Algorithm applies to the passed Class 
   *
   * @param <code>Class</code> to test
   * @return boolean answer
   */
  public boolean doesAlgorithmApply(Class c)
  {
    return mAppliesTo.isAssignableFrom(c);
  }

  /** 
   * returns an instance of the class for this algorithm 
   * 
   * @return instance of algorithm's class
   */
  public Object getInstance()
    throws InstantiationException, IllegalAccessException  
  {
    return mClass.newInstance();
  }

  //
  // private method to validate algorithm -- its class must implement the
  // interface that its task falls under
  //
  private void validateAlgorithm()
  {
    Class[] ints = mClass.getInterfaces();
    Class category = mTask.getCategoryClass();
    int i;
    boolean answer = false;

    for (i=0; i<ints.length && !answer; i++) {
      answer = answer || ints[i].equals(category);
    }

    mIsValid = answer;
  }
}
