import Oid.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;
import java.text.*;

/**
 * Class <code>FiniteFieldMatroidTextArea</code> displays a text area,
 * an error label and a "Continue" button.  The user may enter the
 * values for a representation of a matroid over GF(q) where q is some
 * prime power into the text area, and then press "Continue" to have
 * the object parse the entries and produce an instance of
 * FiniteFieldMatroid - if the object cannot produce a matroid, it
 * displays an error message in its error label.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 4 Jun 2002
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class FiniteFieldMatroidTextArea extends JPanel 
  implements ActionListener
{
  private JTextArea          mTextArea;
  private JLabel             mErrorLabel;
  private int                mRows;
  private int                mCols;
  private FiniteField        mField;
  private FiniteFieldMatroid mMatroid;
  private String             mPrompt;
  private static String      mDefaultPrompt = "Enter the matrix below:";
  private LinkedList         mListeners;
  

  /**
   * Initializes the text area to hold a  5 by 10 matrix over GF(2)
   */
  public FiniteFieldMatroidTextArea()
  {
    mRows = 5;
    mCols = 10;
    mField = FiniteFieldFactory.createFiniteField(2,1);
    mPrompt = new String(mDefaultPrompt);
    initControls();
  }

  /** 
   * The default constructor initializes the text area to hold a 5 by 10
   * matrix.
   *
   * @param field finite field for matrix
   */
  public FiniteFieldMatroidTextArea(FiniteField field)
  {
    mRows = 5;
    mCols = 10;
    mField = field;
    mPrompt = new String(mDefaultPrompt);
    initControls();
  }

  /**
   * This constructor allows the client to specify a prompt string
   *
   * @param field finite field for matrix
   * @param prompt prompt string
   */
  public FiniteFieldMatroidTextArea(FiniteField field, String prompt)
  {
    mRows = 5;
    mCols = 10;
    mField = field;
    mPrompt = new String(prompt);
    initControls();
  }

  /**
   * This constructor allows the client to specify the dimensions of 
   * the matrix.
   *
   * @param field finite field for matrix
   * @param rows number of rows in matrix
   * @param cols number of columns in matrix
   */
  public FiniteFieldMatroidTextArea(FiniteField field, int rows, int cols)
  throws IllegalArgumentException
  {
    if ((rows <= 0) || (cols <= 0)) {
      throw new IllegalArgumentException("Invalid dimensions: " + rows +
					 " " + cols);
    }
    mRows = rows;
    mCols = cols;
    mField = field;
    mPrompt = new String(mDefaultPrompt);
    initControls();
  }

  /** 
   * This constructor allows the client to specify the dimensions of the
   * matrix, the base field for the matrix (as a reference to a FiniteField)
   * and the prompt.
   *
   * @param field finite field for matrix
   * @param rows number of rows in matrix
   * @param cols number of columns in matrix
   * @param prompt prompt string
   */
  public FiniteFieldMatroidTextArea(FiniteField field, 
				    int rows, 
				    int cols, 
				    String prompt)
  throws IllegalArgumentException
  {
    if ((rows <= 0) || (cols <= 0)) {
      throw new IllegalArgumentException("Invalid dimensions: " + rows +
					 " " + cols);
    }
    mRows = rows;
    mCols = cols;
    mField = field;
    mPrompt = new String(prompt);
    initControls();
  }
  
  /** 
   * adds an action listener to the list for this object 
   *
   * @param e <code>ActionListener</code> to add
   */
  public void addActionListener(ActionListener e)
  {
    if (!(mListeners instanceof LinkedList)) {
      mListeners = new LinkedList();
    }
    mListeners.add(e);
  }

  /** 
   * sets enabled status of text area 
   *
   * @param bValue <code>true</code> to enable text area, <code>false</code>
   * to disable it
   */
  public void setEnabled(boolean bValue)
  {
    mTextArea.setEnabled(bValue);
    if (bValue) {
      mErrorLabel.setText(mPrompt);
    }
    else {
      mErrorLabel.setText(" ");
    }
  }

  /** 
   * returns the enabled status of the text area 
   *
   * @return <code>true</code> if and only if text area is enabled
   */
  public boolean isEnabled()
  {
    return mTextArea.isEnabled();
  }

  /** 
   * returns the number of columns 
   *
   * @return number of columns in matrix
   */
  public int getColumns()
  {
    return mCols;
  }

  /** 
   * returns the number of rows 
   *
   * @return number of rows in matrix
   */
  public int getRows()
  {
    return mRows;
  }

  /** 
   * sets the number of rows and columns 
   *
   * @param r number of rows
   * @param c number of columns
   */
  public void setSize(int r, int c)
  {
    if ((r > 0) && (c > 0)) {
      mRows = r;
      mCols = c;
      mTextArea.setRows(r);
      mTextArea.setColumns(c);
    }
    else {
      throw new IllegalArgumentException("Numbers of rows and columns " + 
					 "must be positive");
    }
  }

  /**
   * sets the field
   *
   * @param field <code>FiniteField</code> reference
   */
  public void setField(FiniteField field)
  {
    mField = field;
  }

  /** 
   * returns a matroid on the matrix entered by the user
   *
   * @return <code>FiniteFieldMatroid</code> reference
   */
  public FiniteFieldMatroid getMatroid()
  {
    if (!(mMatroid instanceof FiniteFieldMatroid)) {
      mMatroid = (FiniteFieldMatroid)null;
    }
    return mMatroid;
  }

  /** 
   * returns a reference to the object's <code>FiniteField</code>
   *
   * @return finite field reference
   */
  public FiniteField getField()
  {
    return mField;
  }

  /** 
   * responds to "Continue" button push 
   *
   * @param e <code>ActionEvent</code> from button
   */
  public void actionPerformed(ActionEvent e)
  {
    String theMessage;
    boolean bException = false;
    ActionListener al;
   
    if (mTextArea.isEnabled()) { 
      try {
	makeMatroid();
      }
      catch (Exception ex) {
	theMessage = ex.getMessage();
	if (theMessage instanceof String) {
	  mErrorLabel.setText(theMessage);
	}
	else {
	  mErrorLabel.setText("Error in matrix entry. Please try again.");
	}
	bException = true;
      }
      if (!bException && (mListeners instanceof LinkedList)) {
	for (Iterator i=mListeners.iterator(); i.hasNext();) {
	  al = (ActionListener)i.next();
	  al.actionPerformed(new ActionEvent(this, 
					     ActionEvent.ACTION_PERFORMED,
					     "matroid"));
	}
      }
    }
  }

  //
  // private function to initialize controls
  //
  private void initControls()
  {
    int textRows;
    int textCols;

    setLayout(new BorderLayout());
    
    mErrorLabel = new JLabel(mPrompt);
    add(mErrorLabel, BorderLayout.NORTH);
    
    mTextArea = new JTextArea(mRows+1, mCols * numDigits() * 2);
    
    //
    // set font to monospaced at current size
    //
    Font f = mTextArea.getFont();
    mTextArea.setFont(new Font("Monospaced", Font.PLAIN, f.getSize()));

    JScrollPane scrollPane = 
      new JScrollPane(mTextArea, 
		      JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
		      JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    add(scrollPane, BorderLayout.CENTER);

    JButton continueButton = new JButton("Continue");
    continueButton.addActionListener(this);
    add(continueButton, BorderLayout.SOUTH);
  }

  //
  // private function to determine the number of digits to allow
  // based on the modulus
  //
  private int numDigits()
  { 
    int digits=0;
    int exponent = mField.getFieldSize();

    // enough space for (j) where j = exponent-2
    digits = 2 + String.valueOf(exponent-2).length();

    return digits;
 }

  //
  // private function to make a matroid from the contents
  // of the text area; throws an exception if the contents 
  // aren't sufficient to parse into the matrix or if the
  // calculator is not set
  // 
  private void makeMatroid()
    throws NullPointerException, ParseException, NumberFormatException
  {
    if (!(mField instanceof FiniteField)) {
      throw new NullPointerException("No modulus has been set.");
    }

    String theString, entry;
    int[][] theMatrix = new int[mCols][mRows];
    FiniteFieldVector[] theVector = new FiniteFieldVector[mCols];

    theString = mTextArea.getText();
    StringTokenizer st = new StringTokenizer(theString);
    if (st.countTokens() != mRows * mCols) {
      throw new ParseException("The number of values is incorrect.", 0);
    }

    int i=0, j=0;
    for (;st.hasMoreElements();) {
      entry = (String)st.nextElement();
      if (mField instanceof GFpCalculator) {
	theMatrix[i][j] = mField.sum(Integer.parseInt(entry), 0);
      }
      else if (mField instanceof GF2kCalculator) {
	theMatrix[i][j] = ((GF2kCalculator)mField).powerStringToElt(entry);
      }
      else {
	theMatrix[i][j] = ((GFpkCalculator)mField).powerStringToElt(entry);
      }

      i++;
      if (i >= mCols) {
	i = 0;
	j++;
      }
    }

    for (i=0; i<mCols; i++) {
      theVector[i] = new FiniteFieldVector(theMatrix[i], mRows, mField);
    }

    mMatroid = new FiniteFieldMatroid(theVector);
    displayMatroid(); 
  }

  private void displayMatroid() {
    StringBuffer[] sbLine;
    StringBuffer sbText;
    StringBuffer sbTemp;
    int i,j,k;
    int nd = numDigits()+1;
    int n = mMatroid.getSize();
    int r = mMatroid.getRank();
    FiniteFieldVector[] v;

    //
    // copy vectors of matrix into local array
    // 
    v = new FiniteFieldVector[n];
    for (i=0; i<n; i++) {
      v[i] = mMatroid.getElement(i);
    }

    //
    // build string line by line
    //
    sbLine = new StringBuffer[r];
    for (j=0; j<r; j++) {
      sbLine[j] = new StringBuffer(" ");
    }

    for (i=0; i<n; i++) {
      for (j=0; j<r; j++) {
	if (mField instanceof GFpCalculator) {
	  sbTemp = new StringBuffer(v[i].get(j));
	}
	else if (mField instanceof GF2kCalculator) {
	  sbTemp = new StringBuffer(((GF2kCalculator)mField).
				    eltToPowerString(v[i].get(j)));
	}
	else {
	  sbTemp = new StringBuffer(((GFpkCalculator)mField).
				    eltToPowerString(v[i].get(j)));
	}
	// pad with spaces if necessary
	for (k=sbTemp.length(); k<nd; k++) {
          sbTemp.insert(0, " ");
        }
        sbLine[j].append(sbTemp.toString());
      }
    }

    //
    // put lines together into single string
    //
    sbText = new StringBuffer(sbLine[0].toString());
    for (j=1; j<r; j++) {
      sbText.append("\n");
      sbText.append(sbLine[j].toString());
    }
    mTextArea.setText(sbText.toString());
  }

  /**
   * main method allows this class to be tested from the command line
   *
   * @param arg array of command line arguments: arguments are used as field
   * size, size and rank of matroid
   */
  public static void main(String[] arg) 
  {
    JFrame frame = new JFrame("FiniteFieldMatroidTextArea");
    Container panel = frame.getContentPane();
    int fieldSize=9, rows = 5, cols=10;

    if (arg.length > 0) {
      try {
	fieldSize = Integer.parseInt(arg[0]);
      }
      catch (NumberFormatException nfe) {
	fieldSize = 9;
      }
    }

    if (arg.length > 1) {
      try {
	cols = Integer.parseInt(arg[1]);
      }
      catch (NumberFormatException nfe) {
	cols = 10;
      }
    }

    if (arg.length > 2) {
      try {
	rows = Integer.parseInt(arg[2]);
      }
      catch (NumberFormatException nfe) {
	rows = 5;
      }
    }

    FiniteField field = FiniteFieldFactory.createFiniteField(fieldSize);

    FiniteFieldMatroidTextArea ffmte =
      new FiniteFieldMatroidTextArea(field, rows, cols, "Test");
  
    ffmte.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent e) {
	  int i,j;
	  FiniteFieldMatroid matroid = 
	    ((FiniteFieldMatroidTextArea)e.getSource()).getMatroid();
	  for (i=0; i<matroid.getRank(); i++) {
	    for (j=0; j<matroid.getSize(); j++) {
	      System.out.print(" " + matroid.getElement(j).get(i));
	    }
	    System.out.println();
	  }
	  
	  System.exit(0);
	}
      });

    frame.addWindowListener(new WindowAdapter() {
	public void windowClosing(WindowEvent e) {
	  System.exit(0);
	}
      });

    panel.add(ffmte);
    frame.pack();
    frame.setVisible(true);
  }
}
