import Oid.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

/**
  * This class maintains a collection of representations for a list of loaded
  * matroids, responds to action requests from a graphical user interface, and 
  * implements other utility functions supporting a matroid GUI.
  *
  * @author Robert J. Kingan and Sandra R. Kingan
  * @date 17 Sep 2001
  * @version 2002-08-26
  */
public class MatroidToolkit implements ActionListener
{
  //
  // private data members
  //
  private Vector  mLoadedMatroids;
  private int     mSelectedMatroid;
  private GUI     mGUI;
  private TaskAlgorithmContainer mTaskAlgorithmContainer;
  private Vector  mAlgorithmDescVector;
  private Vector  mAlgorithmVector;
  private Vector  mVisClassNames;
  private Vector  mVisRequiredClasses;
  static final String TASK_ALG_FILENAME = "matroidTaskAlgorithmList.txt";
  static final String VISTASK_NAME = "matroidVisTasks.txt";
  static final String VISLIST_NAME = "matroidVisualizationList.txt";

  /**
    * sets the GUI for this toolkit
    */
  public MatroidToolkit(GUI rGUI)
  {
    mGUI = rGUI;
    mLoadedMatroids = new Vector();
    setupGUI();
  }

  /**
    * sets the GUI for this toolkit and adds one matroid calculator,
    * either the passed matroid (if it is a matroid calculator itself)
    * or a new matroid calculator with m as its first representation
    */
  public MatroidToolkit(GUI rGUI, Matroid m)
  {
    SmartMatroid sm;
    mGUI = rGUI;
    mLoadedMatroids = new Vector();
    setupGUI();
    if (m instanceof SmartMatroid) {
      sm = (SmartMatroid)m;
    }
    else {
      sm = new SmartMatroid(m);
    }
    sm.setTaskAlgorithmContainer(mTaskAlgorithmContainer);
    mLoadedMatroids.add(sm);
  }

  /**
    * responds to ActionEvents by dispatching control to more specific
    * private method
    */
  public void actionPerformed(ActionEvent e)
  { 
    if (e.getSource() instanceof GUI) {
      handleGUIEvent(e);
    }
    else if ((e.getSource() instanceof TaskMenuItem) ||
	     (e.getSource() instanceof TaskMenu)) {
      handleAlgorithmsMenuEvent(e);
    }
    else if ((e.getSource() instanceof JMenuItem) &&
	     (mVisClassNames.contains(((JMenuItem)e.getSource()).getName()))) {
      handleVisToolEvent(e);
    }
    else if (e.getSource() instanceof MultiSelectPanel) {
      handleMultiSelect(e);
    }
    else if (e.getSource() instanceof JPanel) {
      handlePanelEvent(e);
    }
    else if (e.getSource() instanceof SmartMatroid) {
      handleMatroidEvent(e);
    }
  }

  //
  // private method handles matroid event
  //
  private void handleMatroidEvent(ActionEvent e) 
  {
    if (e.getActionCommand().equals("close")) {
      mLoadedMatroids.remove(e.getSource());
    }
  }

  //
  // private method handles actions from a MultiSelectPanel
  //
  private void handleMultiSelect(ActionEvent e) 
  {
    MultiSelectPanel panel = (MultiSelectPanel)e.getSource();
   
    if (!(panel.wasCanceled())) {

      Class combinerClass, cl;
      Combiner alg;
      int i,n;
      Object[] selectedItems;
      Vector items;
      ReadOnlyTextPanel outputPanel;

      try {
	combinerClass = Class.forName("Oid.Combiner");
	cl = Class.forName(panel.getClient());
	if (!combinerClass.isAssignableFrom(cl)) {
	  throw new InternalError("Non-combiner client in MultiSelectPanel");
	}
	alg = (Combiner)cl.newInstance();
      
	// make Vector from selected items
	selectedItems = panel.getSelectedValues();
	n = selectedItems.length;
	items = new Vector();
	for (i=0; i<n; i++) {
	  items.add(selectedItems[i]);
	}

	mGUI.removePanel(panel);

	alg.setCollection(items);
      
	outputPanel = new ReadOnlyTextPanel();
	outputPanel.setText(alg.getConstruction().toString());
	mGUI.displayPanel(outputPanel, 
			  alg.getName() + " results",
			  selectedItems[0]);

      }
      catch (Exception exc) {
	mGUI.showErrorDialog(exc.toString());
      }
    }
  }

  //
  // private method handles actions from a visualization option on
  // the Algorithms menu
  //
  private void handleVisToolEvent(ActionEvent e)
  {
    SmartMatroid SM = (SmartMatroid)mGUI.getSelectedOpenObject();
    Matroid rep = null;
    Visualization vis = null;
    String visClassName = ((JMenuItem)e.getSource()).getName();
    Class requiredClass = null;
    JPanel visPanel = null;
    int i;

    try {
      vis = (Visualization)(Class.forName(visClassName).newInstance());
    }
    catch (Exception f) {
      mGUI.showErrorDialog("Unable to perform operation: " + f.toString());
    }

    if (vis instanceof Visualization) {
      // find out required class name
      i = mVisClassNames.indexOf(visClassName);
      if (i < 0) {
	// this "can't happen"
	throw new InternalError("Internal error in vistool handler");
      }
      try {
	requiredClass = Class.forName((String)mVisRequiredClasses.get(i));
      }
      catch (Exception f) {
	// cannot recognize required representation class
	mGUI.showErrorDialog("Unable to recognize required class");
      }
    }
    
    if (SM instanceof SmartMatroid) {

      if ((vis instanceof Visualization)&&(requiredClass instanceof Class)) {
	rep = SM.getRepresentation(requiredClass);
	if (!(rep instanceof Matroid)) {
	  mGUI.showErrorDialog("Required representation of matroid " +
			       "not found: " + requiredClass.getName());
	}
      }

      if ((vis instanceof Visualization) && (rep instanceof Matroid)) {
	mGUI.addMessage("Generating results...");
	try {
	  visPanel = vis.getVisualization(rep);
	}
	catch (IllegalArgumentException f) {
	  mGUI.showErrorDialog(f.toString());
	}
      }

      mGUI.addMessage("...finished");
      if (visPanel instanceof JPanel) {
	mGUI.displayPanel(visPanel, 
			  vis.getName() + " results for " + SM.getName(),
			  SM);
      }
    }
  }

  //
  // private method handles actions from JPanels (generally creating new
  // matroids)
  //
  private void handlePanelEvent(ActionEvent e)
  {
    if (e.getSource() instanceof NewFiniteFieldMatroidPanel) {
      NewFiniteFieldMatroidPanel entryPanel = (NewFiniteFieldMatroidPanel)e.getSource();
      FiniteFieldMatroid newRep = (FiniteFieldMatroid)entryPanel.getMatroid();
      mGUI.addMessage("Adding new matroid " + newRep.getName());
      SmartMatroid newMatroid = new SmartMatroid(newRep);
      newMatroid.setTaskAlgorithmContainer(mTaskAlgorithmContainer);
      mLoadedMatroids.add(newMatroid);
      mGUI.removePanel(entryPanel);
      mGUI.displayPanel(new FiniteFieldMatroidDisplayPanel(newRep), 
			newRep.getName(),
			newMatroid);
    }
    else if (e.getSource() instanceof PGSpecPanel) {
      PGSpecPanel specPanel = (PGSpecPanel)e.getSource();
      mGUI.addMessage("Adding new matroid " + specPanel.getMatroidName());
      FiniteField p = specPanel.getMatroidField();
      mGUI.removePanel(specPanel);
      int size = specPanel.getMatroidRank();
      createPG(size, p, specPanel.getMatroidName());
    }
    else if (e.getSource() instanceof NewBasisMatroidPanel) {
      NewBasisMatroidPanel newPanel = (NewBasisMatroidPanel)e.getSource();
      BasisMatroid newBMat = newPanel.getMatroid();
      mGUI.addMessage("Adding new matroid " + newBMat.getName());
      SmartMatroid newSM = new SmartMatroid(newBMat);
      newSM.setTaskAlgorithmContainer(mTaskAlgorithmContainer);
      mLoadedMatroids.add(newSM);
      mGUI.removePanel(newPanel);
      BasisMatroidDisplayPanel newBMDisplay = 
	  new BasisMatroidDisplayPanel(newBMat);
      mGUI.displayPanel(newBMDisplay, "Basis Matroid " + newSM.getName(),
			newSM);
    }
    else if (e.getSource() instanceof NewLinearSpacePanel) {
      NewLinearSpacePanel newPanel = (NewLinearSpacePanel)e.getSource();
      LinearSpace newLSpc = newPanel.getLinearSpace();
      mGUI.addMessage("Adding new linear space " + newLSpc.getName());
      SmartMatroid newSM = new SmartMatroid(newLSpc);
      newSM.setTaskAlgorithmContainer(mTaskAlgorithmContainer);
      mLoadedMatroids.add(newSM);
      mGUI.removePanel(newPanel);
      LinearSpaceDisplayPanel newLSDisplay = 
	  new LinearSpaceDisplayPanel(newLSpc);
      mGUI.displayPanel(newLSDisplay, "Linear Space " + newSM.getName(),
			newSM);
    }
    else if (e.getSource() instanceof NewLSByMatrixPanel) {
      NewLSByMatrixPanel newPanel = (NewLSByMatrixPanel)e.getSource();
      LinearSpace newLSpc = newPanel.getLinearSpace();
      mGUI.addMessage("Adding new linear space " + newLSpc.getName());
      SmartMatroid newSM = new SmartMatroid(newLSpc);
      newSM.setTaskAlgorithmContainer(mTaskAlgorithmContainer);
      mLoadedMatroids.add(newSM);
      mGUI.removePanel(newPanel);
      LinearSpaceDisplayPanel newLSDisplay = 
	  new LinearSpaceDisplayPanel(newLSpc);
      mGUI.displayPanel(newLSDisplay, "Linear Space " + newSM.getName(),
			newSM);
    }
  }

  //
  // private method creates a new projective space matroid
  //
  private void createPG(int size, FiniteField p, String name)
  {
    PGFactory factory;
    factory = new PGFactory(size, p);

    FiniteFieldMatroid newRep = 
      new FiniteFieldMatroid(factory.getProjectiveSpace());
    newRep.setName(name);
    SmartMatroid newMatroid = new SmartMatroid(newRep);
    newMatroid.setTaskAlgorithmContainer(mTaskAlgorithmContainer);
    mLoadedMatroids.add(newMatroid);
    mGUI.displayPanel(new FiniteFieldMatroidDisplayPanel(newRep), 
		      newRep.getName(),
		      newMatroid);
  }

  //
  // private method handles GUI events, including opening files,
  // saving files, and "File--New"
  //
  private void handleGUIEvent(ActionEvent e)
  {
    String cmdVerb = (String)null;
    String cmdObject = (String)null;
    StringTokenizer cmdParser = new StringTokenizer(e.getActionCommand());
    if (cmdParser.hasMoreElements()) {
      cmdVerb = (String)cmdParser.nextElement();
    }
    if (cmdParser.hasMoreElements()) {
      cmdObject = (String)cmdParser.nextElement();
    }
    if (cmdVerb instanceof String) {
      if (cmdVerb.equals("create")) {
	if (cmdObject instanceof String) {
	  if (cmdObject.equals("Oid.FiniteFieldMatroid")) {
	    createFiniteFieldMatroidSpecPanel();
	  }
	  else if (cmdObject.equals("Oid.PGFactory")) {
	    createPGSpecPanel();
	  }
	  else if (cmdObject.equals("Oid.BasisMatroid")) {
	    createNewBasisMatroidPanel();
	  }
	  else if (cmdObject.equals("Oid.LinearSpace")) {
	    createNewLinearSpacePanel();
	  }
	}
      }
      else if (cmdVerb.equals("open")) {
	try {
	  mGUI.addMessage("Loading matroid...");
	  loadMatroidFromFile(cmdObject);
	  mGUI.addMessage("...finished");
	}
	catch (IOException f) {
	  mGUI.showErrorDialog(new String("Unable to open file " + cmdObject));
	}
      }
      else if (cmdVerb.equals("save")) {
	try {
	  mGUI.addMessage("Saving matroid...");
	  saveMatroidToFile((Matroid)mGUI.getSelectedOpenObject(), cmdObject);
	  mGUI.addMessage("...finished");
	}
	catch (IOException f) {
	  mGUI.showErrorDialog(new String("Unable to save file " + cmdObject));
	}
      }
    }
  }

  //
  // private method handles events from the Algorithms menu
  //
  // if the user selected a task (without specifying a algorithm) then the
  // routine checks for a "built-in" task (one that SmartMatroid supports
  // intrinsically), or calls performNovelTask if the task is new
  //
  // if the user specified a algorithm, the routine calls applyAlgorithm to 
  // do that
  //
  private void handleAlgorithmsMenuEvent(ActionEvent e)
  {
    boolean isBuiltInTask;
    Algorithm selectedAlgorithm;
    Task selectedTask;
    SmartMatroid SM = (SmartMatroid)mGUI.getSelectedOpenObject();
    String elementLabel;
    int parameter;
    int i, n;

    //
    // first, find out what Task for a "built-in" task, or what
    // task and algorithm
    //
    if ((e.getSource() instanceof TaskMenuItem) &&
	(SM instanceof SmartMatroid)){
      TaskMenuItem selectedTaskItem = (TaskMenuItem)e.getSource();
      selectedTask = selectedTaskItem.getTask();
      String taskName = selectedTask.getName();
      //
      // Is it a built-in task? If not, apply it according to its interface.
      //
      if (taskName.equals("getCircuits")) {
	displaySubsetList(selectedTaskItem.getText(),
			  SM.getCircuits(),
			  SM);
	mGUI.addMessage("Generated circuits using " + 
			SM.lastRepresentation().getClass().getName() +
			" class and " + 
			SM.lastAlgorithm().getClass().getName() +
			" algorithm");
      }
      else if (taskName.equals("getBases")) {
	displaySubsetList(selectedTaskItem.getText(),
			  SM.getBases(),
			  SM);
	mGUI.addMessage("Generated bases using " + 
			SM.lastRepresentation().getClass().getName() +
			" class and " + 
			SM.lastAlgorithm().getClass().getName() +
			" algorithm");
      }
      else if (taskName.equals("getHyperplanes")) {
	displaySubsetList(selectedTaskItem.getText(),
			  SM.getHyperplanes(),
			  SM);
	mGUI.addMessage("Generated hyperplanes using " + 
			SM.lastRepresentation().getClass().getName() +
			" class and " + 
			SM.lastAlgorithm().getClass().getName() +
			" algorithm");
      }
      else if (taskName.equals("getFlats")) {
	displaySubsetList(selectedTaskItem.getText(),
			  SM.getFlats(),
			  SM);
	mGUI.addMessage("Generated flats using " + 
			SM.lastRepresentation().getClass().getName() +
			" class and " + 
			SM.lastAlgorithm().getClass().getName() +
			" algorithm");

      }
      else if (taskName.equals("getIndependentSets")) {
	displaySubsetList(selectedTaskItem.getText(),
			  SM.getIndependentSets(),
			  SM);
	mGUI.addMessage("Generated independent sets using " + 
			SM.lastRepresentation().getClass().getName() +
			" class and " + 
			SM.lastAlgorithm().getClass().getName() +
			" algorithm");

      }
      else if (taskName.equals("getContraction")) {
	parameter = getMatroidElement(SM);
	if (parameter >= 0) {
	  mGUI.addMessage("Adding contraction of " + SM.getName());
	  try {
	    addConstructedMatroid(SM.getContraction(parameter));
	  }
	  catch (Exception ee) {
	    mGUI.showErrorDialog("No contraction algorithm exists for the " +
				 "representations currently loaded.");
	  }
	}
	else {
	  mGUI.showErrorDialog(new String("Invalid element!"));
	}
      }
      else if (taskName.equals("getDeletion")) {
	parameter = getMatroidElement(SM);
	if (parameter >= 0) {
	  mGUI.addMessage("Adding deletion of " + SM.getName());
	  try {
	    addConstructedMatroid(SM.getDeletion(parameter));
	  }
	  catch (Exception ee) {
	    mGUI.showErrorDialog("No deletion algorithm exists for the " + 
				 "representations currently loaded.");
	  }
	}
	else {
	  mGUI.showErrorDialog(new String("Invalid element!"));
	}
      }
      else if (taskName.equals("getDual")) {
	mGUI.addMessage("Adding dual matroid to " + SM.getName());
	try {
	  addConstructedMatroid(SM.getDual());
	}
	catch (Exception ee) {
	  mGUI.showErrorDialog("Unable to generate dual of this " +
			       "representation");
	}
      }
      else {
	// non-built-in tasks
	performNovelTask(selectedTask, SM);
      }
    }
  }

  //
  // private method applies specified algorithm to matroid
  //
  // it has to determine the category of the algorithm, get a parameter
  // if necessary, and then apply the algorithm and display the results
  // 
  // SubsetLister:       display results by calling displaySubsetList
  // PropertyCalculator: throw up a message box
  // SingleConstructor:  add a new matroid to the collection of open ones
  // CollectionConstructor: add the whole collection, one by one
  //
  private void applyAlgorithm(Algorithm selectedAlgorithm, SmartMatroid SM)
  {
    String answer;
    int parameter;
    String params;
    Task selectedTask = selectedAlgorithm.getTask();
    String category = selectedTask.getCategoryClass().getName();
    if (category.equals("SubsetLister")) {
      SubsetLister lister;
      try {
	lister = (SubsetLister)selectedAlgorithm.getInstance();
	displaySubsetList(lister.getName(),
			  SM.applySubsetLister(lister),
			  SM);
      }
      catch (InstantiationException e) {
	mGUI.showErrorDialog("Unable to find selected algorithm");
      }
      catch (Exception e) {
	mGUI.showErrorDialog("Error: " + e.getMessage());
      }
    }
    else if (category.equals("PropertyCalculator")) {
      PropertyCalculator propCalc;
      try {
	propCalc = (PropertyCalculator)selectedAlgorithm.getInstance();
	if (propCalc.requiresParameter()) {
	  // ToDo : get parameter
	  parameter = 0;
	  // ToDo
	  mGUI.showMessageDialog(propCalc.getName() + " result: " + 
				 SM.applyPropertyCalculator(propCalc,
							    parameter));
	}
	else {
	  mGUI.showMessageDialog(propCalc.getName() + " result: " + 
				 SM.applyPropertyCalculator(propCalc));
	}
      }
      catch (InstantiationException e) {
	mGUI.showErrorDialog("Unable to find selected algorithm");
      }
      catch (Exception e) {
	mGUI.showErrorDialog("Error: " + e.getMessage());
      }
    }
    else if (category.equals("SingleConstructor")) {
      SingleConstructor cnstr;
      try {
	cnstr = (SingleConstructor)selectedAlgorithm.getInstance();
	if (cnstr.requiresParameter()) {
	  params = mGUI.showInputDialog(cnstr.getPrompt());
	  addConstructedMatroid((Matroid)SM.applySingleConstructor(cnstr,
								   params));
	}
	else {
	  addConstructedMatroid((Matroid)SM.applySingleConstructor(cnstr));
	}
      }
      catch (InstantiationException e) {
	mGUI.showErrorDialog("Unable to find selected algorithm");
      }
      catch (Exception e) {
	mGUI.showErrorDialog("Error: " + e.getMessage());
      }
    }
    else if (category.equals("CollectionConstructor")) {
      CollectionConstructor colConst;
      Collection newMatroids;
      try {
	colConst = (CollectionConstructor)selectedAlgorithm.getInstance();
	if (colConst.requiresParameter()) {
	  // ToDo : get parameter
	  parameter = 0;
	  // ToDo
	  newMatroids = SM.applyCollectionConstructor(colConst, parameter);
	}
	else {
	  newMatroids = SM.applyCollectionConstructor(colConst);
	}

	for (Iterator pConst = newMatroids.iterator(); pConst.hasNext();) {
	  addConstructedMatroid((Matroid)pConst.next());
	}
      }
      catch (InstantiationException e) {
	mGUI.showErrorDialog("Unable to find selected algorithm");
      }
      catch (Exception e) {
	mGUI.showErrorDialog("Error: " + e.getMessage());
      }
    }
    else if (category.equals("Combiner")) {
      MultiSelectPanel multiPanel = 
	new MultiSelectPanel(new String("Select matroids:"),
			     mLoadedMatroids,
			     selectedAlgorithm.getClassName());
							 
    }
  }   

  //
  // private method performs "non-built-in" tasks
  //
  private void performNovelTask(Task selectedTask,
				SmartMatroid SM)
  {
    String answer;
    int parameter;
    String categoryClassName = selectedTask.getCategoryClass().getName();
    if (categoryClassName.equals("Oid.SubsetLister")) {
      SubsetLister algorithm;
      try {
	algorithm = SM.performSubsetListerTask(selectedTask);
	displaySubsetList(algorithm.getName(),
			  algorithm.getSubsetList(),
			  SM);
      }
      catch (Exception e) {
	mGUI.showErrorDialog("Unable to complete this task with the " +
			     "current representation");
      }
    }
    else if (categoryClassName.equals("Oid.PropertyCalculator")) {
      PropertyCalculator thePropCalc;
      try {
	thePropCalc = SM.performPropertyCalculatorTask(selectedTask);
	if (thePropCalc.requiresParameter()) {
	  answer = mGUI.showInputDialog("Enter parameter");
	  parameter = Integer.valueOf(answer).intValue();
	  thePropCalc.setParameter(parameter);
	}
	mGUI.showMessageDialog(thePropCalc.getName() + " result: " + 
			       thePropCalc.getProperty());
	mGUI.addMessage("Calculated property using " +
			SM.lastRepresentation().getClass().getName() + 
			" class and " + 
			SM.lastAlgorithm().getClass().getName() +
			" algorithm");
      }
      catch (Exception e) {
	mGUI.showErrorDialog("Unable to calculate this property with " +
			     "the current representation");
      }
    }
    else if (categoryClassName.equals("Oid.SingleConstructor")) {
      SingleConstructor theConstructor;
      try {
	theConstructor = SM.performSingleConstructorTask(selectedTask);
	if (theConstructor.requiresParameter()) {
	  answer = mGUI.showInputDialog(theConstructor.getPrompt());
	  theConstructor.setParameters(answer);
	}
	addConstructedMatroid((Matroid)theConstructor.getConstruction());
	mGUI.addMessage("Generated matroid using " +
			SM.lastRepresentation().getClass().getName() + 
			" class and " + 
			SM.lastAlgorithm().getClass().getName() +
			" algorithm");
      }
      catch (Exception e) {
	mGUI.showErrorDialog("Unable to complete this Task with the " +
			     "current representation:\n" +
			     e.toString());
	e.printStackTrace();
      }
    }
    else if (categoryClassName.equals("Oid.CollectionConstructor")) {
      CollectionConstructor theColConstructor;
      try {
	theColConstructor = 
	  SM.performCollectionConstructorTask(selectedTask);
	Collection constructedCollection = 
	  theColConstructor.getConstructions();
	for (Iterator pConst = constructedCollection.iterator();
	     pConst.hasNext();) {
	  addConstructedMatroid((Matroid)pConst.next());
	}
	mGUI.addMessage("Generated collection using " +
			SM.lastRepresentation().getClass().getName() + 
			" class and " + 
			SM.lastAlgorithm().getClass().getName() +
			" algorithm");
      }
      catch (Exception e) {
	mGUI.showErrorDialog("Unable to complete the requested construction " +
			     "with the current representation");
      }
    }
    else if (categoryClassName.equals("Oid.Combiner")) {
      Vector algs = 
	new Vector(mTaskAlgorithmContainer.getAlgorithms(selectedTask));
      Algorithm selectedAlgorithm = (Algorithm)algs.get(0);
      MultiSelectPanel multiPanel = 
	new MultiSelectPanel(new String("Select matroids:"),
			     mLoadedMatroids,
			     selectedAlgorithm.getClassName());
      multiPanel.addActionListener(this);
      mGUI.displayPanel(multiPanel, selectedAlgorithm.getDescription(), SM);
    }
  }

  //
  // private method to query the user to enter an element label, which
  // returns the element's index in the matroid if it is a valid label,
  // or returns -1 if the label is invalid
  // 
  private int getMatroidElement(Matroid m) {
    int i, n = m.getSize();
    String label;

    label = mGUI.showInputDialog(new String("Enter element"));
    for (i=0; i<n; i++) {
      if (m.getLabel(i).equals(label)) {
	break;
      }
    }
    if (i >= n) {
      i = -1;
    }

    return i;
  }

  // 
  // private method to take a collection of subsets generated by a 
  // SubsetLister and display it on the GUI on a panel whose parent 
  // is the passed Matroid
  // 
  private void displaySubsetList(String algorithmName, 
				 Collection subsetList, 
				 Matroid parent)
  {
    Subset ss;
    Iterator p;
    StringBuffer sb = new StringBuffer("Results:\n");
    int n, max;
    int i;
    int[] chain;
    String s;

    n = parent.getSize();
    max = 0;

    chain = new int[n+1];
    for (i=0; i<=n; i++) {
      chain[i] = 0;
    }

    for (p = subsetList.iterator(); p.hasNext();) {
      ss = (Subset)p.next();

      i = ss.numElements();
      chain[i]++;
      max = (i > max) ? i : max;

      sb.append(ss.toString(true));
      sb.append("\n");
    }

    // add chain to output
    sb.append("Chain:\n");
    for (i=0; i<=max; i++) {
      s = String.valueOf(i);
      while(s.length() < 5) {
	s = " " + s;
      }
      sb.append(s);
    }
    sb.append("\n");
    for (i=0; i<=max; i++) {
      sb.append("-----");
    }
    sb.append("\n");
    for (i=0; i<=max; i++) {
      s = String.valueOf(chain[i]);
      while (s.length() < 5) {
	s = " " + s;
      }
      sb.append(s);
    }
    sb.append("\n");
    
    ReadOnlyTextPanel resultsPanel = new ReadOnlyTextPanel();
    resultsPanel.setText(sb.toString());
    mGUI.displayPanel(resultsPanel, parent.getName() + " " +
		      algorithmName + " results", parent);    
  }

  //
  // adds a new Matroid to the collection of loaded matroids based on
  // the results of running a SingleConstructor
  //
  private void addConstructedMatroid(Matroid newMatroid)
  {
    SmartMatroid newSmartMatroid;
    Matroid newRep;
    newSmartMatroid = getSmartMatroid(newMatroid);
    newSmartMatroid.setTaskAlgorithmContainer(mTaskAlgorithmContainer);
    newRep = newSmartMatroid.getDefaultRepresentation();
    if (newRep instanceof FiniteFieldMatroid) {
      mGUI.displayPanel(new FiniteFieldMatroidDisplayPanel((FiniteFieldMatroid)newRep), 
			newSmartMatroid.getName(),
			newSmartMatroid);
    }
    else if (newRep instanceof BasisMatroid) {
      mGUI.displayPanel(new BasisMatroidDisplayPanel((BasisMatroid)newRep),
			newSmartMatroid.getName(),
			newSmartMatroid);
    }
  }

  //
  // returns a SmartMatroid for the passed matroid (an existing one
  // if one of the loaded matroid calculators contains the representation,
  // or a new one otherwise)
  //
  private SmartMatroid getSmartMatroid(Matroid m) 
  {
    SmartMatroid answer=null;
    boolean found;
    int i;

    if (m instanceof SmartMatroid) {
      i = mLoadedMatroids.indexOf(m);
      if (i < 0) {
	mLoadedMatroids.add(m);
	answer = (SmartMatroid)m;
      }
      else {
	answer = (SmartMatroid)mLoadedMatroids.get(i);
      }
    }
    else {
      found = false;
      for (Iterator p = mLoadedMatroids.iterator(); 
	   p.hasNext() && !false;) {
	answer = (SmartMatroid)p.next();
	found = (answer.getRepresentations().contains(m));
      }
      if (!found) {
	answer = new SmartMatroid(m);
	answer.setTaskAlgorithmContainer(mTaskAlgorithmContainer);
	mLoadedMatroids.add(answer);
      }
    }
    return answer;
  }

  /**
   * creates a new FiniteFieldMatroid instance on the screen
   */
  private void createFiniteFieldMatroidSpecPanel()
  {
    NewFiniteFieldMatroidPanel entryPanel = new NewFiniteFieldMatroidPanel();
    entryPanel.addActionListener(this);
    mGUI.displayPanel(entryPanel, "Enter the new matroid", null);
  }

  /**
   * creates spec panel for a new projective space on the screen
   */
  private void createPGSpecPanel()
  {
    PGSpecPanel specPanel = new PGSpecPanel();
    specPanel.addActionListener(this);
    mGUI.displayPanel(specPanel, 
                      "Enter rank and field for projective geometry", 
		      null);
  }

  /** 
   * creates a panel for a new BasisMatroid
   */
  private void createNewBasisMatroidPanel()
  {
    NewBasisMatroidPanel newPanel = new NewBasisMatroidPanel();
    newPanel.addActionListener(this);
    mGUI.displayPanel(newPanel,
		      "Enter name, size and bases for new matroid",
		      null);
  }

  /** 
   * creates a panel for a new LinearSpace
   */
  private void createNewLinearSpacePanel()
  {
    Object[] options = {"Enter a set of lines","Enter a matrix"};
    int n = JOptionPane.showOptionDialog((JFrame)mGUI,
					 "How would you like to specify your linear space?",
					 "Linear Space",
					 JOptionPane.YES_NO_OPTION,
					 JOptionPane.QUESTION_MESSAGE,
					 null,
					 options,
					 options[0]);
    

    JPanel newPanel;
    if (0 == n) {
      NewLinearSpacePanel nlsp = new NewLinearSpacePanel();
      nlsp.addActionListener(this);
      newPanel = (JPanel)nlsp;
    }
    else {
      NewLSByMatrixPanel nlsmp = new NewLSByMatrixPanel();
      nlsmp.addActionListener(this);
      newPanel = (JPanel)nlsmp;
    }

    mGUI.displayPanel(newPanel,
		      "Enter name, size and lines for new linear space",
		      null);
  }
  /**
    * sets the necessary properties of the GUI
    */ 
  private void setupGUI()
  {
    Vector vcc = new Vector(); // creatable class list
    try {
      vcc.add(Class.forName("Oid.FiniteFieldMatroid"));
      vcc.add(Class.forName("Oid.PGFactory"));
      vcc.add(Class.forName("Oid.BasisMatroid"));
      vcc.add(Class.forName("Oid.LinearSpace"));
    }
    catch (ClassNotFoundException e) {
      throw new InternalError("FiniteFieldMatroid class not found");
    }

    Vector vcn = new Vector(); // creatable class names
    vcn.add(new String("Finite field matroid"));
    vcn.add(new String("Projective geometry"));
    vcn.add(new String("Basis Matroid"));
    vcn.add(new String("Linear Space"));

    mGUI.setCreatableClassList(vcc);
    mGUI.setCreatableClassNames(vcn);
    setupTaskAlgorithmContainer();
    mGUI.setTaskMenu(taskMenu());
    mGUI.setViewMenu(setupVisualizations());
    mGUI.addActionListener(this);
  }

  /**
   * sets up the algorithmbox for this object
   */
  private void setupTaskAlgorithmContainer()
  {
    try {
      mTaskAlgorithmContainer = new TaskAlgorithmContainer(TASK_ALG_FILENAME);
    }
    catch (IOException e) {
      throw new InternalError("Error reading algorithmfile");
    }
    catch (ClassNotFoundException e) {
      throw new InternalError("ClassNotFoundException: " + e.getMessage());
    }
  }
  

  /**
    * creates a Algorithms menu for the GUI
    */
  private JMenu taskMenu() 
  {
    JMenu taskMenu = new JMenu("Tasks");
    TaskMenu subMenu;
    TaskMenuItem taskMenuItem;
    Collection tasks;
    Collection algorithms;
    Task task;
    Algorithm algorithm;
    Iterator pTask, pAlgorithm;

    mAlgorithmVector = new Vector();
    mAlgorithmDescVector = new Vector();

    tasks = mTaskAlgorithmContainer.getTasks();
    for (pTask = tasks.iterator(); pTask.hasNext();) {
      task = (Task)pTask.next();
      algorithms = mTaskAlgorithmContainer.getAlgorithms(task);
      taskMenuItem = new TaskMenuItem(task);
      taskMenuItem.addActionListener(this);
      if (task.getPrecedeWithSeparator()) {
	taskMenu.addSeparator();
      }
      taskMenu.add(taskMenuItem);
    }
    setupVisTasks(taskMenu);
    return taskMenu;
  }

  //
  // private method sets up menu items for visualizations that go 
  // on the algorithm menu
  //
  private void setupVisTasks(JMenu taskMenu)
  {
    TextFileParser visParser;

    try {
      visParser = new TextFileParser(VISTASK_NAME);
    }
    catch (Exception e) {
      // must be able to read visualization file
      throw new InternalError(e.toString());
    }
    
    JMenuItem menuItem;
    String visClass, visReqClass, visDesc;
    Vector textFileLine;
    Iterator pWord;

    if (!(mVisClassNames instanceof Vector)) {
      mVisClassNames = new Vector();
      mVisRequiredClasses = new Vector();
    }
    int i, j, n = visParser.numLines();
    for (i=0; i<n; i++) {
      textFileLine = visParser.getLine(i);
      if (textFileLine.size() >= 3) {
	pWord = textFileLine.iterator();
	visClass = (String)pWord.next();
	visReqClass = (String)pWord.next();
	visDesc = new String("");
	for (; pWord.hasNext();) {
	  visDesc += (String)pWord.next();
	  visDesc += ' ';
	}
	menuItem = new JMenuItem(visDesc);
	menuItem.setName(visClass);
	menuItem.addActionListener(this);
	taskMenu.add(menuItem);
	mVisClassNames.add(visClass);
	mVisRequiredClasses.add(visReqClass);
      }
      else if ((textFileLine.size() > 0) &&
	       (((String)textFileLine.get(0)).equals("separator"))) {
	taskMenu.addSeparator();
      }
    }
  }
  
  
  //
  // private method sets up menu items for visualizations
  //
  private JMenu setupVisualizations()
  {
    TextFileParser visParser;
    JMenu visMenu = new JMenu("View");
    try {
      visParser = new TextFileParser(VISLIST_NAME);
    }
    catch (Exception e) {
      // must be able to read visualization file
      throw new InternalError(e.toString());
    }
    
    JMenuItem menuItem;
    String visClass, visReqClass, visDesc;
    Vector textFileLine;
    Iterator pWord;

    if (!(mVisClassNames instanceof Vector)) {
      mVisClassNames = new Vector();
      mVisRequiredClasses = new Vector();
    }

    int i, j, n = visParser.numLines();
    for (i=0; i<n; i++) {
      textFileLine = visParser.getLine(i);
      if (textFileLine.size() >= 3) {
	pWord = textFileLine.iterator();
	visClass = (String)pWord.next();
	visReqClass = (String)pWord.next();
	visDesc = new String("");
	for (; pWord.hasNext();) {
	  visDesc += (String)pWord.next();
	  visDesc += ' ';
	}
	menuItem = new JMenuItem(visDesc);
	menuItem.setName(visClass);
	menuItem.addActionListener(this);
	visMenu.add(menuItem);
	mVisClassNames.add(visClass);
	mVisRequiredClasses.add(visReqClass);
      }
      else if ((textFileLine.size() > 0) &&
	       (((String)textFileLine.get(0)).equals("separator"))) {
	visMenu.addSeparator();
      }
    }
    return visMenu;
  }
  


  // 
  // private method to load a matroid from a file
  //
  private void loadMatroidFromFile(String filename)
  throws IOException
  {
    //
    // ToDo : delegate instantiation to SmartMatroid
    //
    FileReader rdr = new FileReader(filename);
    char[] buffer = new char[1024];
    int nRead;
    StringBuffer sBuffer = new StringBuffer("");
    
    // read the contents of the file into one big string
    nRead = rdr.read(buffer);
    while (nRead >= 0) {
      sBuffer.append(buffer, 0, nRead);
      nRead = rdr.read(buffer);
    }
    rdr.close();

    // tokenize the string into lines
    String contents = sBuffer.toString();
    
    // get type of matroid from first word of file
    StringTokenizer st = new StringTokenizer(contents);
    String matroidType;
    if (st.hasMoreElements()) {
      matroidType = (String)st.nextElement();
      if (matroidType.toLowerCase().equals("finitefieldmatroid")) {
	FiniteFieldMatroid newRep = new FiniteFieldMatroid();
	newRep.setState(contents, "text");
	SmartMatroid newMatroid = new SmartMatroid(newRep);
	newMatroid.setTaskAlgorithmContainer(mTaskAlgorithmContainer);
	mLoadedMatroids.add(newMatroid);
	mGUI.displayPanel(new FiniteFieldMatroidDisplayPanel(newRep), 
			  newRep.getName(),
			  newMatroid);
      }
      else if (matroidType.toLowerCase().equals("basismatroid")) {
	BasisMatroid newBRep = new BasisMatroid();
	newBRep.setState(contents, "text");
	SmartMatroid newBSmartMatroid = new SmartMatroid(newBRep);
	newBSmartMatroid.setTaskAlgorithmContainer(mTaskAlgorithmContainer);
	mLoadedMatroids.add(newBSmartMatroid);
	mGUI.displayPanel(new BasisMatroidDisplayPanel(newBRep),
			  newBSmartMatroid.getName(),
			  newBSmartMatroid);
      }
      else if (matroidType.toLowerCase().equals("linearspace")) {
	LinearSpace newRep = new LinearSpace();
	newRep.setState(contents, "text");
	SmartMatroid newMatroid = new SmartMatroid(newRep);
	newMatroid.setTaskAlgorithmContainer(mTaskAlgorithmContainer);
	mLoadedMatroids.add(newMatroid);
	mGUI.displayPanel(new LinearSpaceDisplayPanel(newRep), 
			  newRep.getName(),
			  newMatroid);
      }
      else {
	throw new IOException("Unknown matroid type: " + matroidType);
      }
    }
  }

  //
  // private method to save a matroid to a file
  //
  private void saveMatroidToFile(Matroid theMatroid, 
				 String filename)
    throws IOException
  {
    

    if (theMatroid instanceof Matroid) {
      FileOutputStream stream = new FileOutputStream(filename);
      PrintWriter writer = new PrintWriter(stream);
      writer.println(theMatroid.getState("text"));
      writer.flush();
      stream.close();
    }
  }
}
