import Oid.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
 * A <code>NewBasisMatroidPanel</code> displays three input areas: one
 * for the user to specify the name of a new <code>BasisMatroid</code>
 * to create, one to specify the size of the new matroid, and the
 * third for the user to identify its bases. It also displays a
 * Continue button.  When the user presses the Continue button, the
 * object attempts to create a new <code>BasisMatroid</code> instance
 * using the input values.  The panel can then provide a reference to
 * the resulting <code>BasisMatroid</code> instance to its client.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 14 Nov 2001
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class NewBasisMatroidPanel extends JPanel implements ActionListener
{
  //
  // private data members
  //
  JTextField   mNameField;
  JTextField   mSizeField;
  JTextArea    mTextArea;
  JLabel       mMessageLabel;
  BasisMatroid mMatroid;
  String       mMatroidName;
  int          mMatroidSize;
  Vector       mSubsets;
  Vector       mActionListeners;
  int          mBadSubsetLine;

  /**
   * sets up the panel
   */
  public NewBasisMatroidPanel()
  {
    mMatroidSize = 0;
    mMatroid = (BasisMatroid)null;
    setLayout(new BorderLayout());
    
    // 
    // set up panel with size & name fields in north area
    //
    JPanel colPanel = new JPanel();
    colPanel.setLayout(new BoxLayout(colPanel, BoxLayout.Y_AXIS));
    JPanel rowPanel;

    // name label & field
    rowPanel = new JPanel();
    rowPanel.setLayout(new BoxLayout(rowPanel, BoxLayout.X_AXIS));
    mNameField = new JTextField();
    rowPanel.add(new JLabel("Name:"));
    rowPanel.add(mNameField);
    colPanel.add(rowPanel);

    // size label & field
    rowPanel = new JPanel();
    rowPanel.setLayout(new BoxLayout(rowPanel, BoxLayout.X_AXIS));
    mSizeField = new JTextField();
    rowPanel.add(new JLabel("Size:"));
    rowPanel.add(mSizeField);
    colPanel.add(rowPanel);
    
    add(colPanel, BorderLayout.NORTH);

    // set up text area for subsets in center area
    mTextArea = new JTextArea(10, 20);
    JScrollPane scrollPane = 
      new JScrollPane(mTextArea,
		      JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
		      JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    add(scrollPane, BorderLayout.CENTER);
   
    // set up continue button and message label in south area
    colPanel = new JPanel();
    colPanel.setLayout(new BoxLayout(colPanel, BoxLayout.Y_AXIS));
    mMessageLabel = new JLabel("Press continue when finished");
    colPanel.add(mMessageLabel);
    JButton continueButton = new JButton("Continue");
    continueButton.addActionListener(this);
    colPanel.add(continueButton);
    add(colPanel, BorderLayout.SOUTH);
  }

  /**
   * returns the matroid entered in the panel
   *
   * @return <code>BasisMatroid</code> reference
   */
  public BasisMatroid getMatroid()
  {
    return mMatroid;
  }

  /**
   * adds an action listener to the panel
   *
   * @param al <code>ActionListener</code> reference
   */
  public void addActionListener(ActionListener al)
  {
    if (!(mActionListeners instanceof Vector)) {
      mActionListeners = new Vector();
    }
    mActionListeners.add(al);
  }

  /**
   * responds to user input
   *
   * @param e <code>ActionEvent</code> from control on the panel
   */
  public void actionPerformed(ActionEvent e) 
  {
    boolean ready = true;
    ready = ready && validateName();
    if (!ready) {
      mMessageLabel.setText("You must enter a name");
    }
    else {
      ready = ready && validateSize();
      if (!ready) {
	mMessageLabel.setText("Incorrect size entered");
      }
    }

    if (ready) {
      ready = ready && validateSubsets();
      if (!ready) {
	mMessageLabel.setText("Incorrect subset in line " + mBadSubsetLine);
      }
    }

    
    if (ready) {
      try {
	mMatroid = new BasisMatroid(mSubsets, mMatroidName);
      }
      catch (Exception g) {
	mTextArea.setText("Error:" + g.toString() + '\n' + 
			  mTextArea.getText());
	mMessageLabel.setText("Subsets failed axioms: see text above");
	ready = false;
      }
    }

    if (ready) {
      ActionListener listener;
      Iterator pListener;
      ActionEvent f = new ActionEvent(this, 
				      ActionEvent.ACTION_PERFORMED,
				      "new BasisMatroid");
      for (pListener = mActionListeners.iterator(); pListener.hasNext(); ) {
	listener = (ActionListener)pListener.next();
	listener.actionPerformed(f);
      }
    }
  }

  //
  // private method validates name of matroid
  //
  private boolean validateName()
  {
    mMatroidName = mNameField.getText().trim();
    return (!(mMatroidName.equals("")));
  }

  //
  // private method validates size of matroid
  //
  private boolean validateSize()
  {
    String s = mSizeField.getText();
    try {
      mMatroidSize = Integer.valueOf(s).intValue();
    }
    catch (Exception e) {
      mMatroidSize = 0;
    }
    return (mMatroidSize > 0);
  }

  //
  // private method validates subsets of matroid -- size must be validated 1st
  //
  private boolean validateSubsets()
  {
    mSubsets = new Vector();
    int lNum = 1;
    boolean answer = false;
    answer = (mMatroidSize > 0);
    if (answer) {
      String text = mTextArea.getText();
      String oneLine;
      StringTokenizer st = new StringTokenizer(text, "\n");
      for (; st.hasMoreElements(); ) {
	oneLine = (String)st.nextElement();
	if (!(getSubset(oneLine))) {
	  mBadSubsetLine = lNum;
	  answer = false;
	  break;
	}
	lNum++;
      }
    }

    return answer;
  }

  // 
  // private method tries to get a subset out of the entries in the passed
  // string, and returns a boolean value indicating whether it worked--if
  // it did, the subset is added to mSubsets
  //
  private boolean getSubset(String s)
  {
    boolean answer = true;
    Subset ss;
    int i;
    ss = new Subset(mMatroidSize);

    for (StringTokenizer st = new StringTokenizer(s); st.hasMoreElements(); ) {
      i = Integer.valueOf((String)st.nextElement()).intValue();
      if ((i > 0) && (i <= mMatroidSize)) {
	ss.set(i-1);
      }
      else {
	answer = false;
	break;
      }
    }
    if (answer) {
      mSubsets.add(ss);
    }
    return answer;
  }

  /**
   * allows this class to be tested from the command line
   *
   * @param arg array of command line arguments; ignored
   */
  public static void main(String[] arg)
  {
    JFrame frame = new JFrame("NewBasisMatroidPanel");
    NewBasisMatroidPanel panel;

    panel = new NewBasisMatroidPanel();

    frame.addWindowListener(new WindowAdapter() {
	public void windowClosing(WindowEvent e) {
	  System.exit(0);
	}
      });

    panel.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent f) {
	  NewBasisMatroidPanel nbmp = (NewBasisMatroidPanel)f.getSource();

	  BasisMatroid matroid = nbmp.getMatroid();

	  System.out.println("Name: " + matroid.getName());
	  System.out.println("Size: " + matroid.getSize());
	  System.out.println("Rank: " + matroid.getRank());
	  System.out.println("Bases:");
	  Vector bases = matroid.getBases();
	  for (Iterator p = bases.iterator(); p.hasNext(); ) {
	    System.out.println(((Subset)p.next()).toString(true));
	  }

	  System.exit(0);
	}
      });
    
    frame.getContentPane().add(panel);
    frame.pack();
    frame.setVisible(true);
  }
}
