import Oid.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
 * A <code>NewFiniteFieldMatroidPanel</code> displays an instance of
 * <code>FiniteFieldMatroidSpecPanel</code>, which allows the user to
 * enter the exponent, size and rank of a matroid represented over
 * GF(q) for some prime power q, and validates the input; and an
 * instance of <code>FiniteFieldMatroidTextArea</code>, which allows
 * the user to enter the matrix representation of a matroid over
 * GF(q). A <code>NewFiniteFieldMatroidPanel</code> instance can then
 * provide a reference to the resulting
 * <code>FiniteFieldMatroid</code> instance to its client.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 3 June 2002
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class NewFiniteFieldMatroidPanel extends JPanel 
  implements ActionListener
{
  FiniteFieldMatroidSpecPanel mSpecPanel;
  FiniteFieldMatroidTextArea  mTextArea;
  FiniteFieldMatroid          mMatroid;
  JTextArea                   mFieldDesc;
  String               mMatroidName;
  Vector               mActionListeners;
  int                  mExponent;
  FiniteField       mField;

  public NewFiniteFieldMatroidPanel()
  {
    mMatroid = (FiniteFieldMatroid)null;
    setLayout(new BorderLayout());

    // 
    // set up spec panel in north area
    //
    mSpecPanel = new FiniteFieldMatroidSpecPanel();
    mSpecPanel.addActionListener(this);
    add(mSpecPanel, BorderLayout.NORTH);
    
    //
    // set up matrix entry panel in center area
    //
    mTextArea = new FiniteFieldMatroidTextArea();
    mTextArea.addActionListener(this);
    add(mTextArea, BorderLayout.CENTER);
    mTextArea.setEnabled(false);

    // 
    // set up explanation label in east area
    //
    mFieldDesc = new JTextArea(10, 40);
    mFieldDesc.setEnabled(true);
    mFieldDesc.setEditable(false);

    //
    // set font to monospaced at current size
    //
    Font f = mFieldDesc.getFont();
    mFieldDesc.setFont(new Font("Monospaced", Font.PLAIN, f.getSize()));

    JScrollPane scrollPane = 
      new JScrollPane(mFieldDesc,
		      JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
		      JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    add(scrollPane, BorderLayout.SOUTH);
  }

  public Matroid getMatroid()
  {
    return mMatroid;
  }

  public void addActionListener(ActionListener al)
  {
    if (!(mActionListeners instanceof Vector)) {
      mActionListeners = new Vector();
    }
    mActionListeners.add(al);
  }

  public void actionPerformed(ActionEvent e) 
  {
    ActionListener al;
    ActionEvent f;
    Object source = e.getSource();
    if (source instanceof FiniteFieldMatroidSpecPanel) {
      mTextArea.setSize(mSpecPanel.getMatroidRank(), 
			mSpecPanel.getMatroidSize());
      mField = mSpecPanel.getField();
      mTextArea.setField(mField);
      mFieldDesc.setText(fieldDesc());
      mMatroidName = mSpecPanel.getMatroidName();
      mTextArea.setEnabled(true);
    }
    else if (source instanceof FiniteFieldMatroidTextArea) {
      if (mActionListeners instanceof Vector) {
	mMatroid = mTextArea.getMatroid();
	mMatroid.setName(mMatroidName);
	f = new ActionEvent(this, 
			    ActionEvent.ACTION_PERFORMED,
			    "new FiniteFieldMatroid");
	for (Iterator pListener = mActionListeners.iterator();
	     pListener.hasNext();) {
	  al = (ActionListener)pListener.next();
	  al.actionPerformed(f);
	}
      }
    }
  }

  //
  // private method generates description of the current field
  //
  private String fieldDesc()
  {
    String ans = "Enter field values using numbers from 0 to " + 
        (mField.getFieldSize() - 1);

    if (mField instanceof GF2kCalculator) {
      GF2kCalculator field2 = (GF2kCalculator)mField;

      ans = "Enter field values as follows:\n";
      ans += "  0 --   0\n";
      ans += "  1 --   1\n";
      for (int i=2; i<mField.getFieldSize(); i++) {
	ans += field2.eltToPowerString(i) + " -- "+field2.eltToString(i)+"\n";
      }
    } 
    else if (mField instanceof GFpkCalculator) {
      GFpkCalculator field = (GFpkCalculator)mField;

      ans = "Enter field values as follows:\n";
      ans += "  0 --   0\n";
      ans += "  1 --   1\n";
      for (int i=2; i<mField.getFieldSize(); i++) {
	ans += field.eltToPowerString(i) + " -- "+field.eltToString(i)+"\n";
      }
    }

    return ans;
  }

  /**
   * allows this class to be tested from the command line
   *
   * @param arg array of command line arguments (ignored)
   */
  public static void main(String[] arg)
  {
    JFrame frame = new JFrame("NewFiniteFieldMatroidPanel");
    NewFiniteFieldMatroidPanel panel = new NewFiniteFieldMatroidPanel();

    panel.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent e) {
	  NewFiniteFieldMatroidPanel p = 
	    (NewFiniteFieldMatroidPanel)e.getSource();
	  FiniteFieldMatroid m = (FiniteFieldMatroid)p.getMatroid();

	  System.out.println("Name:       " + m.getName());
	  System.out.println("Size:       " + m.getSize());
	  System.out.println("Rank:       " + m.getRank());
	  System.out.println("Field size: " + m.getField().getFieldSize());
	  System.out.println("Entries:");
	  
	  int i,j;
	  for (i=0; i<m.getRank(); i++) {
	    for (j=0; j<m.getSize(); j++) {
	      System.out.print(" " + m.getElement(j).get(i));
	    }
	    System.out.println();
	  }
	  System.exit(0);
	}
      });

    frame.addWindowListener(new WindowAdapter() {
	public void windowClosing(WindowEvent e) {
	  System.exit(0);
	}
      });

    frame.getContentPane().add(panel);
    frame.pack();
    frame.setVisible(true);
  }
}
