import Oid.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
 * A <code>NewLSByMatrixPanel</code> displays an instance of
 * <code>FiniteFieldMatroidSpecPanel</code>, which allows the user to
 * enter the field size, size and rank of a matroid represented over
 * GF(q) for some prime power q, and validates the input; and an
 * instance of <code>FiniteFieldMatroidTextArea</code>, which allows
 * the user to enter the matrix representation of a matroid over
 * GF(q). A <code>NewGFpMatroidPanel</code> instance can then generate
 * a <code>LinearSpace</code> instance based on the matroid.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 19 April 2002
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class NewLSByMatrixPanel extends JPanel implements ActionListener
{
  FiniteFieldMatroidSpecPanel mSpecPanel;
  FiniteFieldMatroidTextArea  mTextArea;
  JLabel              mLabel;
  FiniteFieldMatroid          mMatroid;
  LinearSpace         mLinearSpace;
  String              mMatroidName;
  Vector              mActionListeners;

  /**
   * sets up the panel
   */
  public NewLSByMatrixPanel()
  {
    mMatroid = (FiniteFieldMatroid)null;
    setLayout(new BorderLayout());
    
    // 
    // set up spec panel in north area
    //
    mSpecPanel = new FiniteFieldMatroidSpecPanel();
    mSpecPanel.setDefaultRank(3);
    mSpecPanel.addActionListener(this);
    add(mSpecPanel, BorderLayout.NORTH);
    

    mTextArea = new FiniteFieldMatroidTextArea();
    mTextArea.addActionListener(this);
    add(mTextArea, BorderLayout.CENTER);
    mTextArea.setEnabled(false);

    mLabel = new JLabel();
    add(mLabel, BorderLayout.SOUTH);
  }

  /**
   * returns the <code>LinearSpace</code> entered by the user
   *
   * @return reference to space
   */
  public LinearSpace getLinearSpace()
  {
    return mLinearSpace;
  }

  /**
   * returns the matroid representation entered by the user
   *
   * @return <code>Matroid</code> reference
   */
  public Matroid getMatroid()
  {
    return mMatroid;
  }

  /**
   * adds an action listener to this panel
   *
   * @param al <code>ActionListener</code> reference
   */
  public void addActionListener(ActionListener al)
  {
    if (!(mActionListeners instanceof Vector)) {
      mActionListeners = new Vector();
    }
    mActionListeners.add(al);
  }

  /**
   * responds to events generated by panel controls
   *
   * @param e <code>ActionEvent</code> from a panel control
   */
  public void actionPerformed(ActionEvent e) 
  {
    ActionListener al;
    ActionEvent f;
    Object source = e.getSource();
    if (source instanceof FiniteFieldMatroidSpecPanel) {
      mTextArea.setField(mSpecPanel.getField());
      mTextArea.setSize(mSpecPanel.getMatroidRank(), 
			mSpecPanel.getMatroidSize());
      mMatroidName = mSpecPanel.getMatroidName();
      mTextArea.setEnabled(true);
    }
    else if (source instanceof FiniteFieldMatroidTextArea) {
      if (mActionListeners instanceof Vector) {
	mMatroid = mTextArea.getMatroid();
	mMatroid.setName(mMatroidName);
	try {
	  mLinearSpace = new LinearSpace(mMatroid);
	  mLinearSpace.setName(mMatroidName);
	  f = new ActionEvent(this, 
			      ActionEvent.ACTION_PERFORMED,
			      "new FiniteFieldMatroid");
	  for (Iterator pListener = mActionListeners.iterator();
	       pListener.hasNext();) {
	    al = (ActionListener)pListener.next();
	    al.actionPerformed(f);
	  }
	}
	catch (IllegalArgumentException ex) {
	  mLabel.setText("ERROR: " + ex.getMessage());
	}

      }
    }
  }

  /**
   * allows this class to be tested from the command-line
   *
   * @param arg command-line arguments; ignored
   */
  public static void main(String[] arg)
  {
    JFrame frame = new JFrame("NewLSByMatrixPanel");
    NewLSByMatrixPanel panel;

    panel = new NewLSByMatrixPanel();

    panel.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent e) {
	  NewLSByMatrixPanel p = (NewLSByMatrixPanel)e.getSource();
	  LinearSpace ls = p.getLinearSpace();
	  
	  System.out.println("Name: " + ls.getName());
	  System.out.println("Rank: " + ls.getRank());
	  System.out.println("Size: " + ls.getSize());
	  System.out.println("Lines:");
	  
	  Vector lines = ls.getLineVector();
	  for (Iterator pLine = lines.iterator(); pLine.hasNext(); ) {
	    System.out.println(((Subset)pLine.next()).toString(true));
	  }

	  System.exit(0);
	}
      });

    frame.addWindowListener(new WindowAdapter() {
	public void windowClosing(WindowEvent e) {
	  System.exit(0);
	}
      });

    frame.getContentPane().add(panel);
    frame.pack();
    frame.setVisible(true);    
  }
}
