package Oid;
import java.util.*;

/**
 * A BasisMatroid stores a representation of a matroid as a collection of
 * its bases, which are in turn represented as <code>Subset</code>s of a 
 * common ground set.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 9 Nov 2001
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class BasisMatroid implements Matroid, ComplexityProvider
{
  //
  // private data members
  // 
  private int          mSize;
  private int          mRank;
  private Vector       mBases;
  private String[]     mLabel;
  private String       mName;
  private BasisMatroid mDual;

  //
  // constructors
  //
  /**
   * the default constructor sets up the simplest possible matroid, with
   * one basis that has a single element
   */
  public BasisMatroid()
  {
    Vector bases = new Vector();
    try {
      bases.add(new Subset(1,0));
    }
    catch (Exception e) {
      // this "can't happen"
      throw new InternalError(e.toString());
    }
    setDefaultName();
    setDefaultLabels();
  }

  /** 
   * copy constructor 
   *
   * @param matroid another <code>BasisMatroid</code>
   */
  public BasisMatroid(BasisMatroid matroid)
  {
    mSize = matroid.getSize();
    mRank = matroid.getRank();
    mBases = new Vector(matroid.getBases());
    mName = matroid.getName();
    mLabel = new String[mSize];
    for (int i = 0; i<mSize; i++) {
      mLabel[i] = matroid.getLabel(i);
    }
  }

  /** 
   * constructs a new BasisMatroid, if possible, using the passed collection
   * of subsets of a fixed ground set
   *
   * @param bases <code>Collection</code> of <code>Subset</code>s
   * @throws IllegalArgumentException if the collection does not form a 
   * set of bases for a matroid
   */
  public BasisMatroid(Collection bases)
    throws IllegalArgumentException
  {
    checkAxioms(bases);
    setDefaultName();
    setDefaultLabels();
  }

  /**
   * constructs a new BasisMatroid, if possible, using the passed collection
   * of subsets and the passed name
   *
   * @param bases <code>Collection</code> of <code>Subset</code>s
   * @param name name for matroid
   * @throws IllegalArgumentException if the collection does not form a set
   * of bases for a matroid
   */
  public BasisMatroid(Collection bases, String name)
    throws IllegalArgumentException
  {
    checkAxioms(bases);
    mName = new String(name);
    setDefaultLabels();
  }

  /**
   * constructs a new BasisMatroid, if possible, using the passed collection
   * of subsets, name and label names
   *
   * @param bases <code>Collection</code> of <code>Subset</code>s
   * @param name name for matroid
   * @param labels array of element labels
   * @throws IllegalArgumentException if the collection does not form a set
   * of bases for a matroid
   */
  public BasisMatroid(Collection bases, String name, String[] labels)
    throws IllegalArgumentException
  {
    checkAxioms(bases);
    mName = new String(name);
    if (labels.length != mSize) {
      throw new IllegalArgumentException("label array has incorrect length");
    }
    mLabel = new String[mSize];
    for (int i = 0; i<mSize; i++) {
      mLabel[i] = new String(labels[i]);
    }
  }

  /** 
   * returns the collection of bases for this matroid 
   *
   * @return <code>Vector</code> of </code>Subset</code>s listing bases for
   * this matroid
   */
  public Vector getBases()
  {
    return mBases;
  }

  /** 
   * reads number of elements in matroid 
   *
   * @return size of matroid
   */
  public int getSize()
  {
    return mSize;
  }

  /** 
   * reads or determines the rank of the matroid 
   *
   * @return rank of matroid
   */
  public int getRank()
  {
    return mRank;
  }

  /** 
   * sets the matroid's name 
   *
   * @param newName name for matroid
   */
  public void setName(String newName)
  {
    mName = new String(newName);
  }

  /** 
   * returns the matroid's name 
   *
   * @return name of matroid
   */
  public String getName()
  {
    return new String(mName);
  }

  /** 
   * sets the label for one of the matroid's elements
   *
   * @param element index of element (starting with zero)
   * @param label label for element
   * @throws ArrayIndexOutOfBoundsException if <code>element</code> is out
   * of range
   */
  public void setLabel(int element, String label)
    throws ArrayIndexOutOfBoundsException
  {
    if ((element < 0) || (element >= mSize)) {
      throw new ArrayIndexOutOfBoundsException("invalid label index");
    }
    mLabel[element] = new String(label);
  }

  /** 
   * returns the label for one of the matroid's elements 
   *
   * @param element index of element (starting with zero)
   * @return label for element
   * @throws ArrayIndexOutOfBoundsException if <code>element</code> is out
   * of range
   */
  public String getLabel(int element) 
    throws ArrayIndexOutOfBoundsException
  {
    if ((element < 0) || (element >= mSize)) {
      throw new ArrayIndexOutOfBoundsException("invalid label index");
    }
    return new String(mLabel[element]);
  }

  /** 
   * copies the internal state of this matroid to a string in the
   * specified format
   *
   * @param format string specifying format for string ("text" works)
   * @return string with representation of state of this object
   * @throws IllegalArgumentException if <code>format</code> is invalid
   */
  public String getState(String format)
  throws IllegalArgumentException
  {
    String LF = System.getProperty("line.separator");
    StringBuffer state = new StringBuffer("BasisMatroid ");
    Subset b;
    int i;
    if (format.toLowerCase().equals("text") || 
	format.toLowerCase().equals("txt")) {
      state.append(mName);
      state.append(LF);
      state.append("size ");
      state.append(mSize);
      state.append(LF);
      state.append("rank ");
      state.append(mRank);
      state.append(LF);
      state.append("labels" + LF);
      for (i=0; i<mSize; i++) {
	state.append(mLabel[i]);
	state.append(' ');
      }
      state.append(LF + "bases" + LF);
      for (Iterator p = mBases.iterator(); p.hasNext(); ) {
	b = (Subset)p.next();
	try {
	  for (i=0; i<mSize; i++) {
	    if (b.get(i)) {
	      state.append(mLabel[i]);
	      state.append(' ');
	    }
	  }
	  state.append(LF);
	}
	catch (Exception e) {
	  // this "can't happen"
	  throw new InternalError("internal error in BasisMatroid.getState");
	}
      }
    }
    else {
      throw new IllegalArgumentException("Invalid matroid format: " + format);
    }
    return state.toString();
  }

  /** 
   * sets state of this matroid from the specified string using 
   * the specified format
   *
   * @param state string with state information for matroid
   * @param format string specifying format of state string
   * @throws IllegalArgumentException if <code>state</code> or 
   * <code>format</code> is invalid
   */
  public void setState(String state, String format)
  throws IllegalArgumentException
  {
    StringTokenizer stateTokens = new StringTokenizer(state);
    String word, word2;
    String newName = (String)null;
    int newSize=0, newRank=0;
    GFpCalculator newCalc = (GFpCalculator)null;
    Vector newSets = new Vector();
    int elementCount = 0;
    int labelCount = 0;
    String[] newLabel = null;
    int i = 0, j, k, m, n;
    boolean found;
    boolean readingSets=false;
    boolean gotSets=false;
    boolean readingLabels=false;
    boolean gotLabels=false;
    Vector newBases = new Vector();

    // loop through all the words in the string
    for (; stateTokens.hasMoreElements();) {
      word = (String)stateTokens.nextElement();
      // first two tokens must be "BasisMatroid" and the name
      if (readingLabels) {
	newLabel[labelCount++] = new String(word);
	if (labelCount >= newSize) {
	  readingLabels = false;
	  gotLabels = true;
	}
      }
      // if inside the sets portion, just copy the word onto newSets
      else if (readingSets) {
	newSets.add(word);
	elementCount++;
      }
      // if beginning labels, make sure we know size and set up array
      else if (word.toLowerCase().equals("labels")) {
	if (newSize == 0) {
	  throw new IllegalArgumentException("size must be defined " + 
					     "before labels in matroid");
	}
	else {
	  readingLabels = true;
	  newLabel = new String[newSize];
	}
      }
      // make sure first word is "BasisMatroid", not case-sensitive
      else if ((i == 0) && (!word.toLowerCase().equals("basismatroid"))) {
	throw new IllegalArgumentException("Not a BasisMatroid: " + word);
      }
      // second word is the matroid's name
      else if (i == 1) {
	newName = new String(word);
      }
      // beginning of basis listings
      else if (word.toLowerCase().equals("bases")) {
	if ((newSize == 0) || (newRank == 0)) {
	  throw new IllegalArgumentException("bases must be last");
	}
	else {
	  if (!(gotLabels)) {
	    for (j=0; j<mSize; j++) {
	      newLabel[j] = String.valueOf(j+1);
	    }
	    gotLabels = true;
	  }
	  readingSets = true;
	}
      }
      // read the rank
      else if (word.toLowerCase().equals("rank") && 
	       stateTokens.hasMoreElements()) {
	word2 = (String)stateTokens.nextElement();
	try {
	  newRank = Integer.valueOf(word2).intValue();
	}
	catch (NumberFormatException e) {
	  newRank = 0;
	}
	if (newRank <= 0) {
	  throw new IllegalArgumentException("invalid rank " + word2);
	}
      }
      // read the size
      else if (word.toLowerCase().equals("size") && 
	       stateTokens.hasMoreElements()) {
	word2 = (String)stateTokens.nextElement();
	try {
	  newSize = Integer.valueOf(word2).intValue();
	}
	catch (NumberFormatException e) {
	  newSize = 0;
	}
	if (newSize <= 0) {
	  throw new IllegalArgumentException("invalid size " + word2);
	}
      }
      ++i;
    }

    if (elementCount % newRank != 0) {
      throw new IllegalArgumentException("bases not completely defined");
    }

    // define bases -- go through words in definition, search
    // for labels, and define the subsets
    n = elementCount / newRank;
    Iterator pElt = newSets.iterator();
    Subset s;
    String eltLabel;
    mSize = newSize;
    mRank = newRank;
    try {
      for (i=0; i<n; i++) {
	s = new Subset(mSize);
	for (j=0; j<mRank; j++) {
	  eltLabel = (String)pElt.next();
	  found = false;
	  for (k=0; k<mSize; k++) {
	    if (newLabel[k].equals(eltLabel)) {
	      found = true;
	      break;
	    }
	  }
	  if (!found) {
	    throw new IllegalArgumentException("Invalid element: " + 
					       eltLabel);
	  }
	  s.set(k);
	}
	newBases.add(s);
      }
    }
    catch (Exception e) {
      // just throw a description of what went wrong for now
      throw new IllegalArgumentException(e.toString());
    }

    // copy all the new values into the private data members
    checkAxioms(newBases);
    if (newName != null) {
      mName = newName;
    }
    else {
      setDefaultName();
    }
    mLabel = new String[newSize];
    for (i=0; i<mSize; i++) {
      mLabel[i] = newLabel[i];
    }
  }

  /** 
   * determines the rank of a subset of the matroid's elements 
   *
   * @param s subset of base set
   * @return rank of <code>s</code> as a subset of the matroid
   */
  public int subsetRank(Subset s)
  // if the subset is independent, return its size; otherwise, start 
  // with s.numElements() - 1 and loop down through sizes, iterate through
  // subsets, and look for the first independent set
  {
    int answer = 0;
    int k = s.numElements();
    int i, j;
    boolean found;
    SubsetEnumeration se;
    Subset index;
    Subset ss;

    if (s.getSize() == mSize) {
      try {
	if (isIndependent(s)) {
	  answer = k;
	}
	else {
	  found = false;
	  for (j=k-1; j>0 && !found; j--) {
	    for (se = new SubsetEnumeration(k, j); se.hasMoreElements(); ) {
	      index = (Subset)se.nextElement();
	      ss = s.getSubSubset(index);
	      if (isIndependent(ss)) {
		found = true;
		answer = j;
		break;
	      }
	    }
	  }
	}
      }
      catch (Exception e) {
	// this "can't happen"
	throw new InternalError("internal error in subsetRank");
      }
    }

    return answer;
  }

  /** 
   * determines whether a subset of the matroid is a circuit 
   * 
   * @param s subset of the matroid
   * @return <code>true</code> if and only if <code>s</code> is a circuit
   */
  public boolean isCircuit(Subset s)
  // if s is an independent set, the answer is false; otherwise, throw
  // each element of s out, one by one, and make sure the subsets of s
  // are all independent
  {
    boolean answer = true;
    int i;
    Subset s2 = new Subset(s);
    
    if (s.getSize() != mSize) {
      // if the ground set size for s isn't right, always answer false
      answer = false;
    }
    else if (isIndependent(s)) {
      // if s is an independent set, the answer is false
      answer = false;
    }
    else {
      // for each element of s, throw the element out and make sure
      // the resulting set is independent
      try {
	for (i=0; i<mSize; i++) {
	  if (s2.get(i)) {
	    s2.clear(i);
	    if (!isIndependent(s2)) {
	      answer = false;
	      break;
	    }
	    s2.set(i);
	  }
	}
      }
      catch (Exception e) {
	// this "can't happen"
	throw new InternalError("internal error in isCircuit");
      }
    }

    return answer;
  }

  /** 
   * determines whether a subset of the matroid is a flat 
   *
   * @param s subset of the matroid
   * @return <code>true</code> if and only if <code>s</code> is a flat
   */
  public boolean isFlat(Subset s)
  // just see whether s is the same as its closure
  {
    return this.closure(s).equals(s);
  }

  /** 
   * determines whether a subset of the matroid is a basis 
   *
   * @param s subset of the matroid
   * @return <code>true</code> if and only if <code>s</code> is a basis
   */
  public boolean isBasis(Subset s)
  // just check the collection
  {
    return mBases.contains(s);
  }

  /** 
   * determines whether a subset of the matroid is independent 
   *
   * @param s subset of the matroid
   * @return <code>true</code> if and only if <code>s</code> is independent
   */
  public boolean isIndependent(Subset s)
  // look for a member of the bases collection that contains s
  {
    boolean found = false;
    Subset b;

    if (s.numElements() <= mRank) {
      for (Iterator p = mBases.iterator(); p.hasNext(); ) {
	b = (Subset)p.next();
	if (b.subsetContains(s)) {
	  found = true;
	  break;
	}
      }
    }

    return found;
  }

  /** 
   * determines whether a subset of the matroid is a hyperplane 
   *
   * @param s subset of the matroid
   * @return <code>true</code> if and only if <code>s</code> is a hyperplane
   */
  public boolean isHyperplane(Subset s)
  // check rank of s and whether s is closed
  {
    return ((this.subsetRank(s) == mRank-1) && (this.isFlat(s)));
  }

  /** 
   * determines whether an element is in the closure of a set 
   *
   * @param s subset of the matroid
   * @param e index of matroid element
   * @return <code>true</code> if and only if <code>s</code> is in <i>cl(s)</i>
   */
  public boolean isInClosure(Subset s, int e) {
    if ((e < 0) || (e >= mSize) || (s.getSize() != mSize)) {
      return false;
    }

    Subset union = new Subset(s);
    try {
      s.set(e);
    }
    catch (Exception ex) {
      // this "can't happen"
      throw new InternalError("Internal error in isInClosure");
    }
    return (this.subsetRank(union) == this.subsetRank(s));
  }

  /** 
   * determines the closure of a subset of the matroid 
   *
   * @param s subset of the matroid
   * @return closure of <code>s</code>
   */
  public Subset closure(Subset s)
  // throw in each element of the matroid not in s, see which ones
  // don't increase the subsetRank
  {
    Subset answer = null;
    int r;
    int i;

    if (s.getSize() == mSize) {
      answer = new Subset(s);
      r = this.subsetRank(answer);
      try {
	for (i=0; i<mSize; i++) {
	  if (!answer.get(i)) {
	    answer.set(i);
	    if (this.subsetRank(answer) != r) {
	      answer.clear(i);
	    }
	  }
	}
      }
      catch (Exception e) {
	// this "can't happen"
	throw new InternalError("internal error in closure method");
      }
    }

    return answer;
  }

  /** 
   * sets a reference to the dual of this matroid--used when calculating dual
   * to set a reference back to the original matroid
   *
   * @param dual reference to dual
   */
  protected void setDual(BasisMatroid dual)
  {
    mDual = dual;
  }

  /** 
   * returns the dual of this matroid (its bases are the complements of 
   * this matroid's bases)
   * 
   * @return reference to dual of this matroid
   */
  public BasisMatroid getDual()
  // return the complements of all the bases
  {
    BasisMatroid answer = null;
    if (mDual instanceof BasisMatroid) {
      answer = mDual;
    }
    else {
      Vector dualBases = new Vector();
      Subset s;
      for (Iterator pb = mBases.iterator(); pb.hasNext(); ) {
	s = (Subset)pb.next();
	dualBases.add(s.complement());
      }
      
      try {
	answer = new BasisMatroid(dualBases, mName + "*", mLabel);
	answer.setDual(this);
      }
      catch (IllegalArgumentException e) {
	// this "can't happen"
	throw new InternalError("Internal error in getDual: " + e.toString());
      }
    }

    return answer;
  }

  // 
  // private method sets up default labels
  //
  private void setDefaultLabels()
  {
    int i;
    mLabel = new String[mSize];
    for (i=0; i<mSize; i++) {
      mLabel[i] = String.valueOf(i+1);
    }
  }

  // 
  // private method sets up default name
  //
  private void setDefaultName()
  {
    mName = new String("BasisMatroid");
  }

  //
  // private methods for checking the set of axioms for a collection
  // of subsets to be the set of bases for a matroid
  //

  //
  // checkAxioms checks flat axioms and sets private vector of subsets
  // for this matroid
  //
  private void checkAxioms(Collection sets)
  throws IllegalArgumentException
  {
    Iterator pb = sets.iterator();
    Iterator pb2;
    Subset b1;
    Subset b2;
    Object b;

    mBases = new Vector();

    // check axiom B1: collection must be non-empty
    if (sets.size() == 0) {
      throw new IllegalArgumentException("Empty basis collection");
    }

    // check that all sets are subsets of the same ground set
    b = pb.next();
    if (!(b instanceof Subset)) {
      throw new IllegalArgumentException("Collection must contain subsets");
    }
    mSize = ((Subset)b).getSize();
    mRank = ((Subset)b).numElements();
    mBases.add(b);

    for (; pb.hasNext(); ) {
      b = pb.next();
      if (!(b instanceof Subset)) {
	throw new IllegalArgumentException("Collection must contain subsets");
      }
      if (((Subset)b).getSize() != mSize) {
	throw new IllegalArgumentException("Inconsistent ground set sizes");
      }
      if (!(mBases.contains(b))) {
	mBases.add(b);
      }
    }

    // now check axiom B2
    for (pb = mBases.iterator(); pb.hasNext(); ) {
      b1 = (Subset)pb.next();
      for (pb2 = mBases.iterator(); pb2.hasNext(); ) {
	b2 = (Subset)pb2.next();
	if (!b1.equals(b2) && !checkAxiomB2(b1, b2)) {
	  throw new IllegalArgumentException("Axiom B2 fails for subsets: " +
					      b1.toString(true) + " and " +
					      b2.toString(true));
	}
      }
    }
  }

  //
  // private function checks axiom B2 for passed subsets b1 and b2
  //
  private boolean checkAxiomB2(Subset b1, Subset b2)
  {
    Subset d1 = new Subset(b1);
    Subset d2 = new Subset(b2);
    Subset b3;
    boolean found = false;
    boolean answer = true;
    int i, j;

    // loop through elements of d1
    try {
      d1.andNot(b2);
      d2.andNot(b1);
      
      for (i=0; i<mSize; i++) {
	found = true;
	if (d1.get(i)) {
	  // search for element j of d2 such that (b1 - i) U j is a basis
	  found = false;
	  b3 = new Subset(b1);
	  b3.clear(i);
	  for (j=0; j<mSize; j++) {
	    if (d2.get(j)) {
	      b3.set(j);
	      if (isInBases(b3)) {
		found = true;
		break;
	      }
	      b3.clear(j);
	    }
	  }
	}
	if (!found) {
	  answer = false;
	  break;
	}
      }
    }
    catch (Exception e) {
      // this "can't happen"
      throw new InternalError("Internal error in axiom check");
    }
    return answer;
  }

  //
  // private method searches list of bases for a subset
  //
  private boolean isInBases(Subset s)
  {
    boolean answer = false;
    for (Iterator p = mBases.iterator(); p.hasNext(); ) {
      if (s.equals((Subset)p.next())) {
	answer = true;
	break;
      }
    }
    return answer;
  }

  // 
  // ComplexityProvider interface methods
  //
  /** 
   * returns a value estimating the number of steps required to 
   * execute the method with the passed name, or -1 if the method cannot
   * currently be executed
   *
   * @param methodName name of method
   * @return estimate of method's complexity
   */
  public long getComplexityEstimate(String methodName)
  {
    //
    // gives a worst-case estimate for methods that require an argument
    //
    long answer = -1;

    if (methodName.equals("getBases") ||
	methodName.equals("getSize") ||
	methodName.equals("getRank") ||
	methodName.equals("setName") ||
	methodName.equals("getName") ||
	methodName.equals("setLabel") ||
	methodName.equals("getLabel")) {
      answer = 1;
    }
    else if (methodName.equals("getState") ||
	     methodName.equals("setState") ||
	     methodName.equals("isBasis") ||
	     methodName.equals("isIndependent")) {
      answer = mBases.size();
    }
    else if (methodName.equals("subsetRank")) {
      answer = (long)Math.pow(2, mRank) * mBases.size();
    }
    else if (methodName.equals("isCircuit")) {
      answer = mRank * mBases.size();
    }
    else if (methodName.equals("isFlat") ||
	     methodName.equals("isHyperplane") ||
	     methodName.equals("closure")) {
      answer = (long)Math.pow(2, mRank) * mBases.size() * mRank;
    }
    else if (methodName.equals("getDual")) {
      answer = mBases.size();
    }

    return answer;
  }
  
  /** 
   * returns a value estimating the number of steps required to
   * execute the method with the passed name and argument, or -1 if the 
   * method cannot be executed with the passed argument
   *
   * @param methodName name of method
   * @param argument argument for method
   * @return estimate of method's complexity
   */
  public long getComplexityEstimate(String methodName, Object argument)
  {
    long answer = -1;
    Subset s;

    if (argument instanceof Subset) {
      s = (Subset)argument;
      int n = s.numElements();
      if (methodName.equals("subsetRank")) {
	answer = (long)Math.pow(2, n) * mBases.size();
      }
      else if (methodName.equals("isCircuit")) {
	answer = n * mBases.size();
      }
      else if (methodName.equals("isFlat") ||
	       methodName.equals("isHyperplane") ||
	       methodName.equals("closure")) {
	answer = (long)Math.pow(2, n) * mBases.size() * (mSize - n);
      }
      else {
	answer = getComplexityEstimate(methodName);
      }
    }
    else {
      answer = getComplexityEstimate(methodName);
    }
    return answer;
  }

  /** 
   * returns a value estimating the number of steps required to 
   * execute the method with the passed name and collection of arguments
   * (which must be placed in correct order for iteration), or -1 if the
   * method cannot be executed with the passed arguments
   *
   * @param methodName name of method
   * @param arguments arguments for method
   * @return estimate of method's complexity
   */
  public long getComplexityEstimate(String methodName, Collection arguments)
  {
    long answer = -1;

    if (arguments.size() > 0) {
      answer = getComplexityEstimate(methodName,
				     arguments.iterator().next());
    }

    return answer;
  }
}
