package Oid;
import java.util.*;
import java.math.*;

/**
 * The DefaultBasisAlgorithm class, which implements the SubsetLister 
 * interface for use in the Matroid framework, generates a list of the bases 
 * of a passed Matroid.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 9 Oct 2001
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class DefaultBasisAlgorithm implements SubsetLister,
					      ComplexityProvider
{   
  //
  // private data members
  //
  private Matroid mMatroid;
  private Vector mBases;

  /**
   * sets the reference object for this SubsetLister
   *
   * @param arg reference object 
   * @throws IllegalArgumentException if <code>arg</code> is not a 
   * <code>Matroid</code>
   */
  public void setInstance(Object arg)
  throws IllegalArgumentException
  {
    if (!(arg instanceof Matroid)) {
      throw new IllegalArgumentException("Non-matroid passed to " + 
					 "DefaultBasisAlgorithm");
    }
    mMatroid = (Matroid)arg;
    determineBases();
  }

  /**
   * returns the reference object for this SubsetLister 
   * 
   * @return reference object
   */
  public Object getInstance()
  {
    return mMatroid;
  }

  /**
   * generates a list of the relevant Subsets for the reference object 
   * 
   * @return <code>Collection</code> of relevant subsets
   * @throws NullPointerException if no matroid has been set
   */
  public Collection getSubsetList()
  throws NullPointerException
  {
    return mBases;
  }

  /** 
   * generates a Vector containing indexed collections of Subsets for 
   * the reference object -- trivial for this algorithm
   *
   * @return Vector of collections of subsets
   * @throws NullPointerException if no matroid has been set
   */
  public Vector getSubsetListVector()
  throws NullPointerException
  {
    Vector dummy = new Vector();
    dummy.add(mBases);
    return dummy;
  }

  /** 
   * generates a Vector containing the number of Subsets at each index
   * for the reference object -- trivial for this algorithm
   *
   * @return vector of subset counts
   * @throws NullPointerException if no matroid has been set
   */
  public Vector getSubsetCountChain()
  throws NullPointerException
  {
    Vector dummy = new Vector();
    dummy.add(new Integer(mBases.size()));
    return dummy;
  }

  /**
   * returns the name of the property by which the subsets are indexed
   *
   * @return name of property
   * @throws NullPointerException if no matroid has been set
   */
  public String getArrayIndexName()
  throws NullPointerException
  {
    return new String("(no ranking)");
  }

  /**
   * returns a description of the algorithm used to generate subsets 
   *
   * @return algorithm description
   */
  public String getAlgorithmDescription()
  {
    return new String("This algorithm checks all r-element subsets " +
		      "for independence");
  }

  /**
   * returns the name of this algorithm 
   *
   * @return algorithm name
   */
  public String getName()
  {
    return new String("basis calculator");
  }

  // private method determines bases of passed matroid
  private void determineBases()
  {
    int n = mMatroid.getSize();
    int r = mMatroid.getRank();
    Subset s;

    mBases = new Vector();
    try {
      for (SubsetEnumeration pSub = new SubsetEnumeration(n, r);
          pSub.hasMoreElements();) {
        s = (Subset)pSub.nextElement();
        if (mMatroid.isIndependent(s)) {
          mBases.add(s);
        }
      }
    } 
    catch (Exception e) {
      throw new InternalError("Invalid subset size in DefaultBasisAlgorithm");
    }
  }
  //
  // ComplexityProvider interface methods
  //
  /** 
   * returns a value estimating the number of steps required to 
   * execute the method with the passed name, or -1 if the method cannot
   * currently be executed
   *
   * @param methodName name of method
   * @return complexity estimate for method
   */
  public long getComplexityEstimate(String methodName)
  {
    long answer = -1;
    ComplexityProvider cp;

    if ((mMatroid instanceof Matroid) && 
	(mMatroid instanceof ComplexityProvider) &&
	(methodName.equals("getSubsetList") ||
	 methodName.equals("getSubsetListVector") ||
	 methodName.equals("getSubsetCountChain"))) {
      cp = (ComplexityProvider)mMatroid;
      answer = estimateComplexity(mMatroid.getSize(),
				  mMatroid.getRank(),
				  cp.getComplexityEstimate("isIndependent"));
    }

    return answer;
  }
  
  /**
   * returns a value estimating the number of steps required to
   * execute the method with the passed name and argument, or -1 if the 
   * method cannot be executed with the passed argument
   *
   * @param methodName name of method
   * @param argument argument to method
   * @return complexity estimate for method
   */
  public long getComplexityEstimate(String methodName, Object argument)
  {
    long answer = -1;

    if ((argument instanceof Matroid) && 
	(argument instanceof ComplexityProvider) &&
	(methodName.equals("getSubsetList") ||
	 methodName.equals("getSubsetListVector") ||
	 methodName.equals("getSubsetCountChain"))) {
      ComplexityProvider cp = (ComplexityProvider)argument;
      Matroid m = (Matroid)argument;
      answer = estimateComplexity(m.getSize(),
				  m.getRank(),
				  cp.getComplexityEstimate("isIndependent"));
    }

    return answer;
  }

  /**
   * returns a value estimating the number of steps required to 
   * execute the method with the passed name and collection of arguments
   * (which must be placed in correct order for iteration), or -1 if the
   * method cannot be executed with the passed arguments
   *
   * @param methodName name of method
   * @param arguments arguments to method
   * @return complexity estimate for method
   */
  public long getComplexityEstimate(String methodName, Collection arguments)
  {
    long answer = -1;

    if (arguments.size() > 0) {
      Object argument = arguments.iterator().next();
      if ((argument instanceof Matroid) && 
	  (argument instanceof ComplexityProvider) &&
	  (methodName.equals("getSubsetList") ||
	   methodName.equals("getSubsetListVector") ||
	   methodName.equals("getSubsetCountChain"))) {
	ComplexityProvider cp = (ComplexityProvider)argument;
	Matroid m = (Matroid)argument;
	answer = estimateComplexity(m.getSize(),
				    m.getRank(),
				    cp.getComplexityEstimate("isIndependent"));
      }
    }
    return answer;
  }

  //
  // private method that estimates the complexity in terms of the
  // rank of the matroid and the number of steps required to check
  // independence of a set
  //
  private long estimateComplexity(long size, long rank, long steps)
  {
    return steps * nCk(size, rank);
  }

  //
  // private method to get combinations -- only uses BigInteger if 
  // it has to
  //
  private long nCk(long n, long k)
  {
    long answer = 0;
    long i;

    if ((0 == k) || (n == k)) {
      answer = 1;
    }
    else if ((1 == k) || ((n-1) == k)) {
      answer = n;
    }
    else if ((k > 1) && (k < n-1)) {
      if (n > 20) {
	BigInteger bigAns = BigInteger.valueOf(1);
	for (i=n; i>n-k; i--) {
	  bigAns = bigAns.multiply(BigInteger.valueOf((long)i));
	}
	for (i=n-k; i>1; i--) {
	  bigAns = bigAns.divide(BigInteger.valueOf((long)i));
	}
	answer = bigAns.longValue();
      }
      else {
	answer = 1;
	for (i=n; i>n-k; i--) {
	  answer *= i;
	}
	for (i=k; i>1; i--) {
	  answer /= i;
	}
      }
    }
    return answer;
  }
}
