package Oid;
import java.util.*;

/**
 * This class is an implementation of SubsetLister which can determine the
 * circuits of any Matroid implementation.  It works by checking for dependent
 * 1-element sets, and then for k=2, ..., (m.getRank()+1), it checks for 
 * inclusion of smaller circuits first, then checks for dependence.
 * 
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 25 Nov 2001
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class DefaultCircuitAlgorithm implements SubsetLister, 
						ComplexityProvider
{
  //
  // private data members
  // 
  private Matroid mMatroid;
  private Vector  mCircuits;
  private Vector  mChain;

  /**
   * sets the reference object for this SubsetLister 
   *
   * @param arg reference object
   * @throws IllegalArgumentException if <code>arg</code> is not a 
   * <code>Matroid</code>
   */
  public void setInstance(Object arg)
    throws IllegalArgumentException
  {
    if (!(arg instanceof Matroid)) {
      throw new IllegalArgumentException("Matroid required by "
					 + "DefaultCircuitAlgorithm");
    }
    mMatroid = (Matroid)arg;
    determineCircuits();
  }

  /**
   * returns the reference object for this SubsetLister 
   *
   * @return reference object
   */
  public Object getInstance()
  {
    return mMatroid;
  }

  /**
   * generates a list of the relevant Subsets for the reference object 
   * 
   * @return <code>Collection</code> of relevant subsets
   * @throws NullPointerException if no matroid has been set
   */
  public Collection getSubsetList()
    throws NullPointerException
  {
    Vector allCircuits = new Vector();
    Vector v;
    int i;

    for (i=0; i<mCircuits.size(); i++) {
      v = (Vector)mCircuits.get(i);
      allCircuits.addAll(v);
    }
    return allCircuits;
  }

  /** 
   * generates a Vector containing indexed collections of Subsets for 
   * the reference object
   *
   * @return subset list <code>Vector</code>
   * @throws NullPointerException if no matroid has been set
   */
  public Vector getSubsetListVector()
    throws NullPointerException
  {
    if (!(mMatroid instanceof Matroid)) {
      throw new NullPointerException("No matroid set");
    }
    return mCircuits;
  }

  /** 
   * generates a Vector containing the number of Subsets at each index
   * for the reference object
   *
   * @return <code>Vector</code> of subset counts
   * @throws NullPointerException if no matroid has been set
   */
  public Vector getSubsetCountChain()
    throws NullPointerException
  {
    if (!(mMatroid instanceof Matroid)) {
      throw new NullPointerException("No matroid set");
    }
    return mChain;
  }

  /**
   * returns the name of the property by which the subsets are indexed
   *
   * @return name of indexed property
   * @throws NullPointerException if no matroid has been set
   */
  public String getArrayIndexName()
    throws NullPointerException
  {
    return new String("Size");
  }

  /**
   * returns a description of the algorithm used to generate subsets
   *
   * @return algorithm description 
   */
  public String getAlgorithmDescription()
  {
    StringBuffer sb = new StringBuffer();
    sb.append("This class is an implementation of CircuitCalculator which ");
    sb.append("can determine the circuits of any Matroid implementation.  ");
    sb.append("It works by checking for dependent 1-element sets, and then ");
    sb.append("for k=2, ..., (m.getRank()+1), it checks for inclusion ");
    sb.append("of smaller circuits first, then checks for dependence.");
    return sb.toString();
  }

  /**
   * returns this algorithm's name 
   *
   * @return algorithm name
   */
  public String getName() 
  {
    return new String("circuit calculator");
  }

  // determines the matroid's circuits and places them into Vector mCircuits
  private void determineCircuits()
  {
    Iterator pSet;
    int n;
    int kLimit;
    int k, j;
    Subset s, t;
    SubsetEnumeration subEnum;
    boolean bMinimal;
    
    n = mMatroid.getSize();
    /* loop through all candidate circuit sizes: 1 .. rank(M) + 1 */
    kLimit = mMatroid.getRank() + 1;
    mCircuits = new Vector();
    for (k=0; k<=kLimit; k++) {
      mCircuits.add(k, new Vector());
    }
    
    if (kLimit <= n) {
      for (k=1; k<=kLimit; k++) {
	/* loop through all k-subsets of M */
	try {
	  subEnum = new SubsetEnumeration(n, k);
	}
	catch (IllegalArgumentException exSize) {
	  // change this to a proper exception throw later
	  throw new InternalError("Invalid matroid size");
	}
	catch (ArrayIndexOutOfBoundsException exIndex) {
	  // this "can't happen"
	  throw new InternalError("Invalid matroid subset size");
	}
	    
	for (; subEnum.hasMoreElements();) {
	  s = (Subset)subEnum.nextElement();
	  bMinimal = true;
		
	  /* determine whether s contains any previously discovered circuits */
	  for (j=0; j<k; j++) {
	    for (pSet = ((Vector)mCircuits.get(j)).iterator(); 
		 pSet.hasNext() && bMinimal;) {
	      t = (Subset)pSet.next();
	      bMinimal = bMinimal && !s.subsetContains(t);
	    }
	  }
		
	  /* if s contains no circuits and is dependent, it is a circuit */
	  if (bMinimal && !mMatroid.isIndependent(s)) {
	    ((Vector)mCircuits.get(k)).add(s);
	  }
	}
      }
    }
    determineChain();
  }

  private void determineChain()
  {
    Vector v;
    int i;

    mChain = new Vector();
    for (i=0; i<mCircuits.size(); i++) {
      v = (Vector)mCircuits.get(i);
      mChain.add(i, new Integer(v.size()));
    }
  }

  //
  // ComplexityProvider interface methods
  //
  /** 
   * returns a value estimating the number of steps required to 
   * execute the method with the passed name, or -1 if the method cannot
   * currently be executed
   *
   * @param methodName name of method
   * @return complexity estimate
   */
  public long getComplexityEstimate(String methodName)
  {
    long answer = -1;
    ComplexityProvider cp;

    if ((mMatroid instanceof Matroid) && 
	(mMatroid instanceof ComplexityProvider) &&
	(methodName.equals("getSubsetList") ||
	 methodName.equals("getSubsetListVector") ||
	 methodName.equals("getSubsetCountChain"))) {
      cp = (ComplexityProvider)mMatroid;
      answer = estimateComplexity(mMatroid.getRank(),
				  cp.getComplexityEstimate("isIndependent"));
    }

    return answer;
  }
  
  /**
   * returns a value estimating the number of steps required to
   * execute the method with the passed name and argument, or -1 if the 
   * method cannot be executed with the passed argument
   *
   * @param methodName name of method
   * @param argument argument to method
   * @return complexity estimate
   */
  public long getComplexityEstimate(String methodName, Object argument)
  {
    long answer = -1;
    ComplexityProvider cp;
    Matroid m;

    if ((argument instanceof Matroid) && 
	(argument instanceof ComplexityProvider) &&
	(methodName.equals("getSubsetList") ||
	 methodName.equals("getSubsetListVector") ||
	 methodName.equals("getSubsetCountChain"))) {
      cp = (ComplexityProvider)argument;
      m = (Matroid)argument;
      answer = estimateComplexity(m.getRank(),
				  cp.getComplexityEstimate("isIndependent"));
    }

    return answer;
  }

  /**
   * returns a value estimating the number of steps required to 
   * execute the method with the passed name and collection of arguments
   * (which must be placed in correct order for iteration), or -1 if the
   * method cannot be executed with the passed arguments
   *
   * @param methodName name of method
   * @param arguments arguments to method
   * @return complexity estimate
   */
  public long getComplexityEstimate(String methodName, Collection arguments)
  {
    long answer = -1;
    ComplexityProvider cp;
    Matroid m;
    Object argument;

    if (arguments.size() > 0) {
      argument = arguments.iterator().next();
      if ((argument instanceof Matroid) && 
	  (argument instanceof ComplexityProvider) &&
	  (methodName.equals("getSubsetList") ||
	   methodName.equals("getSubsetListVector") ||
	   methodName.equals("getSubsetCountChain"))) {
	cp = (ComplexityProvider)argument;
	m = (Matroid)argument;
	answer = estimateComplexity(m.getRank(),
				    cp.getComplexityEstimate("isIndependent"));
      }
    }
    return answer;
  }

  //
  // private method that estimates the complexity in terms of the
  // rank of the matroid and the number of steps required to check
  // independence of a set
  //
  private long estimateComplexity(long rank, long steps)
  {
    long p = (long)Math.pow(2, rank+1);
    return steps * ((p-1) * (p-2) / 2);
  }
}
