package Oid;
import java.util.*;
import java.io.*;

/**
 * This class accepts two collections of matroids, all of the same
 * size and rank, represented over the same finite field, and lists
 * all single-element extensions of the matroids in the first
 * collection representable over the same field which have no minor
 * (by deletion) among the matroids in the second collection.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 26 Jun 2003
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class ExtMinorCheck
{
  //
  // private data members
  //
  private Vector mColA;
  private Vector mColB;
  private FiniteFieldVector[] mPG;
  private FiniteField mGFq;
  private PrintStream mOut;
  private Vector mResults;

  /**
   * runs the routine with the two passed Vectors of matroids and sends
   * output to the passed PrintStream
   * 
   * @param colA collection of matroids to generate extensions of
   * @param colB collection of minors to avoid
   * @param outStream PrintStream for results
   * @return Vector containing resulting matroids
   * @throws IllegalArgumentException if vectors contain anything other than
   * FiniteFieldMatroids or the matroids are inconsistent
   */
  public Vector findExtensions(Vector colA, Vector colB, PrintStream outStream)
    throws IllegalArgumentException
  {
    checkArguments(colA, colB);
    mOut = outStream;
    computeExtensions();
    return mResults;
  }

  /**
   * runs the routine with the two passed Vectors of matroids and sends
   * output to the standard output stream
   *
   * @param colA collection of matroids to generate extensions of
   * @param colB collection of minors to avoid
   * @return Vector containing resulting matroids
   * @throws IllegalArgumentException if vectors contain anything other than
   * FiniteFieldMatroids or the matroids are inconsistent
   */
  public Vector findExtensions(Vector colA, Vector colB)
    throws IllegalArgumentException
  {
    checkArguments(colA, colB);
    mOut = System.out;
    computeExtensions();
    return mResults;
  }

  public static void main(String[] arg)
  {
    if (arg.length < 2) {
      System.out.println("Usage: java ExtMinorCheck <seed file> " + 
			 "<exclusion file> [<output file>]");
    }
    else {
      Vector seeds = new Vector();
      Vector excluded = new Vector();
      MatrixFileReader reader;

      try {
	for (reader = new MatrixFileReader(arg[0], true);
	     reader.hasMoreElements(); ) {
	  seeds.add(reader.nextElement());
	}
      }
      catch (Exception ex) {
	System.err.println("ERROR: Unable to read seed file");
	System.exit(0);
      }

      try {
	for (reader = new MatrixFileReader(arg[1], true); 
	     reader.hasMoreElements(); ) {
	  excluded.add(reader.nextElement());
	}
      }
      catch (Exception ex) {
	System.err.println("ERROR: Unable to read exclusion file");
	System.exit(0);
      }

      PrintStream ps=(PrintStream)null;
      if (arg.length > 2) {
	try {
	  ps = new PrintStream(new FileOutputStream(arg[2]));
	}
	catch (Exception ex) {
	  System.err.println("Unable to open output file");
	  System.exit(0);
	}
      }
      else {
	ps = System.out;
      }

      ExtMinorCheck checker = new ExtMinorCheck();
      Vector extensions = checker.findExtensions(seeds, excluded, ps);
      
      ps.println("Extension matroids:");
      for (Iterator pExt = extensions.iterator(); pExt.hasNext(); ) {
	ps.println(FiniteFieldMatroidFormatter.displayMatrix
		   ((FiniteFieldMatroid)pExt.next()));
	ps.println();
      }
    }
  }

  //
  // computes extensions 
  //
  private void computeExtensions()
  {
    Vector Bplus = new Vector();
    Vector Cprime = new Vector();
    Vector AChains=new Vector(), ADegrees=new Vector(), ABases=new Vector();
    Vector BChains=new Vector(), BDegrees=new Vector(), BBases=new Vector();
    Vector Ext, ExtDegrees, ExtChains, ExtBases;
    Vector chains=(Vector)null, degrees=(Vector)null, bases=(Vector)null;
    FiniteFieldMatroid M, M1, M2, M3=(FiniteFieldMatroid)null;
    MatroidIsomorphismChecker checker = new MatroidIsomorphismChecker();
    Subset SM, SMExt, SMDel;
    boolean found;
    Iterator p;
    int d, i, j, k, m, n, pg=mPG.length;

    mOut.println("Computing circuit, independent set and basis values " +
		 "for seed matroids...");
    mOut.flush();
    for (p = mColA.iterator(); p.hasNext(); ) {
      M = (FiniteFieldMatroid)p.next();
      checker.setMatroid1(M);
      AChains.add(checker.getChains1());
      ADegrees.add(checker.getDegrees1());
      ABases.add(checker.getBases1());      
    }

    mOut.println("Computing circuit, independent set and basis values " +
		 "for excluded minors...");
    mOut.flush();
    for (p = mColB.iterator(); p.hasNext(); ) {
      M = (FiniteFieldMatroid)p.next();
      SM = PGSubset(M);
      Bplus.add(SM);
      checker.setMatroid1(M);
      BChains.add(checker.getChains1());
      BDegrees.add(checker.getDegrees1());
      BBases.add(checker.getBases1());
    }

    // 
    // loop through first collection
    //
    for (k=0; k<mColA.size(); k++) {
      M = (FiniteFieldMatroid)mColA.get(k);
      
      Ext = new Vector();
      ExtChains = new Vector();
      ExtDegrees = new Vector();
      ExtBases = new Vector();

      SM = PGSubset(M);

      //
      // find non-isomorphic extensions of current matroid
      //
      mOut.println("Generating extensions of matroid " + 
		   M.getName() + "...");
      mOut.flush();
      for (i=0; i<pg; i++) {
	if (!SM.get(i)) {
	  SMExt = new Subset(SM);
	  SMExt.set(i);
	  M1 = PGMatroid(SMExt);
	  found = false;
	  checker.setMatroid1(M1);
	  for (j=0; j<Ext.size(); j++) {
	    checker.setMatroid2((FiniteFieldMatroid)Ext.get(j),
				(Vector)ExtChains.get(j),
				(Vector)ExtDegrees.get(j),
				(Vector)ExtBases.get(j));
	    if (checker.checkMatroids()) {
	      found = true;
	      break;
	    }
	  }
	  
	  if (!found) {
	    M1.setName(M.getName() + "_" + String.valueOf(i));
	    Ext.add(M1);
	    ExtChains.add(checker.getChains1());
	    ExtDegrees.add(checker.getDegrees1());
	    ExtBases.add(checker.getBases1());
	    mOut.println("  " + 
			 String.valueOf(Ext.size()) + " extensions found");
	    mOut.flush();
	  }
	}
      }

      //
      // now check extensions against earlier extensions and for excluded
      // minors
      //
      mOut.println("Checking extensions of matroid " + String.valueOf(k+1) +
		   " against earlier seeds and excluded minors...");
      mOut.flush();
      for (i=0; i<Ext.size(); i++) {
	d = 0;
	M = (FiniteFieldMatroid)Ext.get(i);
	SM = PGSubset(M);

	mOut.println("  Checking extension " + String.valueOf(i+1) +
		     " against earlier extension subsets...");
	mOut.flush();

	//
	// first check subset of PG(R-1,q) against subsets from earlier 
	// extensions
	//
	for (p = Cprime.iterator(); p.hasNext(); ) {
	  if(SM.subsetEquals((Subset)p.next())) {
	    d = 1; // already found
	    mOut.println("    Matroid is an extension of an earlier seed");
	    mOut.flush();
	    break;
	  }
	}

	// 
	// now check single-element deletions of subset of PG(R-1,q)
	// against subsets from excluded minors
	//
	mOut.println("  Checking extension" + String.valueOf(i+1) + 
		     " single-element deletions vs excluded minor subsets...");
	mOut.flush();
	if (0 == d) {
	  for (j=0; j<pg; j++) {
	    if (SM.get(j)) {
	      SMDel = new Subset(SM);
	      SMDel.clear(j);
	      
	      for (p = Bplus.iterator(); p.hasNext(); ) {
		if (SMDel.subsetEquals((Subset)p.next())) { 
		  d = 2; // already rejected
		  mOut.println("    Matroid contains an excluded minor");
		  mOut.flush();
		  break;
		}
	      }
	    }
	    
	    if (0 != d) {
	      break;
	    }
	  }
	}

	//
	// if there still is no conclusion about what to do with M,
	// check single-element deletions against earlier seed matroids
	// for isomorphism, and against excluded minors also
	//
	if (0 == d) {
	  mOut.println("  Checking extension " + String.valueOf(i+1) + 
		       " single-elt deletions against earlier seeds " +
		       "and excluded minors");
	  mOut.flush();
	  for (m=0; m<pg; m++) {
	    if (SM.get(m)) {
	      mOut.print("+");
	      mOut.flush();
	      SMDel = new Subset(SM);
	      SMDel.clear(m);
	      M1 = PGMatroid(SMDel);
	      found = false;
	      checker.setMatroid1(M1);
	      for (n=0; n<k; n++) {
		mOut.print("-");
		mOut.flush();
		checker.setMatroid2((FiniteFieldMatroid)mColA.get(n),
				    (Vector)AChains.get(n),
				    (Vector)ADegrees.get(n),
				    (Vector)ABases.get(n));
		if (checker.checkMatroids()) {
		  found = true;
		  break;
		}
	      }
	      
	      if (found) {
		d = 1;
		mOut.println();
		mOut.println("  Matroid is isomorphic to single-element " +
			     "extension of previous seed");
		mOut.flush();
	      }

	      // 
	      // if the deletion isn't isomorphic to a previous seed,
	      // see if it is isomorphic to one of the excluded minors
	      //
	      if (0 == d) {
		for (n=0; n<Bplus.size(); n++) {
		  mOut.print("|");
		  mOut.flush();
		  M2 = PGMatroid((Subset)Bplus.get(n));
		  checker.setMatroid2(M2,
				      (Vector)BChains.get(n),
				      (Vector)BDegrees.get(n),
				      (Vector)BBases.get(n));
		  if (checker.checkMatroids()) {
		    mOut.println();
		    mOut.println("Matroid has minor isomorphic to an " +
				 "excluded minor");
		    mOut.flush();
		    d = -2; 
		    M3 = M1;
		    chains = checker.getChains1();
		    degrees = checker.getDegrees1();
		    bases = checker.getBases1();
		    break;
		  }
		}
	      }
	      mOut.println();
	      mOut.flush();
	    }

	    if (0 != d) {
	      break;
	    }
	  }
	}

	if (0 == d) {
	  // this is a brand-new extension
	  mOut.println("  Matroid is new; adding to results");
	  mOut.flush();
	  Cprime.add(SM);
	}
	else if (-2 == d) {
	  // M3 is isomorphic to an excluded minor
	  mOut.println("  Adding matroid minor to list of excluded minors");
	  mOut.flush();
	  Bplus.add(PGSubset(M3));
	  BChains.add(chains);
	  BDegrees.add(degrees);
	  BBases.add(bases);
	}
      }
    }

    mResults = new Vector();
    for (p = Cprime.iterator(); p.hasNext(); ) {
      mResults.add(PGMatroid((Subset)p.next()));
    }
  }

  //
  // given a matroid, returns the subset of columns from the PG that
  // correspond to its columns
  //
  private Subset PGSubset(FiniteFieldMatroid M)
  {
    int i, j, n = mPG.length, size = M.getSize(), c;
    FiniteFieldVector c1, c2;
    Subset result = new Subset(n);

    for (i=0; i<size; i++) {
      c1 = M.getElement(i);
      for (j=0; j<n; j++) {
	c2 = mPG[j];
	if (c1.multiple(c2) != 0) {
	  result.set(j);
	  break;
	}
      }
    }
    
    return result;
  }

  //
  // given a subset, returns a FiniteFieldMatroid represented by the matrix
  // made from the PG columns in the subset
  //
  private FiniteFieldMatroid PGMatroid(Subset S)
  {
    FiniteFieldMatroid M = (FiniteFieldMatroid)null;
    int i,j=0,k,n=mPG.length;

    if (S.numElements() > 0) {
      FiniteFieldVector[] v = new FiniteFieldVector[S.numElements()];
      for (i=0; i<n; i++) {
	if (S.get(i)) {
	  v[j++] = new FiniteFieldVector(mPG[i]);
	}
      }
      M = new FiniteFieldMatroid(v);
    }

    return M;
  }

  // 
  // checks vectors for consistency and then makes a local copy
  //
  private void checkArguments(Vector colA, Vector colB)
    throws IllegalArgumentException
  {
    FiniteFieldMatroid M;
    Vector localA = new Vector(), localB = new Vector();
    Iterator pMatroid;
    Object o;
    int rank=-1, size=-1;
    FiniteField GFq = (FiniteField)null;

    for (pMatroid = colA.iterator(); pMatroid.hasNext(); ) {
      o = pMatroid.next();
      if (!(o instanceof FiniteFieldMatroid)) {
	throw new IllegalArgumentException("Seed matroids must be " +
					   "FiniteFieldMatroid instances");
      }
      M = (FiniteFieldMatroid)o;
      if (rank < 0) {
	rank = M.getRank();
	size = M.getSize();
	GFq = M.getField();
      }
      else if (M.getRank() != rank) {
	throw new IllegalArgumentException("Inconsistent matroid ranks");
      }
      else if (M.getSize() != size) {
	throw new IllegalArgumentException("Inconsistent matroid sizes");
      }
      else if (M.getField().getCharacteristic() != GFq.getCharacteristic()) {
	throw new IllegalArgumentException("Inconsistent matrix fields");
      }
      localA.add(M);
    }

    for (pMatroid = colB.iterator(); pMatroid.hasNext(); ) {
      o = pMatroid.next();
      if (!(o instanceof FiniteFieldMatroid)) {
	throw new IllegalArgumentException("Excluded minors must be " +
					   "FiniteFieldMatroid instances");
      }
      M = (FiniteFieldMatroid)o;
      if (rank < 0) {
	rank = M.getRank();
	size = M.getSize();
	GFq = M.getField();
      }
      else if (M.getRank() != rank) {
	throw new IllegalArgumentException("Inconsistent matroid ranks");
      }
      else if (M.getSize() != size) {
	throw new IllegalArgumentException("Inconsistent matroid sizes");
      }
      else if (M.getField().getCharacteristic() != GFq.getCharacteristic()) {
	throw new IllegalArgumentException("Inconsistent matrix fields");
      }
      localB.add(M);
    }

    if (!(GFq instanceof FiniteField)) {
      throw new IllegalArgumentException("No matroids provided");
    }

    mColA = localA;
    mColB = localB;
    mGFq = GFq;
    PGFactory pgf = new PGFactory(rank, mGFq);
    mPG = pgf.getProjectiveSpace();
  }
}
