package Oid;
/**
 * Classes which implement the FiniteField interface perform the basic 
 * arithmetic operations for a finite field, representing field elements
 * as integers (either as elements in GF(p), p prime, or as combinations
 * of powers of the characteristic for GF(p^k)).  This interface also includes
 * some methods for conversion between strings and field elements, mainly
 * for elements of GF(p^k).
 * 
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 14 Aug 2002
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public interface FiniteField
{
  /**
   * returns the field's characteristic
   *
   * @return characteristic prime of this field
   */
  public int getCharacteristic();

  /**
   * returns the power of the characteristic that is the size of the field;
   * that is, the k where the size of the field is p^k
   *
   * @return exponent for this field
   */
  public int getExponent();

  /**
   * returns the size of the field
   *
   * @return field size
   */
  public int getFieldSize();

  /**
   * returns the sum of two field elements
   * 
   * @param x integer representation of a field element
   * @param y integer representation of a field element
   * @return integer representation of the sum of x and y in the field
   */
  public int sum(int x, int y);

  /**
   * returns the difference of two field elements
   *
   * @param x integer representation of a field element
   * @param y integer representation of a field element
   * @return integer representation of x - y in the field
   */
  public int difference(int x, int y);

  /**
   * returns the product of two field elements
   * 
   * @param x integer representation of a field element
   * @param y integer representation of a field element
   * @return integer representation of x * y in the field
   */
  public int product(int x, int y);

  /** 
   * returns the quotient of two field elements
   * 
   * @param x integer representation of a field element 
   * @param y integer representation of a field element (non-zero)
   * @return integer representation of x / y in the field
   * @throws ArithmeticException if y is zero
   */
  public int quotient(int x, int y)
    throws ArithmeticException;

  /**
   * returns the additive inverse of a field element
   * 
   * @param x integer representation of a field element
   * @return negative of x in the field (same as x if characteristic is 2)
   */
  public int negative(int x);

  /**
   * returns the multiplicative inverse of a field element
   *
   * @param x integer representation of a (nonzero) field element
   * @return integer representation of 1/x in the field
   * @throws ArithmeticException if x is zero
   */
  public int inverse(int x) 
    throws ArithmeticException;

  /**
   * returns the indicated power of a field element
   * 
   * @param x integer representation of a field element
   * @param k power to raise x to
   * @return x^k in the field
   * @throws ArithmeticException if k < 0 and x = 0
   */
  public int power(int x, int k)
    throws ArithmeticException;

  /**
   * returns a String representation of a field element
   *
   * @param x integer representation of a field element
   * @return String representation of x
   */
  public String eltToString(int x);

  /**
   * returns the field element represented by a String
   * 
   * @param s String representing a field element
   * @return integer representation of the field element
   * @throws IllegalArgumentException if the format of s is incorrect
   */
  public int stringToElt(String s)
    throws IllegalArgumentException;

  /**
   * returns a String representation of a field element using an additional
   * format parameter
   *
   * @param x integer representation of a field element
   * @param fmt integer specifying a format option
   * @return String representation of x
   */
  public String eltToString(int x, int fmt);

  /**
   * returns the field element represented by a String using an additional
   * format parameter
   *
   * @param s String representation of a field element
   * @param fmt integer specifying a format option
   * @return integer representation of the field element
   * @throws IllegalArgumentException if the format of s is incorrect
   */
  public int stringToElt(String s, int fmt)
    throws IllegalArgumentException;
}
