package Oid;
import java.util.*;

/**
  * Matroid represented as a matrix of column vectors with entries from
  * GF(<i>p</i>^k) for some prime <i>p</i>, k > 0.
  * 
  * @author Robert J. Kingan and Sandra R. Kingan
  * @date 25 Aug 2002
  * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
public class FiniteFieldMatroid implements Matroid, ComplexityProvider
{
  // 
  // private data members
  // 
  int mSize;
  int mRank;
  String mName;
  String[] mLabel;
  FiniteFieldVector[] mMatrix;
  FiniteFieldVector zero;
  FiniteField mField;
  FiniteFieldReduction mReducer;
  FiniteFieldMatroid mDual;

  // 
  // constants
  //
  static final int mDEFAULT_SIZE = 5;
  static final int mDEFAULT_K = 2;
  static final String mBAD_K_MESSAGE = "Invalid K in FiniteFieldMatroid";
  static final String mBAD_ARGUMENT_MESSAGE = 
    "Illegal argument in FiniteFieldMatroid";
  //
  // constructors
  //
  /**
   * The default constructor generates a matroid represented over GF(2) 
   * of rank 5, size 5, whose members are the standard basis.
   */
  public FiniteFieldMatroid()
  {
    mField = new GFpCalculator(2);
    initDefaultMatroid();
    setupLabels();
  }

  /**
   * Given a value for the rank of a matroid, this constructor sets up the
   * matroid on the <code>rank</code> standard basis elements over GF(2)
   * of rank <code>rank</code>.
   *
   * @param rank rank of new matroid
   * @throws IllegalArgumentException if <code>rank</code> <= 0
   */
  public FiniteFieldMatroid(int rank)
    throws IllegalArgumentException
  {
    mField = new GFpCalculator(2);
    initDefaultMatroid(rank);
    setupLabels();
  }

  /**
   * Given values for the rank and base field of a matroid, this constructor
   * sets up the matroid on the <code>rank</code> standard basis elements
   * over GF(<i>p</i>^k) of rank <code>rank</code>.
   *
   * @param rank rank of new matroid
   * @param p characteristic prime of base field for new matroid
   * @param k exponent for base field for new matroid
   * @throws IllegalArgumentException if <code>rank</code> <= 0 or
   * <code>p</code> isn't prime or <code>k</code> < 1
   */
  public FiniteFieldMatroid(int rank, int p, int k)
    throws IllegalArgumentException
  {
    mField = FiniteFieldFactory.createFiniteField(p, k);
    initDefaultMatroid(rank);
    setupLabels();
  }

  /** 
   * Given the rank and base field of a matroid, this constructor sets up
   * the matroid on the <code>rank</code> standard basis elements over the
   * field.
   *
   * @param rank rank of new matroid
   * @param field finite field for new matroid
   * @throws IllegalArgumentException if <code>rank</code> <= 0
   */
  public FiniteFieldMatroid(int rank, FiniteField field)
  {
    mField = field;
    initDefaultMatroid(rank);
    setupLabels();
  }

  /**
   * This constructor sets up a matroid based on the passed two-dimensional
   * array of integers, and the passed base field
   */

  /**
   * Given an array of FiniteFieldVectors and its size, this constructor
   * sets up the matroid on the vectors over GF(<i>p</i>^k), where <i>p</i>
   * and k are common to the vectors.  (An IllegalArgumentException is
   * thrown if the vectors do not share a common field or a common size.)
   *
   * This constructor checks that the size of the FiniteFieldVectors
   * in the passed matrix is the same as the rank of the matrix; i.e.,
   * that the matrix does not need reduction to have the number of
   * rows display its rank.  The matrix does not, however, need to be
   * in standard form.
   *
   * @param matrix array of columns represented as FiniteFieldVectors
   * @throws IllegalArgumentException if the matrix entries are inconsistent
   * @throws ArrayIndexOutOfBoundsException if any matrix entries aren't 
   * instantiated
   */
  public FiniteFieldMatroid(FiniteFieldVector[] matrix)
    throws IllegalArgumentException, ArrayIndexOutOfBoundsException
  {
    int size = matrix.length;
    int i, p, k;
    FiniteField fld;
    checkMatrix(matrix, size);
    
    // 
    // copy reference to calculator, and set size & rank
    //
    mField = matrix[0].getField();
    p = mField.getCharacteristic();
    k = mField.getExponent();
    mRank = matrix[0].getSize();
    mSize = size;

    //
    // make local copies of the vectors
    //
    mMatrix = new FiniteFieldVector[mSize];
    for (i=0; i<mSize; i++) {
      mMatrix[i] = (FiniteFieldVector)matrix[i].clone();
      fld = mMatrix[i].getField();
      if ((mMatrix[i].getSize() != mRank) ||
	  (fld.getCharacteristic() != p) ||
	  (fld.getExponent() != k)) {
	throw new IllegalArgumentException("Inconsistent matrix entries");
      }
    }
    setupZeroVector();
    setupLabels();
  }

  /** copy constructor */
  public FiniteFieldMatroid(FiniteFieldMatroid m)
  {
    int i;
    mSize = m.getSize();
    mRank = m.getRank();
    mMatrix = new FiniteFieldVector[mSize];
    mLabel = new String[mSize];
    mField = m.getField();
    mName = m.getName();

    for (i=0; i<mSize; i++) {
      mMatrix[i] = (FiniteFieldVector)m.getElement(i).clone();
      mLabel[i] = new String(m.getLabel(i));
    }
  }

  //
  // ComplexityProvider interface methods
  //
  /** 
   * returns a value estimating the number of steps required to 
   * execute the method with the passed name, or -1 if the method cannot
   * currently be executed
   *
   * @param methodName name of method to check
   * @return complexity estimate for method, or -1
   */
  public long getComplexityEstimate(String methodName)
  {
    long answer = -1;

    if (methodName.equals("isIndependent") ||
	methodName.equals("isDependent") || 
	methodName.equals("isBasis") ||
	methodName.equals("subsetRank")) {
      answer = (long)mRank * mRank * mRank;
    }
    else if (methodName.equals("closure") ||
	     methodName.equals("isFlat") ||
	     methodName.equals("isHyperplane")) {
      answer = (long)(mSize - mRank) * mRank * mRank * mRank;
    }
    else if (methodName.equals("getDeletion") || 
	     methodName.equals("getContraction")) {
      answer = mSize * mRank;
    }
    else if (methodName.equals("getDual")) {
      if (mDual instanceof FiniteFieldMatroid) {
	answer = 1;
      }
      else if (inStandardForm()) {
	answer = (long)mSize;
      }
      else {
	answer = -1;
      }
    }

    return answer;
  }
  
  /**
   * returns a value estimating the number of steps required to
   * execute the method with the passed name and argument, or -1 if the 
   * method cannot be executed with the passed argument
   *
   * @param methodName name of method to check
   * @param argument argument to check method with
   * @return complexity estimate or -1 
   */
  public long getComplexityEstimate(String methodName, Object argument)
  {
    long answer = -1;

    if (argument instanceof Subset) {
      int size = ((Subset)argument).numElements();
      if (methodName.equals("isIndependent") ||
	  methodName.equals("isDependent") || 
	  methodName.equals("isBasis") ||
	  methodName.equals("subsetRank")) {
	  answer = (long)size * size * size;
      }
      else if (methodName.equals("closure") ||
	       methodName.equals("isFlat") ||
	       methodName.equals("isHyperplane")) {
	  answer = (long)(mSize - size) * size * size * size;
      }
    }
    else if (methodName.equals("getDeletion") || 
	     methodName.equals("getContraction")) {
	answer = mSize * mRank;
    }

    return answer;    
  }

  /**
   * returns a value estimating the number of steps required to 
   * execute the method with the passed name and collection of arguments
   * (which must be placed in correct order for iteration), or -1 if the
   * method cannot be executed with the passed arguments
   *
   * @param methodName name of method to check
   * @param arguments collection of arguments to check method with
   * @return complexity estimate or -1 
   */
  public long getComplexityEstimate(String methodName, Collection arguments)
  {
    long answer = -1;
    Object argument = null;
    Iterator pArg = arguments.iterator();
    if (pArg.hasNext()) {
      argument = pArg.next();
    }

    if (argument instanceof Subset) {
      long size = (long)((Subset)argument).numElements();
      if (methodName.equals("isIndependent") ||
	  methodName.equals("isDependent") || 
	  methodName.equals("isBasis") ||
	  methodName.equals("subsetRank")) {
	  answer = size * size * size;
      }
      else if (methodName.equals("closure") ||
	       methodName.equals("isFlat") ||
	       methodName.equals("isHyperplane")) {
	  answer = (mSize - size) * size * size * size;
      }
    }
    else if (methodName.equals("getDeletion") || 
	     methodName.equals("getContraction")) {
	answer = mSize * mRank;
    }

    return answer;    
  }

  /**
   * overrides Object.toString()
   *
   * @return string representation of this matroid
   */
  public String toString()
  {
    return mName;
  }

  //
  // Matroid interface methods
  //

  /**
   * creates a string describing the state of this matroid in the specified
   * format
   *
   * @param format string describing state format to generate ("text" only
   * for now)
   * @return state of this matroid in the specified format
   */
  public String getState(String format)
  throws IllegalArgumentException
  {
    String LF = System.getProperty("line.separator");
    StringBuffer state = new StringBuffer("FiniteFieldMatroid ");
    int i,j;
    if (format.toLowerCase().equals("text") || 
	format.toLowerCase().equals("txt")) {
      state.append(mName);
      state.append(LF);
      state.append("k ");
      state.append(mField.getExponent());
      state.append(LF);
      state.append("p ");
      state.append(mField.getCharacteristic());
      state.append(LF);
      state.append("size ");
      state.append(mSize);
      state.append(LF);
      state.append("rank ");
      state.append(mRank);
      state.append(LF);
      state.append("labels" + LF);
      for (i=0; i<mSize; i++) {
	state.append(mLabel[i]);
	state.append(' ');
      }
      state.append(LF + "matrix" + LF);
      try {
	for (i=0; i<mRank;i++) {
	  for (j=0; j<mSize; j++) {
	    state.append(mMatrix[j].get(i));
	    state.append(' ');
	  }
	  state.append(LF);
	}
      }
      catch (Exception e) {
	// this "can't happen"
	throw new InternalError("Internal Error in " + 
				"FiniteFieldMatroid.getState");
      }
    }
    else {
      throw new IllegalArgumentException("Invalid matroid format: " + format);
    }
    return state.toString();
  }

  /** 
   * sets the internal state of this matroid using the passed string in the
   * specified format
   *
   * @param state string containing state of matroid in specified format
   * @param format string describing format of state ("text" only right now)
   */
  public void setState(String state, String format)
  throws IllegalArgumentException
  {
    StringTokenizer stateTokens = new StringTokenizer(state);
    String word, word2;
    String newName = (String)null;
    int newK=0, newP = 0, newSize=0, newRank=0;
    FiniteField newField = (FiniteField)null;
    Vector newMatrixEntries = new Vector();
    Vector newLabels = new Vector();
    int i = 0;
    boolean readingMatrix=false;
    boolean gotMatrix=false;
    boolean readingLabels=false;
    boolean gotLabels=false;

    for (; stateTokens.hasMoreElements();) {
      word = (String)stateTokens.nextElement();
      // first two tokens must be "FiniteFieldMatroid" and the name
      if (readingLabels) {
	newLabels.add(word);
	if (newLabels.size() >= newSize) {
	  readingLabels = false;
	  gotLabels = true;
	}
      }
      else if (readingMatrix) {
	newMatrixEntries.add(word);
	if (newMatrixEntries.size() >= newSize * newRank) {
	  gotMatrix = true;
	}
      }
      else if (word.toLowerCase().equals("labels")) {
	if (newSize == 0) {
	  throw new IllegalArgumentException("size must be defined " + 
					     "before labels in matroid");
	}
	else {
	  readingLabels = true;
	}
      }
      else if ((i == 0) && 
	       (!word.toLowerCase().equals("finitefieldmatroid"))) {
	throw new IllegalArgumentException("Not a FiniteFieldMatroid: " + 
					   word);
      }
      else if (i == 1) {
	newName = new String(word);
      }
      else if (word.toLowerCase().equals("matrix")) {
	if ((newSize == 0) || (newRank == 0) || (newK == 0) || (newP == 0)) {
	  throw new IllegalArgumentException("matrix must be last");
	}
	else {
	  readingMatrix = true;
	}
      }
      else if (word.toLowerCase().equals("rank") && 
	       stateTokens.hasMoreElements()) {
	word2 = (String)stateTokens.nextElement();
	try {
	  newRank = Integer.valueOf(word2).intValue();
	}
	catch (NumberFormatException e) {
	  newRank = 0;
	}
	if (newRank <= 0) {
	  throw new IllegalArgumentException("invalid rank " + word2);
	}
      }
      else if (word.toLowerCase().equals("size") && 
	       stateTokens.hasMoreElements()) {
	word2 = (String)stateTokens.nextElement();
	try {
	  newSize = Integer.valueOf(word2).intValue();
	}
	catch (NumberFormatException e) {
	  newSize = 0;
	}
	if (newSize <= 0) {
	  throw new IllegalArgumentException("invalid size " + word2);
	}
      }
      else if (word.toLowerCase().equals("k") &&
	       stateTokens.hasMoreElements()) {
	word2 = (String)stateTokens.nextElement();
	try {
	  newK = Integer.valueOf(word2).intValue();
	  if (newP != 0) {
	    newField = FiniteFieldFactory.createFiniteField(newP, newK);
	  }
	}
	catch (NumberFormatException e) {
	  throw new IllegalArgumentException("invalid K " + word2);
	}
	catch (IllegalArgumentException e) {
	  throw new IllegalArgumentException("K is invalid: " + word2);
	}
      }
      else if (word.toLowerCase().equals("p") &&
	       stateTokens.hasMoreElements()) {
	word2 = (String)stateTokens.nextElement();
	try {
	  newP = Integer.valueOf(word2).intValue();
	  if (newK != 0) {
	    newField = FiniteFieldFactory.createFiniteField(newP, newK);
	  }
	}
	catch (NumberFormatException e) {
	  throw new IllegalArgumentException("invalid prime " + word2);
	}
	catch (IllegalArgumentException e) {
	  throw new IllegalArgumentException("p is not prime: " + word2);
	}
      }
      ++i;
    }

    if (!gotMatrix) {
      throw new IllegalArgumentException("matrix not completely defined");
    }

    // define matrix
    FiniteFieldVector[] newMatrix = new FiniteFieldVector[newSize];
    int j,k;
    for (i=0; i<newSize; i++) {
      newMatrix[i] = new FiniteFieldVector(newRank, newField);
    }
    
    i=0;
    j=0;
    for (Iterator pMatrix = newMatrixEntries.iterator();
	 pMatrix.hasNext() && (i<newRank) && (j<newSize);) {
      try {
	k = Integer.valueOf((String)pMatrix.next()).intValue(); 
      }
      catch (NumberFormatException e) {
	throw new IllegalArgumentException("Invalid number format in row "  +
					   i + " column " + j);
      }
      newMatrix[j].set(i, k);

      if (++j >= newSize) {
	j=0;
	i++;
      }
    }

    // copy all the new values into the private data members
    mRank = newRank;
    mField = newField;
    mSize = newSize;
    mMatrix = newMatrix;
    if (newName != null) {
      mName = newName;
    }
    else {
      mName = new String("FiniteFieldMatroid");
    }
    mLabel = new String[newSize];
    if (gotLabels) {
      i=0;
      for (Iterator pLabels = newLabels.iterator();
	   pLabels.hasNext() && (i<mSize);) {
	mLabel[i++] = (String)pLabels.next();
      }
    }
    else {
      setupLabels();
    }
    zero = new FiniteFieldVector(newSize, newField);
    mDual = null;
  }

  // 
  // set/gets
  //

  /** 
   * sets the matroid representation's name 
   *
   * @param s name for matroid
   */
  public void setName(String s)
  {
    mName = new String(s);
  }

  /** 
   * returns the matroid representation's name 
   *
   * @return name of matroid
   */
  public String getName()
  {
    String sAns = new String(mName);
    return sAns;
  }

  /** 
   * sets the label for one of the matroid's elements 
   * 
   * @param element index of element to set name (0-based)
   * @param label new label for element
   * @throws ArrayIndexOutOfBoundsException if <code>element</code> is invalid
   */
  public void setLabel(int element, String label) 
    throws ArrayIndexOutOfBoundsException
  {
    if ((element < 0) || (element >= mSize)) {
      throw new ArrayIndexOutOfBoundsException("Invalid matroid element");
    }

    mLabel[element] = new String(label);
  }

  /** 
   * returns the label of one of the matroid's elements 
   *
   * @param element index of element (0-based)
   * @return string with element's label
   * @throws ArrayIndexOutOfBoundsException if <code>element</code> is invalid
   */
  public String getLabel(int element)
    throws ArrayIndexOutOfBoundsException
  {
    if ((element < 0) || (element >= mSize)) {
      throw new ArrayIndexOutOfBoundsException("Invalid matroid element");
    }

    return new String(mLabel[element]);
  }
    

  /**
   * getSize returns the size of the matroid
   *
   * @return size of matroid
   */
  public int getSize() 
  {
    return mSize;
  }

  /**
   * getRank returns the rank of the matroid (also the size of its vectors)
   *
   * @return rank of matroid
   */
  public int getRank()
  {
    return mRank;
  }

  /**
   * getFiniteField returns a reference to the matroid's base field
   *
   * @return <code>FiniteField</code> reference
   */
  public FiniteField getField() 
  {
    return mField;
  }

  /**
   * getElement returns a clone of one of the
   * <code>FiniteFieldVectors</code> forming the matroid
   *
   * @param element element index (0-based)
   * @throws ArrayIndexOutOfBoundsException if <code>element</code> is invalid
   */
  public FiniteFieldVector getElement(int element)
    throws ArrayIndexOutOfBoundsException
  {
    return (FiniteFieldVector)mMatrix[element].clone();
  }

  
  //
  // other Matroid interface methods
  //
  /** 
   * determines the rank of a subset of the matroid's elements 
   *
   * @param s subset of matroid's base set
   * @return integer with subset's rank
   */
  public int subsetRank(Subset s)
  {
    int theRank;
    int n;
    int i;
    FiniteFieldVector[] subMatrix;

    checkSubset(s);

    n = s.numElements();
    if (n == 0) {
      return 0;
    }
    else if (n == 1) {
      //
      // find the element -- rank is one if it is nonzero, zero otherwise
      //
      i = s.firstElement();
      
      if (mMatrix[i].isZero()) {
        theRank = 0;
      }
      else {
        theRank = 1;
      }
    }
    else {
      if (!(mReducer instanceof FiniteFieldReduction)) {
        mReducer = new FiniteFieldReduction(mRank, mSize, mField);
      }
      subMatrix = getSubmatrix(s);
      theRank = mReducer.getRank(subMatrix, s.numElements());
    }

    return theRank;
  }

  /** 
   * determines whether a subset of the matroid is a circuit 
   *
   * @param s subset of matroid's base set
   * @return boolean indicating whether <code>s</code> is a circuit
   * @throws IllegalArgumentException if <code>s</code> isn't valid
   */
  public boolean isCircuit(Subset s) 
    throws IllegalArgumentException
  {
    checkSubset(s);
    
    int n = s.numElements();
    int i;
    if ((n > mRank+1) || (0 == n)) {
      return false;
    }
    else if (1 == n) {
      i = s.firstElement();
      return mMatrix[i].isZero();
    }
    else if (this.isIndependent(s)) {
      return false;
    }
    
    Subset sCopy;
    boolean bAns = true;
    for (i=0; i<mSize && bAns; i++) {
      if (s.get(i)) {
	sCopy = (Subset)s.clone();
	sCopy.clear(i);
	bAns = bAns && this.isIndependent(sCopy);
      }
    }
    
    return bAns;
  }

  /** 
   * determines whether a subset of the matroid is a flat 
   *
   * @param s subset of matroid's base set
   * @return boolean indicating whether <code>s</code> is a flat
   * @throws IllegalArgumentException if <code>s</code> is not a valid subset
   */
  public boolean isFlat(Subset s) 
  throws IllegalArgumentException
  {
    return s.equals(this.closure(s));
  }

  /** 
   * determines whether a subset of the matroid is a basis 
   *
   * @param s subset of matroid's base set
   * @return boolean indicating whether <code>s</code> is a basis
   * @throws IllegalArgumentException if <code>s</code> is not a valid subset
   */
  public boolean isBasis(Subset s)
    throws IllegalArgumentException
  {
    checkSubset(s);
    
    int n = s.numElements();
    if (mRank != n) {
      return false;
    }
    
    return this.isIndependent(s);
  }

  /** 
   * determines whether a subset of the matroid is independent
   *
   * @param s subset of matroid's base set
   * @return boolean indicating whether <code>s</code> is independent
   * @throws IllegalArgumentException if <code>s</code> is not a valid subset
   */
  public boolean isIndependent(Subset s)
    throws IllegalArgumentException
  {
    checkSubset(s);
    
    int i, n;
    FiniteFieldVector[] subMatrix;
    
    n = s.numElements();
    if (0 == n) {
      return true;
    }
    else if (1 == n) {
      i = s.firstElement();
      return (!mMatrix[i].isZero());
    }
    
    subMatrix = getSubmatrix(s);
    if (!(mReducer instanceof FiniteFieldReduction)) {
      mReducer = new FiniteFieldReduction(mRank, mSize, mField);
    }
    i = mReducer.getRank(subMatrix, n);
    return (i == n);
  }

  /** 
   * determines whether a subset of the matroid is a hyperplane
   *
   * @param s subset of matroid's base set
   * @return boolean indicating whether <code>s</code> is a hyperplane
   * @throws IllegalArgumentException if <code>s</code> is not a valid subset
   */
  public boolean isHyperplane(Subset s) 
  {
    checkSubset(s);
    
    // 
    // if the number of elements is less than r(M) - 1, the set cannot
    // possibly be closed and of rank r(M) - 1.
    //
    if (s.numElements() < mRank-1) {
      return false;
    }
    
    Subset s1 = this.closure(s);
    if (!s1.equals(s)) {
      return false;
    }
    
    return (this.subsetRank(s) == mRank - 1);
  }
  
  /** 
   * determines whether the matroid's matrix is in standard form; that is,
   * of the form [ I_r | D ], where I_r is the rank x rank identity matrix
   *
   * @return boolean value true if and only if matroid is in standard form
   */
  public boolean inStandardForm()
  {
    int i, j;
    boolean bAns = true;

    for (i=0; i<mRank; i++) {
      for (j=0; j<mRank; j++) {
	if (((i == j) & (mMatrix[i].get(j) != 1)) ||
	    ((i != j) & (mMatrix[i].get(j) != 0))) {
	  bAns = false;
	  break;
	}
      }
      if (!bAns) {
	break;
      }
    }
    
    return bAns;
  }
  
  /** 
   * determines if an element is in the closure of a set 
   *
   * @param s subset of base set of matroid
   * @param e element index (0-based)
   * @return boolean indicating whether e is in the closure of s
   */
  public boolean isInClosure(Subset s, int e) 
  {
    if ((e < 0) || (e >= mSize) || (s.getSize() != mSize)) {
      return false;
    }

    FiniteFieldVector[] subMatrix = getSubmatrix(s);
    int n = s.numElements();
    return mReducer.isInSpan(subMatrix, n, mMatrix[e]);
  }

  /** 
   * determines the closure of a subset of the matroid 
   *
   * @param s subset of base set of matroid
   * @return subset of base set of matroid with closure of <code>s</code>
   */
  public Subset closure(Subset s)
  {
    Subset theClosure = (Subset)s.clone();
    int n = s.numElements();
    int i;
    FiniteFieldVector[] subMatrix;

    checkSubset(s);

    if (0 == n) {
      //
      // first make sure all loops are within closure
      //
      for (i=0; i<mSize; i++) {
	if (mMatrix[i].isZero()) {
	  theClosure.set(i);
	}
      }
    }
    else {
      if (!(mReducer instanceof FiniteFieldReduction)) {
	      mReducer = new FiniteFieldReduction(mRank, mSize, mField);
      }
      //
      // generate array of column vector references for original set
      //
      subMatrix = getSubmatrix(s);
      for (i=0; i<mSize; i++) {
	//
	// check each element not already in closure -- in span of s?
	//
	if (!(theClosure.get(i))) {
	  if (mReducer.isInSpan(subMatrix, n, mMatrix[i])) {
	    theClosure.set(i);
	  }
	}
      }
    }
    
    return theClosure;
  }

  /** 
   * sets a reference to this matroid's dual if it has already been
   * calculated--this is only called by the dual itself
   *
   * @param m dual of this matroid
   */
  protected void setDual(FiniteFieldMatroid m)
  {
    mDual = m;
  }

  /**
   * getDual returns, if possible, a reference to the matroid's dual.  If 
   * the matroid already carries a reference to its dual (either through 
   * generating it previously, or otherwise), it simply returns the reference.
   * Otherwise, it checks to see whether it is in standard form.  If not, 
   * an ImpossibleMatroidOperationException is thrown.  Otherwise, it generates
   * its dual by generating the matrix:
   *
   * [ -D<sup>T</sup> | I<sub>n-r</sub> ]
   * 
   * where the matroid is in the form 
   *
   * [ I<sub>r</sub> | D ]
   *
   * @return dual of this matroid
   * @throws IllegalStateException if the dual cannot be determined
   */ 
  public FiniteFieldMatroid getDual() 
  throws IllegalStateException
  {
    // if the dual is known already, just return it
    if (mDual instanceof FiniteFieldMatroid) {
      return mDual;
    }
    
    // cannot generate the dual if the matroid is not in standard form
    if (!this.inStandardForm()) {
      throw new IllegalStateException();
    }
    
    FiniteFieldVector[] newMatrix;
    FiniteFieldVector[] D;
    FiniteFieldVector[] Dt;
    int i;
    
    // if the rank and the size are the same, the matroid is self-dual
    if (mSize == mRank) {
      newMatrix = mMatrix;
    }
    else {
      // first, get -D<sup>T</sup>
      D = new FiniteFieldVector[mSize-mRank];
      for (i=mRank; i<mSize; i++) {
        D[i-mRank] = mMatrix[i];
	D[i-mRank].multiply(mField.negative(1));
      }
      
      if (!(mReducer instanceof FiniteFieldReduction)) {
        mReducer = new FiniteFieldReduction(mRank, mSize, mField);
      }
      Dt = mReducer.transpose(D, mSize-mRank);
      
      // put -D<sup>T</sup> into the left side
      newMatrix = new FiniteFieldVector[mSize];
      for (i=0; i<mRank; i++) {
        newMatrix[i] = (FiniteFieldVector)Dt[i].clone();
      }
      
      // put I<sub>n-r</sub> into the right side
      for (i=mRank; i<mSize; i++) {
        newMatrix[i] = new FiniteFieldVector(mSize-mRank, mField);
        newMatrix[i].set(i-mRank,1);
      }
    }
    
    mDual = new FiniteFieldMatroid(newMatrix);
    mDual.setName(new String(mName + "*"));
    mDual.setDual(this);
    return mDual;
  }

  /** 
   * generates the matroid determined by deleting the indicated element 
   *
   * @param element element index (0-based)
   * @return matroid with <code>element</code> deleted
   * @throws ArrayIndexOutOfBoundsException if <code>element</code> is invalid
   */
  public FiniteFieldMatroid getDeletion(int element) 
  throws ArrayIndexOutOfBoundsException
  {
    if ((element < 0) || (element >= mSize)) {
      throw new ArrayIndexOutOfBoundsException("Invalid element: " + element);
    }

    FiniteFieldMatroid answer = new FiniteFieldMatroid(this);
    answer.setName(mName + '\\' + mLabel[element]);
    answer.deleteElement(element);

    return answer;
  }

  /** 
   * generates the matroid determined by contracting the indicated element 
   *
   * @param element element index (0-based)
   * @return matroid with <code>element</code> contracted
   * @throws ArrayIndexOutOfBoundsException if <code>element</code> is invalid
   */
  public FiniteFieldMatroid getContraction(int element)
  throws ArrayIndexOutOfBoundsException
  {
    if ((element < 0) || (element >= mSize)) {
      throw new ArrayIndexOutOfBoundsException("Invalid element: " + element);
    }

    FiniteFieldMatroid answer = new FiniteFieldMatroid(this);
    answer.setName(mName + "/" + mLabel[element]);
    answer.contractElement(element);

    return answer;
  }

  /** 
   * deletes the indicated element from the current FiniteFieldMatroid 
   *
   * @param element element index (0-based)
   * @throws ArrayIndexOutOfBoundsException if <code>element</code> is invalid
   */
  public void deleteElement(int element)
    throws ArrayIndexOutOfBoundsException
  {
    //
    // if the matrix is in standard form and 0 <= element < rank, then
    // this routine needs to either find another column in the matrix 
    // with a one in the same position and pivot, or delete the row (and
    // decrease the rank) if there is no other 1 in that row
    //
    // if the matrix is not in standard form, just delete the column, and
    // check for any rows that are all zero
    //
    if ((element < 0) || (element >= mSize)) {
      throw new ArrayIndexOutOfBoundsException("Invalid element: " + element);
    }

    FiniteFieldVector[] newMatrix = deleteEntry(element);
    String[] newLabel = deleteLabel(element);
    int i;
    int newSize = mSize-1, newRank = mRank;

    if (mSize > 1) {
      if (inStandardForm()) {
	if (element < mRank) {
	  i = nonZeroIndex(newMatrix, newSize, element);
	  if (i < 0) {
	    newMatrix = deleteRow(newMatrix, newSize, element);
	    newRank--;
	  }
	  else {
	    pivot(newMatrix, newSize, element, i);
	    newMatrix = movePivotColumn(newMatrix, newSize, element, i);
	    newLabel = movePivotColumnLabel(newLabel, newSize, element, i);
	  }
	}
      }
      else {
	i = firstZeroRow(newMatrix, newSize);
	while ((i >= 0) && (newMatrix instanceof FiniteFieldVector[])) {
	  newMatrix = deleteRow(newMatrix, newSize, i);
	  newRank--;
	  i = firstZeroRow(newMatrix, newSize);
	}
      }
    }

    mRank = newRank;
    mSize = newSize;
    if ((newRank > 0) && (newSize > 0)) {
      mMatrix = new FiniteFieldVector[mSize];
      mLabel = new String[mSize];
      for (i=0; i<mSize; i++) {
	mMatrix[i] = newMatrix[i];
	mLabel[i] = newLabel[i];
      }
      setupZeroVector();
    }
  }

  /** 
   * contracts the indicated element from the current FiniteFieldMatroid 
   *
   * @param element element index (0-based)
   * @throws ArrayIndexOutOfBoundsException if <code>element</code> is invalid
   */
  public void contractElement(int element)
  throws ArrayIndexOutOfBoundsException
  {
    // if the matrix is in standard form and 0 <= element < mRank, then
    // contraction just consists of removing row element and column 
    // element from the matrix
    //
    // if the matrix is in standard form and mRank <= element, then
    // contraction consists of finding a nonzero entry in column element,
    // pivoting on it, deleting the row that we pivoted on and the column
    // element, but then transferring the basis vector corresponding to the
    // pivot row to the old element's location (along with its label)
    //
    // if the matrix is not in standard form, we just find a nonzero entry
    // in the column to be contracted, pivot on it, and then remove the column
    // and the pivoted row
    // 
    // if the element to be contracted is a loop, just delete the column, 
    // contraction does not change the rank
    //
    if ((element < 0) || (element >= mSize)) {
      throw new ArrayIndexOutOfBoundsException("Invalid matroid element");
    }

    int pivotRow;
    int i;
    FiniteFieldVector[] newMatrix = (FiniteFieldVector[])null;
    int newSize;
    int newRank;
    String[] newLabel = deleteLabel(element);

    if (mMatrix[element].isZero()) {
      newMatrix = deleteEntry(element);
      newSize = mSize-1;
      newRank = mRank;
    }
    else {
      if (inStandardForm()) {
	if (element < mRank) {
	  newMatrix = deleteEntry(element);
	  newSize = mSize-1;
	  newMatrix = deleteRow(newMatrix, newSize, element);
	  newRank = mRank-1;
	}
	else {
	  pivotRow = nonZeroVectorIndex(mMatrix[element]);
	  if (pivotRow < 0) {
	    // we've already handled the zero column case, so 
	    // this shouldn't happen
	    throw new InternalError("Internal Error in contraction");
	  }
	  pivot(mMatrix, mSize, pivotRow, element);
	  newMatrix = deleteEntry(element);
	  newSize = mSize-1;
	  newMatrix = deleteRow(newMatrix, newSize, pivotRow);
	  newRank = mRank-1;
	  newMatrix = movePivotColumn(newMatrix, newSize, 
				      element-1, pivotRow);
	  newLabel = movePivotColumnLabel(newLabel, newSize, 
					  element-1, pivotRow);
	}
      }
      else {
	pivotRow = nonZeroVectorIndex(mMatrix[element]);
	if (pivotRow < 0) {
	  // again, we've already handled the zero vector case
	  throw new InternalError("Internal error in contraction");
	}
	pivot(mMatrix, mSize, pivotRow, element);
	newMatrix = deleteEntry(element);
	newSize = mSize-1;
	newMatrix = deleteRow(newMatrix, newSize, pivotRow);
	newRank = mRank-1;
	// eliminate any all-zero rows
	i = firstZeroRow(newMatrix, newSize);
	while ((i >= 0) && (newMatrix instanceof FiniteFieldVector[])) {
	  newMatrix = deleteRow(newMatrix, newSize, i);
	  newRank--;
	  i = firstZeroRow(newMatrix, newSize);
	}
      }
    }

    mRank = newRank;
    mSize = newSize;
    if ((newRank > 0) && (newSize > 0)) {
      mMatrix = new FiniteFieldVector[mSize];
      mLabel = new String[mSize];
      for (i=0; i<mSize; i++) {
	mMatrix[i] = newMatrix[i];
	mLabel[i] = newLabel[i];
      }
      setupZeroVector();
    }
  }

  // 
  // private method returns a copy of the matroid's matrix, without
  // the indicated element
  //
  private FiniteFieldVector[] deleteEntry(int element)
  {
    FiniteFieldVector[] newMatrix = new FiniteFieldVector[mSize-1];
    int i;

    for (i=0; i<mSize-1; i++) {
      if (i < element) {
	newMatrix[i] = mMatrix[i];
      }
      else {
	newMatrix[i] = mMatrix[i+1];
      }
    }
    return newMatrix;
  }

  //
  // private method deletes the indicated label
  //
  private String[] deleteLabel(int element)
  {
    String[] newLabel = new String[mSize-1];
    int i;

    for (i=0; i<mSize-1; i++) {
      if (i < element) {
	newLabel[i] = mLabel[i];
      }
      else {
	newLabel[i] = mLabel[i+1];
      }
    }
    return newLabel;
  }

  //
  // private method moves the pivoted column into place
  //
  private FiniteFieldVector[] movePivotColumn(FiniteFieldVector[] matrix, 
					      int size, int newPos, int oldPos)
  {
    int i;
    FiniteFieldVector[] newMatrix = new FiniteFieldVector[size];
    
    if (newPos < oldPos) {
      for (i=0; i<size; i++) {
	if ((i < newPos) || (i > oldPos)) {
	  newMatrix[i] = matrix[i];
	}
	else if (i == newPos) {
	  newMatrix[i] = matrix[oldPos];
	}
	else {
	  newMatrix[i] = matrix[i-1];
	}
      }
    }
    else if (newPos > oldPos) {
      for (i=0; i<size; i++) {
	if ((i < oldPos) || (i > newPos)) {
	  newMatrix[i] = matrix[i];
	}
	else if (i == newPos) {
	  newMatrix[i] = matrix[oldPos];
	}
	else {
	  newMatrix[i] = matrix[i+1];
	}
      }
    }
    else {
      for (i=0; i<size; i++) {
	newMatrix[i] = matrix[i];
      }
    }
    return newMatrix;
  }

  //
  // private method moves the label from the pivoted column into place
  //
  private String[] movePivotColumnLabel(String []label, int size,
					  int newPos, int oldPos)
  {
    int i;
    String[] newlabel = new String[size];
    
    if (newPos < oldPos) {
      for (i=0; i<size; i++) {
	if ((i < newPos) || (i > oldPos)) {
	  newlabel[i] = label[i];
	}
	else if (i == newPos) {
	  newlabel[i] = label[oldPos];
	}
	else {
	  newlabel[i] = label[i-1];
	}
      }
    } 
    else if (newPos > oldPos) {
      for (i=0; i<size; i++) {
	if ((i < oldPos) || (i > newPos)) {
	  newlabel[i] = label[i];
	}
	else if (i == newPos) {
	  newlabel[i] = label[oldPos];
	}
	else {
	  newlabel[i] = label[i+1];
	}
      }
    }
    else {
      for (i=0; i<size; i++) {
	newlabel[i] = label[i];
      }
    }
    return newlabel;
  }

  // 
  // private method returns the index of the first entry in the 
  // passed FiniteFieldVector that is nonzero, or -1 if the vector is all
  // zeroes
  //
  private int nonZeroVectorIndex(FiniteFieldVector vector)
  {
    int answer = -1;
    boolean found = false;
    int i;

    for (i=0; i<vector.getSize(); i++) {
      if (vector.get(i) != 0) {
	found = true;
	break;
      }
    }
    if (found) {
      answer = i;
    }
    return answer;
  }

  //
  // private method returns the index of the first FiniteFieldVector in the
  // passed array with a nonzero entry in the indicated row; if none
  // exists, it returns -1
  //
  private int nonZeroIndex(FiniteFieldVector[] matrix, int size, int row)
  {
    int i = -1;
    boolean found=false;
    for (i=0; i<size; i++) {
      if (matrix[i].get(row) != 0) {
	found = true;
	break;
      }
    }
    
    if (!found) {
      i = -1;
    }
    return i;
  }

  //
  // private method pivots the passed matrix on the indicated row
  // and column
  //
  private void pivot(FiniteFieldVector[] matrix, int size, int r, int c)
    throws IllegalArgumentException
  {
    int i, j, mult;
    int rows = matrix[0].getSize();

    mult = matrix[c].get(r);
    if (0 == mult) {
      throw new IllegalArgumentException("division by zero error");
    }
    mult = mField.inverse(mult);

    try {
      // make pivot entry 1
      for (i=0; i<size; i++) {
	matrix[i].set(r, mField.product(mult, matrix[i].get(r)));
      }

      // kill off remainder of column
      for (i=0; i<rows; i++) {
	if (i != r) {
	  j = matrix[c].get(i);
	  if (j != 0) {
	    combineRows(matrix, size, i, r, mField.negative(j));
	  }
	}
      }
    }
    catch (Exception e) {
      // this "can't happen"
      throw new InternalError("Error in deletion: " + e.toString());
    }
  }

  //
  // private method adds a multiple of one row of the passed matrix
  // to another
  //
  private void combineRows(FiniteFieldVector[] matrix, int size, int toRow, 
			   int fromRow, int mult)
  {
    int i, j, k;
    try {
      for (i=0; i<size; i++) {
	j = mField.product(matrix[i].get(fromRow), mult);
	k = matrix[i].get(toRow);
	matrix[i].set(toRow, mField.sum(j, k));
      }
    }
    catch (Exception e) {
      // this "can't happen"
      throw new InternalError("Error in deletion: " + e.toString());
    }
  }

  //
  // private method returns the index of the first row in the passed
  // matrix with all zero entries, or -1 if no such row exists
  //
  private int firstZeroRow(FiniteFieldVector[] matrix, int size)
  {
    int answer = -1;
    boolean found = false;
    boolean allZero;
    int i, j;
    
    if (size > 0) {
      int rows = matrix[0].getSize();
      for (i=0; i<rows; i++) {
	allZero = true;
	for (j=0; j<size; j++) {
	  if (matrix[j].get(i) != 0) {
	    allZero = false;
	    break;
	  }
	}
	if (allZero) {
	  found = true;
	  break;
	}
      }
      if (found) {
	answer = i;
      }
    }

    return answer;
  }

  //
  // private method returns a copy of the passed matrix with the
  // indicated row deleted
  //
  private FiniteFieldVector[] deleteRow(FiniteFieldVector[] matrix, 
					int size, int row)
  {
    FiniteFieldVector[] newMatrix = new FiniteFieldVector[size];
    int i, j, n;

    try {
      n = matrix[0].getSize()-1;
      for (i=0; i<size; i++) {
	newMatrix[i] = new FiniteFieldVector(n, mField);
	for (j=0; j<n; j++) {
	  if (j < row) {
	    newMatrix[i].set(j, matrix[i].get(j));
	  }
	  else {
	    newMatrix[i].set(j, matrix[i].get(j+1));
	  }
	}
      }
    }
    catch (Exception e) {
      // this "can't happen"
      throw new InternalError("Error in deletion: " + e.toString());
    }

    return newMatrix;
  }

  // 
  // private methods for setting up instance
  //
  private void setupLabels()
  {
    int i;
    mLabel = new String[mSize];
    for (i=0; i<mSize; i++) {
      mLabel[i] = String.valueOf(i+1);
    }
    
    if (!(mName instanceof String)) {
      mName = new String("FiniteFieldMatroid");
    }
  }

  private void initDefaultMatroid()
  {
    try {
      initDefaultMatroid(mDEFAULT_SIZE);
    }
    catch (IllegalArgumentException e) {
      // this "can't happen"
      throw new InternalError(mBAD_K_MESSAGE + ".FiniteFieldMatroid()");
    }
  }

  private void initDefaultMatroid(int theSize)
    throws IllegalArgumentException
  {
    int i;

    // 
    // set up the rank, size and array of GF2kVectors -- array is just 
    // standard basis vectors
    //
    if (theSize <= 0) {
      throw new IllegalArgumentException("Invalid matroid size: " + theSize);
    }

    mRank = theSize;
    mSize = theSize;
    mMatrix = new FiniteFieldVector[mSize];
    for (i=0; i<mSize; i++) {
      mMatrix[i] = new FiniteFieldVector(mSize, mField);
      mMatrix[i].set(i,1);
    }
    setupZeroVector();
  }

  private void checkMatrix(FiniteFieldVector[] A, int n)
    throws IllegalArgumentException, ArrayIndexOutOfBoundsException
  {
    int i, p, k, r, s;
    FiniteFieldReduction reducer;

    if (n <= 0) {
      throw new IllegalArgumentException("Empty vector matrix");
    }

    p = A[0].getField().getCharacteristic();
    k = A[0].getField().getExponent();
    s = A[0].getSize();
    for (i=1; i<n; i++) {
      if ((A[i].getField().getCharacteristic() != p) ||
	  (A[i].getField().getExponent() != k)) {
	      throw new IllegalArgumentException("Inconsistent vector fields");
      }
      if (A[i].getSize() != s) {
	      throw new IllegalArgumentException("Inconsistent vector sizes");
      }
    }
    
    //
    // check rank of matrix against length of column vectors
    //
    reducer = new FiniteFieldReduction(s, n, A[0].getField());
    r = reducer.getRank(A, n);
    if (r != s) {
      throw new IllegalArgumentException("Matrix rank does not match " + 
					 "number of rows");
    }
     
  }

  //
  // checks a subset to make sure that its size matches mSize
  //
  private void checkSubset(Subset s)
    throws IllegalArgumentException
  {
    if (s.getSize() != mSize) {
      throw new IllegalArgumentException();
    }
  }

  //
  // generates a submatrix of the matrix with the elements specified in
  // the passed Subset -- the size of the resulting array will be 
  // s.numElements()
  // NOTE: this assumes that s.getSize() == this.getSize()
  //
  private FiniteFieldVector[] getSubmatrix(Subset s)
    throws IllegalArgumentException
  {
    FiniteFieldVector[] m;
    int n = s.numElements();
    int i, j=0;

    if (0 == n) {
      return null;
    }

    m = new FiniteFieldVector[n];
    for (i=0; (i<mSize) && (j<n); i++) {
      if (s.get(i)) {
	m[j] = mMatrix[i];
	j++;
      }
    }

    return m;
  }

  //
  // sets up zero vector for comparison
  //
  private void setupZeroVector()
  {
    zero = new FiniteFieldVector(mRank, mField);
  }
}
