package Oid;
import java.util.*;

/**
 * An instance of FiniteFieldMatroidChainChecker accepts as input two
 * FiniteFieldMatroid instances and determines whether their "chains"
 * of circuits, cocircuits, independent sets and coindependent sets
 * are identical.  Here a chain refers to the sequence of numbers of
 * sets of different sizes; for instance, the number of 1-element
 * circuits, 2-element circuits and so on.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 20 Nov 2002
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class FiniteFieldMatroidChainChecker implements Combiner
{
  //
  // private data members
  //
  private FiniteFieldMatroid m1, m2;  // last matroids checked
  private FiniteFieldMatroid d1, d2;  // duals of last matroids checked
  private String mExpl;       // explanation of steps in last check
  private boolean mUseExpl;
  private DefaultCircuitAlgorithm mCCTAlg;
  private DefaultIndependentSetAlgorithm mIndAlg;
  private Vector mDegL;
  private Vector mDegR;
  private Vector mDDegL;
  private Vector mDDegR;
  private Vector mChainL;
  private Vector mChainR;
  private Vector mDChainL;
  private Vector mDChainR;

  /**
   * constructor initializes algorithms for checks
   */
  public FiniteFieldMatroidChainChecker()
  {
    mCCTAlg = new DefaultCircuitAlgorithm();
    mIndAlg = new DefaultIndependentSetAlgorithm();
    mUseExpl = true;
  }

  /**
   * sets value indicating whether checker should generate an explanation
   * each time it checks two matroids
   *
   * @param makeExplanation boolean value
   */
  public void setMakeExplanation(boolean makeExplanation)
  {
    mUseExpl = makeExplanation;
  }

  /** 
   * returns the explanation generated during the last check
   *
   * @return String with explanation (null if no check has been done)
   */
  public String getExplanation()
  {
    String answer = (String)null;
    if (mExpl instanceof String) {
      answer = new String(mExpl);
    }
    return answer;
  }

  /**
   * returns the first matroid used for the last check
   *
   * @return FiniteFieldMatroid reference (null if no check has been done)
   */
  public FiniteFieldMatroid getFiniteFieldMatroid1()
  {
    FiniteFieldMatroid answer = (FiniteFieldMatroid)null;
    if (m1 instanceof FiniteFieldMatroid) {
      answer = m1;
    }
    return answer;
  }

  /** 
   * returns a vector of degree sequences for the first matroid
   *
   * @return Vector with circuit, ind set, cocircuit and coind set seqs
   */
  public Vector getDegrees1()
  {
    Vector answer = (Vector)null;
    if (mDegL instanceof Vector) {
      answer = mDegL;
    }
    return answer;
  }

  /**
   * returns a vector of chains for the first matroid
   *
   * @return Vector with circuit, ind set, cocircuit and coind chains
   */
  public Vector getChains1()
  {
    Vector answer = (Vector)null;
    if (mChainL instanceof Vector) {
      answer = mChainL;
    }
    return answer;
  }

  /** 
   * returns a vector of degree sequences for the first matroid
   *
   * @return Vector with circuit, ind set, cocircuit and coind set seqs
   */
  public Vector getDualDegrees1()
  {
    Vector answer = (Vector)null;
    if (mDDegL instanceof Vector) {
      answer = mDDegL;
    }
    return answer;
  }

  /**
   * returns a vector of chains for the first matroid
   *
   * @return Vector with circuit, ind set, cocircuit and coind chains
   */
  public Vector getDualChains1()
  {
    Vector answer = (Vector)null;
    if (mDChainL instanceof Vector) {
      answer = mDChainL;
    }
    return answer;
  }

  /**
   * returns the second matroid used for the last check
   *
   * @return FiniteFieldMatroid reference (null if no check has been done)
   */
  public FiniteFieldMatroid getFiniteFieldMatroid2()
  {
    FiniteFieldMatroid answer = (FiniteFieldMatroid)null;
    if (m2 instanceof FiniteFieldMatroid) {
      answer = m2;
    }
    return answer;
  }

  /** 
   * returns a vector of degree sequences for the second matroid
   *
   * @return Vector with circuit, ind set, cocircuit and coind set deg seqs
   */
  public Vector getDegrees2()
  {
    Vector answer = (Vector)null;
    if (mDegR instanceof Vector) {
      answer = mDegR;
    }
    return answer;
  }

  /**
   * returns a vector of chains for the second matroid
   *
   * @return Vector with circuit, ind set, cocircuit and coind set chains
   */
  public Vector getChains2()
  {
    Vector answer = (Vector)null;
    if (mChainR instanceof Vector) {
      answer = mChainR;
    }
    return answer;
  }

  /** 
   * returns a vector of degree sequences for the second matroid
   *
   * @return Vector with circuit, ind set, cocircuit and coind set deg seqs
   */
  public Vector getDualDegrees2()
  {
    Vector answer = (Vector)null;
    if (mDDegR instanceof Vector) {
      answer = mDDegR;
    }
    return answer;
  }

  /**
   * returns a vector of chains for the second matroid
   *
   * @return Vector with circuit, ind set, cocircuit and coind set chains
   */
  public Vector getDualChains2()
  {
    Vector answer = (Vector)null;
    if (mDChainR instanceof Vector) {
      answer = mDChainR;
    }
    return answer;
  }

  /**
   * sets reference to first matroid (without accompanying chains, degrees
   * and bases)
   *
   * @param m FiniteFieldMatroid reference
   */
  public void setFiniteFieldMatroid1(FiniteFieldMatroid m)
  {
    m1 = m;
    d1 = m.getDual();

    int n = m1.getSize();

    mChainL = new Vector();
    mDegL = new Vector();
    mDChainL = new Vector();
    mDDegL = new Vector();
      
    mIndAlg.setInstance(m1);
    mChainL.add(mIndAlg.getSubsetCountChain());
    mDegL.add(getTuple((Vector)mIndAlg.getSubsetList(), n));
    
    mIndAlg.setInstance(d1);
    mDChainL.add(mIndAlg.getSubsetCountChain());
    mDDegL.add(getTuple((Vector)mIndAlg.getSubsetList(), n));

    mCCTAlg.setInstance(m1);
    mChainL.add(mCCTAlg.getSubsetCountChain());
    mDegL.add(getTuple((Vector)mCCTAlg.getSubsetList(), n));

    mCCTAlg.setInstance(d1);
    mDChainL.add(mCCTAlg.getSubsetCountChain());
    mDDegL.add(getTuple((Vector)mCCTAlg.getSubsetList(), n));
  }

  /**
   * sets references to first matroid and Vectors of chains, degree sequences
   * and bases
   *
   * @param m FiniteFieldMatroid reference
   * @param chains Vector with chains of circuits, ind sets, spanning sets and
   * bases
   * @param degrees Vector with degree sequences for cct, ind, span, bases
   * @param bases Vector with list of bases
   */
  public void setFiniteFieldMatroid1(FiniteFieldMatroid m,
				     FiniteFieldMatroid dual,
				     Vector chains, 
				     Vector degrees,
				     Vector dualChains,
				     Vector dualDegrees)
  {
    m1 = m;
    d1 = dual;
    mDegL = degrees;
    mChainL = chains;
    mDDegL = dualDegrees;
    mDChainL = dualChains;
  }

  /**
   * sets reference to second matroid (without accompanying chains, degrees
   * and bases)
   *
   * @param m FiniteFieldMatroid reference
   */
  public void setFiniteFieldMatroid2(FiniteFieldMatroid m)
  {
    m2 = m;
    d2 = m.getDual();

    int n = m2.getSize();

    mChainR = new Vector();
    mDegR = new Vector();
    mDChainR = new Vector();
    mDDegR = new Vector();
      
    mIndAlg.setInstance(m2);
    mChainR.add(mIndAlg.getSubsetCountChain());
    mDegR.add(getTuple((Vector)mIndAlg.getSubsetList(), n));
      
    mCCTAlg.setInstance(m2);
    mChainR.add(mCCTAlg.getSubsetCountChain());
    mDegR.add(getTuple((Vector)mCCTAlg.getSubsetList(), n));
      
    mIndAlg.setInstance(d2);
    mDChainR.add(mIndAlg.getSubsetCountChain());
    mDDegR.add(getTuple((Vector)mIndAlg.getSubsetList(), n));
      
    mCCTAlg.setInstance(d2);
    mDChainR.add(mCCTAlg.getSubsetCountChain());
    mDDegR.add(getTuple((Vector)mCCTAlg.getSubsetList(), n));
  }

  /**
   * sets references to second matroid and Vectors of chains, degree sequences
   * and bases
   *
   * @param m FiniteFieldMatroid reference
   * @param chains Vector with chains of circuits, ind sets, spanning sets and
   * bases
   * @param degrees Vector with degree sequences for cct, ind, span, bases
   * @param bases Vector with list of bases
   */
  public void setFiniteFieldMatroid2(FiniteFieldMatroid m,
				     FiniteFieldMatroid dual,
				     Vector chains, 
				     Vector degrees, 
				     Vector dualChains,
				     Vector dualDegrees)
  {
    m2 = m;
    d2 = dual;
    mDegR = degrees;
    mChainR = chains;
    mDDegR = dualDegrees;
    mDChainR = dualChains;
  }


  /**
   * Performs check on matroids, and returns a boolean value indicating
   * whether any isomorphisms were found
   *
   * @param m1 first matroid to check
   * @param m2 second matroid to check
   * @return boolean value indicating result of check
   * @throws NullPointerException if matroids aren't set
   */
  public boolean checkFiniteFieldMatroids()
  throws NullPointerException
  {
    if (!(m1 instanceof FiniteFieldMatroid)) {
      throw new NullPointerException("No left-hand matroid set");
    }

    if (!(m2 instanceof FiniteFieldMatroid)) {
      throw new NullPointerException("No right-hand matroid set");
    }

    boolean answer = false;
    int i,j;
    int n=0, r=0, p=0;
    int index;
    StringBuffer expl=(StringBuffer)null;

    if (mUseExpl) {
      expl = new StringBuffer();
    }

    // 
    // check sizes, ranks and moduli of base fields
    //
    n = m1.getSize();
    r = m1.getRank();
    if ((m2.getSize() != n) || (m2.getRank() != r)) {
      if (mUseExpl) {
	expl.append("matroids have inconsistent sizes and/or, ranks");
	mExpl = expl.toString();
      }
      return answer;
    }

    Vector degrees1 = new Vector(); // holds degree sequences for m1
    Vector degrees2 = new Vector(); // holds degree sequences for m2
    Vector dDegrees1 = new Vector(); // dual of m1 degrees
    Vector dDegrees2 = new Vector(); // dual of m2 degrees
    Vector chain1, chain2, dChain1, dChain2;
    

    // check independent set chains
    index = 0;
    chain1 = (Vector)mChainL.get(index);
    degrees1.add((int [])mDegL.get(index));

    chain2 = (Vector)mChainR.get(index);
    degrees2.add((int [])mDegR.get(index));

    if (mUseExpl) {
      expl.append("independent set chains:\n");
      expl.append(m1.getName());
      expl.append(": ");
      expl.append(getChainDescription(chain1, 0));
      expl.append("\n");
      expl.append(m2.getName());
      expl.append(": ");
      expl.append(getChainDescription(chain2, 0));
      expl.append("\n");
    }
    if (compareChains(chain1, chain2)) {
      if (mUseExpl) {
	expl.append("independent set chains match\n\n");
      }
    }
    else {
      if (mUseExpl) {
	expl.append("independent set chains do not match\n");
	mExpl = expl.toString();
      }
      return answer;
    }

    // check independent set degrees
    if (mUseExpl) {
      expl.append("independent set degree sequences:\n");
      expl.append(m1.getName());
      expl.append(": ");
      expl.append(getTupleDescription((int[])degrees1.get(index),0));
      expl.append("\n");
      expl.append(m2.getName());
      expl.append(": ");
      expl.append(getTupleDescription((int[])degrees2.get(index),0));
      expl.append("\n");
    }

    if (degreesMatch(degrees1, degrees2,n)) {
      if (mUseExpl) {
	expl.append("independent set degree sequences match\n\n");
      }
    }
    else {
      if (mUseExpl) {
	expl.append("independent set degree sequences do not match\n");
	mExpl = expl.toString();
      }
      return answer;
    }
      
    // next check coindependent set chains
    dChain1 = (Vector)mDChainL.get(index);
    dDegrees1.add((int [])mDDegL.get(index));

    dChain2 = (Vector)mDChainR.get(index);
    dDegrees2.add((int [])mDDegR.get(index));

    if (mUseExpl) {
      expl.append("coindependent set chains:\n");
      expl.append(m1.getName());
      expl.append(": ");
      expl.append(getChainDescription(dChain1, index));
      expl.append("\n");
      expl.append(m2.getName());
      expl.append(": ");
      expl.append(getChainDescription(dChain2, index));
      expl.append("\n");
    }
    if (compareChains(dChain1, dChain2)) {
      if (mUseExpl) {
	expl.append("coindependent set chains match\n\n");
      }
    }
    else {
      if (mUseExpl) {
	expl.append("coindependent set chains do not match\n");
	mExpl = expl.toString();
      }
      return answer;
    }

    // check coindependent set degrees
    if (mUseExpl) {
      expl.append("coindependent set degree sequences:\n");
      expl.append(m1.getName());
      expl.append(": ");
      expl.append(getTupleDescription((int[])dDegrees1.get(index),0));
      expl.append("\n");
      expl.append(m2.getName());
      expl.append(": ");
      expl.append(getTupleDescription((int[])dDegrees2.get(index),0));
      expl.append("\n");
    }

    if (degreesMatch(dDegrees1, dDegrees2,n)) {
      if (mUseExpl) {
	expl.append("coindependent set degree sequences match\n\n");
      }
    }
    else {
      if (mUseExpl) {
	expl.append("coindependent set degree sequences do not match\n");
	mExpl = expl.toString();
      }
      return answer;
    }
      
    // now check circuits
    ++index;
    chain1 = (Vector)mChainL.get(index);
    degrees1.add((int [])mDegL.get(index));

    chain2 = (Vector)mChainR.get(index);
    degrees2.add((int [])mDegR.get(index));

    if (mUseExpl) {
      expl.append("circuit chains:\n");
      expl.append(m1.getName());
      expl.append(": ");
      expl.append(getChainDescription(chain1, 0));
      expl.append("\n");
      expl.append(m2.getName());
      expl.append(": ");
      expl.append(getChainDescription(chain2, 0));
      expl.append("\n");
    }


    if (compareChains(chain1, chain2)) {
      if (mUseExpl) {
	expl.append("circuit chains match\n\n");
      }
    }
    else {
      if (mUseExpl) {
	expl.append("circuit chains do not match\n");
	mExpl = expl.toString();
      }
      return answer;
    }

    // check degree sequences
    if (mUseExpl) {
      expl.append("circuit degree sequences:\n");
      expl.append(m1.getName());
      expl.append(": ");
      expl.append(getTupleDescription((int[])degrees1.get(index),0));
      expl.append("\n");
      expl.append(m2.getName());
      expl.append(": ");
      expl.append(getTupleDescription((int[])degrees2.get(index),0));
      expl.append("\n");
    }

    if (degreesMatch(degrees1, degrees2, n)) {
      if (mUseExpl) {
	expl.append("circuit degree sequences match\n\n");
      }
    }
    else {
      if (mUseExpl) {
	expl.append("circuit degree sequences do not match\n");
	mExpl = expl.toString();
      }
      return answer;
    }

    // now check cocircuits
    dChain1 = (Vector)mDChainL.get(index);
    dDegrees1.add((int [])mDDegL.get(index));

    dChain2 = (Vector)mDChainR.get(index);
    dDegrees2.add((int [])mDDegR.get(index));

    if (mUseExpl) {
      expl.append("cocircuit chains:\n");
      expl.append(m1.getName());
      expl.append(": ");
      expl.append(getChainDescription(dChain1, 0));
      expl.append("\n");
      expl.append(m2.getName());
      expl.append(": ");
      expl.append(getChainDescription(dChain2, 0));
      expl.append("\n");
    }


    if (compareChains(dChain1, dChain2)) {
      if (mUseExpl) {
	expl.append("cocircuit chains match\n\n");
      }
    }
    else {
      if (mUseExpl) {
	expl.append("cocircuit chains do not match\n");
	mExpl = expl.toString();
      }
      return answer;
    }

    // check degree sequences
    if (mUseExpl) {
      expl.append("cocircuit degree sequences:\n");
      expl.append(m1.getName());
      expl.append(": ");
      expl.append(getTupleDescription((int[])dDegrees1.get(index),0));
      expl.append("\n");
      expl.append(m2.getName());
      expl.append(": ");
      expl.append(getTupleDescription((int[])dDegrees2.get(index),0));
      expl.append("\n");
    }

    if (degreesMatch(dDegrees1, dDegrees2, n)) {
      if (mUseExpl) {
	expl.append("cocircuit degree sequences match\n\n");
      }
    }
    else {
      if (mUseExpl) {
	expl.append("cocircuit degree sequences do not match\n");
	mExpl = expl.toString();
      }
      return answer;
    }

    answer = true;
    return answer;
  }
  
  //
  // private method compares two chains and returns true if they have
  // the same numbers
  //
  private boolean compareChains(Vector v1, Vector v2)
  {
    boolean answer = true;
    int i,j;
    Iterator p1, p2;
    for (p1=v1.iterator(), p2=v2.iterator(); p1.hasNext() && p2.hasNext();) {
      i = ((Integer)p1.next()).intValue();
      j = ((Integer)p2.next()).intValue();
      if (i != j) {
	answer = false;
	break;
      }
    }

    if (p1.hasNext() || p2.hasNext()) {
      answer = false;
    }

    return answer;
  }

  // 
  // private method generates a descriptive string for a tuple,
  // starting at the passed index
  //
  private String getTupleDescription(int[] tuple, int start)
  {
    int i, n=tuple.length;
    StringBuffer ansBuf = new StringBuffer("(");
    boolean flag = false;

    for (i=start; i<n; i++) {
      if (!flag) {
	flag = true;
      }
      else {
	ansBuf.append(", ");
      }
      ansBuf.append(String.valueOf(tuple[i]));
    }
    ansBuf.append(")");

    return ansBuf.toString(); 
  }

  //
  // private method generates a descriptive string for a chain,
  // starting at the passed index and preceded by a total
  //
  private String getChainDescription(Vector chain, int start)
  {
    int i, n=chain.size(), total = 0;
    boolean flag = false;
    StringBuffer ansBuf;
    
    // get total
    for (i=0; i<n; i++) {
      total += ((Integer)(chain.get(i))).intValue();
    }
    
    ansBuf = new StringBuffer(String.valueOf(total));
    ansBuf.append(" = ");
    for (i=start; i<n; i++) {
      if (!flag) {
	flag = true;
      }
      else {
	ansBuf.append(" + ");
      }
      ansBuf.append(String.valueOf(((Integer)(chain.get(i))).intValue()));
    }
    return ansBuf.toString();
  }
  
  //
  // private method returns a matrix with the number of occurrences
  // of each value in the passed vector of sets, up to the specified
  // maximum
  //
  private int[] getTuple(Vector sets, int n)
  {
    int[] tuple = new int[n];
    int i, j;
    Subset s;

    for (i=0; i<n; i++) {
      tuple[i] = 0;
    }

    // 
    // loop through all sets
    //
    for (Iterator p = sets.iterator(); p.hasNext(); ) {
      s = (Subset)(p.next());
      try {
	for (i=0; i<n; i++) {
	  tuple[i] += (s.get(i)) ? 1 : 0;
	}
      }
      catch (Exception e) {
	// this "can't happen"
	throw new InternalError();
      }
    }
    return tuple;
  }

  //
  // private method returns a vector with the number of elements in 
  // no sets, the number of elements in one set, etc. 
  //
  private Vector getPointChain(int[] tuple)
  {
    int i, j, n = tuple.length;
    Vector answer = new Vector();

    // we need a vector with element counts at the various indices
    for (i=0; i<n; i++) {
      j = tuple[i];
      if (answer.size() <= j) {
	answer.setSize(j+1);
      }
      
      // set the correct value to 1 or increment it
      if (!(answer.get(j) instanceof Integer)) {
	answer.set(j, new Integer(1));
      }
      else {
	answer.set(j, new Integer(((Integer)answer.get(j)).intValue()+1));
      }
    }

    // fill in the remaining values with zeroes
    for (i=0; i<answer.size(); i++) {
      if (!(answer.get(i) instanceof Integer)) {
	answer.set(i, new Integer(0));
      }
    }
    return answer;
  }

  //
  // private function determines whether vectors of degree sequences
  // have a match
  //
  private boolean degreesMatch(Vector degrees1, Vector degrees2, int n)
  {
    boolean answer = true;
    int i, j;
    Iterator p1, p2;
    boolean[] matched = new boolean[n];
    boolean found;
    boolean flag;
    int[] deg1;
    int[] deg2;

    for (i=0; i<n; i++) {
      matched[i] = false;
    }

    // go through first array and find matches in second
    for (i=0; i<n; i++) {
      found = false;
      for (j=0; j<n; j++) {
	if (!matched[j]) {
	  flag = true;
	  for (p1 = degrees1.iterator(), p2 = degrees2.iterator(); 
	       p1.hasNext();) {
	    deg1 = (int[])p1.next();
	    deg2 = (int[])p2.next();
	    if (deg1[i] != deg2[j]) {
	      flag = false;
	      break;
	    }
	  }
	  if (flag) {
	    matched[j] = true;
	    found = true;
	    break;
	  }
	}
      }
      if (!found) {
	answer = false;
	break;
      }
    }

    return answer;
  }

  // Combiner interface implementation

  /** 
   * sets the reference collection for this combiner 
   *
   * @param ref reference collection
   * @throws IllegalArgumentException if <code>ref</code> isn't two matroids
   */
  public void setCollection(Collection ref) 
    throws IllegalArgumentException
  {
    if (ref.size() == 0) {
      throw new IllegalArgumentException("No matroids passed");
    }
    
    Object[] matroids = ref.toArray();
    
    if (matroids.length == 1) {
      if (!(matroids[0] instanceof FiniteFieldMatroid)) {
	throw new IllegalArgumentException("Not a matroid");
      }
      m1 = (FiniteFieldMatroid)matroids[0];
      m2 = (FiniteFieldMatroid)matroids[0];
    }
    else {
      if (!((matroids[0] instanceof FiniteFieldMatroid) && 
	    (matroids[1] instanceof FiniteFieldMatroid)) ) {
	throw new IllegalArgumentException("Not a matroid");
      }
      m1 = (FiniteFieldMatroid)matroids[0];
      m2 = (FiniteFieldMatroid)matroids[1];
    }
    mUseExpl = true;
//      mStopWithOrbits = true; // don't check all n! if using as Combiner
    checkFiniteFieldMatroids();
  }

  /** 
   * returns the reference collection for this constructor 
   *
   * @return reference collection
   */
  public Object getCollection()
  {
    Vector answer = new Vector();
    answer.add(m1);
    answer.add(m2);
    return answer;
  }

  /** 
   * returns the constructed object from the reference collection 
   *
   * @return constructed object
   */
  public Object getConstruction()
  {
    return mExpl;
  }

  /** 
   * returns a description of the algorithm used for the construction 
   *
   * @return algorithm description
   */
  public String getAlgorithmDescription()
  {
    StringBuffer answer = new StringBuffer();
    answer.append("An instance of FiniteFieldMatroidChainChecker ");
    answer.append("accepts as input two ");
    answer.append("FiniteFieldMatroid instances and determines whether ");
    answer.append("they are isomorphic.  It first");
    answer.append("performs some common-sense checks, making sure ");
    answer.append("that both matroids have");
    answer.append("the same size and rank.");
    answer.append("If the matroids pass these checks, ");
    answer.append("then the algorithm finds their ");
    answer.append("independent set degree sequences and compares them.  ");
    answer.append("If there is a match,");
    answer.append("it adds in circuit degrees, then spanning set degrees, ");
    answer.append("then cocircuit ");
    answer.append("degrees.  If there is still a match, the algorithm ");
    answer.append("seeks a mapping based");
    answer.append("on the degree sequences that preserves the circuit ");
    answer.append("structure of the");
    answer.append("matroids.");
    return answer.toString();
  }

  /** 
   * returns the algorithm's name 
   *
   * @return algorithm name
   */
  public String getName()
  {
    return new String("FiniteFieldMatroid Isomorphism Checker");
  }
}
