package Oid;
import java.util.*;

/**
 * <code>SingleConstructor</code> implementation which constructs the
 * dual of a FiniteFieldMatroid as long as it is in standard form
 * (actually, it relies on the <code>FiniteFieldMatroid</code> class's
 * own <code>getDual</code> method).
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 13 Oct 2001
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class FiniteFieldMatroidDualAlgorithm implements SingleConstructor, 
						ComplexityProvider
{
  FiniteFieldMatroid mMatroid;
  FiniteFieldMatroid mDual;

  /** sets the reference object for this constructor */
  public void setInstance(Object ref) 
    throws IllegalArgumentException
  {
    if (!(ref instanceof FiniteFieldMatroid)) {
      throw new IllegalArgumentException("Valid only for FiniteFieldMatroids");
    }
    mMatroid = (FiniteFieldMatroid)ref;
    try {
      mDual = (FiniteFieldMatroid)mMatroid.getDual();
    }
    catch (Exception e) {
      mMatroid = null;
      mDual = null;
      throw new IllegalArgumentException("Cannot calculate matroid's dual");
    }
  }
  
  /** returns the reference object for this constructor */
  public Object getInstance()
  {
    return mMatroid;
  }

  /** indicates that this algorithm does not require a parameter */
  public boolean requiresParameter() 
  {
    return false;
  }

  /** does nothing, since this algorithm does not require a parameter */
  public void setParameters(String parameters)
  throws IllegalArgumentException
  {
  }

  /** returns empty string since this algorithm does not require a parameter */
  public String getParameters()
  {
    return new String("");
  }

  /** returns a blank string */
  public String getPrompt()
  {
    return new String("");
  }

  /** returns the constructed object from the reference object */
  public Object getConstruction()
  {
    return mDual;
  }

  /** returns the constructed object from the reference object & parameter */
  public Object getConstruction(int parameter)
  {
    return mDual;
  }

  /** returns a description of the algorithm used for the construction */
  public String getAlgorithmDescription()
  {
    return new String("Uses the FiniteFieldMatroid getDual method to determine " +
		      "the matroid's dual");
  }

  /** returns this algorithm's name */
  public String getName()
  {
    return new String("dual calculator");
  }

  //
  // ComplexityProvider interface methods
  //
  /** 
   * returns a value estimating the number of steps required to 
   * execute the method with the passed name, or -1 if the method cannot
   * currently be executed
   */
  public long getComplexityEstimate(String methodName)
  {
    long answer=-1;
 
    if (methodName.equals("getConstruction") && 
	(mMatroid instanceof FiniteFieldMatroid)) {
      answer = (long)mMatroid.getSize();
    }
    
    return answer;
  }
  
  /**
   * returns a value estimating the number of steps required to
   * execute the method with the passed name and argument, or -1 if the 
   * method cannot be executed with the passed argument
   */
  public long getComplexityEstimate(String methodName, Object argument)
  {
    long answer = -1;
    if (methodName.equals("getConstruction") &&
	(argument instanceof FiniteFieldMatroid)) {
      answer = ((FiniteFieldMatroid)argument).getSize();
    }
    return answer;
  }

  /**
   * returns a value estimating the number of steps required to 
   * execute the method with the passed name and collection of arguments
   * (which must be placed in correct order for iteration), or -1 if the
   * method cannot be executed with the passed arguments
   */
  public long getComplexityEstimate(String methodName, Collection arguments)
  {
    long answer = -1;
    Iterator pArg = arguments.iterator();
    if (pArg.hasNext()) {
      answer = getComplexityEstimate(methodName, pArg.next());
    }
    return answer;
  }
}
