package Oid;
/**
 * This class contains methods for formatting a <code>FiniteFieldMatroid</code>
 * for text display.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 3 Sep 2002
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class FiniteFieldMatroidFormatter
{
  /**
   * Returns a string containing a formatted descrption of the
   * passed matroid's base matrix.  If the matroid's base field is 
   * GF(q) where q is a prime power greater than one (e.g. 4, 8, 9, etc.)
   * then the field elements are represented as powers of the primitive
   * element; i.e. 0 for 0, 1 for 1, (1) for the primitive element, 
   * (2) for the square of the primitive element, etc.
   *
   * @param matroid matroid to represent
   * @return string with <code>matroid</code> base matrix
   */
  public static String displayMatrix(FiniteFieldMatroid matroid)
  {
    FiniteField field = matroid.getField();
    int colWidth = fieldColSize(field)+1;
    return matrixDisplay(matroid, colWidth);
  }

  /**
   * Returns a string similar to that returned by <code>displayMatrix</code>,
   * but also including the element labels for <code>matroid</code>
   *
   * @param matroid matroid to represent
   * @return string with <code>matroid</code> base matrix and element labels
   */
  public static String displayWithLabels(FiniteFieldMatroid matroid)
  {
    FiniteField field = matroid.getField();
    int colWidth = fieldColSize(field) + 1;
    int i, j, n = matroid.getSize();
    String lf = "\n";

    for (i=0; i<n; i++) {
      j = matroid.getLabel(i).length();
      if (j + 1 > colWidth) {
	colWidth = j + 1;
      }
    }

    StringBuffer sb = new StringBuffer();
    for (i=0; i<n; i++) {
      sb.append(TextAlignment.rightAlign(matroid.getLabel(i), colWidth));
      sb.append(" ");
    }
    sb.append(lf);
    sb.append(matrixDisplay(matroid, colWidth));
    return sb.toString();
  }

  /**
   * Returns a string similar to that returned by <code>displayMatrix</code>,
   * but also including descriptive information about the matroid, including
   * its name, base field, size and rank, and the labels for its elements.
   *
   * @param matroid matroid to represent
   * @return string with <code>matroid</code> base matrix, element labels
   * and description
   */
  public static String displayWithInfo(FiniteFieldMatroid matroid)
  {
    StringBuffer sb = new StringBuffer();
    String lf = "\n";

    sb.append("Name: ");
    sb.append(matroid.getName());
    sb.append(lf);

    sb.append("Field: GF(");
    sb.append(matroid.getField().getFieldSize());
    sb.append(")");
    sb.append(lf);

    sb.append("Size: ");
    sb.append(matroid.getSize());
    sb.append(lf);
    
    sb.append("Rank: ");
    sb.append(matroid.getRank());
    sb.append(lf);

    sb.append(FiniteFieldMatroidFormatter.displayWithLabels(matroid));
   
    return sb.toString();
  }

  //
  // returns matrix display with given column width
  //
  private static String matrixDisplay(FiniteFieldMatroid matroid, int colWidth)
  {
    FiniteField field = matroid.getField();
    int r, c;
    int m = matroid.getRank(), n = matroid.getSize();
    StringBuffer sb = new StringBuffer();
    String s;
    String lf = "\n";

    if (field instanceof GFpCalculator) {
      for (r=0; r<m; r++) {
	for (c=0; c<n; c++) {
	  s = String.valueOf(matroid.getElement(c).get(r));
	  sb.append(TextAlignment.rightAlign(s, colWidth));
	  sb.append(" ");
	}
	sb.append(lf);
      }
    }
    else if (field instanceof GF2kCalculator) {
      GF2kCalculator GF2k = (GF2kCalculator)field;
      for (r=0; r<m; r++) {
	for (c=0; c<n; c++) {
	  s = GF2k.eltToPowerString(matroid.getElement(c).get(r));
	  sb.append(TextAlignment.rightAlign(s, colWidth));
	  sb.append(" ");
	}
	sb.append(lf);
      }
    }
    else {
      GFpkCalculator GFpk = (GFpkCalculator)field;
      for (r=0; r<m; r++) {
	for (c=0; c<n; c++) {
	  s = GFpk.eltToPowerString(matroid.getElement(c).get(r));
	  sb.append(TextAlignment.rightAlign(s, colWidth));
	  sb.append(" ");
	}
	sb.append(lf);
      }
    }
    return sb.toString();
  }

  //
  // returns the column width to use given a finite field
  //
  private static int fieldColSize(FiniteField GF)
  {
    int ans = 0;

    if (GF instanceof GFpCalculator) {
      ans = String.valueOf(GF.getFieldSize()-1).length();
    }
    else {
      //
      // (field size) - 2 is the largest exponent that could show up;
      // it will appear with leading and trailing parentheses
      //
      ans = 2 + String.valueOf(GF.getFieldSize()-2).length();
    }
    return ans;
  }

  /**
   * allows this class to be tested from the command-line
   *
   * @param arg array of strings with command-line arguments; ignored
   */
  public static void main(String[] arg)
  {
    FiniteFieldMatroid m = new FiniteFieldMatroid();

    System.out.println(FiniteFieldMatroidFormatter.displayWithInfo(m));
  }
}
