package Oid;
import java.util.*;

/**
 * This class provides Gauss-Jordan reduction for sets of columns in the
 * form of FiniteFieldVectors.
 * <p><b>Design notes:</b> In order to implement many of the Matroid interface
 * methods for matroids represented as sets of columns over GF(p^k), we need
 * to be able to do two things:</p>
 * <ol><li>Determine the rank of a set of columns</li>
 * <li>Determine whether a column lies within the span of a set of 
 * other columns</li></ol>
 * <p>The FiniteFieldReduction class is designed to perform these two tasks.
 * 
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 25 Aug 2002
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class FiniteFieldReduction
{
  //
  // private data members
  // 
  private FiniteFieldVector[] mMatrix;
  private int mRows;
  private int mMaxCols;
  private int mCols;
  private FiniteFieldVector mObjectVector;
  private int mModulus;
  private FiniteField mField;

  private FiniteFieldReduction() { } // force constructor with parameters

  /**
   * creates a reduction instance for matrices with the specified number
   * of rows, up to the specified number of columns over the specified
   * finite field
   *
   * @param rows number of rows in matrices to be reduced
   * @param maxCols maximum number of columns for matrices to be reduced
   * @param field finite field over which matrices are represented
   * @throws IllegalArgumentException for silly values
   */
  public FiniteFieldReduction(int rows, int maxCols, FiniteField field)
    throws IllegalArgumentException
  {
    if (rows <= 0) {
      throw new IllegalArgumentException("number of rows must be positive");
    }

    if (maxCols <= 0) {
      throw new IllegalArgumentException("max columns must be positive");
    }

    mRows = rows;
    mMaxCols = maxCols;
    mField = field;

    mMatrix = new FiniteFieldVector[mMaxCols];
    for (int i=0; i<mMaxCols; i++) {
      mMatrix[i] = new FiniteFieldVector(mRows, mField);
    }
  }

  /**
   * hasParallel checks the passed array of vectors to see if any one
   * is a scalar multiple of another
   *
   * @param theMatrix one-dimensional array of FiniteFieldVectors
   * @param start starting index in array to check (inclusive)
   * @param end end position to check (non-inclusive)
   * @return boolean value indicating whether parallel elements were found
   * @throws NullPointerException if there are null pointers in the array
   * @throws IllegalArgumentException if there are inconsistent entries
   */
  public static boolean hasParallel(FiniteFieldVector[] theMatrix,
				    int start,
				    int end)
    throws NullPointerException, IllegalArgumentException
  {
//     checkMatrix(theMatrix, end);
    
    int col1, col2, c1, c2;
    int nz1, nz2;
    int i,n;
    boolean result = false;
    boolean areEqual;
    FiniteFieldVector v1, v2;
    FiniteField fld;

    fld = theMatrix[start].getField();

    n = theMatrix[start].getSize();

    for (col1 = start; col1 < end; col1++) {
      v1 = theMatrix[col1];
      nz1 = -1;
      for (i = 0; i<n; i++) {
	if (v1.get(i) != 0) {
	  nz1 = i;
	  break;
	}
      }

      if (nz1 < 0) {
	result = true;
	break;
      }

      for (col2 = col1+1; col2<end; col2++) {
	v2 = theMatrix[col2];
	nz2 = -1;
	for (i=0; i<n; i++) {
	  if (v2.get(i) != 0) {
	    nz2 = i;
	    break;
	  }
	}
	if (nz2 < 0) {
	  result = true;
	  break;
	}

	if (!result && (nz1 == nz2)) {
	  c1 = fld.inverse(v1.get(nz1));
	  c2 = fld.inverse(v2.get(nz2));
	  
	  areEqual = true;
	  for (i=nz1; i<n; i++) {
	    if (fld.product(v1.get(i), c1) != fld.product(v2.get(i), c2)) {
	      areEqual = false;
	      break;
	    }
	  }
	  
	  if (areEqual) {
	    result = true;
	  }
	}
	
	if (result) {
	  break;
	}
      }
      
      if (result) { 
	break;
      }
    }

    return result;
  }

  /**
   * hasSeries takes an array of FiniteFieldVector instances, transposes
   * the resulting matrix, and looks for parallel columns
   *
   * @param theMatrix array of FiniteFieldVector references
   * @param start start position to check (inclusive)
   * @param end end position to check (non-inclusive)
   * @return boolean value indicating whether series columns were found
   * @throws NullPointerException if there are null pointers in the array
   * @throws IllegalArgumentException if there are inconsistent entries
   */
  public static boolean hasSeries(FiniteFieldVector[] theMatrix,
				  int start,
				  int end)
    throws NullPointerException, IllegalArgumentException
  {
    if (end < start) {
      throw new IllegalArgumentException("start must be before end");
    }

//     checkMatrix(theMatrix, end);

    FiniteFieldReduction ffr = new FiniteFieldReduction();
    FiniteFieldVector myMatrix[] = new FiniteFieldVector[end-start];
    FiniteFieldVector Mt[];
    int i, n = end - start;
    

    for (i=start; i<end; i++) {
      myMatrix[i] = theMatrix[i+start];
    }

    Mt = ffr.transpose(myMatrix, n);

    return FiniteFieldReduction.hasParallel(Mt, 0, Mt.length);
  }


  /**
   * getRank calculates the rank of the passed array of
   * FiniteFieldVectors.  It makes a private copy of theMatrix for its
   * calculations.
   *
   * @param theMatrix one-dimensional array of FiniteFieldVectors with matrix
   * @param nCols number of columns of the matrix to consider
   * @return rank of matrix entries 
   * @throws NullPointerException if there are null pointers in the array
   * @throws IllegalArgumentException if there are inconsistent entries
   */
  public int getRank(FiniteFieldVector[] theMatrix, int nCols)
    throws NullPointerException, IllegalArgumentException
  {
    checkMatrix(theMatrix, nCols);
    localCopyMatrix(theMatrix, nCols);

    int r = 0, c = 0;

    while ((r < mRows) && (c < mCols)) {
      if (seekPivot(r, c)) {
        killMatrixRow(r, c);
        c++;
      }
      r++;
    }
    
    r = 0;
    // check until out of columns or rank is same as number of rows
    for (c=0; (c<mCols) && (r < mRows); c++) {
      if (!mMatrix[c].isZero()) {
        r++;
      }
    }
    return r;
  }

  /**
   * isInSpan determines whether theVec lies within the span of
   * theMatrix It makes a local copy of theMatrix for its
   * calculations.
   *
   * @param theMatrix one-dimensional array of FiniteFieldVectors with matrix
   * @param nCols number of columns of theMatrix to consider
   * @param theVec vector to check for inclusion in span(theMatrix)
   * @return true if theVec is in span(theMatrix), false otherwise
   * @throws NullPointerException if there are null pointers in the array
   * @throws IllegalArgumentException if there are inconsistent entries
   */
  public boolean isInSpan(FiniteFieldVector[] theMatrix, 
			  int nCols, 
			  FiniteFieldVector theVec)
    throws NullPointerException, IllegalArgumentException
  {
    checkSystem(theMatrix, nCols, theVec);
    localCopySystem(theMatrix, nCols, theVec);
    FiniteFieldVector zero = new FiniteFieldVector(mRows, mField);
    int c=0, r=0;

    while ((r < mRows) && !mObjectVector.isZero()) {
      if (seekPivot(r, c)) {
        killMatrixRow(r, c);
        killObjectRow(r, c);
        c++;
      }
      r++;
    }

    return mObjectVector.isZero();
  }
  
  /**
   * performs column-wise Gauss-Jordan reduction on the passed matrix; it works
   * with the passed vectors in-place.
   *
   * @param theMatrix one-dimensional array of FiniteFieldVectors with matrix
   * @param nCols number of columns of theMatrix to reduce
   */
  public void reduce(FiniteFieldVector[] theMatrix, int nCols)
  {
    checkMatrix(theMatrix, nCols);
    localCopyMatrix(theMatrix, nCols);
    
    int r=0, c=0;
    while ((r < mRows) && (c < mCols)) {
      if (seekPivot(r, c)) {
        killMatrixRowBothDir(r, c);
        c++;
      }
      r++;
    }
    
    for (c=0; c<mCols; c++) {
      theMatrix[c] = mMatrix[c];
    }
  }
  
  /**
   * returns an array of FiniteFieldVectors containing the transpose of
   * the passed array.
   *
   * @param theMatrix one-dimensional array of FiniteFieldVectors with matrix
   * @param nCols number of columns of theMatrix to use
   * @return transpose of theMatrix (number of entries in array is the size
   *         of the vectors in theMatrix, and the size of each of the vectors
   *         is nCols)
   */
  public FiniteFieldVector[] transpose(FiniteFieldVector[] theMatrix, 
					      int nCols)
  {
    checkMatrix(theMatrix, nCols);
    int oldRows = theMatrix[0].getSize();
    int oldCols = nCols;
    int newRows = oldCols;
    int newCols = oldRows;
    int i,j;
    FiniteFieldVector[] T;
    FiniteField fld = theMatrix[0].getField();

    T = new FiniteFieldVector[newCols];
    
    for (i=0; i<newCols; i++) {
      T[i] = new FiniteFieldVector(newRows, fld);
      for (j=0; j<newRows; j++) {
        T[i].set(j, theMatrix[j].get(i));
      }
    }
    
    return T;
  }

  // 
  // private methods
  //

  //
  // checkMatrix makes sure all column vectors have the same size & modulus
  // and that rows and cols match what this instance was created with
  //
  private void checkMatrix(FiniteFieldVector[] theMatrix, int nCols)
    throws NullPointerException, IllegalArgumentException
  {
    int nCheckRows = theMatrix[0].getSize();
    FiniteField fld0 = theMatrix[0].getField(), fld1;
    int i;

    if (nCheckRows != mRows) {
      throw new IllegalArgumentException("invalid row count");
    }

    if (nCols > mMaxCols) {
      throw new IllegalArgumentException("invalid column count");
    }

    if (theMatrix[0].getField().getFieldSize() != mField.getFieldSize()) {
      throw new IllegalArgumentException("invalid finite field");
    }

    for (i=1; i<nCols; i++) {
      if (!(theMatrix[i] instanceof FiniteFieldVector)) {
	      throw new NullPointerException("Null pointer passed to "
					     + "FiniteFieldReduction");
      }
      if (theMatrix[i].getSize() != nCheckRows) {
	      throw new IllegalArgumentException("Inconsistent row count");
      }
      fld1 = theMatrix[i].getField();
      if ((fld1.getCharacteristic() != fld0.getCharacteristic()) ||
	  (fld1.getExponent() != fld0.getExponent())) {
	      throw new IllegalArgumentException("Inconsistent fields");
      }
    }
  }

  // checkSystem makes sure all column vectors and objective vector have
  // the same size & modulus
  private void checkSystem(FiniteFieldVector[] theMatrix, 
			   int nCols, 
			   FiniteFieldVector theVec)
    throws NullPointerException, IllegalArgumentException
  {
    checkMatrix(theMatrix, nCols);
    if (!(theVec instanceof FiniteFieldVector)) {
      throw new NullPointerException("Null pointer passed to " + 
				     "FiniteFieldReduction");
    }

    if (theMatrix[0].getSize() != theVec.getSize()) {
      throw new IllegalArgumentException("Inconsistent row count");
    }
    
    FiniteField fld0 = theMatrix[0].getField(), fld1 = theVec.getField();

    if ((fld0.getCharacteristic() != fld1.getCharacteristic()) ||
	(fld0.getExponent() != fld1.getExponent())) {
      throw new IllegalArgumentException("Inconsistent fields");
    }
  }

  // localCopyMatrix makes an internal copy of the vectors in the passed
  // matrix
  private void localCopyMatrix(FiniteFieldVector[] theMatrix, int nCols)
  {
    int i, j;

    for (i=0; i<nCols; i++) {
      for (j=0; j<mRows; j++) {
	mMatrix[i].set(j, theMatrix[i].get(j));
      }
    }

    mCols = nCols;
  }

  // localCopySystem makes an internal copy of the vectors in the 
  // passed matrix and also the passed objective vector
  private void localCopySystem(FiniteFieldVector[] theMatrix, int nCols, 
			       FiniteFieldVector theVec)
  {
    localCopyMatrix(theMatrix, nCols);
    mObjectVector = (FiniteFieldVector)theVec.clone();
  }

  // switchCols switches the indicated column references and records
  // the switch if necessary
  private void switchCols(int col1, int col2)
  {
    FiniteFieldVector temp;
    temp = mMatrix[col1];
    mMatrix[col1] = mMatrix[col2];
    mMatrix[col2] = temp;
  }

  // multCol multiplies the indicated column by the passed scalar and
  // records the operation if necessary
  private void multCol(int col, int c)
  {
    mMatrix[col].multiply(c);
  }

  // combineCols adds the specified multiple of the column with index col1
  // to the column with index col2, and records the operation if necessary
  private void combineCols(int col1, int col2, int nCoeff)
  {
    mMatrix[col2].add(mMatrix[col1], nCoeff);
  }

  // combineObj adds the specified multiple of the column with index col1
  // to the objective column, and records the operation if necessary
  private void combineObj(int col1, int nCoeff)
  {
    mObjectVector.add(mMatrix[col1], nCoeff);
  }

  // seekPivot begins with the vector in position nStart and seeks
  // a vector with a nonzero entry in row nStart.  If it finds one, it 
  // switches the vector into position nStart, multiplies the vector
  // by the multiplicative inverse of the entry in position nStart, and
  // returns true. Otherwise, it returns false.
  private boolean seekPivot(int nRow, int nStart)
  {
    int i;
    int c;
    boolean ans = false;

    for (i=nStart; (i<mCols) && !ans; i++) {
      ans = (mMatrix[i].get(nRow) != 0);
    }

    if (ans) {
      i -= 1;
      if (i != nStart) {
	switchCols(i, nStart);
      }

      c = mMatrix[nStart].get(nRow);
      if (c != 1) {
	multCol(nStart, mField.inverse(c));
      }
    }

    return ans;
  }

  // killMatrixRow seeks all vectors in the matrix beyond nStart with
  // a nonzero entry in row nStart.  Wherever it finds a nonzero entry, 
  // it adds the additive inverse of that entry, times column nStart,
  // to that column in order to "kill" the nonzero entry
  private void killMatrixRow(int nRow, int nStart)
  {
    int i;
    int c;

    for (i=nStart+1; i<mCols; i++) {
      c = mMatrix[i].get(nRow);
      if (0 != c) {
	combineCols(nStart, i, mField.negative(c));
      }
    }
  }
  
  // killMatrixRowBothDir does the same thing as killMatrixRow, but it
  // also "kills off" entries before column nStart.
  private void killMatrixRowBothDir(int nRow, int nStart)
  {
    int i;
    int c;
    
    for (i=0; i<mCols; i++) {
      if (i != nStart) {
        c = mMatrix[i].get(nRow);
        if (0 != c) {
          combineCols(nStart, i, mField.negative(c));
        }
      }
    }
  }

  // killObjectRow checks the entry in row nStart of the objective column.
  // If that entry is not zero, it adds the additive inverse of that entry,
  // times the column with index nStart, to the objective column in order
  // to "kill" the nonzero entry.
  private void killObjectRow(int nRow, int nStart)
  {
    int c;
    c = mObjectVector.get(nRow);
    if (c != 0) {
      combineObj(nStart, mField.negative(c));
    }
  }

//   //
//   // private method writes the matrix to the screen (for debugging)
//   //
//   private void dumpMatrix(String message)
//   {
//     int r, c;

//     System.out.println();
//     System.out.println(message);
//     for (r=0; r<mRows; r++) {
//       for (c=0; c<mCols; c++) {
// 	System.out.print(" " + mMatrix[c].get(r));
//       }
//       System.out.println();
//     }
//     System.out.println();
//   }
}



