package Oid;
/**
 * <code>FiniteFieldVector</code> implements basic operations for
 * vectors over finite fields
 * 
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 25 Aug 2002
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class FiniteFieldVector implements Cloneable {
  // 
  // private data members
  //
  private int[] mVector;
  private int mSize;
  private FiniteField mField;
  static final int mDEFAULT_VECT_SIZE = 10;

  /**
   * copies another <code>FiniteFieldVector</code> instance
   * 
   * @param v vector to copy
   */
  public FiniteFieldVector(FiniteFieldVector v)
  {
    mField = v.getField();
    mSize = v.getSize();
    mVector = new int[mSize];
    for (int i = 0; i<mSize; i++) {
      mVector[i] = v.get(i);
    }
  }

  /** 
   * constructs a vector of the indicated size using the passed array and
   * <code>FiniteField</code> reference
   * 
   * @param matrix one-dimensional array of integers to use for vector entries
   * @param size size of new vector
   * @param field field to use for new vector
   * @throws ArrayIndexOutOfBoundsException if matrix is not big enough
   * @throws IllegalArgumentException if size is not positive
   */
  public FiniteFieldVector(int[] matrix, int size, FiniteField field)
  throws ArrayIndexOutOfBoundsException, IllegalArgumentException
  {
    checkSize(size);

    if (size != matrix.length) {
      throw new ArrayIndexOutOfBoundsException("Inconsistent array size");
    }

    mField = field;
    mSize = size;
    mVector = new int[mSize];
    for (int i=0; i<mSize; i++) {
      mVector[i] = mField.sum(matrix[i],0);
    }
  }

  /**
   * constructs a vector of the indicated size and accepts a reference to a 
   * <code>FiniteField</code>
   *
   * @param size size of new vector
   * @param field field for new vector
   * @throws IllegalArgumentException if size is not positive
   */
  public FiniteFieldVector(int size, FiniteField field)
    throws IllegalArgumentException
  {
    if (field == (FiniteField)null) {
      throw new IllegalArgumentException("Null field pointer");
    }
    checkSize(size);
    mField = field;
    mSize = size;
    mVector = new int[mSize];
    for (int i=0; i<mSize; i++) {
      mVector[i] = 0;
    }
  }

  /**
   * constructs a vector of default size (10) and accepts a reference to a 
   * FiniteField
   *
   * @param field FiniteField to use for vector
   */
  public FiniteFieldVector(FiniteField field)
  {
    if (field == (FiniteField)null) {
      throw new IllegalArgumentException("Null field pointer");
    }
    mSize = mDEFAULT_VECT_SIZE;
    mField = field;
    mVector = new int[mSize];
  }

  /**
   * constructs a vector of default size (10) and default field GF(2)
   */
  public FiniteFieldVector()
  {
    mSize = 10;
    mField = new GFpCalculator(2);
    mVector = new int[mSize];
    
    for (int i=0; i<mSize; i++) {
      mVector[i] = 0;
    }
  }

  /** 
   * overrides Object.clone() method
   */
  public Object clone()
  {
    FiniteFieldVector newVec = new FiniteFieldVector(mVector, mSize, mField);

    return (Object)newVec;
  }

  /**
   * overrides Object.toString method
   */
  public String toString() {
    StringBuffer answer = new StringBuffer("[ ");
    int i;
    
    if (mField instanceof GFpCalculator) {
      for (i=0; i<mSize; i++) {
	answer.append(mVector[i]);
	answer.append(" ");
      }
      answer.append("]");
    }
    else if (mField instanceof GF2kCalculator) {
      GF2kCalculator calc2k = (GF2kCalculator)mField;
      for (i=0; i<mSize; i++) {
	answer.append(calc2k.eltToPowerString(mVector[i]));
	answer.append(" ");
      }
      answer.append("]");      
    }
    else {
      GFpkCalculator calcpk = (GFpkCalculator)mField;
      for (i=0; i<mSize; i++) {
	answer.append(calcpk.eltToPowerString(mVector[i]));
	answer.append(" ");
      }
      answer.append("]");      
    }

    return answer.toString();
  }

  /**
   * overrides <code>Object.equals</code> method to check for equality
   * with another vector
   *
   * @param o Object to check for equality
   * @return true if o is an instance of FiniteFieldVector with same
   * modulus, size and entries, false otherwise
   */
  public boolean equals(Object o)
  {
    if (!(o instanceof FiniteFieldVector)) {
      return false;
    }

    FiniteFieldVector theVec = (FiniteFieldVector)o;
    if ((theVec.getSize() != mSize) || 
	(theVec.getField().getCharacteristic()!=mField.getCharacteristic()) ||
	(theVec.getField().getExponent() != mField.getExponent())) {
      return false;
    }

    boolean bAns = true;
    int i;
    for (i=0; (i<mSize) && bAns; i++) {
      bAns = bAns && (mVector[i] == theVec.get(i));
    }
    return bAns;
  }
  
  /**
   * checks whether all vector entries are zero
   *
   * @return true if all entries are zero, false otherwise
   */
  public boolean isZero()
  {
    boolean bAns = true;
    int i;
    for (i=0; (i<mSize) && bAns; i++) {
      bAns = bAns && (0 == mVector[i]);
    }
    
    return bAns;
  }

  /**
   * sets a value in the vector
   *
   * @param i index of position in vector (must be in the range 0..size-1)
   * @param x value to set in vector 
   * @throws ArrayIndexOutOfBoundsException if i is out of bounds
   */
  public void set(int i, int x)
    throws ArrayIndexOutOfBoundsException
  {
    checkIndex(i);
    mVector[i] = mField.sum(x, 0);
  }

  /**
   * retrieves a value from the vector 
   *
   * @param i index of position in vector (must be in the range 0..size-1)
   * @return value at position i (always in the range 0..p-1)
   * @throws ArrayIndexOutOfBoundsException if i is out of bounds
   */
  public int get(int i) 
    throws ArrayIndexOutOfBoundsException
  {
    checkIndex(i);
    return mVector[i];
  }

  /**
   * returns the size of the vector
   *
   * @return integer with size of vector
   */
  public int getSize() {
    return mSize;
  }

  /**
   * returns a reference to the base field for the vector
   */
  public FiniteField getField() {
    return mField;
  }

  /**
   * adds another <code>FiniteFieldVector</code>, times a constant, to this one
   *
   * @param v vector to add (must have the same size and field)
   * @param c multiply v by c before adding
   * @throws IllegalArgumentException if size and/or base field do not agree
   */
  public void add(FiniteFieldVector v, int c)
    throws IllegalArgumentException
  {
    checkVector(v);

    int i;
    int d = mField.sum(c,0);
    int s;
    try {
      for (i=0; i<mSize; i++) {
	s = mField.product(d, v.get(i));
	mVector[i] = mField.sum(mVector[i], s);
      }
    }
    catch (ArrayIndexOutOfBoundsException ex) {
      // this "can't happen"
      throw new InternalError("Internal error in FiniteFieldVector " +
			      "add--invalid index");
    }
  }

  /**
   * adds another <code>FiniteFieldVector</code> to this one
   *
   * @param v vector to add (must have the same size and base field)
   * @throws IllegalArgumentException  if size and/or base field do not agree
   */
  public void add(FiniteFieldVector v)
    throws IllegalArgumentException
  {
    this.add(v, 1);
  }

  /**
   * scalar-multiplies this by <code>c</code>
   *
   * @param c scalar to multiply
   */
  public void multiply(int c)
  {
    int i;
    int d = mField.sum(c,0);

    for (i=0; i<mSize; i++) {
      mVector[i] = mField.product(mVector[i], d);
    }
  }

  /** 
   * calculates the dot product of this with <code>v</code>; that is,
   * returns the value <code>this * v<sup>T</sup></code>
   *
   * @param v vector to calculate dot product with (must have the same
   *          size and base field)
   * @return dot product of this vector and v
   * @throws IllegalArgumentException if size and/or base field do not agree
   */
  public int dotProduct(FiniteFieldVector v) 
    throws IllegalArgumentException
  {
    checkVector(v);

    int ans = 0;
    int i;

    try {
      for (i=0; i<mSize; i++) {
	ans = mField.sum(ans, mField.product(mVector[i], v.get(i)));
      }
    }
    catch (ArrayIndexOutOfBoundsException exi) {
      // this "can't happen"
      throw new InternalError("Invalid index error in " +
			      "FiniteFieldVector.dotProduct");
    }

    return ans;
  }

  /**
   * if the passed <code>FiniteFieldVector</code> is a multiple of
   * this one, this method returns the nonzero scalar that the passed
   * vector must be multiplied by to obtain this one; otherwise, the
   * method returns 0
   *
   * @param v vector to compare
   * @return scalar quotient, or zero
   * @throws IllegalArgumentException if size and/or base field do not agree
   */
  public int multiple(FiniteFieldVector v)
  {
    checkVector(v);

    int answer = 0;
    int quotient;
    int i, j, k;

    if (v.getSize() == mSize) {
      for (i=0; i<mSize; i++) {
	j = mField.sum(v.get(i), 0);
	k = mField.sum(mVector[i], 0);
	if (((j != 0) && (k == 0)) ||
	    ((j == 0) && (k != 0))) {
	  answer = 0;
	  break;
	}
	else if ((j != 0) && (k != 0)) {
	  quotient = mField.quotient(k, j);
	  if (answer == 0) {
	    answer = quotient;
	  }
	  else if (answer != quotient) {
	    answer = 0;
	    break;
	  }
	}
      }
    }

    return answer;
  }

  //
  // private utility functions
  //
  private void checkVector(FiniteFieldVector theOther) 
    throws IllegalArgumentException
  {
    if (!(theOther instanceof FiniteFieldVector)) {
      throw new IllegalArgumentException("FiniteFieldVector required");
    }

    if (mSize != theOther.getSize()) {
      throw new IllegalArgumentException("Inconsistent vector sizes");
    }

    FiniteField fld = theOther.getField();

    if ((fld.getCharacteristic() != mField.getCharacteristic()) ||
	(fld.getExponent() != mField.getExponent())) {
      throw new IllegalArgumentException("Inconsistent base fields");
    }
  }

  private void checkSize(int theSize)
    throws IllegalArgumentException
  {
    if (theSize <= 0) {
      throw new IllegalArgumentException();
    }
  }

  private void checkIndex(int theIndex)
    throws ArrayIndexOutOfBoundsException
  {
    if ((theIndex < 0) || (theIndex >= mSize)) {
      throw new ArrayIndexOutOfBoundsException();
    }
  }
}



