package Oid;
import java.util.*;

/**
 * An instance of <code>Flat</code> contains the Subset representing
 * the members in a flat of a matroid, as well as the flat's rank and
 * collections of references to its sub- and superflats.  Instances of
 * <code>Flat</code> are immutable in rank and subset, but super- and 
 * subset references can be added and/or deleted after construction.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 26 May 2003
 * @version 3.0
 *
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class Flat
{
  //
  // private data members
  //
  private Vector mSubFlats;
  private Vector mSupFlats;
  private Subset mSubset;
  private String mLabel;
  private boolean mIdentified;     // used for modular cut enumeration
  private boolean mSelected;       // used for modular cut enumeration
  private int mRank;

  /**
   * Simplest constructor requires only the rank.
   *
   * @param rank rank of this flat
   * @throws IllegalArgumentException if the rank is negative
   */
  public Flat(int rank)
    throws IllegalArgumentException
  {
    if (rank < 0) {
      throw new IllegalArgumentException("negative rank: " + 
					 String.valueOf(rank));
    }

    mRank = rank;
    mSubFlats = new Vector();
    mSupFlats = new Vector();
  }

  /**
   * Constructor using the rank and a Subset reference.
   *
   * @param rank rank of this flat
   * @param sub subset indicating members of this flat
   * @throws IllegalArgumentException if the rank is negative or larger
   * than the number of elements in the subset
   */
  public Flat(int rank, Subset sub)
    throws IllegalArgumentException
  {
    if (rank < 0) {
      throw new IllegalArgumentException("negative rank: " + 
					 String.valueOf(rank));
    }
    
    if (rank > sub.numElements()) {
      throw new IllegalArgumentException("rank exceeds size of set: " + 
					 String.valueOf(rank));
    }

    mRank = rank;
    mSubFlats = new Vector();
    mSupFlats = new Vector();
    mSubset = sub;
  }
 
  /** 
   * Constructor using the rank, a Subset reference, and Vectors of 
   * sub- and superflats
   *
   * @param rank rank of this flat
   * @param sub subset indicating members of this flat
   * @param subFlats Vector of references to subflats of this flat
   * @param supFlats Vector of references to superflats of this flat
   * @throws IllegalArgumentException if rank is negative or exceeds
   * number of elements in subset, or if subflats are not all lower in
   * rank, or if superflats are not all greater in rank, or if subsets 
   * aren't null and containments aren't correct
   */
  public Flat(int rank, Subset sub, Vector subFlats, Vector supFlats)
    throws IllegalArgumentException
  {
    if (rank < 0) {
      throw new IllegalArgumentException("negative rank: " + 
					 String.valueOf(rank));
    }
    
    if (rank > sub.numElements()) {
      throw new IllegalArgumentException("rank exceeds size of set: " + 
					 String.valueOf(rank));
    }

    Iterator pFlat;
    Flat f;
    Subset s;
    // check subflats -- all must have lower rank than me 
    if (subFlats instanceof Vector) {
      for (pFlat = subFlats.iterator(); pFlat.hasNext(); ) {
	f = (Flat)pFlat.next();
	if (f.getRank() >= rank) {
	  throw new IllegalArgumentException("subflats must have lower rank");
	}
      }
      
      if (sub instanceof Subset) {
	for (pFlat = subFlats.iterator(); pFlat.hasNext(); ) {
	  s = ((Flat)pFlat.next()).getSubset();
	  if (s instanceof Subset) {
	    if (!sub.subsetContains(s)) {
	      throw new IllegalArgumentException("subflats must be contained" +
						 " in flat");
	    }
	  }
	}
      }

      mSubFlats = new Vector();
      mSubFlats.addAll(subFlats);
    }
    else {
      mSubFlats = new Vector();
    }

    // check superflats -- all must have greater rank than me
    if (supFlats instanceof Vector) {
      for (pFlat = supFlats.iterator(); pFlat.hasNext(); ) {
	f = (Flat)pFlat.next();
	if (f.getRank() <= rank) {
	  throw new IllegalArgumentException("superflats must have greater" +
					     " rank");
	}
      }

      if (sub instanceof Subset) {
	for (pFlat = supFlats.iterator(); pFlat.hasNext(); ) {
	  s = ((Flat)pFlat.next()).getSubset();
	  if (s instanceof Subset) {
	    if (!s.subsetContains(sub)) {
	      throw new IllegalArgumentException("superflats must contain" +
						 " flat");
	    }
	  }
	}
      }

      mSupFlats = new Vector();
      mSupFlats.addAll(supFlats);
    }
    else {
      mSupFlats = new Vector();
    }
  }

  /**
   * copy constructor
   *
   * @param f reference to another flat
   */
  public Flat(Flat f) 
  {
    mRank = f.getRank();
    mSubset = new Subset(f.getSubset());
    mSubFlats = new Vector(f.getSubFlats());
    mSupFlats = new Vector(f.getSuperFlats());
  }

  /** 
   * checks for equality with another flat by checking subsets
   *
   * @param f flat to check
   * @return true if and only if this flat and f have the same subset
   */
  public boolean equals(Object o) 
  {
    boolean answer = false;
    
    if (o instanceof Flat) {
      Flat f = (Flat)o;
      answer = mSubset.subsetEquals(f.getSubset());
    }
    return answer;
  }

  /**
   * sets the label for this flat
   *
   * @param label String with new label
   */
  public void setLabel(String label)
  {
    mLabel = new String(label);
  }

  /**
   * returns the label for this flat
   * 
   * @return String with copy of label
   */
  public String getLabel()
  {
    String result = (String)null;

    if (mLabel instanceof String) {
      result = new String(mLabel);
    }
    else {
      result = new String("");
    }

    return result;
  }

  /**
   * returns identification status of this flat
   * 
   * @return true if and only if this flat has been identified as candidate
   * for a modular cut
   */
  public boolean getIdentified()
  {
    return mIdentified;
  }

  /**
   * sets identification status of this flat
   * 
   * @param identified true if and only if this flat has been identified as
   * a candidate for a modular cut
   */
  public void setIdentified(boolean identified)
  {
    mIdentified = identified;
  }

  /**
   * returns selection status of this flat
   *
   * @return true if and only if this flat has been selected as a member
   * of a modular cut
   */
  public boolean getSelected()
  {
    return mSelected;
  }

  /**
   * sets selection status of this flat
   *
   * @param selected true if and only if this flat has been selected
   * as a candidate for a modular cut
   */
  public void setSelected(boolean selected)
  {
    mSelected = selected;
  }

  /** 
   * returns the rank of this flat
   *
   * @return rank of this flat
   */
  public int getRank() 
  {
    return mRank;
  }

  /**
   * returns a reference to this flat's subset
   *
   * @return Subset reference
   */
  public Subset getSubset() 
  {
    return mSubset;
  }

  /**
   * returns a reference to this flat's Vector of subflats
   *
   * @return reference to Vector of subflats
   */
  public Vector getSubFlats()
  {
    return mSubFlats;
  }

  /**
   * returns a reference to this flat's Vector of superflats
   *
   * @return reference to Vector of superflats
   */
  public Vector getSuperFlats()
  {
    return mSupFlats;
  }

  /**
   * Adds a superflat reference to this flat
   * 
   * @param f reference to superflat
   * @throws IllegalArgumentException if rank of flat is not greater than
   * this flat's rank, or flat's subset does not contain this flat's subset
   */
  public void addSuperFlat(Flat f)
    throws IllegalArgumentException
  {
    if (f.getRank() <= mRank) {
      throw new IllegalArgumentException("superflat rank must be larger");
    }

    Subset s = f.getSubset();
    if ((mSubset instanceof Subset) && (s instanceof Subset)) {
      if (!s.subsetContains(mSubset)) {
	throw new IllegalArgumentException("superflat must be contained in " + 
					   "flat");
      }
    }

    mSupFlats.add(f);
  }

  /** 
   * Removes a superflat reference from this flat
   * 
   * @param f reference to superflat
   */
  public void removeSuperFlat(Flat f)
  {
    mSupFlats.remove(f);
  }

  /**
   * Adds a subflat reference to this flat
   * 
   * @param f reference to subflat
   * @throws IllegalArgumentException if rank of flat is not less than
   * this flat's rank, or flat's subset is not contained within this
   * flat's subset
   */
  public void addSubFlat(Flat f)
    throws IllegalArgumentException
  {
    if (f.getRank() >= mRank) {
      throw new IllegalArgumentException("subflat rank must be smaller");
    }

    Subset s = f.getSubset();
    if ((mSubset instanceof Subset) && (s instanceof Subset)) {
      if (!mSubset.subsetContains(s)) {
	throw new IllegalArgumentException("subflat must be contained in " + 
					   "flat");
      }
    }

    mSubFlats.add(f);
  }

  /** 
   * Removes a subflat reference from this flat
   * 
   * @param f reference to subflat
   */
  public void removeSubFlat(Flat f)
  {
    mSubFlats.remove(f);
  }
}
