package Oid;
import java.lang.*;
import java.util.*;

/** 
 * <code>GFpCalculator</code> is designed to encapsulate common GF(p) 
 * arithmetic operations.  It implements the FiniteField interface.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 22 Aug 2002
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class GFpCalculator implements FiniteField
{
  private int p;
  private int[] mInv;   // mInv[i] holds 1/i in GF(p) for 1 <= i < p
  private int[][] mDiv; // mDiv[i][j] holds i/j in GF(p) for 1 <= i,j < p
  static final int DEFAULT_MODULUS = 2;

  //
  // constructors
  //
  /** the default constructor sets the modulus (p) to 2 */
  public GFpCalculator()
  {
    p = DEFAULT_MODULUS;
    initArrays();
  }

  /** 
   * this constructor allows you to set the modulus to any prime 
   * @param nModulus modulus (must be prime)
   * @throws IllegalArgumentException if nModulus is not prime
   */
  public GFpCalculator(int nModulus)
  throws IllegalArgumentException
  {
    checkModulus(nModulus);

    p = nModulus;
    initArrays();
  }

  //
  // public methods
  //
  
  /** 
   * returns the modulus for the field 
   *
   * @return integer with prime modulus
   */
  public int getCharacteristic() 
  {
    return p;
  }

  /**
   * returns the exponent for this field (always 1)
   *
   * @return exponent as integer
   */
  public int getExponent()
  {
    return 1;
  }

  /**
   * returns the size of the field (p)
   *
   * @return field size
   */
  public int getFieldSize()
  {
    return p;
  }

  /** 
   * sum always returns a value between 0 and p-1, inclusive 
   * @param x first value to add
   * @param y second value to add
   * @return sum of x and y mod p
   */
  public int sum(int x, int y)
  {
    return fix(x + y);
  }

  /** 
   * difference calculates x - y (mod p) and always returns a value between 
   * 0 and p-1, inclusive 
   * @param x value to subtract from
   * @param y value to subtract
   * @return difference of x and y mod p
   */
  public int difference(int x, int y)
  {
    return fix(x - y);
  }

  /** 
   * product calculates x * y (mod p) and always returns a value between
   * 0 and p-1, inclusive
   * @param x first value to multiply
   * @param y second value to multiply
   * @return product of x and y mod p
   */
  public int product(int x, int y)
  {
    return fix(x * y);
  }

  /**
   * quotient divides x by y (mod p) and always returns a value between
   * 0 and p-1, inclusive
   * @param x numerator
   * @param y denominator
   * @return quotient of x and y mod p
   */
  public int quotient(int x, int y) 
  throws ArithmeticException
  {
    int fx = fix(x);
    int fy = fix(y);
    int ans = 0;
    if (fy == 0) {
      throw new ArithmeticException("Division by zero (mod p)");
    }

    if (fx > 0) {
      ans = mDiv[fx][fy];
    }

    return ans;    
  }

  /**
   * negative returns -x (mod p) and always returns a value between 
   * 0 and p-1, inclusive
   * @param x value to negate
   * @return -x (mod p)
   */
  public int negative(int x) {
    return fix(-x);
  }

  /**
   * inverse returns (1/x) (mod p) and always returns a value between
   * 0 and p-1, inclusive
   * @param x value to invert
   * @return multiplicative inverse of x (mod p)
   */
  public int inverse(int x)
  throws ArithmeticException
  {
    int fx = fix(x);
    if (fx == 0) {
      throw new ArithmeticException("Division by zero (mod p)");
    }
    return mInv[fx];
  }

  /**
   * returns an element raised to a power
   *
   * @param x element
   * @param k power
   * @return x^k in GF(p)
   * @throws ArithmeticException if k < 0 and x = 0
   */
  public int power(int x, int k)
    throws ArithmeticException
  {
    if ((0 == x) && (k < 0)) {
      throw new ArithmeticException("Divide by zero (mod p)");
    }

    int y = (k < 0) ? this.inverse(fix(x)) : fix(x);
    int j = (k < 0) ? ((-k) % (p-1)) : (k % (p-1));
    int answer = 1;

    for (int i=0; i<j; i++) {
      answer *= y;
    }
    
    return fix(answer);
  }

  /**
   * returns a string represenation of a field element
   * 
   * @param x integer representing a field element
   * @return string representation of x
   */
  public String eltToString(int x)
  {
    return String.valueOf(this.sum(x,0));
  }

  /**
   * returns an integer representation of a field element converted from
   * a string
   *
   * @param s String representation of a field element
   * @return integer representation of s
   */
  public int stringToElt(String s)
    throws IllegalArgumentException
  {
    Integer answer = new Integer(0);
    
    try {
      answer = Integer.valueOf(s);
    }
    catch (NumberFormatException ex) {
      throw new IllegalArgumentException(ex.getMessage());
    }
    
    return this.sum(answer.intValue(), 0);
  }

  /**
   * returns a String representation of a field element using an additional
   * format parameter--GFpCalculator ignores the format specifier
   *
   * @param x integer representation of a field element
   * @param fmt integer specifying a format option
   * @return String representation of x
   */
  public String eltToString(int x, int fmt)
  {
    return this.eltToString(x);
  }

  /**
   * returns the field element represented by a String using an additional
   * format parameter--GFpCalculator ignores the format specifier
   *
   * @param s String representation of a field element
   * @param fmt integer specifying a format option
   * @return integer representation of the field element
   * @throws IllegalArgumentException if the format of s is incorrect
   */
  public int stringToElt(String s, int fmt)
    throws IllegalArgumentException
  {
    return this.stringToElt(s);
  }

  //
  // private methods
  //

  // checkModulus checks for non-positive values and does a prime check
  private void checkModulus(int nModulus) 
  throws IllegalArgumentException
  {
    if (nModulus <= 0) {
      throw new IllegalArgumentException();
    }

    // quick n' dirty prime check
    if (nModulus < 40) {
      if (!((nModulus ==  2) || (nModulus ==  3) || (nModulus ==  5) || 
            (nModulus ==  7) || (nModulus == 11) || (nModulus == 13) || 
            (nModulus == 17) || (nModulus == 19) || (nModulus == 23) || 
            (nModulus == 29) || (nModulus == 31) || (nModulus == 37))) {
	      throw new IllegalArgumentException();
      }
    }
    else {
      double nLimit = Math.sqrt((double)nModulus);
      int i;
      boolean bOk = true;
      
      for (i = 2; i<nLimit && bOk; i++) {
	      bOk = (nModulus % i != 0);
      }
      if (!bOk) {
	      throw new IllegalArgumentException();
      }
    }
  }

  // initArrays initializes the arrays of inverses and quotients mod p
  private void initArrays() {
    mInv = new int[p];
    mDiv = new int[p][p];

    int i, j;
    
    // inverses
    for (i=1; i<p; i++) {
      mInv[i] = -1;
      for (j=1; (j<p) && (mInv[i] == -1); j++) {
	if ((i * j) % p == 1) {
	  mInv[i] = j;
	}
      }
      if (mInv[i] < 0) {
	// this "can't happen"
	throw new InternalError("Unable to find inverse of "+ i +" mod " + p);
      }
    }

    // quotients
    for (i=1; i<p; i++) {
      for (j=1; j<p; j++) {
	mDiv[i][j] = (i * mInv[j]) % p;
      }
    }
  }

  // fix adds or subtracts to n to guarantee 0 <= n < p
  private int fix(int n)
  {
    int ans = n;

    if (ans < 0) {
      ans += (-ans / p + 1) * p;
    }
    ans = ans % p;
    return ans;
  }
}
