package Oid;
import java.io.*;

/**
 * An instance of <code>GFpPoly</code> stores a polynomial in one
 * variable with coefficients int GF(p) for some prime p.  It can do
 * basic polynomial operations (add, subtract, multiply with other
 * instances and with integers) and can evaluate itself at a value.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 15 Jun 2002
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class GFpPoly
{
  //
  // private data members
  //
  private int mDegree;
  private int[] mCoeff;
  private GFpCalculator mGFp;

  /**
   * returns an instance of GFpPoly over the specified base field that
   * consists of one term with the specified power of x
   *
   * @param GFp base field
   * @param n power of x
   * @returns GFpPoly reference with x^n, over GF(p)
   * @throws IllegalArgumentException if n < 0
   */
  public static GFpPoly xPower(GFpCalculator GFp, int n)
    throws IllegalArgumentException
  {
    if (n < 0) {
      throw new IllegalArgumentException("Power must be non-negative");
    }

    GFpPoly answer;

    if (0 == n) {
      answer = new GFpPoly(GFp);
      answer.add(1);
    }
    else {
      int[] coeff = new int[n+1];
      coeff[n] = 1;
      for (int i=0; i<n; i++) {
	coeff[i] = 0;
      }
      answer = new GFpPoly(coeff, GFp);
    }
    return answer;
  }

  /** 
   * the default constructor creates the zero polynomial over GF(2)
   */
  public GFpPoly()
  {
    mDegree = -1;
  }

  /**
   * this constructor creates the zero polynomial over a specified base
   * field
   * 
   * @param GFp <code>GFpCalculator</code instance
   */
  public GFpPoly(GFpCalculator GFp)
  {
    mGFp = GFp;
    mDegree = -1;
  }


  /**
   * the copy constructor copies the coefficients and base field from another 
   * <code>GFpPoly</code> instance
   * 
   * @param poly another <code>GFpPoly</code> instance
   */
  public GFpPoly(GFpPoly poly)
  {
    mDegree = poly.getDegree(); 
    mGFp = poly.getField();
    
    if (mDegree >= 0) {
      int[] newCoeff = poly.getCoefficientArray();
      int i;
      
      mCoeff = new int[mDegree+1];
      for (i=0; i<=mDegree; i++) {
	mCoeff[i] = newCoeff[i];
      }
    }
  }

  /**
   * constructs an instance from an array of integers with the 
   * coefficients and a field reference
   *
   * @param coeff array of coefficients, must be of size (degree+1)
   * @param GFp <code>GFpCalculator</code> reference
   */
  public GFpPoly(int[] coeff, GFpCalculator GFp)
  {
    
    int n = coeff.length-1;
    int i;

    mGFp = GFp;

    // find last nonzero coefficient
    for (i=n; i>=0; i--) {
      if (mGFp.sum(coeff[i],0) != 0) {
	break;
      }
    }

    mDegree = i;

    if (mDegree >= 0) {
      mCoeff = new int[mDegree+1];
      for (i=0; i<=mDegree; i++) {
	mCoeff[i] = mGFp.sum(coeff[i],0);
      }
    }
  }

  /** 
   * returns a reference to the base field for this polynomial
   *
   * @return <code>GFpCalculator</code> reference
   */
  public GFpCalculator getField()
  {
    return mGFp;
  }

  /** 
   * returns this polynomial's array of coefficients
   * @return integer array of coefficients
   */
  public int[] getCoefficientArray()
  {
    return mCoeff;
  }

  /**
   * returns this polynomial's degree
   * @return integer with degree
   */
  public int getDegree()
  {
    return mDegree;
  }

  /**
   * returns the coefficient of x to the specified power
   * @param k exponent
   * @return coefficient of x^k
   */
  public int getCoefficient(int k) 
  {
    int answer = 0;
    if ((k >= 0) && (k <= mDegree)) {
      answer = mCoeff[k];
    }
    return answer;
  }

  /**
   * returns a <code>String</code> expression of this polynomial using the 
   * passed character as the variable
   *
   * @param v character to use as the variable
   * @return polynomial in <code>String</code> form
   */
  public String toString(char v)
  {
    StringBuffer answer = new StringBuffer();
    boolean first=true;

    if (mDegree < 0) {
      answer.append("0");
    }
    else {
      for (int i=mDegree; i>=0; i--) {
	if (i > 0) {
	  if (mCoeff[i] != 0) {
	    if (mCoeff[i] > 0) {
	      if (!first) {
		answer.append(" + ");
	      }
	      if (mCoeff[i] > 1) {
		answer.append(mCoeff[i]);
	      }
	    }
	    else {
	      answer.append(" - ");
	      if (mCoeff[i] < -1) {
		answer.append(-mCoeff[i]);
	      }
	    }
	    answer.append(v);
	    if (i > 1) {
	      answer.append("^");
	      answer.append(i);
	    }
	    first = false;
	  }
	}
	else {
	  if (mCoeff[0] > 0) {
	    if (!first) {
	      answer.append(" + ");
	    }
	    answer.append(mCoeff[0]);
	  }
	  else if (mCoeff[0] < 0) {
	    answer.append(" - ");
	    answer.append(-mCoeff[0]);
	  }
	  first = false;
	}
      }
    }
    return answer.toString();
  }

  /**
   * returns a <code>String</code> expression of this polynomial in terms of x
   *
   * @return <code>String</code> with polynomial
   */
  public String toString() 
  {
    return this.toString('x');
  }

  /**
   * evaluates this polynomial at an integer value and returns the result
   *
   * @param x integer value to use
   * @return value of this polynomial evaluated at x
   */
  public int evaluate(int x) 
  {
    int answer = 0;
    int i;
    int p = 1;

    for (i=0; i<=mDegree; i++) {
      answer += mCoeff[i] * p;
      p *= x;
    }
    return mGFp.sum(answer,0);
  }

  /**
   * substitutes another polynomial for the variable in this one and 
   * returns the result
   *
   * @param poly polynomial to substitute
   * @return resulting polynomial
   * @throws IllegalArgumentException if the base field of <code>poly</code>
   * is different from this polynomial's
   */
  public GFpPoly substitute(GFpPoly poly)
    throws IllegalArgumentException
  {
    if (poly.getField().getCharacteristic() != mGFp.getCharacteristic()) {
      throw new IllegalArgumentException("Inconsistent fields");
    }

    GFpPoly answer = new GFpPoly(mGFp);
    GFpPoly p = new GFpPoly(mGFp);
    int i;

    p.add(1);
    for (i=0; i<=mDegree; i++) {
      answer.add(p, mCoeff[i]);
      p.multiply(poly);
    }
    return answer;
  }

  /** 
   * adds another polynomial, times an integer constant to this one
   *
   * @param poly another GFpPoly instance
   * @param c integer constant
   * @throws IllegalArgumentException if the other polynomial is over a 
   * different base field 
   */
  public void add(GFpPoly poly, int c)
    throws IllegalArgumentException
  {
    if (poly.getField().getCharacteristic() != mGFp.getCharacteristic()) {
      throw new IllegalArgumentException("Inconsistent fields");
    }

    int degree = poly.getDegree();
    int[] coeff = poly.getCoefficientArray();
    int i;

    if ((mGFp.sum(c,0) != 0) && (degree >= 0)) {
      if (degree > mDegree) {
	// make me a bigger coefficient array
	int[] newCoeff = new int[degree+1];
	for (i=0; i<=mDegree; i++) {
	  newCoeff[i] = mCoeff[i];
	}
	mCoeff = (int[])null;
	mCoeff = newCoeff;
	mDegree = degree;
      }
      
      // now add 'em up
      for (i=0; i<=mDegree; i++) {
	mCoeff[i] = (i > degree) ? mCoeff[i] : (mCoeff[i] + c * coeff[i]);
	mCoeff[i] = mGFp.sum(mCoeff[i],0);
      }
      
      // check whether leading coefficient(s) were cancelled out
      adjustDegree();
    }
  }

  /** 
   * adds an integer to this polynomial
   * @param c integer to add
   */
  public void add(int c) 
  {
    if (c != 0) {
      if (mDegree <= 0) {
	mDegree = 0;
	mCoeff = new int[1];
	mCoeff[0] = c;
      }
      else {
	mCoeff[0] += c;
	// what if this is a constant and we just cancelled the constant term?
	if ((mDegree == 0) && (mCoeff[0] == 0)) {
	  mDegree = -1;
	}
      }
    }
    mCoeff[0] = mGFp.sum(mCoeff[0], 0);
  }

  /**
   * adds another polynomial to this one
   *
   * @param poly another <code>GFpPoly</code> instance
   * @throws IllegalArgumentException if the base field of <code>poly</code>
   * is different
   */
  public void add(GFpPoly poly) 
    throws IllegalArgumentException
  {
    this.add(poly, 1);
  }

  /**
   * subtracts another polynomial from this one
   *
   * @param poly the other polynomial
   * @throws IllegalArgumentException if the base field of <code>poly</code>
   * is different
   */
  public void subtract(GFpPoly poly)
    throws IllegalArgumentException
  {
    this.add(poly, -1);
  }

  /**
   * subtracts an integer constant from this polynomial (for completness' sake)
   *
   * @param c constant
   */
  public void subtract(int c) 
  {
    this.add(-c);
  }

  /**
   * multiplies this polynomial by an integer constant
   *
   * @param c constant
   */
  public void multiply(int c)
  {
    if (mGFp.sum(c,0) == 0) {
      mCoeff = (int[])null;
      mDegree = -1;
    }
    else {
      for (int i=0; i<=mDegree; i++) {
	mCoeff[i] = mGFp.product(mCoeff[i], c);
      }
    }
  }

  /**
   * multiplies this polynomial by another one
   *
   * @param poly another <code>GFpPoly</code>
   * @throws IllegalArgumentException if the base field of <code>poly</code>
   * is different
   */
  public void multiply(GFpPoly poly)
    throws IllegalArgumentException
  {
    if (poly.getField().getCharacteristic() != mGFp.getCharacteristic()) {
      throw new IllegalArgumentException("Inconsistent fields");
    }

    int degree = poly.getDegree();
    int[] coeff = poly.getCoefficientArray();

    if (degree < 0) {
      mDegree = -1;
      mCoeff = (int[])null;
    }
    else if (degree == 0) {
      this.multiply(coeff[0]);
    }
    else if (mDegree >= 0) {
      int newDegree = mDegree + degree;
      int[] newCoeff = new int[newDegree+1];
      int i,j;

      for (i=0; i<=newDegree; i++) {
	newCoeff[i] = 0;
      }

      for (i=0; i<=mDegree; i++) {
	for (j=0; j<=degree; j++) {
	  newCoeff[i+j] += mCoeff[i] * coeff[j];
	}
      }

      mDegree = newDegree;
      mCoeff = (int[])null;
      mCoeff = newCoeff;
      for (i=0; i<mDegree; i++) {
	mCoeff[i] = mGFp.sum(mCoeff[i],0);
      }
    }
  }

  /**
   * divides this polynomial by another one and returns the remainder
   *
   * @param poly another <code>GFpPoly</code>
   * @return remainder after this polynomial is divided by <code>poly</code>
   * @throws IllegalArgumentException if <code>poly</code> is over a different
   * field
   * @throws ArithmeticException if division by zero is attempted
   */
  public GFpPoly divide(GFpPoly poly)
    throws IllegalArgumentException, ArithmeticException
  {
    if (poly.getField().getCharacteristic() != mGFp.getCharacteristic()) {
      throw new IllegalArgumentException("Inconsistent base fields");
    }
    
    if (poly.getDegree() < 0) {
      throw new ArithmeticException("Division by zero");
    }

    GFpPoly r;
    int i,c;

    if (poly.getDegree() == 0) {
      r = new GFpPoly(mGFp);
      c = poly.getCoefficient(0);
      if (0 == c) {
	throw new ArithmeticException("Division by zero");
      }

      for (i=0; i<=mDegree; i++) {
	mCoeff[i] = mGFp.quotient(mCoeff[i], c);
      }
    }
    else {
      GFpPoly q=new GFpPoly(mGFp), xPow, mult;
      int degDiff;

      r = new GFpPoly(this);

      while (r.getDegree() >= poly.getDegree()) {
	degDiff = r.getDegree() - poly.getDegree();
	xPow = GFpPoly.xPower(mGFp, degDiff); 
	mult = new GFpPoly(poly); 
	xPow.multiply(mGFp.quotient(r.getCoefficient(r.getDegree()),
				    poly.getCoefficient(poly.getDegree())));
	mult.multiply(xPow);
	r.subtract(mult);
	q.add(xPow);
      }

      int[] newCoeff;
      mDegree = q.getDegree();
      if (mDegree >= 0) {
	mCoeff = new int[mDegree+1];
	newCoeff = q.getCoefficientArray();

	for (i=0; i<=mDegree; i++) {
	  mCoeff[i] = newCoeff[i];
	}
      }
    }
    
    return r;
  }

  /**
   * checks this polynomial for equality with another one (or with an integer)
   * @param obj object to compare
   */
  public boolean equals(Object obj)
  {
    boolean answer = false;

    if (obj instanceof GFpPoly) {
      GFpPoly poly = (GFpPoly)obj;
      int d = poly.getDegree();
      int[] coeff = poly.getCoefficientArray();
      int i;

      answer = (d == mDegree) && 
	(poly.getField().getCharacteristic() == mGFp.getCharacteristic());

      if (answer) {
	for (i=0; i<=mDegree; i++) {
	  if (coeff[i] != mCoeff[i]) {
	    answer = false;
	    break;
	  }
	}
      }
    }
    else if (obj instanceof Integer) {
      answer = (mDegree == 0) && 
	(mCoeff[0] == mGFp.sum(((Integer)obj).intValue(),0));
    }
    
    return answer;
  }

  //
  // private method adjusts the degree in case the leading coefficient is 0
  //
  private void adjustDegree()
  {
    int i;

    // find last nonzero coefficient
    for (i=mDegree; i>=0; i--) {
      if (mCoeff[i] != 0) {
	break;
      }
    }

    mDegree = i;
  }
}
