package Oid;
import java.math.*;
import java.util.*;

/**
 * A <code>GFpkCalculator</code> instance can perform arithmetic operations 
 * in GF(p^k) where 1 < p^k < 2^31.  (Integers are used to represent field
 * elements, using the p-ary expansion of an integer to represent coefficients
 * of field elements.)
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 24 Aug 2002
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class GFpkCalculator implements FiniteField
{
  //
  // private data members
  //
  private int mK;
  private int mP;
  private int mFieldSize;
  private GFpCalculator mGFp;
  private GFpPoly mPrimPoly;
  private int[] mLog;
  private int[] mPow;
  private int[][] mSum;
  private static final int DEFAULT_P = 3;
  private static final int DEFAULT_K = 2;
  private static final char DEFAULT_X = 'x';

  /**
   * default constructor sets k = 2, p = 3
   */
  public GFpkCalculator()
  {
    mP = DEFAULT_P;
    mK = DEFAULT_K;
    initArrays();
  }

  /**
   * constructor accepts values for k and p
   * @param p prime for characteristic of field
   * @param k power of 2 for size of field
   * @throws IllegalArgumentException thrown if k <= 1 or if p is not prime
   * @throws IndexOutOfBoundsException if p^k > 2^31
   */
  public GFpkCalculator(int p, int k) 
    throws IllegalArgumentException, IndexOutOfBoundsException
  {
    if (k <= 0) {
      throw new IllegalArgumentException("k must be greater than one");
    }

    if (k * Math.log(p) / Math.log(2) > 31) {
      throw new IndexOutOfBoundsException(String.valueOf(p) + " " +
					  String.valueOf(k));
    }

    mK = k;
    mP = p;
    initArrays();
  }

  /**
   * returns the value of k used to initialize the instance
   * @return power of 2 that is size of the field
   */
  public int getExponent() {
    return mK;
  }

  /**
   * returns the characteristic of this field (2)
   * @return field characteristic as integer
   */
  public int getCharacteristic() 
  {
    return mP;
  }

  /**
   * returns the field size (p^k)
   *
   * @return field size
   */
  public int getFieldSize()
  {
    return mFieldSize;
  }

  /**
   * returns the sum of two field elements, expressed as ints
   * @param x int representing one field element
   * @param y int representing second field element
   * @return sum of x and y in field
   */
  public int sum(int x, int y)
  {
    return mSum[checkValue(x)][checkValue(y)];

//     int answer = 0;

//     if (0 == x) {
//       answer = checkValue(y);
//     }
//     else if (0 == y) {
//       answer = checkValue(x);
//     }
//     else {
//       int[] coeff1 = new int[mK];
//       int[] coeff2 = new int[mK];
      
//       int2Coeff(checkValue(x), coeff1);
//       int2Coeff(checkValue(y), coeff2);
      
//       for (int i = 0; i<mK; i++) {
// 	coeff1[i] = mGFp.sum(coeff1[i], coeff2[i]);
//       }

//       answer = coeff2Int(coeff1);
//     }
    
//     return answer;
  }
  
  /**
   * returns the difference of two field elements, expressed as ints
   * (included for completeness' sake, since with characteristic 2 addition
   * and subtraction are the same thing).
   * @param x int representing one field element
   * @param y int representing second field element
   * @return difference of x and y in field
   */
  public int difference(int x, int y)
  {
    return sum(x,this.negative(y));
  }

  /** 
   * returns the product of two field elements, expressed as ints
   * @param x int representing one field element
   * @param y int representing second field element
   * @return product of x and y in field
   * @throws IllegalArgumentException if x or y has a bit beyond the kth pos'n
   */
  public int product(int x, int y)
  {
    int answer = 0;

    if ((0 != x) && (0 != y)) {
      answer = mPow[checkExponent(mLog[checkValue(x)] + 
				  mLog[checkValue(y)])];
    }

    return answer;
  }

  /**
   * returns the quotient of two field elements, expressed as ints
   * @param x int representing one field element
   * @param y int representing second field element
   * @return quotient of x and y in field
   * @throws ArithmeticException if y is zero
   */
  public int quotient(int x, int y)
  throws ArithmeticException
  {
    int answer = 0;

    if (0 == y) {
      throw new ArithmeticException("Divide by zero");
    }

    if (0 != x) {
      answer = this.product(x, mPow[checkExponent(-mLog[checkValue(y)])]);
    }
    
    return answer;
  }

  /** 
   * returns the additive inverse of the passed field element (included for
   * completeness' sake, but with characteristic 2 each element is its own
   * negative).
   * @param x int with field element
   * @return additive inverse of x in field
   */
  public int negative(int x)
  {
    int answer = 0;

    if (0 != x) {
      int[] coeff = new int[mK];
      
      int2Coeff(checkValue(x), coeff);
      for (int i=0; i<mK; i++) {
	coeff[i] = mGFp.negative(coeff[i]);
      }
      answer = coeff2Int(coeff);
    }

    return answer;
  }

  /**
   * returns the multiplicative inverse of the passed field element
   * @param x int with field element
   * @return multiplicative inverse of x 
   * @throws ArithmeticException if x is zero
   */
  public int inverse(int x)
    throws ArithmeticException
  {
    if (0 == x) {
      throw new ArithmeticException("Divide by zero");
    }
    int pow = -mLog[checkValue(x)];
    int answer = mPow[checkExponent(pow)];

    return answer;
  }

  /**
   * returns a field element raised to a specified power
   * 
   * @param x field element
   * @param k power
   * @return x^k in this field
   * @throws ArithmeticException if x = 0 and k < 0
   */
  public int power(int x, int k) 
    throws ArithmeticException
  {
    if ((0 == x) && (k < 0)) {
      throw new ArithmeticException("Divide by zero");
    }

    int answer = 0;

    if (0 != x) {
      int j = mLog[checkValue(x)] * k;
      answer = mPow[checkExponent(j)];
    }
    
    return answer;
  }

  /**
   * returns a String describing the passed field entry in terms of a 
   * polynomial representative in GF(2)[x] / p(x) (where p(x) is the
   * primitive polynomial in use).
   * @param x int representing a field element
   * @return String with polynomial description
   */
  public String eltToString(int x)
  {
    return this.eltToString(x, (int)'x');
  }

  /**
   * returns a String describing the passed field entry in terms of a
   * sum of powers of a primitive element, where the passed character
   * indicates the primitive element
   * @param x int representing field element
   * @param fmt character code representing primitive element
   * @return String with description
   */
  public String eltToString(int x, int fmt) 
  {
    String answer = (String)null;

    if (0 == x) {
      answer = new String("0");
    }
    else {
      StringBuffer buf = new StringBuffer();
      int[] coeff = new int[mK];
      int i;
      boolean first = true;
      
      int2Coeff(x, coeff);
      for (i=mK-1; i>0; i--) {
	if (coeff[i] != 0) {
	  if (!first) {
	    buf.append(" + ");
	  }
	  else {
	    first = false;
	  }
	  if (coeff[i] > 1) {
	    buf.append(coeff[i]);
	  }
	  buf.append((char)fmt);
	  if (i > 1) {
	    buf.append("^");
	    buf.append(i);
	  }
	}
      }
      
      if (coeff[0] != 0) {
	if (!first) {
	  buf.append(" + ");
	}
	buf.append(coeff[0]);
      }
      answer = buf.toString();
    }
      
    return answer;
  }

  /**
   * returns the integer representing a field element given a String
   * representation of the element in the form returned by elt2String
   *
   * @param s String with representation of element
   * @return integer representation of element
   * @throws IllegalArgumentException if s is not in correct form
   */
  public int stringToElt(String s)
    throws IllegalArgumentException
  {
    return this.stringToElt(s, (int)'x');
  }

  /**
   * returns the integer representing a field element given a String
   * representation of the element in the form returned by elt2String with
   * an integer representing the character code to use to represent the 
   * equivalence class of x
   *
   * @param s String with representation of element
   * @param fmt character code of character to use in place of x
   * @return integer representation of element
   * @throws IllegalArgumentException if s is not in correct form
   */
  public int stringToElt(String s, int fmt)
    throws IllegalArgumentException
  {
    String token;
    int ans = 0;
    int exp = 0;
    char c = (char)fmt;
    int[] coeff = new int[mK];
    int i;

    for (i=0; i<mK; i++) {
      coeff[i] = 0;
    }

    for (StringTokenizer st = new StringTokenizer(s, "+"); 
	 st.hasMoreElements();) {
      token = st.nextElement().toString().trim();

      i = token.indexOf(fmt);
      
      try {
	if (i < 0) {
	  coeff[0] = Integer.valueOf(token).intValue();
	}
	else  {
	  if (i == token.length()-1) {
	    exp = 1;
	  }
	  else {
	    exp = Integer.valueOf(token.substring(i+2)).intValue();
	  }
	  if (0 == i) {
	    coeff[exp] = 1;
	  }
	  else {
	    coeff[exp] = Integer.valueOf(token.substring(0,i)).intValue();
	  }
	}
      }
      catch (NumberFormatException ex) {
	throw new IllegalArgumentException("Invalid term: " + token);
      }
    }

    ans = coeff2Int(coeff);
    return ans;
  }

  /**
   * returns a String with a representation of a field element either as 
   * a 0 or 1 or as a power of the primitive element in the form (k), where
   * the passed number represents w^k, where w is the primitive element
   * @param x integer representation of a field element
   * @return string with representation of x
   */
  public String eltToPowerString(int x)
  {
    String ans = (String)null;

    if (0 == x) {
      ans = new String("0");
    }
    else if (1 == x) {
      ans = new String("1");
    }
    else {
      ans = "(" + String.valueOf(mLog[checkValue(x)]) + ")";
    }

    return ans;
  }

  /**
   * returns an integer representing a field element, given its representation
   * in the form expressed in elt2PowerString; that is, either 0 or (j), where
   * the field element is w^j, where w generates the group of units of the
   * field. If j is less than 0 or greater than 2^k-2 (where the field is
   * of size 2^k), it is reduced modulo 2^k-1.
   * @param s String with representation of field element
   * @return integer representation of field element
   * @throws NumberFormatException if the string is not in the correct fmt
   */
  public int powerStringToElt(String s)
    throws NumberFormatException
  {
    int answer = 0;
    int len = s.length();
    int pow;

    if (s.equals("0")) {
      answer = 0;
    }
    else if (s.equals("1")) {
      answer = 1;
    }
    else {
      if ((s.charAt(0) != '(') || (s.charAt(len-1) != ')')) {
	throw new NumberFormatException("Invalid number format: " + s);
      }
      pow = Integer.valueOf(s.substring(1, len-1)).intValue();
      answer = mPow[checkExponent(pow)];
    }

    return answer;
  }

  //
  // private sub initializes lookup arrays
  //
  private void initArrays()
  {
    PrimPolyCalculator ppc = new PrimPolyCalculator();
    mPrimPoly = ppc.findPrimPoly(mP, mK);
    mFieldSize = (int)Math.pow((double)mP, (double)mK);

    mLog = new int[mFieldSize];
    mPow = new int[mFieldSize];

    mGFp = new GFpCalculator(mP);

    int i,j,k;
    int[] coeff = new int[mK];
    int[] coeff1 = new int[mK];
    int[] coeff2 = new int[mK];
    GFpPoly pow = new GFpPoly(mGFp), rem, x = GFpPoly.xPower(mGFp, 1);

    pow.add(1);
    k = 0;
    do {
      //
      // get integer from coefficient array and update log array
      //
      for (i=0; i<mK; i++) {
	coeff[i] = pow.getCoefficient(i);
      }
      j = coeff2Int(coeff);
      mLog[j] = k;
      mPow[k] = j;

      ++k;

      //
      // multiply pow by x, then divide by mPrimPoly and keep remainder
      //
      pow.multiply(x);
      pow = pow.divide(mPrimPoly);
    } while ((pow.getDegree() > 0) || 
	     ((pow.getDegree() == 0) && (pow.getCoefficient(0) > 1)));

    //
    // set up array of sums
    //
    mSum = new int[mFieldSize][mFieldSize];
    for (i=0; i<mFieldSize; i++) {
      for (j=0; j<mFieldSize; j++) {
	if (0 == i) {
	  mSum[i][j] = checkValue(j);
	}
	else if (0 == j) {
	  mSum[i][j] = checkValue(i);
	}
	else {
	  int2Coeff(checkValue(i), coeff1);
	  int2Coeff(checkValue(j), coeff2);
      
	  for (k=0; k<mK; k++) {
	    coeff1[k] = mGFp.sum(coeff1[k], coeff2[k]);
	  }

	  mSum[i][j] = coeff2Int(coeff1);
	}

      }
    }
  }

  //
  // private function converts an integer to a coefficient array
  //
  private void int2Coeff(int x, int[] coeff)
  {
    int i, q=x, r;
    
    for (i=0; i<mK; i++) {
      r = q % mP;
      coeff[i] = r;
      q = (q-r) / mP;
    }
  }

  //
  // private function converts a coefficient array to an integer
  //
  private int coeff2Int(int[] coeff)
  {
    int i, answer=0, q=1;

    for (i=0; i<mK; i++) {
      answer += q * coeff[i];
      q *= mP;
    }

    return answer;
  }

  //
  // private method converts passed value to a valid representation 
  // of a field element
  //
  private int checkValue(int b)
  {
    int answer = b % mFieldSize;
    if (answer < 0) {
      answer += mFieldSize;
    }

    return answer;
  }

  //
  // private method converts passed exponent to one in the range [0, p^k-2]
  //
  private int checkExponent(int e)
  {
    int answer = e % (mFieldSize-1);
    if (answer < 0) {
      answer += (mFieldSize-1);
    }
    return answer;
  }
}
