package Oid;
import java.util.*;
import java.io.*;

/**
 * An instance of IncidenceMatrixCombiner takes two graph incidence matrices
 * and attempts to combine them by finding identical rows in each, and creating
 * a new incidence matrix with the non-identical rows.  If this cannot be done,
 * it looks for a 2-vertex reversal that will allow the incidence matrices
 * to be combined.  (See (ToDo) for details.)
 * 
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 23 Nov 2001
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class IncidenceMatrixCombiner 
{
  //
  // private data members
  //
  private int[][] mLIncMatrix;
  private int[]   mLRowNum;
  private int     mLRows;
  private int[][] mRIncMatrix;
  private int[]   mRRowNum;
  private int     mRRows;
  private int     mCols;
  private int[][] mLRevMatrix;
  private int[][] mRRevMatrix;
  private int[][] mAnswer;
  private int[]   mAnsRowNum;
  private boolean mCanCombine;
  private int     mLDelRow;
  private int     mRDelRow;
  private Vector  mLRevRows;        // Arrays (int[2]) of reversal rows
  private Vector  mLRevComponents;  // Arrays (int[mRowNumn]) of component ids
  private Vector  mLCompNumbers;    // Integers with numbers of components
  private int     mLRevCount;       // Total # of pairs/components for LHS
  private int[]   mLPairNumber;     // Index into mLRevRows for each pair/comp
  private int[]   mLCompNumber;     // Component number for each pair/comp
  private Vector  mRRevRows;        // Arrays (int[2]) of reversal rows
  private Vector  mRRevComponents;  // Arrays (int[mRowNumn]) of component ids
  private Vector  mRCompNumbers;    // Integers with numbers of components
  private int     mRRevCount;       // Total # of pairs/components for RHS
  private int[]   mRPairNumber;     // Index into mRRevRows for each pair/comp
  private int[]   mRCompNumber;     // Component number for each pair/comp
  private String  mExplanation;     // string explaining steps taken by object
  private String  LF;
  private int     mLen;
  private String  mVS;             // format string for chosen vertices

  public IncidenceMatrixCombiner()
  {
    LF = System.getProperty("line.separator");
  }

  /** 
   * sets the two incidence matrices for the combiner to use, and arrays
   * of their row numbers
   * @param incMatrix1 first incidence matrix
   * @param rowNum1 row numbers for first incidence matrix
   * @param incMatrix2 second incidence matrix
   * @param rowNum2 row numbers for second incidence matrix
   * @throws IllegalArgumentException thrown if array sizes don't match up
   */
  public void setIncidenceMatrices(int[][] incMatrix1,
				   int[] rowNum1,
				   int[][] incMatrix2,
				   int[] rowNum2)
    throws IllegalArgumentException
  {
    mLRows = incMatrix1.length;
    if (mLRows != rowNum1.length) {
      throw new IllegalArgumentException("First incidence matrix size " +
					 "does not match row numbers");
    }
    mRRows = incMatrix2.length;
    if (mRRows != rowNum2.length) {
      throw new IllegalArgumentException("Second incidence matrix size " +
					 "does not match row numbers");
    }
    
    mCols = incMatrix1[0].length;
    if (mCols != incMatrix2[0].length) {
      throw new IllegalArgumentException("Incidence matrices do not have " +
					 "the same number of columns");
    }
    
    // 
    // make local copy of arrays
    //
    mLIncMatrix = new int[mLRows][mCols];
    mLRowNum = new int[mLRows];

    mRIncMatrix = new int[mRRows][mCols];
    mRRowNum = new int[mRRows];
    
    int i,j;
    for (i=0; i<mLRows; i++) {
      mLRowNum[i] = rowNum1[i];
      for (j=0; j<mCols; j++) {
	mLIncMatrix[i][j] = incMatrix1[i][j];
      }
    }

    for (i=0; i<mRRows; i++) {
      mRRowNum[i] = rowNum2[i];
      for (j=0; j<mCols; j++) {
	mRIncMatrix[i][j] = incMatrix2[i][j];
      }
    }
    
    // 
    // set up integer formatter -- how many places are needed for the
    // biggest row number?
    //
    i = 0;
    for (j=0; j<mLRows; j++) {
      if (mLRowNum[j] > i) {
	i = mLRowNum[j];
      }
    }

    for (j=0; j<mRRows; j++) {
      if (mRRowNum[j] > i) {
	i = mRRowNum[j];
      }
    }

    j = 0;
    do {
      j++;
      i /= 10;
    } while (i > 0);
    mLen = j+1;

    mVS = new String();
    for (i=0; i<j; i++) {
      mVS += " ";
    }
    mVS += "x";

    mExplanation = new String();
    mCanCombine = combineMatrices();
  }

  /**
   * returns a string with an explanation of the steps taken to combine
   * the incidence matrices
   * @return String value wiht explanation
   */
  public String getExplanation()
  {
    return new String(mExplanation);
  }

  /** 
   * returns a boolean value indicating whether the object was able to 
   * combine the two incidence matrices
   * @return true if matrices could be combined, otherwise false
   */
  public boolean canCombineMatrices()
  {
    return mCanCombine;
  }

  /**
   * returns the combined incidence matrix, if the two matrices could be
   * combined
   * @return 2-dimensional array of integers containing combined matrix
   */
  public int[][] getCombinedMatrix()
  {
    int[][] answer = (int[][])null;
    if (mCanCombine) {
      answer = mAnswer;
    }
    return answer;
  }

  /**
   * returns the row number array for the combined matrix, if the two matrices
   * could be combined
   * @return 1-dimensional array of integers containing row numbers
   */
  public int[] getCombinedRowNumbers()
  {
    int[] answer = (int[])null;
    if (mCanCombine) {
      answer = mAnsRowNum;
    }
    return answer;
  }

  //
  // private method attempts to combine matrices, and returns boolean value
  // indicating whether it succeeded
  //
  private boolean combineMatrices()
  {
    boolean answer = false;
    boolean finished = false;
    SubsetEnumeration seLeft, seRight;
    int kLeft, kRight;
    Subset subLeft, subRight;

    // determine and output reversals on both sides
    setupReversals();
    outputReversals();
    
    // allocate space for the reversals on both sides
    mLRevMatrix = new int[mLRows][mCols];
    mRRevMatrix = new int[mRRows][mCols];

    try {
      if ((mLRevCount == 0) && (mRRevCount == 0)) {
	//
	// if there are no reversals on the left or the right, just check
	// the matrices for a common row
	//
	if (matricesHaveCommonRow(mLIncMatrix, mRIncMatrix)) {
	  //
	  // found a common row -- generate output
	  //
	  mExplanation += "Common row found: Left matrix row  " +
	    String.valueOf(mLRowNum[mLDelRow]) + LF;
	  mExplanation += "                  Right matrix row " +
	    String.valueOf(mRRowNum[mRDelRow]) + LF;
	  //
	  // combine matrices, make sure result is an incidence matrix
	  //
	  combineMatricesWithoutDeletedRows(mLIncMatrix, mRIncMatrix);
	  mExplanation += "Combined matrix:" + LF;
	  addMatrixToExplanation(mAnswer, mAnsRowNum);
	  if (!isIncidenceMatrix(mAnswer)) {
	    throw new IllegalStateException("Combined matrix is not " +
					    "an incidence matrix");
	  }
	  finished = true;
	  answer = true;
	}
      }
      else if (mLRevCount == 0) {
	//
	// if there are no reversals on the left but there are reversals
	// on the right, loop on the right only
	// 
	// 
	// loop through possible numbers of reversals on each side
	//
	for (kRight=0; (kRight <= mRRevCount) && !finished; kRight++) {
	  //
	  // loop through subsets of given sizes
	  //
	  for (seRight = new SubsetEnumeration(mRRevCount, kRight);
	       seRight.hasMoreElements() && !finished;) {
	    subRight = (Subset)seRight.nextElement();
	    if (subRight.numElements() > 0) {
	      mExplanation += "Checking new right-hand reversal " +
		"combination..." + LF;
	    }
	    else {
	      mExplanation += "Checking matrices for common row..." + LF;
	    }
	    //
	    // apply reversals indicated by subsets
	    //
	    applyRightReversals(subRight);
	    if (matricesHaveCommonRow(mLIncMatrix, mRRevMatrix)) {
	      //
	      // found a common row -- generate output
	      //
	      mExplanation += "Common row found: Left matrix row  " +
		String.valueOf(mLRowNum[mLDelRow]) + LF;
	      mExplanation += "                  Right matrix row " +
		String.valueOf(mRRowNum[mRDelRow]) + LF;
	      //
	      // combine matrices, make sure result is an incidence matrix
	      //
	      combineMatricesWithoutDeletedRows(mLIncMatrix, mRRevMatrix);
	      mExplanation += "Combined matrix:" + LF;
	      addMatrixToExplanation(mAnswer, mAnsRowNum);
	      if (!isIncidenceMatrix(mAnswer)) {
		throw new IllegalStateException("Combined matrix is not " +
						"an incidence matrix");
	      }
	      finished = true;
	      answer = true;
	      break;
	    }
	  }
	}
      }
      else if (mRRevCount == 0) {
	// 
	// if there are reversals on the left, but not on the right, loop
	// on the left only
	//
	// 
	// loop through possible numbers of reversals on each side
	//
	for (kLeft=0; (kLeft <= mLRevCount) && !finished; kLeft++) {
	  //
	  // loop through subsets of given sizes
	  //
	  for (seLeft = new SubsetEnumeration(mLRevCount, kLeft);
	       seLeft.hasMoreElements() && !finished;) {
	    subLeft = (Subset)seLeft.nextElement();
	    if (subLeft.numElements() > 0) {
	      mExplanation += "Checking new left-hand reversal " +
		"combination..." + LF;
	    }
	    else {
	      mExplanation += "Checking matrices for common row..." + LF;
	    }
	    //
	    // apply reversals indicated by subsets
	    //
	    applyLeftReversals(subLeft);
	    if (matricesHaveCommonRow(mLRevMatrix, mRIncMatrix)) {
	      //
	      // found a common row -- generate output
	      //
	      mExplanation += "Common row found: Left matrix row  " +
		String.valueOf(mLRowNum[mLDelRow]) + LF;
	      mExplanation += "                  Right matrix row " +
		String.valueOf(mRRowNum[mRDelRow]) + LF;
	      //
	      // combine matrices, make sure result is an incidence matrix
	      //
	      combineMatricesWithoutDeletedRows(mLRevMatrix, mRIncMatrix);
	      mExplanation += "Combined matrix:" + LF;
	      addMatrixToExplanation(mAnswer, mAnsRowNum);
	      if (!isIncidenceMatrix(mAnswer)) {
		throw new IllegalStateException("Combined matrix is not " +
						"an incidence matrix");
	      }
	      finished = true;
	      answer = true;
	      break;
	    }
	  }
	}
      }
      else {
	// 
	// loop through possible numbers of reversals on each side
	//
	for (kLeft=0; (kLeft <= mLRevCount) && !finished; kLeft++) {
	  for (kRight=0; (kRight <= mRRevCount) && !finished; kRight++) {
	    //
	    // loop through subsets of given sizes
	    //
	    for (seLeft = new SubsetEnumeration(mLRevCount, kLeft);
		 seLeft.hasMoreElements() && !finished;) {
	      subLeft = (Subset)seLeft.nextElement();
	      for (seRight = new SubsetEnumeration(mRRevCount, kRight);
		   seRight.hasMoreElements() && !finished;) {
		subRight = (Subset)seRight.nextElement();
		if ((subLeft.numElements() == 0) && 
		    (subRight.numElements() == 0)) {
		  mExplanation += "Checking matrices for common row..." + LF;
		}
		else if (subLeft.numElements() == 0) {
		  mExplanation += "Checking left-hand matrix against " +
		    "reversal combination applied to right-hand matrix..." + LF;
		}
		else if (subRight.numElements() == 0) {
		  mExplanation += "Checking right-hand matrix against " +
		    "reversal combination applied to left-hand matrix..." + LF;
		}
		else {
		  mExplanation += "Checking right-hand left-hand reversal " +
		    "combination..." + LF;
		}
		//
		// apply reversals indicated by subsets
		//
		applyReversals(subLeft, subRight);
		if (matricesHaveCommonRow(mLRevMatrix, mRRevMatrix)) {
		  //
		  // found a common row -- generate output
		  //
		  mExplanation += "Common row found: Left matrix row  " +
		    String.valueOf(mLRowNum[mLDelRow]) + LF;
		  mExplanation += "                  Right matrix row " +
		    String.valueOf(mRRowNum[mRDelRow]) + LF;
		  //
		  // combine matrices, make sure result is an incidence matrix
		  //
		  combineMatricesWithoutDeletedRows(mLRevMatrix, mRRevMatrix);
		  mExplanation += "Combined matrix:" + LF;
		  addMatrixToExplanation(mAnswer, mAnsRowNum);
		  if (!isIncidenceMatrix(mAnswer)) {
		    throw new IllegalStateException("Combined matrix is not " +
						    "an incidence matrix");
		  }
		  finished = true;
		  answer = true;
		  break;
		}
	      }
	    }
	  }
	}
      }
    }
    catch (Exception e) {
      // this "can't happen"
      throw new InternalError("Internal error in combineMatrices");
    }

    if (!answer) {
      mExplanation += "Unable to combine matrices" + LF;
    }

    return answer;
  }

  //
  // private method determines reversals for both sides, and also sets
  // up index arrays
  //
  private void setupReversals()
  {
    Iterator p;
    int[] revRow;
    int[] revComp;
    int i,j,k,n;
    // 
    // determine reversals for left-hand side
    //
    mLRevRows = new Vector();
    mLRevComponents = new Vector();
    mLCompNumbers = new Vector();
    determineReversals(mLIncMatrix, mLRevRows, mLRevComponents, mLCompNumbers);

    // 
    // set up indices for reversals on left-hand side
    //
    mLRevCount = 0;
    for (p = mLCompNumbers.iterator(); p.hasNext(); ) {
      mLRevCount += ((Integer)p.next()).intValue();
    }
    mLPairNumber = new int[mLRevCount];
    mLCompNumber = new int[mLRevCount];
    i = 0;
    j = 0;
    for (p = mLCompNumbers.iterator(); p.hasNext(); ) {
      n = ((Integer)p.next()).intValue();
      for (k=0; k<n; k++) {
	mLPairNumber[i] = j;
	mLCompNumber[i] = k;
	i++;
      }
      j++;
    }

    // 
    // determine reversals for right-hand side
    //
    mRRevRows = new Vector();
    mRRevComponents = new Vector();
    mRCompNumbers = new Vector();
    determineReversals(mRIncMatrix, mRRevRows, mRRevComponents, mRCompNumbers);

    // 
    // set up indices for reversals on right-hand side
    //
    mRRevCount = 0;
    for (p = mRCompNumbers.iterator(); p.hasNext(); ) {
      mRRevCount += ((Integer)p.next()).intValue();
    }
    mRPairNumber = new int[mRRevCount];
    mRCompNumber = new int[mRRevCount];
    i = 0;
    j = 0;
    for (p = mRCompNumbers.iterator(); p.hasNext(); ) {
      n = ((Integer)p.next()).intValue();
      for (k=0; k<n; k++) {
	mRPairNumber[i] = j;
	mRCompNumber[i] = k;
	i++;
      }
      j++;
    }
  }

  //
  // private method adds a list of the reversals on both sides to the
  // explanation
  //
  private void outputReversals()
  {
    Iterator pRow, pComp;
    int i,n;
    int[] revRow;
    int[] revComp;
    //
    // left-hand reversals
    //
    if (mLRevCount == 0) {
      mExplanation += "No reversals found for left-hand matrix" + LF;
    }
    else {
      mExplanation += "Reversals and components found for left-hand matrix:" + LF;
      mExplanation += "Rows";
      for (i=0; i<2*mLen; i++) {
	mExplanation += " ";
      }
      for (i=0; i<mLRows; i++) {
	mExplanation += TextAlignment.rightAlign(String.valueOf(mLRowNum[i]), mLen);
      }
      mExplanation += LF;
			    
      pRow = mLRevRows.iterator();
      pComp = mLRevComponents.iterator();
      for (; pRow.hasNext(); ) {
	revRow = (int[])pRow.next();
	revComp = (int[])pComp.next();
	mExplanation += "( " + TextAlignment.rightAlign(String.valueOf(mLRowNum[revRow[0]]), mLen);
	mExplanation += TextAlignment.rightAlign(String.valueOf(mLRowNum[revRow[1]]), mLen) + " )";
	for (i=0; i<mLRows; i++) {
	  if (revComp[i] < 0) {
	    mExplanation += mVS;
	  }
	  else {
	    mExplanation += TextAlignment.rightAlign(String.valueOf(revComp[i]), mLen);
	  }
	}
	mExplanation += LF;
      }
    }
    //
    // right-hand reversals
    //
    if (mRRevCount == 0) {
      mExplanation += "No reversals found for right-hand matrix" + LF;
    }
    else {
      mExplanation +="Reversals and components found for right-hand matrix:" + LF;
      mExplanation += "Rows";
      for (i=0; i<2*mLen; i++) {
	mExplanation += " ";
      }
      for (i=0; i<mRRows; i++) {
	mExplanation += TextAlignment.rightAlign(String.valueOf(mRRowNum[i]), mLen);
      }
      mExplanation += LF;
			    
      pRow = mRRevRows.iterator();
      pComp = mRRevComponents.iterator();
      for (; pRow.hasNext(); ) {
	revRow = (int[])pRow.next();
	revComp = (int[])pComp.next();
	mExplanation += "( " + TextAlignment.rightAlign(String.valueOf(mRRowNum[revRow[0]]), mLen);
	mExplanation += TextAlignment.rightAlign(String.valueOf(mRRowNum[revRow[1]]), mLen) + " )";
	for (i=0; i<mRRows; i++) {
	  if (revComp[i] < 0) {
	    mExplanation += mVS;
	  }
	  else {
	    mExplanation += TextAlignment.rightAlign(String.valueOf(revComp[i]), mLen);
	  }
	}
	mExplanation += LF;
      }
    }
  }

  //
  // private method applies the reversals whose indices are set in the
  // passed Subsets to the two sides, placing the results in mLRevMatrix
  // and mRRevMatrix
  //
  private void applyReversals(Subset left, Subset right)
  {
    applyLeftReversals(left);
    applyRightReversals(right);
  }

  // 
  // private method applies the reversals whose indices are set in the
  // passed Subset to the left side, placing the results in mLRevMatrix
  //
  private void applyLeftReversals(Subset revs)
  {
    int i;
    int[] revRow;
    int[] revComp;

    copyMatrix(mLIncMatrix, mLRevMatrix);
    try {
      for (i=0; i<mLRevCount; i++) {
	if (revs.get(i)) {
	  revRow = (int[])mLRevRows.get(mLPairNumber[i]);
	  revComp = (int[])mLRevComponents.get(mLPairNumber[i]);
	  mExplanation += "Applying reversal to rows " + 
	    String.valueOf(mLRowNum[revRow[0]]) + " and " +
	    String.valueOf(mLRowNum[revRow[1]]) + " component " + 
	    String.valueOf(mLCompNumber[i]) + " of left-hand matrix..." + LF;
	  applyReversal(mLRevMatrix, revRow, mLCompNumber[i], revComp); 
	}
      }
    }
    catch (Exception e) {
      // this "can't happen"
      throw new InternalError("Internal error applying left-hand reversals");
    }
  }

  // 
  // private method applies the reversals whose indices are set in the
  // passed Subset to the right side, placing the results in mRRevMatrix
  //
  private void applyRightReversals(Subset revs)
  {
    int i;
    int[] revRow;
    int[] revComp;

    copyMatrix(mRIncMatrix, mRRevMatrix);
    try {
      for (i=0; i<mRRevCount; i++) {
	if (revs.get(i)) {
	  revRow = (int[])mRRevRows.get(mRPairNumber[i]);
	  revComp = (int[])mRRevComponents.get(mRPairNumber[i]);
	  mExplanation += "Applying reversal to rows " + 
	    String.valueOf(mRRowNum[revRow[0]]) + " and " +
	    String.valueOf(mRRowNum[revRow[1]]) + " component " + 
	    String.valueOf(mRCompNumber[i]) + " or right-hand matrix..." + LF;
	  applyReversal(mRRevMatrix, revRow, mRCompNumber[i], revComp);
	}
      }
    }
    catch (Exception e) {
      // this "can't happen"
      throw new InternalError("Internal error applying right-hand reversals");
    }
  }

  //
  // private method adds current matrix to the explanation string
  //
  private void addMatrixToExplanation(int[][] matrix, int[] rowNum)
  {
    int i,j,m,n;

    m = matrix.length;
    n = matrix[0].length;
    
    // add rows of matrix, preceded by row labels
    for (i=0; i<m; i++) {
      mExplanation += TextAlignment.rightAlign(String.valueOf(rowNum[i]), mLen);
      for (j=0; j<n; j++) {
	mExplanation += TextAlignment.rightAlign(String.valueOf(matrix[i][j]), mLen);
      }
      mExplanation += LF;
    }
  }
 
  //
  // private method copies source matrix to destination
  //
  private void copyMatrix(int[][] source, int[][] destination)
  {
    int i, j, m = source.length, n = source[0].length;

    for (i=0; i<m; i++) {
      for (j=0; j<n; j++) {
	destination[i][j] = source[i][j];
      }
    }
  }

  // 
  // private method checks to see if matrices have a common row, and sets
  // mLDelRow and mRDelRow if they do -- matrices must be mLRows x mCols and
  // mRRows x mCols
  //
  private boolean matricesHaveCommonRow(int[][] m1, int[][] m2)
  {
    boolean answer = false;
    int i,j;

    for (i=0; (i<mLRows) && !answer; i++) {
      for (j=0; j<mRRows; j++) {
	if (compareRows(m1, i, m2, j)) {
	  mLDelRow = i;
	  mRDelRow = j;
	  answer = true;
	  break;
	}
      }
    }

    return answer;
  }

  //
  // private method checks indicated rows of the two incidence matrices
  // and returns true if they are identical; false otherwise
  //
  private boolean compareRows(int[][] m1, int r1, int[][] m2, int r2)
  {
    boolean answer = true;
    int i;
    for (i=0; i<mCols; i++) {
      if (m1[r1][i] != m2[r2][i]) {
	answer = false;
	break;
      }
    }
    return answer;
  }

  //
  // private method combines matrices without row mLDelRow from the 
  // first matrix, and without row mRDelRow from the second -- the matrices
  // may not be mLIncMatrix and mRIncMatrix, but the row numbers are always
  // taken from mLRowNum and mRRowNum (matrices may be other copies with
  // reversals applied)
  //
  private void combineMatricesWithoutDeletedRows(int[][] m1, int[][]m2)
  {
    int ansRows = mLRows + mRRows - 2;
    int i,j,m;
    mAnswer = new int[ansRows][mCols];
    mAnsRowNum = new int[ansRows];
    
    // get rows from matrix 1
    m = 0;
    for (i=0; i<mLRows; i++) {
      if (i != mLDelRow) {
	mAnsRowNum[m] = mLRowNum[i];
	for (j=0; j<mCols; j++) {
	  mAnswer[m][j] = m1[i][j];
	}
	m++;
      }
    }

    // get rows from matrix 2
    for (i=0; i<mRRows; i++) {
      if (i != mRDelRow) {
	mAnsRowNum[m] = mRRowNum[i];
	for (j=0; j<mCols; j++) {
	  mAnswer[m][j] = m2[i][j];
	}
	m++;
      }
    }
  }

  //
  // private method determines whether passed matrix is an incidence matrix
  // (a column must have either 2 or no nonzero entries)
  //
  private boolean isIncidenceMatrix(int[][] matrix)
  {
    boolean answer = true;
    int i,j,m,n,c;
    m = matrix.length;
    n = matrix[0].length;

    for (i=0; i<n; i++) {
      c = 0;
      for (j=0; j<m; j++) {
	c += (matrix[j][i] != 0) ? 1 : 0;
      }
      if ((c != 0) && (c != 2)) {
	answer = false;
	break;
      }
    }

    return answer;
  }

  //
  // private method determines all 2-element subsets of the vertex set from
  // an incidence matrix whose deletion leaves more than one component,
  // placing the element numbers in mRevRows, the component identifications
  // in mRevComponents and the component counts in mCompNumbers
  //
  private void determineReversals(int[][] matrix,
				  Vector revRowVec,
				  Vector revComponentVec,
				  Vector revCompNumberVec)
  {
    int v1, v2, i, j, m=matrix.length, n=matrix[0].length;
    int[][] deletion;
    int[] component = new int[m-2];
    int[] revEdges;
    int[] revComponents;
    int ccount;


    for (v1 = 1; v1 < m; v1++) {
      for (v2 = 0; v2 < v1; v2++) {
	deletion = getDeletion(matrix, v1, v2);
	ccount = 0;
	if (deletion.length > 0) {
	  ccount = getComponents(deletion, component);
	}

	if (ccount > 1) {
	  // build matrices and add to Vectors
	  revEdges = new int[2];
	  revEdges[0] = v1;
	  revEdges[1] = v2;
	  revComponents = new int[m];
	  j = 0;
	  for (i=0; i<m; i++) {
	    if ((i == v1) || (i == v2)) {
	      revComponents[i] = -1;
	    }
	    else {
	      revComponents[i] = component[j];
	      j++;
	    }
	  }
	  revRowVec.add(revEdges);
	  revComponentVec.add(revComponents);
	  revCompNumberVec.add(new Integer(ccount));
	}
      }
    }
  }

  //
  // private method deletes two vertices from the passed incidence matrix
  // (along with any edges that meet them) and returns the resulting
  // matrix
  //
  private int[][] getDeletion(int[][] matrix, int v1, int v2) 
  {
    int i, j, i2, j2, m = matrix.length, n = matrix[0].length;
    int n2;
    int [][] answer;

    // determine number of columns that meet neither v1 nor v2
    n2 = 0;
    for (j=0; j<n; j++) {
      n2 += ((matrix[v1][j] == 0) && (matrix[v2][j] == 0)) ? 1 : 0;
    }
    
    answer = new int[m-2][n2];

    i2 = 0; 
    for (i=0; i<m; i++) {
      if ((i != v1) && (i != v2)) {
	j2 = 0;
	for (j=0; j<n; j++) {
	  if ((matrix[v1][j] == 0) && (matrix[v2][j] == 0)) {
	    answer[i2][j2] = matrix[i][j];
	    j2++;
	  }
	}
	i2++;
      }
    }
    
    return answer;
  }

  //
  // private method determines the connected components of the graph
  // with the passed incidence matrix, placing the component assignments
  // in the passed array component, and returning the number of components
  //
  private int getComponents(int[][] matrix, int[] component)
  {
    int curComp = 0;
    int rows = matrix.length, cols = matrix[0].length;
    Vector toBeAssigned = new Vector();
    Iterator p;
    int numAssigned = 0;
    boolean found;
    int i, j, k, m, n;

    for (i=0; i<rows; i++) {
      component[i] = -1;
    }

    while(numAssigned < rows) {
      // find an unassigned vertex
      found = false;
      for (i=0; i<rows; i++) {
	if (component[i] < 0) {
	  found = true;
	  break;
	}
      }
      if (!found) {
	// this "can't happen"
	throw new InternalError("Internal error in getComponents");
      }
      toBeAssigned = new Vector();
      toBeAssigned.add(new Integer(i));
      while (toBeAssigned.size() > 0) {
	i = ((Integer)toBeAssigned.iterator().next()).intValue();
	for (j=0; j<cols; j++) {
	  if (matrix[i][j] != 0) {
	    for (k=0; k<rows; k++) {
	      if ((matrix[k][j] != 0) &&
		  (component[k] < 0) &&
		  (k != i) &&
		  (!toBeAssigned.contains(new Integer(k)))) {
		toBeAssigned.add(new Integer(k));
	      }
	    }
	  }
	}
	p = toBeAssigned.iterator();
	component[((Integer)p.next()).intValue()] = curComp;
	numAssigned++;
	p.remove();
      }
      curComp++;
    }
    return curComp;
  }

  // 
  // private method applies a 2-vertex reversal to the passed incidence
  // matrix: reverseVertices is a 2-long array with the 2 vertices to be 
  // reversed, component is the connected component to be reversed, and
  // componentID is an array identifying the component for each vertex
  // other than the ones being reversed
  //
  private void applyReversal(int[][] matrix, int[] reverseVertices,
			     int component, int[] componentID)
  {
    int v1 = reverseVertices[0], v2 = reverseVertices[1];
    int i,j,m,n,rows = matrix.length, cols = matrix[0].length;
    boolean found;

    // examine each edge
    for (i=0; i<cols; i++) {
      // if edge meets v1...
      if (matrix[v1][i] != 0) {
	// ...and edge meets the correct component...
	found = false;
	for (j=0; j<rows; j++) {
	  if ((matrix[j][i] != 0) && (componentID[j] == component)) {
	    found = true;
	    break;
	  }
	}
	if (found) {
	  // make the edge meet v2 instead of v1
	  matrix[v1][i] = 0;
	  matrix[v2][i] = 1;
	}
      }
      // otherwise if edge meets v2...
      else if (matrix[v2][i] != 0) {
	// ...and edge meets the correct component...
	found = false;
	for (j=0; j<rows; j++) {
	  if ((matrix[j][i] != 0) && (componentID[j] == component)) {
	    found = true;
	    break;
	  }
	}
	if (found) {
	  // make the edge meet v1 instead of v2
	  matrix[v1][i] = 1;
	  matrix[v2][i] = 0;
	}
      }
    }
  }
}
