package Oid;
import java.util.*;

/**
 * An instance of <code>LinearSpace</code> stores an affine linear
 * geometry, as a collection of points and lines.  It also implements
 * the <code>Matroid</code> interface, and so supports matroid
 * operations.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 17 April 2002
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class LinearSpace implements Matroid, ComplexityProvider
{
  //
  // private data members
  //
  private int mSize;
  private int mRank; // always 1, 2 or 3
  private Vector mLines;
  private String mName;
  private String[] mLabel;
  private Matroid mMatroid;

  //
  // constructors
  //

  public LinearSpace()
  {
    mSize = 0;
    mRank = 0;
  }

  /**
   * constructs a LinearSpace instance given its size, rank and
   * an array of lines (subsets of the base set)
   * @param size Size of the space
   * @param rank Rank of the space
   * @param lines Vector of Subsets with lines
   * @throws IllegalArgumentException thrown if the collection of lines does
   * not define a linear space
   */
  public LinearSpace(int size, int rank, Collection lines)
  throws IllegalArgumentException
  {
    checkArguments(size, rank, lines);
    mName = new String("LinearSpace");
    setDefaultLabels();
    mMatroid = (Matroid)null;
  }

  /**
   * constructs a LinearSpace instance from the points and lines of a
   * Matroid
   * @param M Matroid to use to construct LinearSpace
   * @throws IllegalArgumentException if the rank of M is not valid 
   * (must be 1, 2 or 3), or its lines do not define a linear space
   */
  public LinearSpace(Matroid M)
  throws IllegalArgumentException
  {
    int size, rank;
    Vector lines = new Vector();

    // check matroid for loops and parallel elements
    checkMatroid(M);

    size = M.getSize();
    rank = M.getRank();

    // get values and list of lines; check
    DefaultFlatAlgorithm alg = new DefaultFlatAlgorithm();
    alg.setInstance(M);

    Vector flats = alg.getSubsetListVector();
    int i, n=flats.size();

    // get all flats except entire space
    for (i=2; i<n-1; i++) {
      lines.addAll((Collection)flats.elementAt(i));
    }
    
    checkArguments(size, rank, lines);
    mName = new String("LinearSpace");
    setDefaultLabels();
    mMatroid = M;
  }

  /** 
   * returns the Matroid used to create this linear space, or null if
   * the space was created with a set of lines
   *
   * @return Matroid instance, or null
   */
  public Matroid getMatroid()
  {
    return mMatroid;
  }

  /**
   * returns a Vector of Vectors of Integers describing the lines
   * for this linear space--the indices begin with 1, not 0
   *
   * @return Vector of Vectors of Integer instances
   */
  public Vector getLineIntVector()
  {
    Vector answer = new Vector();
    Vector line;
    Subset s;
    int i;

    for (Iterator pLine = mLines.iterator(); pLine.hasNext(); ) {
      s = (Subset)pLine.next();
      line = new Vector();
      for (i=0; i<mSize; i++) {
	if (s.get(i)) {
	  line.add(new Integer(i+1));
	}
      }
      answer.add(line);
    }

    return answer;
  }

  /** 
   * returns the array of lines for this linear space 
   *
   * @return array of <code>Subset</code> references
   */
  public Subset[] getLineArray()
  {
    return (Subset[])mLines.toArray();
  }

  /** 
   * returns a vector with the linear space's lines 
   *
   * @return <code>Vector</code> of <code>Subset</code> references
   */
  public Vector getLineVector()
  {
    return mLines;
  }

  /** 
   * returns the number of elements in matroid 
   *
   * @return number of elements
   */
  public int getSize()
  {
    return mSize;
  }

  /** 
   * returns the rank of the matroid 
   *
   * @return rank of matroid
   */
  public int getRank()
  {
    return mRank;
  }
  
  /** 
   * sets the matroid's name 
   *
   * @param newName name for matroid
   */
  public void setName(String newName)
  {
    mName = new String(newName);
  }

  /** 
   * returns the matroid's name 
   *
   * @return name of matroid
   */
  public String getName()
  {
    return new String(mName);
  }

  /** 
   * sets the label for one of the matroid's elements 
   *
   * @param element element number (0-based)
   * @param label label for element
   * @throws ArrayIndexOutOfBoundsException if <code>element</code> is out
   * of range
   */
  public void setLabel(int element, String label)
    throws ArrayIndexOutOfBoundsException
  {
    if ((element < 0) || (element >= mSize)) {
      throw new ArrayIndexOutOfBoundsException("Invalid element index");
    }
    mLabel[element] = new String(label);
  }

  /** 
   * returns the label for one of the matroid's elements 
   *
   * @param element element number (0-based)
   * @return label for element
   * @throws ArrayIndexOutOfBoundsException if <code>element</code> is out
   */
  public String getLabel(int element)
    throws ArrayIndexOutOfBoundsException
  {
    if ((element < 0) || (element >= mSize)) {
      throw new ArrayIndexOutOfBoundsException("Invalid element index");
    }
    return new String(mLabel[element]);
  }

  /** 
   * copies the internal state of this matroid to a string in the
   * specified format
   *
   * @param format format specifier
   * @return state of matroid in specified format
   * @throws IllegalArgumentException if <code>format</code> is invalid
   */
  public String getState(String format)
  throws IllegalArgumentException
  {
    String LF = System.getProperty("line.separator");
    StringBuffer state = new StringBuffer("LinearSpace ");
    Subset b;
    int i;
    if (format.toLowerCase().equals("text") || 
	format.toLowerCase().equals("txt")) {
      state.append(mName);
      state.append(LF);
      state.append("size ");
      state.append(mSize);
      state.append(LF);
      state.append("rank ");
      state.append(mRank);
      state.append(LF);
      state.append("labels" + LF);
      for (i=0; i<mSize; i++) {
	state.append(mLabel[i]);
	state.append(' ');
      }
      state.append(LF + "lines" + LF);
      for (Iterator p = mLines.iterator(); p.hasNext(); ) {
	state.append("line ");
	b = (Subset)p.next();
	for (i=0; i<mSize; i++) {
	  if (b.get(i)) {
	    state.append(mLabel[i]);
	    state.append(' ');
	  }
	}
	state.append(LF);
      }
    }
    else {
      throw new IllegalArgumentException("Invalid matroid format: " + 
					 format);
    }
    return state.toString();
  }

  /** 
   * sets state of this matroid from the specified string using 
   * the specified format
   *
   * @param state string with state of matroid
   * @param format format specifier for <code>state</code>
   * @throws IllegalArgumentException if <code>state</code> or 
   * <code>format</code> is invalid
   */
  public void setState(String state, String format)
  throws IllegalArgumentException
  {
    StringTokenizer stateTokens = new StringTokenizer(state);
    String word, word2;
    String newName = (String)null;
    int newSize=0, newRank=0;
    Vector newSets = new Vector();
    Vector newSet = (Vector)null;
    int labelCount = 0;
    String[] newLabel = null;
    int i = 0, j, k, m, n;
    boolean found;
    boolean readingSets=false;
    boolean gotSets=false;
    boolean readingLabels=false;
    boolean gotLabels=false;
    Vector newLines = new Vector();

    // loop through all the words in the string
    for (; stateTokens.hasMoreElements();) {
      word = (String)stateTokens.nextElement();
      // first two tokens must be "LinearSpace" and the name
      if (readingLabels) {
	newLabel[labelCount++] = new String(word);
	if (labelCount >= newSize) {
	  readingLabels = false;
	  gotLabels = true;
	}
      }
      // if inside the sets portion, check for new line, then add
      else if (readingSets) {
	if (word.toLowerCase().equals("line")) {
	  if (newSet instanceof Vector) {
	    newSets.add(newSet);
	  }
	  newSet = new Vector();
	}
	else {
	  newSet.add(word);
	}
      }
      // if beginning labels, make sure we know size and set up array
      else if (word.toLowerCase().equals("labels")) {
	if (newSize == 0) {
	  throw new IllegalArgumentException("size must be defined " + 
					     "before labels in linear space");
	}
	else {
	  readingLabels = true;
	  newLabel = new String[newSize];
	}
      }
      // make sure first word is "LinearSpace", not case-sensitive
      else if ((i == 0) && (!word.toLowerCase().equals("linearspace"))) {
	throw new IllegalArgumentException("Not a LinearSpace: " + word);
      }
      // second word is the space's name
      else if (i == 1) {
	newName = new String(word);
      }
      // beginning of basis listings
      else if (word.toLowerCase().equals("lines")) {
	if ((newSize == 0) || (newRank == 0)) {
	  throw new IllegalArgumentException("lines must be last");
	}
	else {
	  if (!(gotLabels)) {
	    for (j=0; j<mSize; j++) {
	      newLabel[j] = String.valueOf(j+1);
	    }
	    gotLabels = true;
	  }
	  readingSets = true;
	}
      }
      // read the rank
      else if (word.toLowerCase().equals("rank") && 
	       stateTokens.hasMoreElements()) {
	word2 = (String)stateTokens.nextElement();
	try {
	  newRank = Integer.valueOf(word2).intValue();
	}
	catch (NumberFormatException e) {
	  newRank = 0;
	}
	if (newRank <= 0) {
	  throw new IllegalArgumentException("invalid rank " + word2);
	}
      }
      // read the size
      else if (word.toLowerCase().equals("size") && 
	       stateTokens.hasMoreElements()) {
	word2 = (String)stateTokens.nextElement();
	try {
	  newSize = Integer.valueOf(word2).intValue();
	}
	catch (NumberFormatException e) {
	  newSize = 0;
	}
	if (newSize <= 0) {
	  throw new IllegalArgumentException("invalid size " + word2);
	}
      }
      ++i;
    }
    
    if ((newSet instanceof Vector) && (newSet.size() > 0)) {
      newSets.add(newSet);
    }

    // define lines -- go through words in definition, search
    // for labels, and define the subsets
    Iterator pLines;
    Iterator pLine;
    Vector line;
    Subset s;
    String eltLabel;
    mSize = newSize;
    mRank = newRank;
    for (pLines = newSets.iterator(); pLines.hasNext(); ) {
      line = (Vector)pLines.next();
      s = new Subset(mSize);
      for (pLine = line.iterator(); pLine.hasNext(); ) {
	eltLabel = (String)pLine.next();
	found = false;
	for (k=0; k<mSize; k++) {
	  if (newLabel[k].equals(eltLabel)) {
	    found = true;
	    break;
	  }
	}
	if (!found) {
	  throw new IllegalArgumentException("Invalid element: " + 
					     eltLabel);
	}
	s.set(k);
      }
      newLines.add(s);
    }

    // copy all the new values into the private data members
    checkArguments(newSize, newRank, newLines);
    if (newName != null) {
      mName = newName;
    }
    else {
      mName = new String("LinearSpace");
    }
    mLabel = new String[newSize];
    for (i=0; i<mSize; i++) {
      mLabel[i] = newLabel[i];
    }
    mLines = newLines;
  }

  /** 
   * determines the rank of a subset of the matroid's elements 
   *
   * @param s subset of matroid base set
   * @return rank of <code>s</code> in this matroid
   */
  public int subsetRank(Subset s)
  {
    int answer=0;
    int n = s.numElements();
    Subset line;

    if (s.getSize() == mSize) {
      switch (n) {
      case 0:
	answer = 0;
	break;
      case 1:
	answer = 1;
	break;
      case 2:
	answer = 2;
	break;
      default:
	// answer is 2 if set is contained in a line, 3 otherwise
	answer = 3;
	for (Iterator p = mLines.iterator(); p.hasNext(); ) {
	  line = (Subset)p.next();
	  if (line.subsetContains(s)) {
	    answer = 2;
	    break;
	  }
	}
      }
    }

    return answer;
  }

  /** 
   * determines whether a subset of the matroid is a circuit 
   *
   * @param s subset of matroid ground set
   * @return <code>true</code> if and only if <code>s</code> is a circuit
   */
  public boolean isCircuit(Subset s)
  {
    boolean answer = false;
    int n = s.numElements();
    int i;
    Subset s2;
    Iterator pLine;
    boolean found;

    if (s.getSize() == mSize) {
      switch (mRank) {
      case 1:
	answer = false;
	break;
      case 2:
	// in a rank 2 matroid with no parallel elements, circuits
	// are all 3-element sets
	answer = (s.numElements() == 3) ? true : false;
	break;
      case 3:
	// 3-element lines are circuits
	// 4-element sets with no 3 elements on a line are circuits
	switch (n) {
	case 0:
	case 1:
	case 2:
	  answer = false;
	  break;
	case 3:
	  answer = false;
	  for (pLine = mLines.iterator(); pLine.hasNext(); ) {
	    if (s.subsetEquals((Subset)pLine.next())) {
	      answer = true;
	      break;
	    }
	  }
	  break;
	case 4:
	  // see if any line contains a 3-element subset of s
	  answer = true;
	  s2 = new Subset(s);
	  found = false;
	  for (i=0; i<mSize; i++) {
	    if (s2.get(i)) {
	      s2.clear(i);
	      for (pLine = mLines.iterator(); pLine.hasNext(); ) {
		if (((Subset)pLine.next()).subsetContains(s2)) {
		  found = true;
		  break;
		}
	      }
	      s2.set(i);
	      if (found) {
		answer = false;
		break;
	      }
	    }
	  }
	  break;
	default:
	  answer = false;
	}
      }
    }

    return answer;
  }
  /** 
   * determines whether a subset of the matroid is a flat 
   *
   * @param s subset of matroid ground set
   * @return <code>true</code> if and only if <code>s</code> is a flat
   */
  public boolean isFlat(Subset s)
  {
    boolean answer = false;
    int n = s.numElements();

    if (s.getSize() == mSize) {
      switch (mRank) {
      case 1:
	answer = (n > 0) ? true : false;
	break;
      case 2:
	answer = (n == mSize) ? true : false;
	break;
      case 3:
	if (0 == n) {
	  answer = false;
	}
	else if ((1 == n) || (mSize == n)) {
	  answer = true;
	}
	else {
	  // search lines
	  for (Iterator pLine = mLines.iterator(); pLine.hasNext(); ) {
	    if (((Subset)pLine.next()).subsetEquals(s)) {
	      answer = true;
	      break;
	    }
	  }
	}
      }
    }

    return answer;
  }

  /** 
   * determines whether a subset of the matroid is a basis 
   *
   * @param s subset of matroid ground set
   * @return <code>true</code> if and only if <code>s</code> is a basis
   */
  public boolean isBasis(Subset s)
  {
    boolean answer = false;
    
    if ((s.getSize() == mSize) && (s.numElements() == mRank)) {
      answer = (subsetRank(s) == mRank);
    }

    return answer;
  }

  /** 
   * determines whether a subset of the matroid is independent 
   *
   * @param s subset of matroid ground set
   * @return <code>true</code> if and only if <code>s</code> is independent
   */
  public boolean isIndependent(Subset s)
  {
    boolean answer = false;
    int n = s.numElements();

    if ((s.getSize() == mSize) && (n <= mRank) && (n > 0)) {
      if ((1 == n) || (2 == n)) {
	answer = true;
      }
      else {
	answer = (subsetRank(s) == n);
      }
    }

    return answer;
  }

  /** 
   * determines whether a subset of the matroid is a hyperplane 
   *
   * @param s subset of matroid ground set
   * @return <code>true</code> if and only if <code>s</code> is a hyperplane
   */
  public boolean isHyperplane(Subset s)
  {
    boolean answer = false;
    
    if (s.getSize() == mSize) {
      for (Iterator pLine = mLines.iterator(); pLine.hasNext(); ) {
	if (((Subset)pLine.next()).subsetEquals(s)) {
	  answer = true;
	  break;
	}
      }
    }

    return answer;
  }

  /** 
   * determines whether an element is in the closure of a set 
   *
   * @param s subset of matroid ground set
   * @param e element index (0-based)
   * @return <code>true</code> if and only if <code>e</code> is in the closure
   * of <code>s</code> 
   */
  public boolean isInClosure(Subset s, int e)
  {
    boolean answer = false;
    int n = s.numElements();
    boolean found = false;
    Subset line=(Subset)null;

    if (s.getSize() == mSize) {
      switch (n) {
      case 0:
	answer = false;
	break;
      case 1:
	answer = s.get(e);
	break;
      default:
	// s must have rank of 2 or 3 (if |s| = 2, r(s) = 2)
	if ((2 == n) || (subsetRank(s) == 2)) {
	  // find a line containing s and see if e is on it
	  for (Iterator pLine = mLines.iterator(); pLine.hasNext(); ) {
	    line = (Subset)pLine.next();
	    if (line.subsetContains(s)) {
	      found = true;
	      break;
	    }
	  }
	  if (!found) {
	    throw new InternalError("Inconsistent lines in LinearSpace");
	  }
	  answer = line.get(e);
	}
	else {
	  // s must have rank 3 so its closure is the whole space
	  answer = true;
	}
      }
    }

    return answer;
  }

  /** 
   * determines the closure of a subset of the matroid 
   * 
   * @param s subset of matroid ground set
   * @return closure of <code>s</code>
   */
  public Subset closure(Subset s)
  {
    Subset answer = (Subset)null;
    int n = s.numElements();
    int i;
    Subset line = (Subset)null;
    boolean found = false;
    Iterator pLine;

    if (s.getSize() == mSize) {
      if (0 == n) {
	answer = new Subset(s);
      }
      else if (1 == n) {
	if (mRank > 1) {
	  answer = new Subset(s);
	}
	else {
	  answer = new Subset(mSize);
	  for (i=0; i<mSize; i++) {
	    answer.set(i);
	  }
	}
      }
      else {
	// closure is line containing s if there is one, whole set otherwise
	for (pLine = mLines.iterator(); pLine.hasNext();) {
	  line = (Subset)pLine.next();
	  if (line.subsetContains(s)) {
	    found = true;
	    answer = new Subset(line);
	    break;
	  }
	}

	if (!found) {
	  // s is not contained in a line--what is its rank?
	  if (this.subsetRank(s) < mRank) {
	    answer = new Subset(s);
	  }
	  else {
	    answer = new Subset(mSize);
	    for (i=0; i<mSize; i++) {
	      answer.set(i);
	    }
	  }
	}
      }
    }

    return answer;
  }

  //
  // ComplexityProvider interface methods
  //
  /** 
   * returns a value estimating the number of steps required to
   * execute the method with the passed name, or -1 if the method
   * cannot currently be executed
   *
   * @param methodName name of method to check
   * @return estimate of complexity of method
   */
  public long getComplexityEstimate(String methodName)
  {
    long answer = -1;

    if (methodName.equals("getLineArray") ||
	methodName.equals("getLineVector") ||
	methodName.equals("getSize") ||
	methodName.equals("getRank") ||
	methodName.equals("setName") ||
	methodName.equals("getName") ||
	methodName.equals("setLabel") ||
	methodName.equals("getLabel")) {
      answer = 1;
    }
    else if (methodName.equals("getState") ||
	     methodName.equals("setState") ||
	     methodName.equals("subsetRank") ||
	     methodName.equals("isCircuit") ||
	     methodName.equals("isFlat") ||
	     methodName.equals("isBasis") ||
	     methodName.equals("isIndependent") ||
	     methodName.equals("isHyperplane") ||
	     methodName.equals("isInClosure") ||
	     methodName.equals("closure")) {
      if (mLines instanceof Vector) {
	answer = mLines.size();
      }
    }
    return answer;
  }
  
  /**
   * returns a value estimating the number of steps required to
   * execute the method with the passed name and argument, or -1 if the 
   * method cannot be executed with the passed argument
   *
   * @param methodName name of method to check
   * @param argument argument to method
   * @return estimate of complexity of method
   */
  public long getComplexityEstimate(String methodName, Object argument)
  {
    return getComplexityEstimate(methodName);
  }

  /**
   * returns a value estimating the number of steps required to
   * execute the method with the passed name and collection of
   * arguments (which must be placed in correct order for iteration),
   * or -1 if the method cannot be executed with the passed arguments
   *
   * @param methodName name of method to check
   * @param arguments arguments to method
   * @return estimate of complexity of method
   */
  public long getComplexityEstimate(String methodName, Collection arguments)
  {
    return getComplexityEstimate(methodName);
  }


  //
  // private method checks whether an array of Subsets defines a linear
  // space of size size and rank rank, throws an IllegalArgumentException
  // if not, copies values into private data members if so
  //
  private void checkArguments(int size, int rank, Collection lines)
    throws IllegalArgumentException
  {
    Vector lineVect = new Vector(lines);

    // check size and rank
    if (size <= 0) {
      throw new IllegalArgumentException ("Invalid linear space size");
    }

    if ((rank < 1) || (rank > 3)) {
      throw new IllegalArgumentException ("Invalid linear space rank");
    }

    int n = lineVect.size();
    int i,j;

    // check line base set sizes and element counts
    for (i=0; i<n; i++) {
      if (((Subset)lineVect.elementAt(i)).getSize() != size) {
	throw new IllegalArgumentException("Inconsistent base set sizes");
      }

      j = ((Subset)lineVect.elementAt(i)).numElements();
      if (j<2) {
	throw new IllegalArgumentException("Line without at least two points");
      }
    }

    Subset line1, line2;

    // check line axiom: every two lineVect must intersect in exactly 
    //  one point 
	Subset intersection;
    for (i=0; i<n; i++) {
      line1 = (Subset)lineVect.elementAt(i);
      for (j=i+1; j<n; j++) {
	intersection = new Subset(line1);
	line2 = (Subset)lineVect.elementAt(j);
	intersection.and(line2);
	if (intersection.numElements() > 1) {
	  throw new IllegalArgumentException("Lines " + line1.toString(true)
					     + " and " + line2.toString(true)
					     + " intersect in more than"
					     + " one point");
	}
      }
    }

    // if we got this far, everything is okay
    mSize = size;
    mRank = rank;
    mLines = new Vector(lineVect);
  }

  // private method sets default labels
  private void setDefaultLabels()
  {
    int i;

    mLabel = new String[mSize];
    for (i=0; i<mSize; i++) {
      mLabel[i] = String.valueOf(i+1);
    }
  }

  // private method checks a matroid for loops and parallel 
// elements--if there are any, it throws an IllegalArgumentException
  private void checkMatroid(Matroid M)
  throws IllegalArgumentException
  {
    Subset s;
    int i,j,n=M.getSize();


    // loops - check all one-element sets for rank
    for (i=0; i<n; i++) {
      s = new Subset(n, i);

      
      if (M.subsetRank(s) < 1) {
	throw new IllegalArgumentException("Element " + M.getLabel(i)
					   + " of " + M.getName() 
					   + " is a loop");
      }	
    }

    // parallel elements - check all two-element subsets
    for (i=0; i<n; i++) {
      for (j=i+1; j<n; j++) {
	s = new Subset(n);
	s.set(i);
	s.set(j);

	if (M.subsetRank(s) < 2) {
	  throw new IllegalArgumentException("Elements " + M.getLabel(i)
					     + " and " + M.getLabel(j) 
					     + " in " + M.getName()
					     + " are parallel");
	}
      }
    }
  }
}

