package Oid;
import java.io.*;
import java.util.*;

/**
 * Reads a file of stored linear space instances, returning them using
 * the Enumeration interface.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 30 Sep 2003
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class LinearSpaceFileReader implements Enumeration {

  private boolean mHasMore;
  private LinearSpace mNextElt;
  private LineNumberReader mReader;
  private boolean mHasNames;
  private int mCount;

  /**
   * Constructs a reader using the name of a file containing linear space 
   * definitions, including names.
   *
   * @param filename String with name of file to read
   * @throws FileNotFoundException if file does not exist
   * @throws IOException if file cannot be opened
   */
  public LinearSpaceFileReader(String filename) 
    throws FileNotFoundException, IOException {

    initReader(filename, true);
  }

  /**
   * Constructs a reader using the indicated filename, and indicates whether
   * the file contains matroid names.
   *
   * @param filename name of file to read
   * @param hasNames true if file contains matroid names
   * @throws FileNotFoundException if file does not exist
   * @throws IOException if file cannot be opened
   */
  public LinearSpaceFileReader(String filename, boolean hasNames) 
    throws FileNotFoundException, IOException {
    
    initReader(filename, hasNames);
  }

  /**
   * Returns a boolean value indicating whether more linear spaces
   * exist to be read.
   *
   * @return true if there are more spaces
   */
  public boolean hasMoreElements() {
    return mHasMore;
  }

  /**
   * Returns the next linear space from the file.
   *
   * @return linear space
   * @throws IOException if an error occurs while reading the next linear space
   */
  public Object nextElement() 
    throws NoSuchElementException {
    Object answer = (Object)mNextElt;
    
    try {
      readNextElement();
    }
    catch (Exception ex) {
      throw new NoSuchElementException(ex.getMessage());
    }

    return answer;
  }

  //
  // initializes the reader
  //
  private void initReader(String filename, boolean hasNames) 
    throws IOException, FileNotFoundException {
    FileReader freader = new FileReader(filename);
    mReader = new LineNumberReader(freader);
    mHasNames = hasNames;
    mHasMore = true;
    readNextElement();
    mReader.setLineNumber(1);
    mCount = 0;
  }

  //
  // reads next linear space 
  //
  private void readNextElement() 
    throws IOException {

    String name = readName();

    try {
      if (!(name instanceof String) || (name.trim().length() == 0)) {
	mHasMore = false;
	mReader.close();
      }
      else {

	int size = readSize();
	int rank = readRank();
	int numLines = readNumLines();
	Vector lines = new Vector();

	for (int i=0; i<numLines; i++) {
	  lines.add(readSpaceLine(size));
	}
	mNextElt = new LinearSpace(size, rank, lines);
	mNextElt.setName(name);
      }
    }
    catch (Exception ex) {
      throw new IOException(ex.getMessage());
    }
  }

  // 
  // reads/generates next linear space name
  //
  private String readName() 
    throws IOException {
    
    String answer = (String)null;

    if (mHasNames) {
      answer = mReader.readLine();
    }
    else {
      answer = "A_" + String.valueOf(++mCount);
    }

    return answer;
  }

  // 
  // reads the next linear space's size
  // 
  private int readSize() 
    throws IOException {
    
    int answer = -1;

    try {
      answer = Integer.parseInt(mReader.readLine());
    }
    catch (NumberFormatException ex) {
      throw new IOException("Linear space size is not a number in line " +
			    String.valueOf(mReader.getLineNumber()));
    }

    if (answer <= 0) {
      throw new IOException("Linear space size is not positive in line " +
			    String.valueOf(mReader.getLineNumber()));
    }

    return answer;
  }

  // 
  // reads the next linear space's rank
  // 
  private int readRank() 
    throws IOException {
    
    int answer = -1;

    try {
      answer = Integer.parseInt(mReader.readLine());
    }
    catch (NumberFormatException ex) {
      throw new IOException("Linear space rank is not a number in line " +
			    String.valueOf(mReader.getLineNumber()));
    }

    if (answer <= 0) {
      throw new IOException("Linear space rank is not positive in line " +
			    String.valueOf(mReader.getLineNumber()));
    }

    return answer;
  }

  // 
  // reads the next linear space's number of lines
  // 
  private int readNumLines() 
    throws IOException {
    
    int answer = -1;

    try {
      answer = Integer.parseInt(mReader.readLine());
    }
    catch (NumberFormatException ex) {
      throw new IOException("Linear space number of lines " +
			    "is not a number in line " +
			    String.valueOf(mReader.getLineNumber()));
    }

    if (answer <= 0) {
      throw new IOException("Linear space number of lines " + 
			    "is not positive in line " +
			    String.valueOf(mReader.getLineNumber()));
    }

    return answer;
  }

  //
  // reads a subset indicating a line for the next linear space
  //
  private Subset readSpaceLine(int n) 
    throws IOException {
    
    Subset answer = (Subset)null;

    try {
      answer = new Subset(mReader.readLine(), n, false);
    }
    catch (Exception ex) {
      throw new IOException("Error reading subset in line " + 
			    String.valueOf(mReader.getLineNumber()) +
			    ": " + ex.getMessage());
    }

    return answer;
  }

}
