package Oid;
import java.util.*;

/**
 * An instance of PGFactory produces an array of
 * <code>FiniteFieldVector</code>s representing PG(n,q) where n is the
 * size of the vectors and q is the size of the field.
 * 
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 27 Oct 2001
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class PGFactory
{
  //
  // private data members
  //
  private int                 mFieldSize;
  private int                 mSize;
  private int                 mArraySize;
  private FiniteFieldVector[] mArray;
  private FiniteField         mField;
  private static final int    mDEFAULT_FIELDSIZE = 2;
  private static final int    mDEFAULT_SIZE = 10;

  /** 
   * creates an instance with default field size (2) and size (10) 
   */
  public PGFactory()
  {
    mFieldSize = mDEFAULT_FIELDSIZE;
    mSize = mDEFAULT_SIZE;
    try {
      mField = FiniteFieldFactory.createFiniteField(mFieldSize);
    }
    catch (IllegalArgumentException e) {
      // this "can't happen"
      throw new InternalError("Internal Error in PGFactory.PGFactory()");
    }
    generateSpace();
  }

  /** 
   * creates an instance with the passed field size and vector size 10
   *
   * @param q field size for vectors
   * @throws IllegalArgumentException if <code>q</code> isn't a prime power
   */
  public PGFactory(int q)
  throws IllegalArgumentException
  {
    mFieldSize = q;
    mSize = mDEFAULT_SIZE;
    mField = FiniteFieldFactory.createFiniteField(mFieldSize);
    generateSpace();
  }

  /** 
   * creates an instance using the passed FiniteField for the field size
   * and default size (10)
   *
   * @param field FiniteField to use for calculation
   */
  public PGFactory(FiniteField field)
  {
    mFieldSize = field.getFieldSize();
    mSize = mDEFAULT_SIZE;
    mField = field;
    generateSpace();
  }

  /** 
   * creates an instance using the passed field size and vector size
   *
   * @param size size of vectors in space to return
   * @param q field size for field
   * @throws IllegalArgumentException if <code>size</code> isn't positive or
   * <code>q</code> isn't a prime power
   */
  public PGFactory(int size, int q) 
    throws IllegalArgumentException
  {
    if (size <= 0) {
      throw new IllegalArgumentException("Invalid size for PGFactory");
    }
    mSize = size;
    mField = FiniteFieldFactory.createFiniteField(q);
    mFieldSize = q;
    generateSpace();
  }

  /** 
   * creates an instance using the passed size and FiniteField
   * (for the modulus)
   * @param size size of vectors in space to return
   * @param field FiniteField to use for calculation
   */
  public PGFactory(int size, FiniteField field)
  throws IllegalArgumentException
  {
    if (size <= 0) {
      throw new IllegalArgumentException("Invalid size for PGFactory");
    }
    mSize = size;
    mField = field;
    mFieldSize = field.getFieldSize();
    generateSpace();
  }

  /** 
   * returns an array of FiniteFieldVectors representing the projective space
   * @return array of (p^n - 1)/(p - 1) FiniteFieldVectors
   */
  public FiniteFieldVector[] getProjectiveSpace()
  {
    return mArray;
  }

  /**
   * returns the size of the projective space, (p^n - 1) / (p - 1)
   * @return size of projective space generated by this instance
   */
  public int getProjectiveSpaceSize() 
  {
    return mArraySize;
  }

  /**
   * returns a reference to the factory's FiniteField
   */
  public FiniteField getFiniteField() 
  {
    return mField;
  }

  // 
  // private method generates the space
  //
  private void generateSpace()
  {
    int e,i,j,k;
    int pPow = 1;
    FiniteFieldVector genVect;
    FiniteFieldVectorEnumeration enum;
 
    mArraySize = ((int)Math.pow(mFieldSize, mSize) - 1) / (mFieldSize - 1);
    mArray = new FiniteFieldVector[mArraySize];

    i = mSize;
    k = mSize-1;
    for (e = 0; e < mSize; e++) {
      if (e == 0) {
	mArray[k] = new FiniteFieldVector(mSize, mField);
	mArray[k].set(mSize-1, 1);
	k--;
      }
      else {
	// enumerate vectors
	enum = new FiniteFieldVectorEnumeration(e, mField);
	
	// first vector is all zeroes, with additional 1 becomes 
	// a basis vector
	genVect = (FiniteFieldVector)enum.nextElement();
	mArray[k] = new FiniteFieldVector(mSize, mField);
	mArray[k].set(mSize-1-e, 1);
	k--;

	// remaining vectors go into right-hand side
	for (; enum.hasMoreElements(); ) {
	  genVect = (FiniteFieldVector)enum.nextElement();
	  mArray[i] = new FiniteFieldVector(mSize, mField);
	  mArray[i].set(mSize-1-e, 1);
	  for (j = 0; j<e; j++) {
	    mArray[i].set(mSize-e+j, genVect.get(j));
	  }
	  i++;
	}
      }
    }
  }
}
