package Oid;
import java.io.*;
import java.util.*;

/**
 * An instance of PlanarReader reads a planar graph file in the format output
 * by the program plantri, and acts as an Enumeration implementation, returning
 * the incidence matrices of the graphs read.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class PlanarReader implements Enumeration
{
  private byte[] mFileBody;
  private int mFilePos;
  private int mBufPos;
  private static final int BUF_LEN = 6000000;

  public PlanarReader(String filename)
    throws NullPointerException, FileNotFoundException, 
	   IllegalArgumentException, IOException
  {
    File theFile = new File(filename);
    int fileLen = (int)theFile.length();
    FileReader reader = new FileReader(theFile);
    char[] fileBody = new char[fileLen];
    int i;

    mFileBody = new byte[fileLen];
    reader.read(fileBody, 0, fileLen);
    reader.close();

    for (i=0; i<fileLen; i++) {
      mFileBody[i] = (byte)fileBody[i];
    }

    if (!checkFmt()) {
      throw new IllegalArgumentException("Not a planar graph file");
    }
  }

  private boolean checkFmt()
  {
    StringBuffer hdr = new StringBuffer("");
    String test = new String(">>planar_code<<");
    int i, n = test.length();

    if (n > mFileBody.length) {
      return false;
    }

    for (i=0; i<n; i++) {
      hdr.append((char)mFileBody[i]);
    }
    mFilePos = n;

    return (hdr.toString().equals(test));
  }

  public boolean hasMoreElements()
  {
    return (mFilePos < mFileBody.length);
  }

  public Object nextElement()
    throws NoSuchElementException
  {
    if (!hasMoreElements()) {
      throw new NoSuchElementException();
    }

    int nV, nE, i, j, k, pos;

    nV = (int)mFileBody[mFilePos];
    pos = mFilePos;

    // read through the graph once to count the edges
    nE = 0;
    for (i=0; i<nV; i++) {
      j = (int)mFileBody[++pos];
      while (0 != j) {
	if (i < j-1) {
	  ++nE;
	}
	++pos;
	j = (int)mFileBody[pos];
      }
    }

    // now set up the array and read the values
    int[][] incMat = new int[nV][nE];
    for (i=0; i<nV; i++) {
      for (j=0; j<nE; j++) {
	incMat[i][j] = 0;
      }
    }

    pos = mFilePos;
    k = 0;
    for (i=0; i<nV; i++) {
      j = (int)mFileBody[++pos];
      while (0 != j) {
	if (i < j-1) {
	  incMat[i][k] = 1;
	  incMat[j-1][k] = 1;
	  ++k;
	}
	++pos;
	j = (int)mFileBody[pos];
      }
    }

    mFilePos = pos+1;

    return (Object)incMat;
  }
  
  public static void main(String[] arg)
    throws IOException
  {
    int[][] incMat;

    if (0 == arg.length) {
      System.out.println("Usage: java PlanarReader <filename>");
    }
    else {

      System.out.println("reading graphs");
      System.out.flush();

      for (PlanarReader rdr = new PlanarReader(arg[0]);
	   rdr.hasMoreElements();) {
	incMat = (int [][])rdr.nextElement();
	outputIncMat(incMat);
      }
    }
  }

  public static void outputIncMat(int[][] incMat)
  {
    int i, j, m, n;
    m = incMat.length;
    n = incMat[0].length;

    for (i=0; i<m; i++) {
      for (j=0; j<n; j++) {
	System.out.print(" " + String.valueOf(incMat[i][j]));
      }
      System.out.println();
    }
    System.out.println();
  }
}
