package Oid;
import java.io.*;

/**
 * <code>PrimPolyCalculator</code>, given a prime p and a positive
 * degree k, finds a primitive polynomial of degree k over GF(p) that
 * is <em>primitive</em>; that is, an irreducible polynomial f(x) with
 * coefficients in GF(p) such that, in the field GF(p)[x] / (f(x)),
 * the element [x] generates the (cyclic) group of units.  The method
 * is based on that described by Sean E. O'Connor in the documentation
 * for his C program which performs the same function [ref].  That is,
 * in turn, based on a method described by Alanen and Knuth in a 1964
 * paper entitled "Tables of finite fields" and published in the
 * journal <em>Sankhya</em>, Series A, Vol. 26, Part 4, December 1964,
 * pp. 305-328.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 24-Aug-2002
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class PrimPolyCalculator 
{
  PrimeFactorization mPF;

  /**
   * default constructor
   */
  public PrimPolyCalculator()
  {
    mPF = new PrimeFactorization();
  }

  /**
   * finds and returns a primitive polynomial of degree <code>k</code>
   * with coefficients in GF(p)
   * 
   * @param p prime 
   * @param k degree
   * @return GFpPoly reference with polynomial
   * @throws IllegalArgumentException if k < 2
   * @throws ArithmeticException if p is not prime
   */
  public GFpPoly findPrimPoly(int p, int k)
    throws IllegalArgumentException, ArithmeticException
  {
    if (mPF.getFactors(p).length > 1) {
      throw new ArithmeticException(p + " is not prime");
    }

    if (k < 2) {
      throw new IllegalArgumentException("degree is less than 2");
    }

    //
    // factor r = (p^k-1)/(p-1) = 1 + p + p^2 + ... + p^(k-1) into 
    // distinct primes
    //
    int r=0, i, j=1, a0, a=0, m;

    for (i=0; i<k; i++) {
      r += j;
      j *= p;
    }
    int[][]rFactor = mPF.getFactors(r);
    int[][]pFactor = mPF.getFactors(p-1);

    GFpPoly f, q, rem, xpr, xpm;
    GFpCalculator GFp = new GFpCalculator(p);
    boolean reject = false;
    boolean found = false;

    xpr = GFpPoly.xPower(GFp, r);

    f = getCandidate(GFp, k);
    while (f instanceof GFpPoly) {
      // 
      // make sure constant term of f is a primitive root of p; need only
      // check that a0 ^ ((p-1)/q) is not 1 (mod p) for all prime q | (p-1)
      //
      reject = false;
      a0 = f.getCoefficient(0);
      a0 *= 1 - 2 * (k % 2); // multiply by (-1)^k
      a0 = GFp.sum(a0, 0);   // get into range [0,p-1]
      
      for (i=0; i<pFactor.length; i++) {
	if (GFp.power(a0, (p-1)/pFactor[i][0]) == 1) {
	  reject = true;
	  break;
	}
      }

      // 
      // if f passed the last test, check for the absence of linear factors
      // by testing that f(a) != 0 (mod p) for 1 <= a <= p-1
      //
      if (!reject) {
	for (i=1; i<p; i++) {
	  if (f.evaluate(i) == 0) {
	    reject = true;
	    break;
	  }
	}
      }

      //
      // check if x^r = a (mod f(x), p) for some integer a; that is, divide
      // f by x^r and see if the remainder is an integer
      //
      if (!reject) {
	q = new GFpPoly(xpr);
	rem = q.divide(f);
	if (rem.getDegree() > 0) {
	  reject = true;
	}
	else {
	  a = GFp.sum(rem.getCoefficient(0),0);
	}
      }

      // 
      // check if the remainder constant above is the same as a0
      //
      if (!reject) {
	if (a != a0) {
	  reject = true;
	}
      }

      //
      // check x^m is not an integer (mod f(x), p) for m = r/q, q any prime
      // factor of r, but skip the test for any m = r/q where q|(p-1)
      //
      if (!reject) {
	for (i=0; i<rFactor.length; i++) {
	  if ((p-1) % rFactor[i][0] != 0) {
	    m = r / rFactor[i][0];
	    xpm = GFpPoly.xPower(GFp, m);
	    rem = xpm.divide(f);
	    if (rem.getDegree() == 0) {
	      reject = true;
	      break;
	    }
	  }
	}
      }

      if (!reject) {
	found = true;
	break;
      }

      f = getCandidate(f);
    }
    
    if (!found) {
      f = (GFpPoly)null;
    }

    return f;
  }

  //
  // private method generates first candidate for primitive polynomial
  //
  private GFpPoly getCandidate(GFpCalculator GFp, int k)
  {
    //
    // start off with x^k + 1
    //
    GFpPoly answer = GFpPoly.xPower(GFp, k);
    answer.add(1);
    return answer;
  }

  //
  // private method, given a candidate for primitive polynomial, generates
  // the "next" one, or returns a null pointer if there are no more
  //
  private GFpPoly getCandidate(GFpPoly lastCandidate)
  {
    // 
    // find first coefficient that is smaller than p-1, increment it,
    // and set all lower coefficients to minimal values (0 for everthing
    // except the constant, which is 1)
    //
    int deg = lastCandidate.getDegree();
    int[] coeff = lastCandidate.getCoefficientArray();
    int i, j=0;
    boolean found = false;
    int p = lastCandidate.getField().getCharacteristic();
    GFpPoly answer = (GFpPoly)null;

    for (i=0; i<=deg; i++) {
      if (coeff[i] < (p-1)) {
	j = i;
	found = true;
	break;
      }
    }

    if (found) {
      coeff[j]++;
      for (i=0; i<j; i++) {
	coeff[i] = (i == 0) ? 1 : 0;
      }
      answer = new GFpPoly(coeff, lastCandidate.getField());
    }

    return answer;
  }

  /**
   * main method lets this class be used on a standalone basis
   *
   * @param arg two required arguments are p and k
   */
  public static void main(String[] arg)
  {
    if (arg.length < 2) {
      System.out.println("Usage: PrimPolyCalculator p k");
    }
    else {
      int p = Integer.valueOf(arg[0]).intValue();
      int k = Integer.valueOf(arg[1]).intValue();
      PrimPolyCalculator calc = new PrimPolyCalculator();
      GFpPoly f = calc.findPrimPoly(p, k);
      GFpPoly xp, x, rm;
      int i;

      if (f != (GFpPoly)null) {
	System.out.println("Check:");
	xp = new GFpPoly(f.getField());
	xp.add(1);
	x = GFpPoly.xPower(f.getField(), 1);
	i = 0;
	do {
	  System.out.println("x^" + (i++) + " = " + xp.toString());
	  xp.multiply(x);
	  rm = xp.divide(f);
	  xp = rm;
	} while ((xp.getDegree() > 0) || 
		 ((xp.getDegree() == 0) && (xp.getCoefficient(0) > 1)));
	
	System.out.println("Primitive polynomial: " + f.toString());
      }
    }
  }

}
