package Oid;
import java.util.*;

/**
 * <code>SFDual</code> is a <code>SingleConstructor</code>
 * implementation which generates the dual of a
 * <code>FiniteFieldMatroid</code> in standard form, if possible.  If
 * a matroid of another type is passed to it, it generates its dual if
 * possible, but throws an exception otherwise.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 9 Jul 2002
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class SFDual implements SingleConstructor, ComplexityProvider
{
  //
  // private data members
  //
  private Matroid mMatroid;
  private Matroid mDual;

  /** 
   * sets the reference object for this constructor 
   *
   * @param ref reference object
   */
  public void setInstance(Object ref) 
    throws IllegalArgumentException
  {
    if (!(ref instanceof Matroid)) {
      throw new IllegalArgumentException("must be a matroid");
    }

    if (ref instanceof FiniteFieldMatroid) {
      getGFpDual((FiniteFieldMatroid)ref);
    }
  }

  //
  // private method gets the dual of a FiniteFieldMatroid
  //
  private void getGFpDual(FiniteFieldMatroid m)
    throws IllegalArgumentException
  {
    if (m.getComplexityEstimate("getDual") < 0) {
      throw new IllegalArgumentException("cannot generate dual of this "
					 + "matroid");
    }

    FiniteFieldMatroid dual = m.getDual();

    if (dual.inStandardForm()) {
      mDual = dual;
    }
    else if (m.inStandardForm()) {
      // dual's matrix is in the form [-D^t | I_{n-r}]; build new matrix
      int r = dual.getRank(), n = dual.getSize();
      int[][] newMatrix = new int[n][r];
      int i,j;

      for (i=0; i<r; i++) {
	for (j=0; j<r; j++) {
	  newMatrix[j][i] = dual.getElement(n-r+j).get(i);
	}
	for (j=r; j<n; j++) {
	  newMatrix[j][i] = dual.getElement(j-r).get(i);
	}
      }

      // build array of FiniteFieldVectors
      FiniteField field = dual.getField();
      FiniteFieldVector[] newColumns = new FiniteFieldVector[n];
      for (i=0; i<n; i++) {
	newColumns[i] = new FiniteFieldVector(newMatrix[i], r, field);
      }
      mDual = new FiniteFieldMatroid(newColumns);
      mDual.setName(dual.getName());
    }
    else {
      throw new IllegalArgumentException("matroid or dual must be in " +
					 "standard form");
    }
    
    mMatroid = m;
    
  }


  /** returns the reference object for this constructor */
  public Object getInstance()
  {
    return (Object)mMatroid;
  }

  /** indicates whether the algorithm requires a parameter */
  public boolean requiresParameter()
  {
    return false;
  }

  /** sets the algorithm's parameters */
  public void setParameters(String parameters)
    throws IllegalArgumentException
  {
  }

  /** returns empty string since this algorithm does not require a parameter */
  public String getParameters()
  {
    return new String("");
  }

  /** returns a blank string */
  public String getPrompt()
  {
    return new String("");
  }

  /** returns the constructed object from the reference object */
  public Object getConstruction()
  {
    return (Object)mDual;
  }

  /** returns a description of the algorithm used for the construction */
  public String getAlgorithmDescription()
  {
    StringBuffer sb = new StringBuffer("");
    sb.append("SFDual is a SingleConstructor implementation which ");
    sb.append("generates the ");
    sb.append("dual of a FiniteFieldMatroid in standard form, ");
    sb.append("if possible.  ");
    sb.append("If a matroid of another type is passed to it, it ");
    sb.append("generates its dual ");
    sb.append("if possible, but throws an exception otherwise.");
    return sb.toString();
  }

  /** returns this algorithm's name */
  public String getName()
  {
    return new String("Standard Form Dual Generator");
  }

  /** 
   * returns a value estimating the number of steps required to 
   * execute the method with the passed name, or -1 if the method cannot
   * currently be executed
   */
  public long getComplexityEstimate(String methodName)
  {
    long answer = -1;

    if (methodName.equals("setInstance")) {
      answer = 50;
    }
    else if (methodName.equals("getInstance") ||
	     methodName.equals("requiresParameter") ||
	     methodName.equals("setParameter") || 
	     methodName.equals("getParameter") || 
	     methodName.equals("getConstruction") ||
	     methodName.equals("getAlgorithmDescription") ||
	     methodName.equals("getName")) {
      answer = 1;
    }
    return answer;      
  }
  
  /**
   * returns a value estimating the number of steps required to
   * execute the method with the passed name and argument, or -1 if the 
   * method cannot be executed with the passed argument
   */
  public long getComplexityEstimate(String methodName, Object argument)
  {
    long answer = -1;

    if ((argument instanceof Matroid) && 
	(methodName.equals("setInstance"))) {
      Matroid m = (Matroid)argument;
      answer = m.getSize() * m.getRank();
    }
    else {
      answer = getComplexityEstimate(methodName);
    }
    return answer;
  }

  /**
   * returns a value estimating the number of steps required to 
   * execute the method with the passed name and collection of arguments
   * (which must be placed in correct order for iteration), or -1 if the
   * method cannot be executed with the passed arguments
   */
  public long getComplexityEstimate(String methodName, Collection arguments)
  {
    long answer=-1;
    
    if ((methodName.equals("setInstance")) &&
	(arguments.size() > 0)) {
      if (arguments.iterator().next() instanceof Matroid) {
	Matroid m = (Matroid)arguments.iterator().next();
	answer = m.getSize() * m.getRank();
      }
    }
    else {
      answer = getComplexityEstimate(methodName);
    }
    return answer;
  }
}
