package Oid;
import java.util.*;
import java.math.*;

/**
 * SpanningSetAlgorithm implements the SubsetLister interface, and generates
 * the spanning sets of a matroid, organized by size.  It works for any
 * Matroid representation, but can take advantage of a BasisMatroid if it
 * is given one.
 *
 * It works by generating a list of the matroid's bases (either trivially
 * in the case of a BasisMatroid or using DefaultBasisAlgorithm), and then
 * iterating through all subsets of the matroid which are larger than 
 * the rank, checking to see if each one contains a basis.  The subsets
 * which do contain a basis are kept.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 14 Nov 2001
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class SpanningSetAlgorithm implements SubsetLister,
					     ComplexityProvider
{
  //
  // private data members
  //
  private Matroid mMatroid;
  private boolean mIsBasisMatroid;
  private Vector  mSpanningSets;
  private Vector  mChain;

  // 
  // SubsetLister interface methods
  //

  /** 
   * sets the reference object for this SubsetLister
   *
   * @param arg reference object
   * @throws IllegalArgumentException if <code>arg</code> is not a 
   * <code>Matroid</code> 
   */
  public void setInstance(Object arg)
    throws IllegalArgumentException
  {
    if (!(arg instanceof Matroid)) {
      throw new IllegalArgumentException("SpanningSetAlgorithm requires a "
					 + "Matroid implementation");
    }

    mMatroid = (Matroid)arg;
    mIsBasisMatroid = (mMatroid instanceof BasisMatroid);
    determineSpanningSets();
    determineChain();
  }

  /** 
   * returns the reference object for this SubsetLister
   *
   * @return reference object
   */
  public Object getInstance()
  {
    Object answer = null;
    if (mMatroid instanceof Matroid) {
      answer = mMatroid;
    }
    return answer;
  }

  /** 
   * generates a list of the relevant Subsets for the reference object
   *
   * @return <code>Collection</code> of subsets
   * @throws NullPointerException if matroid has not been set
   */
  public Collection getSubsetList()
    throws NullPointerException
  {
    Vector answer = (Vector)null;
    if (mSpanningSets instanceof Vector) {
      answer = new Vector();
      for (Iterator pSS = mSpanningSets.iterator(); pSS.hasNext(); ) {
	answer.addAll((Vector)pSS.next());
      }
    }
    return answer;
  }

  /** 
   * generates a Vector containing indexed collections of Subsets for 
   * the reference object  
   *
   * @return <code>Vector</code> of subset collections
   * @throws NullPointerException if matroid has not been set
   */
  public Vector getSubsetListVector()
    throws NullPointerException
  {
    Vector answer = (Vector)null;
    if (mSpanningSets instanceof Vector) {
      answer = mSpanningSets;
    }
    return answer;
  }

  /** 
   * generates a Vector containing the number of Subsets at each index
   * for the reference object
   *
   * @return <code>Vector</code> of subset counts
   * @throws NullPointerException if matroid has not been set
   */
  public Vector getSubsetCountChain()
    throws NullPointerException
  {
    Vector answer = (Vector)null;
    if (mChain instanceof Vector) {
      answer = mChain;
    }
    return answer;
  }

  /** 
   * returns the name of the property by which the subsets are indexed
   *
   * @return name of indexed property
   * @throws NullPointerException if matroid has not been set
   */
  public String getArrayIndexName()
    throws NullPointerException
  {
    return new String("Size");
  }

  /** 
   * returns a description of the algorithm used to generate subsets
   *
   * @return algorithm description
   */
  public String getAlgorithmDescription()
  {
    String answer = new String("");
    answer += 
      "It works by generating a list of the matroid's bases (either trivially";
    answer += 
      "for a BasisMatroid or using DefaultBasisAlgorithm), and then";
    answer += 
      "iterating through all subsets of the matroid which are larger than ";
    answer += 
      "the rank, checking to see if each one contains a basis.  The subsets";
    answer += 
      "which do contain a basis are kept.";
    return answer;
  }

  /** 
   * returns the name of the algorithm
   *
   * @return algorithm name
   */
  public String getName()
  {
    return new String("Spanning Set Algorithm");
  }

  /**
   * private method determines the spanning sets
   */
  private void determineSpanningSets()
  {
    int r = mMatroid.getRank(), n = mMatroid.getSize(), k;
    SubsetEnumeration se;
    Iterator pBases;
    Subset s;
    Vector bases;
    boolean found;

    // first we need a Vector with all the matroid's bases -- if the 
    // matroid is a BasisMatroid, just use the getBases method; otherwise,
    // create an instance of DefaultBasisAlgorithm and use that
    if (mIsBasisMatroid) {
      bases = ((BasisMatroid)mMatroid).getBases();
    }
    else {
      DefaultBasisAlgorithm basisAlg = new DefaultBasisAlgorithm();
      basisAlg.setInstance(mMatroid);
      bases = new Vector(basisAlg.getSubsetList());
    }

    //
    // set up the vector that will hold the vectors of subsets
    // all the entries for 0 .. r-1 will be empty
    //
    mSpanningSets = new Vector();
    for (k=0; k<r; k++) {
      mSpanningSets.add(k, new Vector());
    }
    mSpanningSets.add(r, bases); // r-th entry is the bases
    for (k=r+1; k<=n; k++) {
      mSpanningSets.add(k, new Vector());
    }
    
    // now loop through all the subset sizes greater than r
    try {
      for (k=r+1; k<=n; k++) {
	for (se = new SubsetEnumeration(n, k); se.hasMoreElements(); ) {
	  s = (Subset)se.nextElement();
	  // find out if there is a basis that contains this subset
	  found = false;
	  for (pBases = bases.iterator(); pBases.hasNext(); ) {
	    if (s.subsetContains((Subset)pBases.next())) {
	      found = true;
	      break;
	    }
	  }
	  // if so, add it to the correct vector in the list of vectors
	  if (found) {
	    ((Vector)mSpanningSets.get(k)).add(s);
	  }
	}
      }
    }
    catch (Exception e) {
      // this "can't happen"
      throw new InternalError("Internal error in SpanningSetAlgorithm: " +
			      e.toString());
    }
  }

  /**
   * private method determines the subset count chain
   */
  private void determineChain()
  {
    mChain = new Vector();
    int k, n = mMatroid.getSize();
    for (k=0; k<n; k++) {
      mChain.add(new Integer(((Vector)mSpanningSets.get(k)).size()));
    }
  }

  //
  // ComplexityProvider interface methods
  //
  /** 
   * returns a value estimating the number of steps required to 
   * execute the method with the passed name, or -1 if the method cannot
   * currently be executed
   *
   * @param methodName name of method
   * @return complexity estimate for method
   */
  public long getComplexityEstimate(String methodName)
  {
    long answer = -1;
    
    // if there is a matroid for this representation, we can estimate
    // complexity
    if ((methodName.equals("getSubsetList") ||
	 methodName.equals("getSubsetListVector") ||
	 methodName.equals("getSubsetCountChain")) &&
	(mMatroid instanceof Matroid)) {
      answer = getComplexity(mMatroid);
    }

    return answer;
  }
  
  /**
   * returns a value estimating the number of steps required to
   * execute the method with the passed name and argument, or -1 if the 
   * method cannot be executed with the passed argument
   *
   * @param methodName name of method
   * @param argument argument for method
   * @return complexity estimate for method
   */
  public long getComplexityEstimate(String methodName, Object argument)
  {
    long answer = -1;
    
    // if there is a matroid for this representation, we can estimate
    // complexity
    if ((methodName.equals("getSubsetList") ||
	 methodName.equals("getSubsetListVector") ||
	 methodName.equals("getSubsetCountChain")) &&
	(argument instanceof Matroid)) {
      answer = getComplexity((Matroid)argument);
    }

    return answer;    
  }

  /**
   * returns a value estimating the number of steps required to 
   * execute the method with the passed name and collection of arguments
   * (which must be placed in correct order for iteration), or -1 if the
   * method cannot be executed with the passed arguments
   *
   * @param methodName name of method
   * @param arguments arguments for method
   * @return complexity estimate for method
   */
  public long getComplexityEstimate(String methodName, Collection arguments)
  {
    long answer = -1;
    Object argument;

    if (arguments.size() > 0) {
      argument = arguments.iterator().next();

      // if there is a matroid for this representation, we can estimate
      // complexity
      if ((methodName.equals("getSubsetList") ||
	   methodName.equals("getSubsetListVector") ||
	   methodName.equals("getSubsetCountChain")) &&
	  (argument instanceof Matroid)) {
	answer = getComplexity((Matroid)argument);
      }
    }

    return answer;    
  }

  /**
   * private method actually estimates the complexity
   */
  private long getComplexity(Matroid m)
  {
    long answer = -1;
    int n = m.getSize();
    int r = m.getRank();
    long c = nCk(n,r);
    int k;

    if (m instanceof BasisMatroid) {
      answer = 1;
    }
    else {
      answer = 
	((ComplexityProvider)m).getComplexityEstimate("isIndependent") * c;
    }

    for (k=r+1; k<=n; k++) {
      answer += c * nCk(n, k);
    }

    return answer;
  }

  //
  // private method to get combinations -- only uses BigInteger if 
  // it has to
  //
  private long nCk(long n, long k)
  {
    long answer = 0;
    long i;

    if ((0 == k) || (n == k)) {
      answer = 1;
    }
    else if ((1 == k) || ((n-1) == k)) {
      answer = n;
    }
    else if ((k > 1) && (k < n-1)) {
      if (n > 20) {
	BigInteger bigAns = BigInteger.valueOf(1);
	for (i=n; i>n-k; i--) {
	  bigAns = bigAns.multiply(BigInteger.valueOf((long)i));
	}
	for (i=n-k; i>1; i--) {
	  bigAns = bigAns.divide(BigInteger.valueOf((long)i));
	}
	answer = bigAns.longValue();
      }
      else {
	answer = 1;
	for (i=n; i>n-k; i--) {
	  answer *= i;
	}
	for (i=n-k; i>1; i--) {
	  answer /= i;
	}
      }
    }

    return answer;
  }

}
