package Oid;
/**
 * This class is used to align characters for fixed-width text displays,
 * and includes <code>static</code> methods to left-justify, right-justify
 * and center text in a field of spaces of a given size.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 4 Sep 2002
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class TextAlignment
{
  private static final int BUF_LEN = 80;
  private static final char[] BUF = new char[] {
    ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',
    ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',
    ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',
    ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',
    ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' '
  };
    

  /**
   * Returns a string with the passed string left-justified within a 
   * string of spaces of the indicated size; if <code>fieldSize</code> is
   * less than the length of <code>s</code> the leftmost <code>fieldSize</code>
   * characters of <code>s</code> are returned.
   *
   * @param s string to include
   * @param fieldSize number of total spaces
   * @return string of length <code>fieldSize</code>
   */
  public static String leftAlign(String s, int fieldSize)
  {
    String ans = (String)null;
    int n = s.length();

    if (n > fieldSize) {
      ans = s.substring(0, fieldSize);
    }
    else if (n == fieldSize) {
      ans = new String(s);
    }
    else if (fieldSize - n <= BUF_LEN) {
      ans = s + new String(BUF, 0, fieldSize-n);
    }
    else {
      StringBuffer sb = new StringBuffer(s);
      String sapp = new String(BUF, 0, BUF_LEN);

      int k = (fieldSize - n) / BUF_LEN;
      int r = (fieldSize - n) % BUF_LEN;
      for (int i = 0; i<k; i++) {
	sb.append(sapp);
      }
      sb.append(new String(BUF, 0, r));
      ans = sb.toString();
    }

    return ans;
  }

  /**
   * Returns a string with the passed string right-justified within a 
   * string of spaces of the indicated size; if <code>fieldSize</code> is less
   * than the <code>s.length()</code>, the rightmost <code>fieldSize</code>
   * characters of <code>s</code> are returned.
   *
   * @param s string to include
   * @param fieldSize number of total spaces
   * @return string of length <code>fieldSize</code>
   */
  public static String rightAlign(String s, int fieldSize)
  {
    String ans = (String)null;
    int n = s.length();

    if (n > fieldSize) {
      ans = s.substring(n-fieldSize);
    }
    else if (n == fieldSize) {
      ans = new String(s);
    }
    else if (fieldSize - n <= BUF_LEN) {
      ans = new String(BUF, 0, fieldSize-n) + s;
    }
    else {
      StringBuffer sb = new StringBuffer(s);
      String sapp = new String(BUF, 0, BUF_LEN);

      int k = (fieldSize - n) / BUF_LEN;
      int r = (fieldSize - n) % BUF_LEN;
      for (int i = 0; i<k; i++) {
	sb.insert(0, sapp);
      }
      sb.insert(0, new String(BUF, 0, r));
      ans = sb.toString();
    }


    return ans;
  }

  /**
   * Returns a string with the passed string centered within a 
   * string of spaces of the indicated size; if <code>fieldSize</code> is less
   * than the <code>s.length()</code>, the leftmost <code>fieldSize</code>
   * characters of <code>s</code> are returned.
   *
   * @param s string to include
   * @param fieldSize number of total spaces
   * @return string of length <code>fieldSize</code>
   */
  public static String centerAlign(String s, int fieldSize)
  {
    String ans = (String)null;
    int n = s.length();

    if (n > fieldSize) {
      ans = s.substring(0, fieldSize);
    }
    else if (n == fieldSize) {
      ans = new String(s);
    }
    else {
      int r = (fieldSize - n) / 2;
      int l = (fieldSize - n) - r;

      if (l <= BUF_LEN) {
	ans = new String(BUF, 0, l) + s + new String(BUF, 0, r);
      }
      else {
	StringBuffer sb = new StringBuffer(s);
	String sapp = new String(BUF, 0, BUF_LEN);
	int qr = r / BUF_LEN, rr = r % BUF_LEN;
	int ql = l / BUF_LEN, rl = l % BUF_LEN;
	int i;

	for (i=0; i<ql; i++) {
	  sb.insert(0, sapp);
	}
	sb.insert(0, new String(BUF, 0, rl));
	
	for (i=0; i<qr; i++) {
	  sb.append(sapp);
	}
	sb.append(new String(BUF, 0, rr));

	ans = sb.toString();
      }
    }

    return ans;
  }

  /**
   * allows this class to be tested from the command line
   *
   * @param arg array of command line arguments; first argument is string, 
   * second is length of field, third is "left", "right" or "center"
   */
  public static void main(String[] arg)
  {
    int fieldSize;

    if (arg.length < 3) {
      System.out.println("Usage: java TextAlignment <string> <field size> " +
			 "[left|right|center]");
    }
    else {
      try {
	fieldSize = Integer.parseInt(arg[1]);
	if (arg[2].toLowerCase().equals("left")) {
	  System.out.println(TextAlignment.leftAlign(arg[0], fieldSize));
	}
	else if (arg[2].toLowerCase().equals("right")) {
	  System.out.println(TextAlignment.rightAlign(arg[0], fieldSize));
	}
	else if (arg[2].toLowerCase().equals("center")) {
	  System.out.println(TextAlignment.centerAlign(arg[0], fieldSize));
	}
	else {
	  System.out.println("alignment must be left, right or center");
	}
      }
      catch (NumberFormatException nfe) {
	System.out.println("Field size must be a number");
      }
    }
  }
}
