import Oid.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
 * This class displays a panel with two text boxes, for the fieldSize
 * and rank of a projective space, and a button.  When the user
 * presses the button, the class validates the input as correct
 * specifications for a projective space and then raises an event for
 * its listeners, who can then get its values via <code>getRank</code>
 * and <code>getFieldSize</code>.
 * 
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 27 Oct 2001
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class PGSpecPanel extends JPanel implements ActionListener
{
  //
  // private member data
  //
  private JTextField mFieldSizeTextField;
  private JTextField mRankTextField;
  private JTextField mNameTextField;
  private JButton mButton;
  private JLabel mMessageLabel;
  private FiniteField mField;
  private int mRank;
  private String mName;
  private boolean mHasValidInput;
  private LinkedList mListeners;

  private static String FIELDSIZE_TEXT = "FieldSize: ";
  private static String RANK_TEXT = "Rank: ";
  private static String NAME_TEXT = "Name: ";
  private static String PROMPT = 
    "Enter the name, fieldSize, rank and size below:    ";

  /**
   * sets up panel
   */
  PGSpecPanel()
  {
    JPanel rowPanel;
    JPanel colPanel;

    setLayout(new BorderLayout());

    //
    // initialize fieldSize calculator
    //
    mField = (FiniteField)null;

    colPanel = new JPanel();
    colPanel.setLayout(new BoxLayout(colPanel, BoxLayout.Y_AXIS));

    //
    // add name label & text box
    //
    rowPanel = new JPanel();
    rowPanel.setLayout(new BoxLayout(rowPanel, BoxLayout.X_AXIS));
    rowPanel.add(new JLabel(NAME_TEXT));
    mNameTextField = new JTextField(5);
    rowPanel.add(mNameTextField);
    colPanel.add(rowPanel);

    //
    // add fieldSize label & text box
    //
    rowPanel = new JPanel();
    rowPanel.setLayout(new BoxLayout(rowPanel, BoxLayout.X_AXIS));
    rowPanel.add(new JLabel(FIELDSIZE_TEXT));
    mFieldSizeTextField = new JTextField(5);
    rowPanel.add(mFieldSizeTextField);
    colPanel.add(rowPanel);

    //
    // add rank label & text box
    //
    rowPanel = new JPanel();
    rowPanel.setLayout(new BoxLayout(rowPanel, BoxLayout.X_AXIS));
    rowPanel.add(new JLabel(RANK_TEXT));
    mRankTextField = new JTextField(5);
    rowPanel.add(mRankTextField);
    colPanel.add(rowPanel);

    add(colPanel, BorderLayout.CENTER);
    //
    // add continue button
    //
    mButton = new JButton("Continue");
    mButton.addActionListener(this);

    //
    // add message label
    //
    mMessageLabel = new JLabel(PROMPT);
    
    add(mMessageLabel, BorderLayout.NORTH);

    add(mButton, BorderLayout.SOUTH);
  }

  /** 
   * adds an actionListener  
   *
   * @param l <code>ActionListener</code> to add
   */
  public void addActionListener(ActionListener l)
  {
    if (!(mListeners instanceof LinkedList)) {
      mListeners = new LinkedList();
    }

    mListeners.add(l);
  }

  /**
   * listens to button
   *
   * @param e <code>ActionEvent</code>
   */
  public void actionPerformed(ActionEvent e)
  {
    ActionListener l;
    ActionEvent f;

    if (inputIsValid()) {
      mMessageLabel.setText(PROMPT);
      if (mListeners instanceof LinkedList) {
	for (Iterator i=mListeners.iterator(); i.hasNext();) {
	  l = (ActionListener)i.next();
	  l.actionPerformed(new ActionEvent(this,
					    ActionEvent.ACTION_PERFORMED,
					    "Specs selected"));
	}
      }
      else {
	mMessageLabel.setText("Okay!");
      }
    }
  }

  /** 
   * validates input 
   *
   * @return <code>true</code> if and only if input is valid
   */
  public boolean inputIsValid()
  {
    mHasValidInput = (retrieveName() && retrieveFieldSize() && 
		      retrieveRank());
    return mHasValidInput;
  }

  /** 
   * get the finite field
   *
   * @return reference to selected field
   */
  public FiniteField getMatroidField()
  {
    return mField;
  }

  /** 
   * get the rank 
   *
   * @return rank of matroid
   */
  public int getMatroidRank()
  {
    int nAnswer = -1;
    if (mHasValidInput) {
      nAnswer = mRank;
    }
    return nAnswer;
  }

  /** 
   * get the name 
   * 
   * @return name for new matroid
   */
  public String getMatroidName()
  {
    String answer = (String)null;
    if (mHasValidInput) {
      answer = new String(mName);
    }
    return answer;
  }

  //
  // make sure the name string is not empty
  //
  private boolean retrieveName()
  {
    boolean bAns = false;
    if (mNameTextField.getText().length() > 0) {
      mName = mNameTextField.getText();
      bAns = true;
    }
    return bAns;
  }

  //
  // get the fieldSize and check to make sure it is positive and prime
  //
  private boolean retrieveFieldSize()
  {
    boolean bAnswer = true;
    String theText;
    int q;

    theText = mFieldSizeTextField.getText();
    if (theText.equals("")) {
      mMessageLabel.setText("No fieldSize entered");
      bAnswer = false;
    }
    else {
      try {
	q = Integer.parseInt(theText);
	mField = FiniteFieldFactory.createFiniteField(q);
      }
      catch (NumberFormatException e) {
	mMessageLabel.setText("FieldSize is not a number");
	bAnswer = false;
      }
      catch (IllegalArgumentException e) {
	mMessageLabel.setText("FieldSize is not prime");
	bAnswer = false;
      }
    }
    
    if (!bAnswer) {
      mField = (FiniteField)null;
    }

    return bAnswer;
  }
  
  // 
  // get the rank off the form and make sure it is positive
  //
  private boolean retrieveRank()
  {
    boolean bAnswer = true;
    int nValue=0;
    String theText;

    theText = mRankTextField.getText();
    if (theText.equals("")) {
      mMessageLabel.setText("No rank entered");
      bAnswer = false;
    }
    else {
      try {
	nValue = Integer.parseInt(theText);
      }
      catch (NumberFormatException e) {
	mMessageLabel.setText("Rank is not a number");
	bAnswer = false;
      }
      if (bAnswer) {
	if (nValue > 0) {
	  mRank = nValue;
	}
	else {
	  mMessageLabel.setText("Rank is not positive");
	  bAnswer = false;
	}
      }
    }

    return bAnswer;
  }

  /**
   * allows this class to be tested from the command line
   *
   * @param arg array of command-line arguments; ignored here
   */
  public static void main(String[] arg)
  {
    JFrame frame = new JFrame("PGSpecPanel");
    PGSpecPanel panel;

    panel = new PGSpecPanel();

    frame.addWindowListener(new WindowAdapter() {
	public void windowClosing(WindowEvent e) {
	  System.exit(0);
	}
      });

    panel.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent f) {
	  PGSpecPanel pgs = (PGSpecPanel)f.getSource();

	  System.out.println("Name: " + pgs.getMatroidName());
	  System.out.println("Rank: " + pgs.getMatroidRank());
	  System.out.println("Field size: " + 
			     pgs.getMatroidField().getFieldSize());

	  System.exit(0);
	}
      });
    
    frame.getContentPane().add(panel);
    frame.pack();
    frame.setVisible(true);

  }

}
