import Oid.*;
import java.io.*;
import java.util.*;

/**
 * Reads a specified file, tokenizes it, and provides an
 * <code>Vector</code> of <code>Vectors</code> of <code>Strings</code>
 * containing the whitespace-delimited contents of each of the file's
 * lines.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 1 Oct 2001
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class TextFileParser
{
  //
  // private data members
  //
  private String mFilename;
  private Vector mContentVector;
  static final int BUF_SIZE = 1024;

  /** 
   * constructs a text file parser for the file with the passed filename 
   *
   * @param filename name of file to read
   * @throws FileNotFoundException if file cannot be found
   * @throws IOException if file cannot be read
   */
  public TextFileParser(String filename)
    throws FileNotFoundException, IOException
  {
    FileReader rdr = new FileReader(filename);
    char[] buffer = new char[BUF_SIZE];
    int nRead;
    StringBuffer sBuffer = new StringBuffer("");
    
    // read the contents of the file into one big string
    nRead = rdr.read(buffer);
    while (nRead >= 0) {
      sBuffer.append(buffer, 0, nRead);
      nRead = rdr.read(buffer);
    }
    rdr.close();

    // tokenize the string into lines
    String contents = sBuffer.toString();
    StringTokenizer fileTokenizer = new StringTokenizer(contents, "\n");
    
    // tokenize each line into a Vector of strings
    String oneLine;
    StringTokenizer lineTokenizer;
    Vector lineVector;
    mContentVector = new Vector();
    for (; fileTokenizer.hasMoreElements();) {
      oneLine = (String)fileTokenizer.nextElement();
      lineVector = new Vector();
      for (lineTokenizer = new StringTokenizer(oneLine); 
	   lineTokenizer.hasMoreElements();) {
	lineVector.add((String)lineTokenizer.nextElement());
      }
      mContentVector.add(lineVector);
    }
  }

  /** 
   * returns a reference to the Vector holding the file's contents 
   * 
   * @return <code>Vector</code> of <code>Vectors</code> of strings from file
   */
  public Vector getContents()
  {
    return mContentVector;
  }

  /** 
   * returns a reference to the Vector holding the specified line of the 
   * file
   *
   * @param n line number
   * @return <code>Vector</code> of words on line <code>n</code>
   * @throws ArrayIndexOutOfBoundsException if <code>n</code> is out of range
   */
  public Vector getLine(int n)
  throws ArrayIndexOutOfBoundsException
  {
    return (Vector)mContentVector.get(n);
  }

  /**
   * returns the number of lines in the file
   *
   * @return number of lines in file
   */
  public int numLines() {
    return mContentVector.size();
  }

  /** 
   * returns a vector containing every word of the file (no lines) 
   *
   * @param <code>Vector</code> with all words in the file in one dimension
   */
  public Vector getAllWords()
  {
    Vector allWords = new Vector();
    Vector oneLine;
    Iterator pLine, pWord;

    for (pLine = mContentVector.iterator(); pLine.hasNext();) {
      oneLine = (Vector)pLine.next();
      for (pWord = oneLine.iterator(); pWord.hasNext();) {
	allWords.add(pWord.next());
      }
    }
    return allWords;
  }
}
