import Oid.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/** 
 * <code>VisFiniteFieldExtensions</code> generates all of the
 * single-element extensions of a GF(p)-representable matroid, and
 * displays their circuit, cocircuit, independent set and
 * coindependent set chains.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 6 Nov 2001
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class VisFiniteFieldExtensions implements Visualization
{
  private String LF;
  private String mName;

  /** 
   * generates a visualization of the passed object and returns it as
   * a JPanel
   */
  public JPanel getVisualization(Object obj)
    throws IllegalArgumentException
  {
    if (!(obj instanceof FiniteFieldMatroid)) {
      throw new IllegalArgumentException("GF(p) matroid needed");
    }

    if (!((FiniteFieldMatroid)obj).inStandardForm()) {
      throw new IllegalArgumentException("Matroid must be in standard form");
    }

    LF = "\n";
    FiniteFieldMatroid base = (FiniteFieldMatroid)obj;
    int n = base.getSize();
    int r = base.getRank();
    FiniteField calc = base.getField();
    FiniteFieldVector[] element = new FiniteFieldVector[n+1];
    int i,j;
    PGFactory factory = new PGFactory(r, calc);
    FiniteFieldVector[] projSpace = factory.getProjectiveSpace();
    int pn = factory.getProjectiveSpaceSize();
    FiniteFieldMatroid extension, dual;
    boolean found;
    Vector cctChains = new Vector();
    Vector cocctChains = new Vector();
    Vector indChains = new Vector();
    Vector coindChains = new Vector();
    Vector chainVectors = new Vector();
    StringBuffer ansBuf;
    DefaultCircuitAlgorithm cctAlg = new DefaultCircuitAlgorithm();
    DefaultIndependentSetAlgorithm indAlg = 
      new DefaultIndependentSetAlgorithm();

    mName = "GF(" + calc.getFieldSize() + ") extensions";
    ansBuf  = new StringBuffer(mName + LF);

    // get vectors from original matroid
    for (i=0; i<n; i++) {
      element[i] = base.getElement(i);
    }

    // loop through vectors in projective space
    for (i=0; i<pn; i++) {
      // search for projective space vector in matroid
      found = false;
      for (j=0; j<n; j++) {
	if (element[j].multiple(projSpace[i]) != 0) {
	  found = true;
	  break;
	}
      }
      if (!found) {
	// put projective space vector into element matrix, make
	// FiniteFieldMatroid and add chain descriptions
	element[n] = projSpace[i];
	extension = new FiniteFieldMatroid(element);
	dual = extension.getDual();
	processChains(extension, dual, projSpace[i], cctAlg, indAlg,
		      cctChains, cocctChains, indChains, coindChains,
		      chainVectors);
      }
    }

    Iterator p1 = cctChains.iterator();
    Iterator p2 = cocctChains.iterator();
    Iterator p3 = indChains.iterator();
    Iterator p4 = coindChains.iterator();
    Iterator p5 = chainVectors.iterator();
    Iterator pv;
    i=1;
    for (; p1.hasNext(); ) {
      ansBuf.append("Extension: ");
      ansBuf.append(i++);
      ansBuf.append(LF);
      for (pv = ((Vector)p5.next()).iterator(); pv.hasNext(); ) {
	ansBuf.append(((FiniteFieldVector)pv.next()).toString());
	ansBuf.append(LF);
      }
      appendChains(ansBuf, 
		   r,
		   n+1-r,
		   (Vector)p1.next(),
		   (Vector)p2.next(),
		   (Vector)p3.next(),
		   (Vector)p4.next());
      ansBuf.append(LF);
    }

    ReadOnlyTextPanel answer = new ReadOnlyTextPanel();
    answer.setText(ansBuf.toString());
    return answer;
  }

  // 
  // private method determines circuit, cocircuit, independent set and
  // coindependent set chains, for an extension, checks them against the 
  // previously calculated chains, and places the vector accordingly
  //
  private void processChains(FiniteFieldMatroid extension,
			     FiniteFieldMatroid dual,
			     FiniteFieldVector element,
			     DefaultCircuitAlgorithm cctAlg,
			     DefaultIndependentSetAlgorithm indAlg,
			     Vector cctChains,
			     Vector cocctChains,
			     Vector indChains,
			     Vector coindChains,
			     Vector extensionVectors)
  {
    Vector cctChain, cocctChain, indChain, coindChain;
    boolean found = false;

    //
    // get chains
    //
    try {
      cctAlg.setInstance(extension);
      cctChain = cctAlg.getSubsetCountChain();
      cctAlg.setInstance(dual);
      cocctChain = cctAlg.getSubsetCountChain();
      indAlg.setInstance(extension);
      indChain = indAlg.getSubsetCountChain();
      indAlg.setInstance(dual);
      coindChain = indAlg.getSubsetCountChain();
    }
    catch (Exception e) {
      // this shouldn't happen
      throw new InternalError(e.toString());
    }
    
    // search for chains among previous chains
    Iterator pcct, pcocct, pind, pcoind, pext;
    Vector extVector=null;
    Vector curCCTCh;
    Vector curCoCCTCh;
    Vector curIndCh;
    Vector curCoIndCh;
    pcct = cctChains.iterator();
    pcocct = cocctChains.iterator();
    pind = indChains.iterator();
    pcoind = coindChains.iterator();
    pext = extensionVectors.iterator();
    for(; pcct.hasNext();) {
      extVector = (Vector)pext.next();
      curCCTCh = (Vector)pcct.next();
      curCoCCTCh = (Vector)pcocct.next();
      curIndCh = (Vector)pind.next();
      curCoIndCh = (Vector)pcoind.next();
      if (compareChains(curCCTCh, cctChain) &&
	  compareChains(curCoCCTCh, cocctChain) &&
	  compareChains(curIndCh, indChain) &&
	  compareChains(curCoIndCh, coindChain)) {
	found = true;
	break;
      }
    }
    if (found) {
      extVector.add(element);
    }
    else {
      cctChains.add(cctChain);
      cocctChains.add(cocctChain);
      indChains.add(indChain);
      coindChains.add(coindChain);
      extVector = new Vector();
      extVector.add(element);
      extensionVectors.add(extVector);
    }
  }

  //
  // private method compares two chains and returns true if they have
  // the same numbers
  //
  private boolean compareChains(Vector v1, Vector v2)
  {
    boolean answer = true;
    int i,j;
    Iterator p1, p2;
    for (p1=v1.iterator(), p2=v2.iterator(); p1.hasNext() && p2.hasNext();) {
      i = ((Integer)p1.next()).intValue();
      j = ((Integer)p2.next()).intValue();
      if (i != j) {
	answer = false;
	break;
      }
    }

    if (p1.hasNext() || p2.hasNext()) {
      answer = false;
    }

    return answer;
  }

  //
  // private method appends chain descriptions to string buffer
  //
  private void appendChains(StringBuffer buf, 
			    int r1,
			    int r2,
			    Vector cctChain,
			    Vector cocctChain,
			    Vector indChain,
			    Vector coindChain)
  {
    
    int i, j, r;
    
    if (r1 > r2) {
      r = r1+1;
    }
    else {
      r = r2+1;
    }
    
    // first row is size values
    buf.append("Size        ");
    for (i=0; i<=r; i++) {
      buf.append(TextAlignment.rightAlign(String.valueOf(i),8));
    }
    buf.append(LF);

    buf.append("------------");
    for (i=0; i<=r; i++) {
      buf.append("--------");
    }
    buf.append(LF);

    // next circuits and cocircuits
    buf.append("Circuits    ");
    for (i=0; i<=r1+1; i++) {
      j = ((Integer)cctChain.get(i)).intValue();
      buf.append(TextAlignment.rightAlign(String.valueOf(j),8));
    }
    buf.append(LF);
    buf.append("Cocircuits  ");
    for (i=0; i<=r2+1; i++) {
      j = ((Integer)cocctChain.get(i)).intValue();
      buf.append(TextAlignment.rightAlign(String.valueOf(j),8));
    }
    buf.append(LF);
    
    // next independent sets and coindependent sets
    buf.append("Ind sets    ");
    for (i=0; i<=r1; i++) {
      j = ((Integer)indChain.get(i)).intValue();
      buf.append(TextAlignment.rightAlign(String.valueOf(j),8));
    }
    buf.append(LF);
    buf.append("Coind sets  ");
    for (i=0; i<=r2; i++) {
      j = ((Integer)coindChain.get(i)).intValue();
      buf.append(TextAlignment.rightAlign(String.valueOf(j),8));
    }
    buf.append(LF);    
  }

  /**
   * returns the visualization's name 
   */
  public String getName()
  {
    String ans = (String)null;
    
    if (mName instanceof String) {
      ans = mName;
    }
    else {
      ans = new String("GF(p) single-element extension viewer");
    }
    return ans;
  }

  /**
   * returns a string describing the visualization
   */
  public String getDescription()
  {
    return new String("VisFiniteFieldExtensions generates all of the single-element extensions of  a GF(p)-representable matroid, and displays their circuit, cocircuit, independent set and coindependent set chains.");
  }
}
