import Oid.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/** 
 * VisGF2ChainTutteExtF generates a list of the single-element extensions
 * of a FiniteFieldMatroid and checks for unique matroids using the 
 * FiniteFieldMatroidChainChecker class. 
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 6 Jun 2002
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class VisGF2ChainTutteExtF implements Visualization
{
  private String LF;

  /** 
   * generates a visualization of the passed object and returns it as
   * a JPanel -- in this case, the passed object must be an instance
   * of FiniteFieldMatroid, and the returned JPanel is a read-only text panel 
   * with a list of the single-element extensions of the matroid
   */
  public JPanel getVisualization(Object obj)
    throws IllegalArgumentException
  {
    ReadOnlyTextPanel answer = new ReadOnlyTextPanel();
    answer.setText(getDescription(obj));
    return answer;
  }

  /**
   * accepts a FiniteFieldMatroid as input and returns a list of
   * graphic and cographic single-element extensions
   */

  public String getDescription(Object obj)
  {
    if (!(obj instanceof FiniteFieldMatroid)) {
      throw new IllegalArgumentException("Requires a GF(p) matroid");
    }

    FiniteFieldMatroid base = (FiniteFieldMatroid)obj;

    if (2 != base.getField().getFieldSize()) {
      throw new IllegalArgumentException("Requires a GF(2) matroid");
    }

    LF = "\n";
    int n = base.getSize();
    int r = base.getRank();
    FiniteField calc = base.getField();
    FiniteFieldVector[] element = new FiniteFieldVector[n+1];
    int i,j,k;
    PGFactory factory = new PGFactory(r, calc);
    FiniteFieldVector[] projSpace = factory.getProjectiveSpace();
    int pn = factory.getProjectiveSpaceSize();
    int[] index = new int[pn];
    FiniteFieldMatroid extension, dual;
    FiniteFieldMatroid compMatroid;
    Vector compChains, compDegrees, compDChains, compDDegrees;
    Vector currChains, currDegrees, currDChains, currDDegrees;
    Vector uniqExtensions = new Vector();
    Vector extChains = new Vector();
    Vector extDegrees = new Vector();
    Vector extDChains = new Vector();
    Vector extDDegrees = new Vector();
    boolean found;
    boolean anyExtensions=false;
    FiniteFieldMatroidChainChecker checker = 
      new FiniteFieldMatroidChainChecker();
    checker.setMakeExplanation(false);
    StringBuffer matroidOutput;
    StringBuffer output = new StringBuffer(getName() + LF);

    // Tutte's algorithm stuff
    if (!base.inStandardForm()) {
      throw new IllegalArgumentException("Matroid must be in standard form");
    }

    int[][]matrix = new int[r][n+1];
    int[]colNum = new int[n+1];
    int[]rowNum = new int[r];
    
    int r2 = n+1-r;
    int[][] dualMatrix = new int[r2][n+1];
    int[] dualRowNum = new int[r2];

    FiniteFieldVector v;
    boolean isFirst;
    boolean isGraphic=false, isCoGraphic=false;

    // form as much of matrices as we can with matroid itself
    for (i=0; i<r; i++) {
      rowNum[i] = i+1;
    }

    for (i=0; i<r2; i++) {
      dualRowNum[i] = i+1;
    }

    for (i=0; i<n; i++) {
      v = base.getElement(i);
      colNum[i] = i+1;
      for (j=0; j<r; j++) {
	matrix[j][i] = v.get(j);
      }
    }
    colNum[n] = n+1;

    // form most of dual matrix
    for (i=0; i<r2; i++) {
      for (j=0; j<r2; j++) {
	dualMatrix[i][j] = (i == j) ? 1 : 0;
      }
    }
    for (i=0; i<r2-1; i++) {
      for (j=r2; j<=n; j++) {
	dualMatrix[i][j] = matrix[j-r2][i+r];
      }
    }

    // create matrix checker
    TutteMatrixChecker tmc = new TutteMatrixChecker();
//      tmc.setMatrix("M1", matrix, colNum, rowNum);



    // initialize indices to -1 (prevents matroid columns from being listed)
    for (i=0; i<pn; i++) {
      index[i] = -1;
    }

    // build base for extension matrix
    for (i=0; i<n; i++) {
      element[i] = base.getElement(i);
    }

    // loop through elements of the projective space
    System.out.println(String.valueOf(pn));
    System.out.flush();
    for (i=0; i<pn; i++) {
      System.out.println(projSpace[i].toString());
      System.out.flush();

      if (!matroidContains(base, projSpace[i])) {
	anyExtensions = true;

	// build extension matrix
	element[n] = projSpace[i];
	extension = new FiniteFieldMatroid(element);
	currChains = (Vector)null;
	currDegrees = (Vector)null;
	currDChains = (Vector)null;
	currDDegrees = (Vector)null;

	System.out.println("Checking...");
	System.out.flush();

	// check against previously discovered extensions
	found = false;
	for (j=0; j<uniqExtensions.size() && !found; j++) {
	  compMatroid = (FiniteFieldMatroid)uniqExtensions.get(j);
	  compChains = (Vector)extChains.get(j);
	  compDegrees = (Vector)extDegrees.get(j);
	  compDChains = (Vector)extDChains.get(j);
	  compDDegrees = (Vector)extDDegrees.get(j);
	  
	  // if we've already computed the chains & degrees & bases, use them
	  if (compChains instanceof Vector) {
	    checker.setFiniteFieldMatroid1(compMatroid,
				compMatroid.getDual(),
				compChains,
				compDegrees,
				compDChains,
				compDDegrees);
	  }
	  else {
	    checker.setFiniteFieldMatroid1(compMatroid);
	  }
	  
	  // also use current extensions chains & degrees & bases if possible
	  if (currChains instanceof Vector) {
	    checker.setFiniteFieldMatroid2(extension,
				extension.getDual(),
				currChains,
				currDegrees,
				currDChains,
				currDDegrees);
	  }
	  else {
	    checker.setFiniteFieldMatroid2(extension);
	  }

	  // check for isomorphism
	  if (checker.checkFiniteFieldMatroids()) {
	    found = true;
	    index[i] = j;
	  }

	  // if we haven't stored the vectors, get them now
	  if (!(compChains instanceof Vector)) {
	    extChains.set(j, checker.getChains1());
	    extDegrees.set(j, checker.getDegrees1());
	    extDChains.set(j, checker.getDualChains1());
	    extDDegrees.set(j, checker.getDualDegrees1());
	  }

	  // if we haven't stored the current extension's vectors get them now
	  if (!(currChains instanceof Vector)) {
	    currChains = checker.getChains2();
	    currDegrees = checker.getDegrees2();
	    currDChains = checker.getDualChains2();
	    currDDegrees = checker.getDualDegrees2();
	  }
	}

	if (!found) {
	  System.out.println("Adding...");
	  System.out.flush();

	  // make sure we have vectors for this matroid (checker can get them)
	  if (!(currChains instanceof Vector)) {
	    checker.setFiniteFieldMatroid1(extension);
	    currChains = checker.getChains1();
	    currDegrees = checker.getDegrees1();
	    currDChains = checker.getDualChains1();
	    currDDegrees = checker.getDualDegrees1();
	  }

	  // add new matroid to the list
	  uniqExtensions.add(extension);
	  extChains.add(currChains);
	  extDegrees.add(currDegrees);
	  extDChains.add(currDChains);
	  extDDegrees.add(currDDegrees);
	  index[i] = uniqExtensions.size()-1;
	}
      }
    }

    // make output string
    if (anyExtensions) {
      output.append(FiniteFieldMatroidFormatter.displayWithLabels(base));
      output.append(LF);

      for (i=0; i<uniqExtensions.size(); i++) {
	isFirst = true;
	matroidOutput = new StringBuffer("");
	for (j=0; j<pn; j++) {
	  // go through vectors and check graphic, cographic for first
	  if (index[j] == i) {
	    if (isFirst) {
	      // fill in extension column in matrix, row in dual matrix
	      v = projSpace[j];
	      for (k=0; k<r; k++) {
		matrix[k][n] = v.get(k);
		dualMatrix[r2-1][k+r2] = v.get(k);
	      }
	      // check whether matroid is graphic, cographic
	      tmc.setMatrix("M1", matrix, colNum, rowNum);
	      isGraphic = tmc.isGraphic();
	      tmc.setMatrix("M1'", dualMatrix, colNum, dualRowNum);
	      isCoGraphic = tmc.isGraphic();

	      isFirst = false;
	    }
	    matroidOutput.append(projSpace[j].toString());
	    matroidOutput.append(LF);
	  }
	}
	if (isGraphic && isCoGraphic) {
	  output.append("================================"+LF);
	  output.append("Extension ");
	  output.append(i+1);
	  output.append(":"+LF);
	  output.append(FiniteFieldMatroidFormatter.displayWithLabels
			((FiniteFieldMatroid)uniqExtensions.get(i)));
	  output.append(matroidDesc((FiniteFieldMatroid)uniqExtensions.get(i),
				    (Vector)extChains.get(i),
				    (Vector)extDChains.get(i)));
	  output.append("Vectors:"+LF);
	  output.append(matroidOutput.toString());
	  output.append(((isGraphic) ? "Graphic " : "Not graphic "));
	  output.append(((isCoGraphic) ? "CoGraphic"+LF : "Not cographic"+LF));
	}
      }
    }
    else {
      output.append("No extensions found"+LF);
    }
    
    return output.toString();
  }
  
  /**
   * returns the visualization's name 
   */
  public String getName()
  {
    return new String("GF(p) matroid isomorphism extension generator");
  }

  /**
   * returns a string describing the visualization
   */
  public String getDescription()
  {
    return new String("Generates single-element extensions of a GF(p) " + 
		      "matroid and checks them using the matroid " + 
		      "isomorphism checker");
  }

  //
  // private method determines if a FiniteFieldMatroid instance contains a 
  // specified vector
  //
  private boolean matroidContains(FiniteFieldMatroid M, FiniteFieldVector v)
  {
    int i, n=M.getSize();
    boolean found=false;

    for (i=0; i<n; i++) {
      if (M.getElement(i).multiple(v) != 0) {
	found = true;
	break;
      }
    }
    return found;
  }


  //
  // private method outputs a matroid and the chains produced by the
  // MatroidIsomorphismChecker class
  //
  private String matroidDesc(FiniteFieldMatroid matroid, 
			     Vector chains,
			     Vector dChains)
  {
    int r=matroid.getRank(), n=matroid.getSize();
    int i, j, w = 5;
    Vector chain;
    
    StringBuffer answer = new StringBuffer();
    // header
    answer.append("          ");
    for (i=0; i<=n; i++) {
      answer.append(TextAlignment.rightAlign(String.valueOf(i), w));
    }
    answer.append(LF);
    j = n+2;
    for (i=0; i<j; i++) {
      answer.append("-----");
    }
    answer.append(LF);

    // independent sets
    answer.append("Ind Sets  ");
    chain = (Vector)chains.get(0);
    for (i=0; i<=r; i++) {
      answer.append(TextAlignment.rightAlign(chain.get(i).toString(), w));
    }
    answer.append(LF);

    // circuits
    answer.append("Circuits  ");
    chain = (Vector)chains.get(1);
    for (i=0; i<=r+1; i++) {
      answer.append(TextAlignment.rightAlign(chain.get(i).toString(), w));
    }
    answer.append(LF);
    
    r = n-r;
    // coindependent sets
    answer.append("CoInd Sets");
    chain = (Vector)dChains.get(0);
    for (i=0; i<=r; i++) {
      answer.append(TextAlignment.rightAlign(chain.get(i).toString(), w));
    }
    answer.append(LF);

    // cocircuits
    answer.append("CoCircuits");
    chain = (Vector)dChains.get(1);
    for (i=0; i<=r+1; i++) {
      answer.append(TextAlignment.rightAlign(chain.get(i).toString(), w));
    }
    answer.append(LF);

    return answer.toString();
  }

  public static void main (String[] arg)
  {
    VisGF2ChainTutteExtF inst;

    if (0 == arg.length) {
      System.out.println("Usage: java VisGF2ChainTutteExtF <matroid file>");
    }
    else {
      try {
	inst = new VisGF2ChainTutteExtF();
	MatrixFileReader reader = 
	  new MatrixFileReader(arg[0]);
	for (; reader.hasMoreElements(); ) {
	  System.out.println(inst.getDescription(reader.nextElement()));
	}
      }
      catch (Exception ex) {
	System.out.println(ex.toString());
      }
    }
  }
}
