import Oid.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
 * Takes an instance of <code>Matroid</code>, calculates its dual, and
 * uses the matroid and its dual to generate its chains of circuits,
 * cocircuits, independent sets and coindependent sets (using default
 * algorithms). It returns a <code>ReadOnlyTextPanel</code> instance
 * with these chains displayed in table form.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 6 Nov 2001
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class VisMatroidIDMatrix implements Visualization
{
  //
  // private data members
  //
  private Matroid mMatroid;
  private Matroid mDual;

  /** returns a ReadOnlyTextPanel with the matroid's id matrix */
  public JPanel getVisualization(Object obj)
    throws IllegalArgumentException
  {
    if (!(obj instanceof Matroid)) {
      throw new IllegalArgumentException("Requires a matroid");
    }
    if (((ComplexityProvider)obj).getComplexityEstimate("getDual") < 0) {
      throw new IllegalArgumentException("Cannot get dual");
    }
    mMatroid = (Matroid)obj;
    if (mMatroid instanceof FiniteFieldMatroid) {
      mDual = ((FiniteFieldMatroid)mMatroid).getDual();
    }
    else if (mMatroid instanceof BasisMatroid) {
      mDual = ((BasisMatroid)mMatroid).getDual();
    }


    // set up algorithms
    DefaultCircuitAlgorithm cctAlg = new DefaultCircuitAlgorithm();
    DefaultIndependentSetAlgorithm indAlg =
      new DefaultIndependentSetAlgorithm();

    // get chains
    Vector cctChain;
    Vector cocctChain; 
    Vector indChain;
    Vector coindChain;
    try {
      cctAlg.setInstance(mMatroid);
      cctChain = cctAlg.getSubsetCountChain();
      cctAlg.setInstance(mDual);
      cocctChain = cctAlg.getSubsetCountChain();
      indAlg.setInstance(mMatroid);
      indChain = indAlg.getSubsetCountChain();
      indAlg.setInstance(mDual);
      coindChain = indAlg.getSubsetCountChain();
    }
    catch (Exception e) {
      // by now, this shouldn't happen
      throw new InternalError(e.toString());
    }

    //
    // build the matrix
    //
    StringBuffer matrix = new StringBuffer();
    int n = mMatroid.getSize();
    int i, j, r1 = mMatroid.getRank(), r2 = mDual.getRank();
    
    // first row is rank values
    matrix.append("Size        ");
    for (i=0; i<=n; i++) {
      matrix.append(TextAlignment.rightAlign(String.valueOf(i), 8));
    }
    matrix.append('\n');

    matrix.append("------------");
    for (i=0; i<=n; i++) {
      matrix.append("--------");
    }
    matrix.append('\n');

    // next circuits and cocircuits
    matrix.append("Circuits    ");
    for (i=0; i<=r1+1; i++) {
      j = ((Integer)cctChain.get(i)).intValue();
      matrix.append(TextAlignment.rightAlign(String.valueOf(j), 8));
    }
    matrix.append('\n');
    matrix.append("Cocircuits  ");
    for (i=0; i<=r2+1; i++) {
      j = ((Integer)cocctChain.get(i)).intValue();
      matrix.append(TextAlignment.rightAlign(String.valueOf(j), 8));
    }
    matrix.append('\n');
    
    // next independent sets and coindependent sets
    matrix.append("Ind sets    ");
    for (i=0; i<=r1; i++) {
      j = ((Integer)indChain.get(i)).intValue();
      matrix.append(TextAlignment.rightAlign(String.valueOf(j), 8));
    }
    matrix.append('\n');
    matrix.append("Coind sets  ");
    for (i=0; i<=r2; i++) {
      j = ((Integer)coindChain.get(i)).intValue();
      matrix.append(TextAlignment.rightAlign(String.valueOf(j), 8));
    }
    matrix.append('\n');
    
    ReadOnlyTextPanel answer = new ReadOnlyTextPanel();
    answer.setText(matrix.toString());
    return answer;
  }
  
  /** returns the name of this visualization */
  public String getName()
  { 
    return new String("Matroid identification matrix");
  }
  
  public String getDescription()
  { 
    return new String("An VisMatroidIDMatrix takes an instance of Matroid, calculates its dual, and uses the matroid and its dual to generate its chains of circuits, cocircuits, flats, coflats, independent sets and coindependent sets (using default algorithms). It returns a ReadOnlyTextPanel instance with these chains displayed in table form.");
  }
}
