import Oid.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.text.*;

/** 
 * Determines whether a matroid is <strong>negatively
 * correlated</strong>; that is, whether for all e, f in E(M), B(M) <=
 * B(M/e) * B(M/f) / B(M/e,f), where B(M) is the number of bases in M.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 2003-09-03
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class VisNegativeCorrelated implements Visualization
{
  private static String mLF;

  private static String LF() 
  {
    if (!(mLF instanceof String)) {
      mLF = "\n";
    }
    return mLF;
  }

  /** 
   * generates a visualization of the passed object and returns it as
   * a JPanel
   */
  public JPanel getVisualization(Object obj)
    throws IllegalArgumentException {

    if (!(obj instanceof FiniteFieldMatroid)) {
      throw new IllegalArgumentException("Requires a finite field matroid");
    }

    FiniteFieldMatroid M = (FiniteFieldMatroid)obj, Mcef;
    int n=M.getSize(), i, j, e;
    FiniteFieldMatroid[] contraction = new FiniteFieldMatroid[n];
    int MBases, basesC2;
    int[] bases = new int[n];
    boolean negCorr = true;
    StringBuffer txt = new StringBuffer();
    DefaultBasisAlgorithm basAlg = new DefaultBasisAlgorithm();
    DecimalFormat fmt6 = new DecimalFormat("000000");
    DecimalFormat fmt2 = new DecimalFormat("00");

    // 
    // get basis count for M
    //
    basAlg.setInstance(M);
    MBases = basAlg.getSubsetList().size();

    txt.append("B(M) = " + String.valueOf(MBases) + "\n");
    txt.append(" e  f B(M/e) B(M/f) B(M/e,f)\n");
    txt.append("----------------------------\n");

    //
    // want to minimize calls to the basis algorithm
    //
    contraction[0] = M.getContraction(0);
    basAlg.setInstance(contraction[0]);
    bases[0] = basAlg.getSubsetList().size();
    for (i=0; i<n-1 && negCorr; i++) {
      for (j=i+1; j<n; j++) {
	if (!(contraction[j] instanceof FiniteFieldMatroid)) {
	  contraction[j] = M.getContraction(j);
	  basAlg.setInstance(contraction[j]);
	  bases[j] = basAlg.getSubsetList().size();
	}
	e = matroidLabelIndex(contraction[i], M.getLabel(j));
	if (e < 0) {
	  throw new InternalError("label missing");
	}
	Mcef = contraction[i].getContraction(e);
	basAlg.setInstance(Mcef);
	basesC2 = basAlg.getSubsetList().size();
	
	txt.append(padLeft(M.getLabel(i), 2));
	txt.append(" ");
	txt.append(padLeft(M.getLabel(j), 2));
	txt.append(" ");
	txt.append(spaceFmt(fmt6.format(bases[i])));
	txt.append(" ");
	txt.append(spaceFmt(fmt6.format(bases[j])));
	txt.append(" ");
	txt.append(spaceFmt(fmt6.format(basesC2)));
	txt.append("\n");

	if (MBases * basesC2 > bases[i] * bases[j]) {
	  // breaks the condition
	  negCorr = false;
	  txt.append("\nThe matroid is NOT negatively correlated\n");
	  break;
	}
      }
    }

    if (negCorr) {
      txt.append("\nThe matroid IS negatively correlated\n");
    }

    ReadOnlyTextPanel panel = new ReadOnlyTextPanel();
    panel.setText(txt.toString());
    return panel;
  }

  private int matroidLabelIndex(Matroid M, String label) {
    int i, n=M.getSize(), answer=-1;
    for (i=0; i<n; i++) {
      if (M.getLabel(i).equals(label)) {
	answer = i;
	break;
      }
    }
    return answer;
  }

  private String spaceFmt(String s) {
    StringBuffer sb = new StringBuffer(s);
    int n = sb.length();
    for (int i=0; i<n-1; i++) {
      if (sb.charAt(i) == '0') {
	sb.setCharAt(i, ' ');
      }
      else if (sb.charAt(i) != ' ') {
	break;
      }
    }
    return sb.toString();
  }

  private String padLeft(String s, int len) {
    StringBuffer answer = new StringBuffer(s);
    while (answer.length() < len) {
      answer.insert(0, ' ');
    }
    return answer.toString();
  }
  
  /**
   * returns the visualization's name 
   */
  public String getName()
  {
    return new String("Negative correlation checker");
  }

  /**
   * returns a string describing the visualization
   */
  public String getDescription()
  {
    StringBuffer answer = new StringBuffer();
    answer.append("Checks whether a matroid is negatively correlated; ");
    answer.append("that is, B(M) <= B(M/e) * B(M/f) / B(M/e,f) for all ");
    answer.append("e,f in E(M), where B(M) is the number of bases in M.");
    return answer.toString();
  }
}
