import Oid.*;
import java.util.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/** 
 * <code>VisRank3ModularCuts</code> takes a rank 3 matroid and
 * determines its set of modular cuts.  With rank 3, this is just all
 * families of mutually disjoint flats of size greater than one.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 17 April 2003
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class VisRank3ModularCuts implements Visualization
{
  private static String mLF;

  private static String LF() 
  {
    if (!(mLF instanceof String)) {
      mLF = "\n";
    }
    return mLF;
  }

  /** 
   * generates a visualization of the passed object and returns it as
   * a JPanel
   */
  public JPanel getVisualization(Object obj)
    throws IllegalArgumentException
  {
    int i,j,m,n;
    int nFlats, size;
    boolean flag;

    if (!(obj instanceof Matroid)) {
      throw new IllegalArgumentException("requires a matroid");
    }

    Matroid M = (Matroid)obj;

    if (M.getRank() != 3) {
      throw new IllegalArgumentException("requires a rank-3 matroid");
    }

    StringBuffer txt = new StringBuffer();

    try {
      FlatAlgorithm flatAlg = new FlatAlgorithm();

      flatAlg.setInstance(M);
    
      // get all flats with rank 2 into a single vector
      Vector flats = (Vector)((Vector)flatAlg.getSubsetListVector()).get(2);
    
      nFlats = flats.size();

      // output list of flats
      mLF = this.LF();
      txt.append("Rank 2 Flats:" + mLF);
      for (i=0; i<nFlats; i++) {
	txt.append("F");
	txt.append(String.valueOf(i+1));
	txt.append(" = ");
	txt.append(((Subset)flats.get(i)).toString(true) + mLF);
      }
      txt.append(mLF + mLF);

      //
      // list disjoint families of rank 2 flats to make modular cuts
      //
      boolean[] members;
      Subset cut;
      Vector allCuts = new Vector();
      DisjointFamilyEnumeration enum = 
	new DisjointFamilyEnumeration(flats, true);

      txt.append("Modular cuts:" + mLF);
      for (; enum.hasMoreElements(); ) {
	members = (boolean[])enum.nextElement();
	cut = new Subset(nFlats);
	for (i=0; i<nFlats; i++) {
	  if (members[i]) {
	    cut.set(i);
	  }
	}
      
	// output this cut
	txt.append("{");
	flag = false;
	for (i=0; i<nFlats; i++) {
	  if (members[i]) {
	    if (!flag) {
	      flag = true;
	    }
	    else {
	      txt.append(", ");
	    }
	    txt.append("F" + String.valueOf(i+1));
	  }
	}
	txt.append("}" + mLF);

	allCuts.add(cut);
      }

      Vector exts = new Vector();
      Vector extChains = new Vector();
      Vector extDegrees = new Vector();
      Vector extBases = new Vector();
      Vector curChains, curDegrees, curBases;
      Vector lines;
      Iterator pLines;
      LinearSpace curExt, cmpExt;
      int nExt = 0;
      MatroidIsomorphismChecker checker = new MatroidIsomorphismChecker();
      boolean found;

      // now do single-element extensions based on cuts
      txt.append(mLF + mLF + "Cuts producing single-element extensions" 
		 + mLF + mLF);
      for (Iterator pCut = allCuts.iterator(); pCut.hasNext(); ) {
	cut = (Subset)pCut.next();
	curExt = ModCutExtension(M.getSize(), flats, cut);
	checker.setMatroid1(curExt);
	n = exts.size();
	found = false;
	for (i=0; i<n; i++) {
	  checker.setMatroid2((Matroid)exts.get(i),
			      (Vector)extChains.get(i),
			      (Vector)extDegrees.get(i),
			      (Vector)extBases.get(i));
	  if (checker.checkMatroids()) {
	    found = true;
	    break;
	  }
	}
	if (!found) {
	  curExt.setName(M.getName() + "_" + String.valueOf(++nExt));
	  exts.add(curExt);
	  extChains.add(checker.getChains1());
	  extDegrees.add(checker.getDegrees1());
	  extBases.add(checker.getBases1());
	  txt.append(cutDesc(cut) + mLF);
	  lines = curExt.getLineVector();
	  for (pLines = lines.iterator(); pLines.hasNext(); ) {
	    txt.append(" ");
	    txt.append(((Subset)pLines.next()).toStringNoFmt());
	    txt.append(mLF);
	  }
	}
      }
    }
    catch (Exception ex) {
      txt.append(mLF + "Error:" + mLF + ex.toString());
    }

    ReadOnlyTextPanel panel = new ReadOnlyTextPanel();
    panel.setText(txt.toString());
    return panel;
  }

  private String cutDesc(Subset cut)
  {
    StringBuffer answer = new StringBuffer();
    int j, n = cut.getSize();
    boolean flag = false;

    answer.append("{");
    for (j=0; j<n; j++) {
      if (cut.get(j)) {
	if (flag) {
	  answer.append(", ");
	}
	else {
	  flag =true;
	}
	answer.append("F" + String.valueOf(j+1));
      }
    }
    answer.append("}");
    
    return answer.toString();
  }
  
  private LinearSpace ModCutExtension(int n, Vector flats, Subset modCut)
  {
    int i, j, nFlats = flats.size();
    boolean[] inCutLine = new boolean[n];
    Vector newLines = new Vector();
    Subset oldLine, newLine;
    LinearSpace answer;

    for (j=0; j<nFlats; j++) {
      oldLine = (Subset)flats.get(j);
      if (oldLine.numElements() < n) {
	// make new line from old line
	newLine = new Subset(n+1);
	for (i=0; i<n; i++) {
	  if (oldLine.get(i)) {
	    newLine.set(i);
	  }
	}
	// if this line is part of the modular cut, add new element
	if (modCut.get(j)) {
	  newLine.set(n);
	}
	newLines.add(newLine);
      }

      // if line is in modular cut, mark its elements
      if (modCut.get(j)) {
	for (i=0; i<n; i++) {
	  inCutLine[i] = inCutLine[i] || (oldLine.get(i));
	}
      }
	
    }

    // now go back through and create lines for all elements not in cut
    for (i=0; i<n; i++) {
      if (!inCutLine[i]) {
	newLine = new Subset(n+1);
	newLine.set(i);
	newLine.set(n);
	newLines.add(newLine);
      }
    }

    // now create linear space
    answer = new LinearSpace(n+1, 3, newLines);
    return answer;
  }

  /**
   * returns the visualization's name 
   */
  public String getName()
  {
    return new String("Rank 3 modular cuts generator");
  }

  /**
   * returns a string describing the visualization
   */
  public String getDescription()
  {
    StringBuffer answer = new StringBuffer();
    answer.append("VisRank3ModularCuts</code> takes a rank 3 matroid and ");
    answer.append("determines its modular cuts.  With rank 3, this is ");
    answer.append("just all families of mutually disjoint flats of size ");
    answer.append("greater than one.");
    return answer.toString();
  }
}
