import Oid.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/** 
 * <code>VisSizeRankMatrix</code> implements the
 * <code>Visualization</code> interface, and produces a
 * <code>ReadOnlyTextPanel</code> with the Size/Rank matrix for the
 * passed <code>Matroid</code>. It uses the class
 * <code>SizeRankMatrixCalculator</code> to produce the matrix.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 15 June 2002
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class VisSizeRankMatrix implements Visualization
{
  private static String mLF;

  private static String LF() 
  {
    if (!(mLF instanceof String)) {
      mLF = "\n";
    }
    return mLF;
  }

  /** 
   * generates a visualization of the passed object and returns it as
   * a JPanel
   */
  public JPanel getVisualization(Object obj)
    throws IllegalArgumentException
  {
    if (!(obj instanceof Matroid)) {
      throw new IllegalArgumentException("Requires a matroid");
    }

    Matroid M = (Matroid)obj;
    SizeRankMatrixCalculator calc = new SizeRankMatrixCalculator();

    calc.setMatroid(M);
    StringBuffer txt = new StringBuffer();
    int[][] srm = calc.getSizeRankMatrix();
    int i,j,k,l,n=M.getSize(),r=M.getRank();
    int digits;

    // how many digits?
    digits = 0;
    for (i=0; i<=n; i++) {
      for (j=0; j<=r; j++) {
	k = String.valueOf(srm[i][j]).length();
	digits = (k > digits) ? k : digits;
      }
    }
    digits += 1;

    String sp = new String("                    ");
    sp = sp.substring(0,digits);
    StringBuffer cell;

    // header
    txt.append(sp);
    for (j=0; j<=r; j++) {
      cell = new StringBuffer(sp);
      cell.append(j);
      txt.append(cell.substring(cell.length()-digits));
    }
    txt.append(LF());
    
    // matrix
    for (i=0; i<=n; i++) {
      cell = new StringBuffer(sp);
      cell.append(i);
      txt.append(cell.substring(cell.length()-digits));
      for (j=0; j<=r; j++) {
	cell = new StringBuffer(sp);
	cell.append(srm[i][j]);
	txt.append(cell.substring(cell.length()-digits));
      }
      txt.append(LF());
    }

    IntPoly2 rankGenPoly = calc.getRankGeneratingPolynomial();
    txt.append("Rank generating polynomial:" + LF());
    txt.append("R(M; x, y) = ");
    txt.append(rankGenPoly.toString());
    txt.append(LF());
    
    IntPoly chromatic = calc.getChromaticPolynomial();
    txt.append("Chromatic polynomial:" + LF());
    txt.append("P(M; x) =    ");
    txt.append(chromatic.toString());
    txt.append(LF());

    IntPoly2 TuttePoly = calc.getTuttePolynomial();
    txt.append("Tutte polynomial:" + LF());
    txt.append("T(M; x, y) = ");
    txt.append(TuttePoly.toString());
    txt.append(LF());

    IntPoly connPoly = calc.getConnectivityPolynomial();
    txt.append("Connectivity polynomial:" + LF());
    txt.append("C(M; x) =    ");
    txt.append(connPoly.toString());
    txt.append(LF());
    

    // chromatic number
    i = 1;
    j = chromatic.evaluate(i);
    while (j <= 0) {
      i++;
      j = chromatic.evaluate(i);
    }
    txt.append("Chromatic number: ");
    txt.append(i);
    txt.append(LF());

    // critical number
    if (M instanceof FiniteFieldMatroid) {
      i = ((FiniteFieldMatroid)M).getField().getFieldSize();

      j = 1;
      l = 0;
      k = chromatic.evaluate(j);
      while (k <= 0) {
	j *= i;
	k = chromatic.evaluate(j);
	l++;
      }
      txt.append("Critical number: ");
      txt.append(l);
      txt.append(LF());
    }

    txt.append("Number of bases: R(M; 0,0) = ");
    txt.append(rankGenPoly.evaluate(0,0));
    txt.append(LF());

    txt.append("Number of independent sets: R(M; 1, 0) = ");
    txt.append(rankGenPoly.evaluate(1,0));
    txt.append(LF());

    txt.append("Number of spanning sets: R(M; 0, 1) = ");
    txt.append(rankGenPoly.evaluate(0,1));
    txt.append(LF());

    txt.append("Beta invariant: Coefficient on x in T(M; x, y) = ");
    int[][]coeff = TuttePoly.getCoefficientArray();
    txt.append(coeff[1][0]);
    txt.append(LF());

    ReadOnlyTextPanel panel = new ReadOnlyTextPanel();
    panel.setText(txt.toString());
    return panel;
  }
  
  /**
   * returns the visualization's name 
   */
  public String getName()
  {
    return new String("Size/Rank matrix calculator");
  }

  /**
   * returns a string describing the visualization
   */
  public String getDescription()
  {
    StringBuffer answer = new StringBuffer();
    answer.append("VisSizeRankMatrix implements the Visualization interface, and produces");
    answer.append("a ReadOnlyTextPanel with the Size/Rank matrix for the passed Matroid. It");
    answer.append("uses the class SizeRankMatrixCalculator to produce the matrix.");
    return answer.toString();
  }
}
